      subroutine accel

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  accel    Updated     2007 January 16 17:20.
c               accel    Originated  1996 April 1 15:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To interpret data on "accel" input lines, to find the path of
c               a linearly accelerated particle, and its intersection or
c               proximal point with a point, a line, a plane or a quadric
c               surface.
c
c     Logic:    Entry points:  accel.
c accel         Called by:  helpall, main.
c
c accel         Calls:  aptparb, aptparh, aptparl, aptparp, aptparq, aptpars,
c               aptpart, aptparx, aptvaxb, aptvdis, aptvdot, aptvunb, clfind,
c               linefind, plout, ptfind, quout, varfind, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      dimension atype(4)              ! Type of intersection.
      character*8 atype               ! Type of intersection.
      dimension dint(4)               ! Path length to intersection or prox.
      dimension dprox(4)              ! Proximal distance from path.
      dimension dside(4)              ! Value of quadric equation.
      dimension pintx(4)              ! Coordinate x of intersection or prox.
      dimension pinty(4)              ! Coordinate y of intersection or prox.
      dimension pintz(4)              ! Coordinate z of intersection or prox.
      dimension prlnx(4)              ! Coordinate x of proximal point on line.
      dimension prlny(4)              ! Coordinate y of proximal point on line.
      dimension prlnz(4)              ! Coordinate z of proximal point on line.
      dimension ptx(4)                ! Coordinate x of position on trajectory.
      dimension pty(4)                ! Coordinate y of position on trajectory.
      dimension ptz(4)                ! Coordinate y of position on trajectory.
      dimension tint(4)               ! Time to intersection or proximal pt.
      dimension timag(4)              ! Imaginary component of time.
      dimension timep(4)              ! Time of position on trajectory.
      dimension vintx(4)              ! Component x of velocity at intersection.
      dimension vinty(4)              ! Component y of velocity at intersection.
      dimension vintz(4)              ! Component z of velocity at intersection.
      dimension vint(4)               ! Magnitude of velocity at intersection.
      dimension vmx(4)                ! Component x of new initial velocity.
      dimension vmy(4)                ! Component y of new initial velocity.
      dimension vmz(4)                ! Component z of new initial velocity.

c.... Input line:  accelerate [help]

 8110 format ('  Command "accel":  acceptable input lines include:' /
     &  '  "accelerate [help]"          Display this message.' //
     &  '  "accelerate path PINIT VINIT VACC"' /
     &  '     For a particle initially at point PINIT,' /
     &  '     with initial velocity vector VINIT and acceleration',
     &  ' vector VACC,' /
     &  '     find the position, path length and velocity' /
     &  '     at the vertex of its parabolic trajectory.' //
     &  '  "accelerate path PINIT VINIT VACC time TMIN [DT TMAX]"' /
     &  '     For a particle initially at point PINIT,' /
     &  '     with initial velocity vector VINIT and acceleration',
     &  ' vector VACC,' /
     &  '     find the position, path length and velocity at the',
     &  ' vertex of its path,' /
     &  '     and for time[s] TMIN [(DT) TMAX].' //
     &  '  "accelerate path PINIT VINIT VACC point PNAME"' /
     &  '     For a particle initially at point PINIT,' /
     &  '     with initial velocity vector VINIT and acceleration',
     &  ' vector VACC,' /
     &  '     find the points on its path that intersect or are',
     &  ' proximal to' /
     &  '     the point PNAME, and for each such point, the',
     &  ' coordinates,' /
     &  '     the path length and the time.' )

 8112 format (/ '  "accelerate path PINIT VINIT VACC',
     &  ' cluster CLNAME"' /
     &  '     For a particle initially at point PINIT,' /
     &  '     with initial velocity vector VINIT and acceleration',
     &  ' vector VACC,' /
     &  '     find the points on its path that intersect or are',
     &  ' proximal to' /
     &  '     each point in cluster CLNAME, and for each such point,' /
     &  '     the coordinates, the path length and the time.' //
     &  '  "accelerate path PINIT VINIT VACC line ALNAME"' /
     &  '     For a particle initially at point PINIT,' /
     &  '     with initial velocity vector VINIT and acceleration',
     &  ' vector VACC,' /
     &  '     find the points on its path that intersect or are',
     &  ' proximal to' /
     &  '     the line ALNAME, and for each such point, the',
     &  ' coordinates,' /
     &  '     the path length and the time.' //
     & '  "accelerate path PINIT VINIT VACC plane PLNAME"' /
     &  '     For a particle initially at point PINIT,' /
     &  '     with initial velocity vector VINIT and acceleration',
     &  ' vector VACC,' /
     &  '     find the points on its path that intersect or are',
     &  ' proximal to' /
     &  '     the plane PLNAME, and for each such point, the',
     &  ' coordinates,' /
     &  '     the path length and the time.' //
     &  '  Type <return> for more help.' )

 8113 format (a)

 8114 format ('  "accelerate path PINIT VINIT VACC quadric QNAME"' /
     &  '     For a particle initially at point PINIT,' /
     &  '     with initial velocity vector VINIT and acceleration',
     &  ' vector VACC,' /
     &  '     find the points on its path that intersect the quadric',
     &  ' suface QNAME,' /
     &  '     and for each such point, the coordinates,',
     &  ' the path length and the time.' //
     &  '  "accelerate fit P1 [T1 P2 T2 [P3 T3]]"' /
     &  '     For a particle at points P1, P2 and P3 at times',
     &  ' T1, T2 and T3,' /
     &  '     find the position and velocity at time zero, and the',
     &  ' acceleration vector.' //
     &  '  Synonyms:  [accelerate, accel], [help, h], [line, ln, l],',
     &  ' [plane, pl],' /
     &  '    [quadric, quad, q], [time, t].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8113) ans
        endif
        write (   6, 8114)
        write (iout, 8114)
        go to 999
      endif

c=======================================================================********

c.... Input line:  accelerate fit P1 [T1 P2 T2 [P3 T3]]

      if (aword(2) .eq. 'fit') then

        nerr = 0

        if (nword .eq. 3) then
          nword    = 4
          mtype(4) = 3
          fword(4) = 0.0
        endif

 8610   format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 4, 6 or 8 words on "accel =" command:  ', i3)

        if ((nword .ne. 4) .and.
     &      (nword .ne. 6) .and.
     &      (nword .ne. 8)) then
          write (   6, 8610) ainput, ninput, nword
          write (iout, 8610) ainput, ninput, nword
          nerr = 1
        endif

c....   Find the points on the trajectory.

        call ptfind (3, np1)
        if (np1 .eq. 0) nerr = 1

        if (nword .ge. 5) then
          call ptfind (5, np2)
          if (np2 .eq. 0) nerr = 1
        endif

        if (nword .ge. 7) then
          call ptfind (7, np3)
          if (np3 .eq. 0) nerr = 1
        endif

c....   Find the times at the trajectory points.

        do 610 nw = 4, nword, 2       ! Loop over even input line words.

          call varfind (nw, nw)
  
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
            nerr = 1
          endif

  610   continue                      ! End of loop over even input line words.

        if (nerr .eq. 0) then

c....     Find the initial velocity, position and acceleration.

          ptx(1)   = x(np1)
          pty(1)   = y(np1)
          ptz(1)   = z(np1)
          timep(1) = fword(4)

          if (nword .ge. 5) then
            ptx(2)   = x(np2)
            pty(2)   = y(np2)
            ptz(2)   = z(np2)
            timep(2) = fword(6)
          endif

          if (nword .ge. 7) then
            ptx(3)   = x(np3)
            pty(3)   = y(np3)
            ptz(3)   = z(np3)
            timep(3) = fword(8)
          endif
  
          np = 0
          if (nword .eq. 4) np = 1
          if (nword .eq. 6) np = 2
          if (nword .eq. 8) np = 3

          call aptpart (ptx, pty, ptz, timep, np, tol,
     &                  px, py, pz, vx, vy, vz, vlenv,
     &                  ax, ay, az, vlena, nerra)

 8620     format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Two or more times are the same.')

          if (nerra .eq. 1) then
            write (   6, 8610) ainput, ninput, nword
            write (iout, 8610) ainput, ninput, nword
            nerr = 1
          elseif (nerra .eq. 2) then
            write (   6, 8620) ainput, ninput
            write (iout, 8620) ainput, ninput
            nerr = 1
          endif

        endif

        if (nerr .ne. 0) go to 999

 8660   format (/ 'Initial values, at time zero:' )
 8665   format (  'Position (xyz)    ',1p3e20.12)
 8670   format (  'Velocity (xyz)    ',1p3e20.12 /
     &            '  speed           ',1pe20.12 )
 8675   format (  'Accel    (xyz)    ',1p3e20.12 /
     &            '  magnitude       ',1pe20.12 )

        write (   6, 8660)
        write (iout, 8660)

        write (   6, 8665) px, py, pz
        write (iout, 8665) px, py, pz

        write (   6, 8670) vx, vy, vz, vlenv
        write (iout, 8670) vx, vy, vz, vlenv

        write (   6, 8675) ax, ay, az, vlena
        write (iout, 8675) ax, ay, az, vlena

        go to 999

      endif                           ! Input line "accelerate fit ...".

c=======================================================================********

c.... Input line:  accelerate path PINIT VINIT VACC
c.... Input line:  accelerate path PINIT VINIT VACC time TMIN [DT TMAX]
c.... Input line:  accelerate path PINIT VINIT VACC point   PNAME
c.... Input line:  accelerate path PINIT VINIT VACC cluster CLNAME
c.... Input line:  accelerate path PINIT VINIT VACC line    ALNAME
c.... Input line:  accelerate path PINIT VINIT VACC plane   PLNAME
c.... Input line:  accelerate path PINIT VINIT VACC quadric QNAME

      if (aword(2) .eq. 'path') then

c.... Find the initial position point.

      nerr = 0

      call ptfind (3, np)

      if (np .eq. 0) nerr = 1

c.... Find the initial velocity vector.

      call vectfind (4, nv)

      if (nv .eq. 0) nerr = 1

c.... Find the acceleration vector.

      call vectfind (5, na)

      if (na .eq. 0) nerr = 1

      if (nerr .eq. 0) then

c.... Find the dot product of the velocity and acceleration vectors.

      call aptvdot (xcom(nv), ycom(nv), zcom(nv),
     &              xcom(na), ycom(na), zcom(na), 1, tol, vdota, nerra)

c.... Find the vector normal to the plane of the particle path.

      call aptvaxb (xcom(nv), ycom(nv), zcom(nv),
     &              xcom(na), ycom(na), zcom(na), 1, tol,
     &              xpln,     ypln,     zpln,     vnlen, nerra)

 8150 format (/ 'Particle path is a point.')
 8152 format (/ 'Particle path is a simple straight line.')
 8154 format (/ 'Particle path is a degenerate linear parabola.')
 8160 format (/ 'Particle path is parabolic, in a plane with',
     &  ' normal vector:' /
     &          'Normal vect (xyz) ',1p3e20.12)

      if (vnlen .eq. 0.0) then
        if (vlen(na) .eq. 0.0) then
          if (vlen(nv) .eq. 0.0) then
            write (   6, 8150)
            write (iout, 8150)
          else
            write (   6, 8152)
            write (iout, 8152)
          endif
        else
          write (   6, 8154)
          write (iout, 8154)
        endif
      else
        write (   6, 8160) xpln, ypln, zpln
        write (iout, 8160) xpln, ypln, zpln
      endif

c.... Display the results at any vertex point of the particle path.

 8210 format (/ 'Time, path length ',1p2e20.12)
 8240 format (  'Position (xyz)    ',1p3e20.12)
 8230 format (  'Velocity (xyz)    ',1p3e20.12 /
     &          '  speed           ',1pe20.12 )
 8235 format (' ')
 8250 format (/ 'At vertex of path of particle:')

      if (vlen(na) .gt. 0.0) then     ! Acceleration is non-zero.

        write (   6, 8250)
        write (iout, 8250)

        tver  = -vdota / vlen(na)**2  ! Time to reach vertex of curve.

        call aptparb (x(np), y(np), z(np),
     &                xcom(nv), ycom(nv), zcom(nv),
     &                xcom(na), ycom(na), zcom(na), tver, tol,
     &                ppx, ppy, ppz, vtx, vty, vtz, vtlen,
     &                dpath, ntype)

        write (   6, 8210) tver, dpath
        write (iout, 8210) tver, dpath

        write (   6, 8240) ppx, ppy, ppz
        write (iout, 8240) ppx, ppy, ppz

        write (   6, 8230) vtx, vty, vtz, vtlen
        write (iout, 8230) vtx, vty, vtz, vtlen

        write (   6, 8235)
        write (iout, 8235)

      endif                           ! Acceleration is non-zero.

      endif                           ! No input errors in first 5 words.

      if (nword .eq. 5) go to 999

c=======================================================================********

c.... Input line:  accelerate path PINIT VINIT VACC time TMIN [DT TMAX]

      if ((aword(6) .eq. 'time') .or.
     &    (aword(6) .eq. 't'))   then

c.... Test for wrong number of words on "accel ... time" command.

 8120 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 or 9 words on "accel ... time ..." command:  ', i3)
      if ((nword .ne. 7) .and. (nword .ne. 9)) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

      if (nword .eq. 7) then
        nword = 9
        aword(8) = '0'
        mtype(8) = 2
        iword(8) = 0
        fword(8) = 0.0
        aword(9) = aword(7)
        mtype(9) = mtype(7)
        iword(9) = iword(7)
        fword(9) = fword(7)
      endif

c.... Find the time limits and increment.

 8130 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      call varfind (7, 9)

      do 130 nw = 7, 9
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          nerr = 1
        endif
  130 continue

 8140 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  TMIN > TMAX:  ',1p2e20.12)

      if (fword(9) .lt. fword(7)) then
        write (   6, 8140) ainput, ninput, fword(7), fword(9)
        write (iout, 8140) ainput, ninput, fword(7), fword(9)
        nerr = 1
      endif

 8145 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  DT < 0:  ',1pe20.12)

      if (fword(8) .lt. 0.0) then
        write (   6, 8145) ainput, ninput, fword(8)
        write (iout, 8145) ainput, ninput, fword(8)
        nerr = 1
      endif

 8148 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  DT = 0 and TMAX > TMIN:  ',1p2e20.12)

      if ((fword(8) .eq. 0.0)       .and.
     &    (fword(9) .gt. fword(7))) then
        write (   6, 8145) ainput, ninput, fword(7), fword(9)
        write (iout, 8145) ainput, ninput, fword(7), fword(9)
        nerr = 1
      endif

      if (nerr .ne. 0) go to 999

c.... Initialize.

      tmin  = fword(7)
      dt    = fword(8)
      tmax  = fword(9)
      terr  = tol * (abs (tmin) + abs (tmax))

c.... Display the results at times tmin (dt) tmax.

 8260 format (/ 'At other specified times:')

      write (   6, 8260)
      write (iout, 8260)

      time = tmin

  140 call aptparb (x(np), y(np), z(np), xcom(nv), ycom(nv), zcom(nv),
     &              xcom(na), ycom(na), zcom(na), time, tol,
     &              ppx, ppy, ppz, vtx, vty, vtz, vtlen, dpath, ntype)

      write (   6, 8210) time, dpath
      write (iout, 8210) time, dpath

      write (   6, 8240) ppx, ppy, ppz
      write (iout, 8240) ppx, ppy, ppz

      write (   6, 8230) vtx, vty, vtz, vtlen
      write (iout, 8230) vtx, vty, vtz, vtlen

c.... See if the final time has been reached.

      if ((tmax - time) .gt. terr) then
        time = time + dt
        if (time .gt. tmax) time = tmax
        go to 140
      endif

      go to 999

      endif                       ! Input line "accelerate path ... time ...".

c=======================================================================********

c.... Input line:  accelerate path PINIT VINIT VACC point PNAME

      if ((aword(6) .eq. 'point') .or.
     &    (aword(6) .eq. 'pnt')   .or.
     &    (aword(6) .eq. 'pt')    .or.
     &    (aword(6) .eq. 'p'))    then

c.... Test for wrong number of words on "accel ... point ..." command.

 8710 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 words on "accel ... point ..." command:  ', i3)
      if (nword .ne. 7) then
        write (   6, 8710) ainput, ninput, nword
        write (iout, 8710) ainput, ninput, nword
        go to 999
      endif

c.... Find the point.

      call ptfind (7, npt)

      if (npt .eq. 0) go to 999

c.... Find the points on the particle path nearest the target point.

      call aptpars (x(np),     y(np),     z(np),
     &              xcom(nv),  ycom(nv),  zcom(nv),
     &              xcom(na),  ycom(na),  zcom(na),
     &              x(npt),    y(npt),    z(npt), tol,
     &              nints, atype, pintx, pinty, pintz,
     &              vintx, vinty, vintz, vint,
     &              dint, dprox, tint)

 8720 format (/ 'Number of proximal points = ',i2)
 8730 format (/ 'Path point type     ',a /
     &          'Proximal distance ',1pe20.12)
 8735 format (  'Time, path length ',1p2e20.12)
 8740 format (  'Position (xyz)    ',1p3e20.12)
 8750 format (  'Velocity (xyz)    ',1p3e20.12 /
     &          '  speed           ',1pe20.12 )

      write (   6, 8720) nints
      write (iout, 8720) nints

c.... Find the minimum proximal point.

      nmin     = 0
      dproxmin = 1.e99

      do 710 n = 1, nints             ! Loop over proximal points.
        if (abs (dprox(n)) .lt. dproxmin) then
          nmin     = n
          dproxmin = abs (dprox(n))
        endif
  710 continue
      if (atype(nmin) .eq. 'proximal') atype(nmin) = 'prox min'

      do 720 n = 1, nints             ! Loop over proximal points.

        eprox = tol * (abs (dprox(n)) + abs (dproxmin))
        diff  = abs (dprox(n)) - dproxmin

        if ((diff .le. eprox)           .and.
     &      (atype(n) .eq. 'proximal')) then
          atype(n) = 'prox min'
        endif

        write (   6, 8730) atype(n), dprox(n)
        write (iout, 8730) atype(n), dprox(n)

        write (   6, 8735) tint(n), dint(n)
        write (iout, 8735) tint(n), dint(n)

        write (   6, 8740) pintx(n), pinty(n), pintz(n)
        write (iout, 8740) pintx(n), pinty(n), pintz(n)

        write (   6, 8750) vintx(n), vinty(n), vintz(n), vint(n)
        write (iout, 8750) vintx(n), vinty(n), vintz(n), vint(n)

  720 continue                        ! End of loop over proximal points.

c.... Store the proximal point and proximal vector.

      pdist   = dprox(nmin)
      proxptx = pintx(nmin)
      proxpty = pinty(nmin)
      proxptz = pintz(nmin)

      call aptvdis (pintx(nmin), pinty(nmin), pintz(nmin),
     &              x(npt), y(npt), z(npt), 1, tol,
     &              proxvx, proxvy, proxvz, proxlen, nerra)

c.... Find the initial vector vm = (vmx, vmy, vmz) to hit the point exactly.

      if (dproxmin .eq. 0.0) go to 999
 
      call aptparh (x(np), y(np), z(np), vlen(nv),
     &              xcom(na), ycom(na), zcom(na),
     &              x(npt), y(npt), z(npt), tol,
     &              nints, vmx, vmy, vmz, vintx, vinty, vintz, vint,
     &              dint, tint)

 8765 format (/ 'Can not intersect with this initial speed.')

      if (nints .eq. 0) then
        write (   6, 8765)
        write (iout, 8765)
        go to 999
      endif

 8770 format (/ 'Number of intersections with same speed',
     &  ' and new initial velocity = ',i2)
 8780 format (/ 'Init velocity(xyz)',1p3e20.12 /
     &          '  speed           ',1pe20.12 )
 8785 format (  'Velocity (xyz)    ',1p3e20.12 /
     &          '  speed           ',1pe20.12 )

      write (   6, 8770) nints
      write (iout, 8770) nints

      do 780 n = 1, nints

        write (   6, 8780) vmx(n), vmy(n), vmz(n), vlen(nv)
        write (iout, 8780) vmx(n), vmy(n), vmz(n), vlen(nv)
  
        write (   6, 8735) tint(n), dint(n)
        write (iout, 8735) tint(n), dint(n)
  
        write (   6, 8785) vintx(n), vinty(n), vintz(n), vint(n)
        write (iout, 8785) vintx(n), vinty(n), vintz(n), vint(n)

  780 continue

      go to 999

      endif                      ! Input line "accelerate path ... point ...".

c=======================================================================********

c.... Input line:  accelerate path PINIT VINIT VACC cluster CLNAME

      if ((aword(6) .eq. 'cluster') .or.
     &    (aword(6) .eq. 'cl'))     then

c.... Test for wrong number of words on "accel ... cluster ..." command.

 8630 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 words on "accel ... cluster ..." command:  ', i3)
      if (nword .ne. 7) then
        write (   6, 8630) ainput, ninput, nword
        write (iout, 8630) ainput, ninput, nword
        go to 999
      endif

c.... Find the cluster.

      call clfind (7, ncl)

      if (ncl .eq. 0) go to 999

c.... Loop over points in cluster.

      do 660 npr = 1, npairs          ! Loop over cluster pairs.

      if (apaircl(npr) .ne. aclust(ncl)) go to 660

      nwx = 1
      aword(1) = apairpt(npr)

      write (   6, 8525)
      write (iout, 8525)

      call ptfind (nwx, np)

      if (np .eq. 0) go to 660

c.... Find the points on the particle path nearest the target point.

      call aptpars (x(np),     y(np),     z(np),
     &              xcom(nv),  ycom(nv),  zcom(nv),
     &              xcom(na),  ycom(na),  zcom(na),
     &              x(npt),    y(npt),    z(npt), tol,
     &              nints, atype, pintx, pinty, pintz,
     &              vintx, vinty, vintz, vint,
     &              dint, dprox, tint)

      write (   6, 8720) nints
      write (iout, 8720) nints

c.... Find the minimum proximal point.

      nmin     = 0
      dproxmin = 1.e99

      do 630 n = 1, nints             ! Loop over proximal points.
        if (abs (dprox(n)) .lt. dproxmin) then
          nmin     = n
          dproxmin = abs (dprox(n))
        endif
  630 continue
      if (atype(nmin) .eq. 'proximal') atype(nmin) = 'prox min'

      do 640 n = 1, nints             ! Loop over proximal points.

        eprox = tol * (abs (dprox(n)) + abs (dproxmin))
        diff  = abs (dprox(n)) - dproxmin

        if ((diff .le. eprox)           .and.
     &      (atype(n) .eq. 'proximal')) then
          atype(n) = 'prox min'
        endif

        write (   6, 8730) atype(n), dprox(n)
        write (iout, 8730) atype(n), dprox(n)

        write (   6, 8735) tint(n), dint(n)
        write (iout, 8735) tint(n), dint(n)

        write (   6, 8740) pintx(n), pinty(n), pintz(n)
        write (iout, 8740) pintx(n), pinty(n), pintz(n)

        write (   6, 8750) vintx(n), vinty(n), vintz(n), vint(n)
        write (iout, 8750) vintx(n), vinty(n), vintz(n), vint(n)

  640 continue                        ! End of loop over proximal points.

c.... Store the proximal point and proximal vector.

      pdist   = dprox(nmin)
      proxptx = pintx(nmin)
      proxpty = pinty(nmin)
      proxptz = pintz(nmin)

      call aptvdis (pintx(nmin), pinty(nmin), pintz(nmin),
     &              x(npt), y(npt), z(npt), 1, tol,
     &              proxvx, proxvy, proxvz, proxlen, nerra)

c.... Find the initial vector vm = (vmx, vmy, vmz) to hit the point exactly.

      if (dproxmin .eq. 0.0) go to 660
 
      call aptparh (x(np), y(np), z(np), vlen(nv),
     &              xcom(na), ycom(na), zcom(na),
     &              x(npt), y(npt), z(npt), tol,
     &              nints, vmx, vmy, vmz, vintx, vinty, vintz, vint,
     &              dint, tint)

      if (nints .eq. 0) then
        write (   6, 8765)
        write (iout, 8765)
        go to 660
      endif

      write (   6, 8770) nints
      write (iout, 8770) nints

      do 650 n = 1, nints

        write (   6, 8780) vmx(n), vmy(n), vmz(n), vlen(nv)
        write (iout, 8780) vmx(n), vmy(n), vmz(n), vlen(nv)
  
        write (   6, 8735) tint(n), dint(n)
        write (iout, 8735) tint(n), dint(n)
  
        write (   6, 8785) vintx(n), vinty(n), vintz(n), vint(n)
        write (iout, 8785) vintx(n), vinty(n), vintz(n), vint(n)

  650 continue

  660 continue                        ! End of loop over cluster pairs.

      go to 999

      endif                           ! Input line "accel path ... cluster ...".

c=======================================================================********

c.... Input line:  accelerate path PINIT VINIT VACC line ALNAME

      if ((aword(6) .eq. 'line') .or.
     &    (aword(6) .eq. 'ln')   .or.
     &    (aword(6) .eq. 'l'))   then

c.... Test for wrong number of words on "accel ... line ..." command.

 8755 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 words on "accel ... line ..." command:  ', i3)
      if (nword .ne. 7) then
        write (   6, 8755) ainput, ninput, nword
        write (iout, 8755) ainput, ninput, nword
        go to 999
      endif

c.... Find the line.

      call linefind (7, nl, np1, np2, nerrln)

      if (nerrln .ne. 0) go to 999

c.... Find the points on the particle path nearest the line, and the
c....   corresponding proximal points on the line.

      call aptparl (x(np),     y(np),     z(np),
     &              xcom(nv),  ycom(nv),  zcom(nv),
     &              xcom(na),  ycom(na),  zcom(na),
     &              x(np1),    y(np1),    z(np1),
     &              x(np2),    y(np2),    z(np2), tol,
     &              nints, atype, pintx, pinty, pintz,
     &              vintx, vinty, vintz, vint, dint, dprox, tint,
     &              prlnx, prlny, prlnz, nerr)

 8760 format (  'Line point (xyz)  ',1p3e20.12)

      write (   6, 8720) nints
      write (iout, 8720) nints

c.... Find the minimum proximal point.

      nmin     = 0
      dproxmin = 1.e99

      do 760 n = 1, nints             ! Loop over proximal points.
        if (dprox(n) .lt. dproxmin) then
          nmin     = n
          dproxmin = dprox(n)
        endif
  760 continue
      if (atype(nmin) .eq. 'proximal') atype(nmin) = 'prox min'

      do 770 n = 1, nints             ! Loop over proximal points.

        eprox = tol * (abs (dprox(n)) + abs (dproxmin))
        diff  = abs (dprox(n) - dproxmin)

        if ((diff .le. eprox)           .and.
     &      (atype(n) .eq. 'proximal')) then
          atype(n) = 'prox min'
        endif

        write (   6, 8730) atype(n), dprox(n)
        write (iout, 8730) atype(n), dprox(n)

        write (   6, 8735) tint(n), dint(n)
        write (iout, 8735) tint(n), dint(n)

        write (   6, 8740) pintx(n), pinty(n), pintz(n)
        write (iout, 8740) pintx(n), pinty(n), pintz(n)

        write (   6, 8750) vintx(n), vinty(n), vintz(n), vint(n)
        write (iout, 8750) vintx(n), vinty(n), vintz(n), vint(n)

        write (   6, 8760) prlnx(n), prlny(n), prlnz(n)
        write (iout, 8760) prlnx(n), prlny(n), prlnz(n)

  770 continue                        ! End of loop over proximal points.

c.... Store the proximal point and proximal vector.

      pdist   = dprox(nmin)
      proxptx = pintx(nmin)
      proxpty = pinty(nmin)
      proxptz = pintz(nmin)

      call aptvdis (pintx(nmin), pinty(nmin), pintz(nmin),
     &              x(npt), y(npt), z(npt), 1, tol,
     &              proxvx, proxvy, proxvz, proxlen, nerra)

      go to 999

      endif                      ! Input line "accelerate path ... line ...".

c=======================================================================********

c.... Input line:  accelerate path PINIT VINIT VACC plane PLNAME

      if ((aword(6) .eq. 'plane') .or.
     &    (aword(6) .eq. 'pl'))   then

c.... Test for wrong number of words on "accel ... plane ..." command.

 8310 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 words on "accel ... plane ..." command:  ', i3)
      if (nword .ne. 7) then
        write (   6, 8310) ainput, ninput, nword
        write (iout, 8310) ainput, ninput, nword
        go to 999
      endif

c.... Find the plane.

 8320 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Plane does not exist:  ',a)

      do 320 n = 1, nquads
        if (aqtype(n) .ne. aqtypes(0)) go to 320
        if (aword(7) .eq. aquad(n)) then
          npl = n
          call plout (n, n)
          go to 330
        endif
  320 continue
      write (   6, 8320) ainput, ninput, aword(7)
      write (iout, 8320) ainput, ninput, aword(7)
      nerr = 1

c.... Find the magnitude of the normal vector to the plane.

  330 call aptvunb (xcom(npl), ycom(npl), zcom(npl), 1, tol,
     &              ux, uy, uz, vlenpl, nerra)

 8325 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Normal vector of plane has zero length')

      if (vlenpl .eq. 0.0) then
        write (   6, 8325) ainput, ninput
        write (iout, 8325) ainput, ninput
        nerr = 1
      endif

      if (nerr .ne. 0) go to 999

c.... Find any intersections of the particle path and the plane.

      call aptparp (x(np),     y(np),     z(np),
     &              xcom(nv),  ycom(nv),  zcom(nv),
     &              xcom(na),  ycom(na),  zcom(na),
     &              xcen(npl), ycen(npl), zcen(npl),
     &              qx(npl),   qy(npl),   qz(npl), tol,
     &              nints, pintx, pinty, pintz,
     &              vintx, vinty, vintz, vint, dint, tint, nerra)


      if (nints .eq. 0) then          ! Proximal point only.

 8405   format (/ 'No intersections.  One pair of proximal points:')
 8420   format ('Prox pt path (xyz)',1p3e20.12)
 8430   format ('Prox pt plane(xyz)',1p3e20.12)
 8435   format ('Prox distance     ',1pe20.12)

        write (   6, 8405)
        write (iout, 8405)

        write (   6, 8210) tint(1), dint(1)
        write (iout, 8210) tint(1), dint(1)

        write (   6, 8420) pintx(1), pinty(1), pintz(1)
        write (iout, 8420) pintx(1), pinty(1), pintz(1)

        write (   6, 8230) vintx(1), vinty(1), vintz(1), vint(1)
        write (iout, 8230) vintx(1), vinty(1), vintz(1), vint(1)

        write (   6, 8430) pintx(2), pinty(2), pintz(2)
        write (iout, 8430) pintx(2), pinty(2), pintz(2)

        write (   6, 8435) dint(2)
        write (iout, 8435) dint(2)

c....   Find the proximal point and vector.

        pdist   = dint(2)
        proxptx = pintx(1)
        proxpty = pinty(1)
        proxptz = pintz(1)

        call aptvdis (pintx(1), pinty(1), pintz(1),
     &                pintx(2), pinty(2), pintz(2), 1, tol,
     &                proxvx, proxvy, proxvz, proxlen, nerra)

      elseif (nints .eq. -1) then     ! One tangent point.

 8440   format (/ 'One tangent point:')

        write (   6, 8440)
        write (iout, 8440)

        write (   6, 8210) tint(1), dint(1)
        write (iout, 8210) tint(1), dint(1)

        write (   6, 8240) pintx(1), pinty(1), pintz(1)
        write (iout, 8240) pintx(1), pinty(1), pintz(1)

        write (   6, 8230) vintx(1), vinty(1), vintz(1), vint(1)
        write (iout, 8230) vintx(1), vinty(1), vintz(1), vint(1)

      elseif (nints .eq. 1) then      ! One intersection.

 8445   format (/ 'One intersection:')

        write (   6, 8445)
        write (iout, 8445)

        write (   6, 8210) tint(1), dint(1)
        write (iout, 8210) tint(1), dint(1)

        write (   6, 8240) pintx(1), pinty(1), pintz(1)
        write (iout, 8240) pintx(1), pinty(1), pintz(1)

        write (   6, 8230) vintx(1), vinty(1), vintz(1), vint(1)
        write (iout, 8230) vintx(1), vinty(1), vintz(1), vint(1)

      elseif (nints .eq. 2) then      ! Parabolic path, two intersections.

 8465   format (/ 'Two intersections:')

        write (   6, 8465)
        write (iout, 8465)

        write (   6, 8210) tint(1), dint(1)
        write (iout, 8210) tint(1), dint(1)

        write (   6, 8240) pintx(1), pinty(1), pintz(1)
        write (iout, 8240) pintx(1), pinty(1), pintz(1)

        write (   6, 8230) vintx(1), vinty(1), vintz(1), vint(1)
        write (iout, 8230) vintx(1), vinty(1), vintz(1), vint(1)

        write (   6, 8210) tint(2), dint(2)
        write (iout, 8210) tint(2), dint(2)

        write (   6, 8240) pintx(2), pinty(2), pintz(2)
        write (iout, 8240) pintx(2), pinty(2), pintz(2)

        write (   6, 8230) vintx(2), vinty(2), vintz(2), vint(2)
        write (iout, 8230) vintx(2), vinty(2), vintz(2), vint(2)

      endif                           ! Tested number of intersections.

      go to 999

      endif                      ! Input line "accelerate path ... plane ...".

c=======================================================================********

c.... Input line:  accelerate path PINIT VINIT VACC quadric QNAME

      if ((aword(6) .eq. 'quadric') .or.
     &    (aword(6) .eq. 'quad')    .or.
     &    (aword(6) .eq. 'q'))      then

c.... Test for wrong number of words on "accel ... quadric ..." command.

 8510 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 words on "accel ... quadric ..." command:  ', i3)
      if (nword .ne. 7) then
        write (   6, 8510) ainput, ninput, nword
        write (iout, 8510) ainput, ninput, nword
        go to 999
      endif

c.... Find the quadric surface.

 8520 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric surface does not exist:  ',a)
 8525 format (/ )

      do 520 n = 1, nquads
        if (aword(7) .eq. aquad(n)) then
          nq = n
          write (   6, 8525)
          write (iout, 8525)
          call quout (n, n, 'min', nerrq)
          go to 530
        endif
  520 continue
      write (   6, 8520) ainput, ninput, aword(7)
      write (iout, 8520) ainput, ninput, aword(7)
      nerr = 1

  530 continue

      if (nerr .ne. 0) go to 999

c.... Find the intersections of the parabolic path with the quartic surface.

      call aptparq (x(np), y(np), z(np), xcom(nv), ycom(nv), zcom(nv),
     &              xcom(na), ycom(na), zcom(na),
     &              qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), tol,
     &              nints, atype, pintx, pinty, pintz,
     &              vintx, vinty, vintz, vint, dint, tint, nerra)


      if (nints .eq. 0) then          ! No intersections.

 8535   format (/ 'No intersections.')

        write (   6, 8535)
        write (iout, 8535)

c....   Find points of extreme values of the quadric equation.
 
        call aptparx (x(np), y(np), z(np), xcom(nv), ycom(nv), zcom(nv),
     &                xcom(na), ycom(na), zcom(na),
     &                qc(nq), qx(nq), qy(nq), qz(nq),
     &                qxy(nq), qyz(nq), qzx(nq),
     &                qxx(nq), qyy(nq), qzz(nq), tol,
     &                nprox, atype, pintx, pinty, pintz,
     &                vintx, vinty, vintz, vint, dint, tint,
     &                dside, nerra)

 8538   format (/ 'Number of extrema of side measure on path =',i2)

        write (   6, 8538) nprox
        write (iout, 8538) nprox

        if (nprox .gt. 0) then        ! One or moree extrema of dside.

 8540     format (/ 'Extremum type       ',a /
     &              'Side measure      ',1pe20.12 /
     &              'Time, path length ',1p2e20.12 /
     &              'Position (xyz)    ',1p3e20.12 /
     &              'Velocity (xyz)    ',1p3e20.12 /
     &              '  speed           ',1pe20.12 )
 
          write (   6, 8540) (atype(n), dside(n), tint(n),  dint(n),
     &                        pintx(n), pinty(n), pintz(n),
     &                        vintx(n), vinty(n), vintz(n), vint(n),
     &                        n = 1, nprox)
          write (iout, 8540) (atype(n), dside(n), tint(n),  dint(n),
     &                        pintx(n), pinty(n), pintz(n),
     &                        vintx(n), vinty(n), vintz(n), vint(n),
     &                        n = 1, nprox)

        endif                         ! One or more extrema of dside.

      else                            ! One or more intersections.

 8565   format (/ 'Number of intersections =',i2)

        write (   6, 8565) nints
        write (iout, 8565) nints

 8532   format (/ 'Intersection type   ',a /
     &            'Time, path length ',1p2e20.12 /
     &            'Position (xyz)    ',1p3e20.12 /
     &            'Velocity (xyz)    ',1p3e20.12 /
     &            '  speed           ',1pe20.12 )
 
        write (   6, 8532) (atype(n), tint(n),  dint(n),
     &                      pintx(n), pinty(n), pintz(n),
     &                      vintx(n), vinty(n), vintz(n), vint(n),
     &                      n = 1, nints)
        write (iout, 8532) (atype(n), tint(n),  dint(n),
     &                      pintx(n), pinty(n), pintz(n),
     &                      vintx(n), vinty(n), vintz(n), vint(n),
     &                      n = 1, nints)

      endif                           ! Tested number of intersections.

      go to 999

      endif                    ! Input line "accelerate path ... quadric ...".

      endif                    ! Input line "accelerate path ...">

c=======================================================================********

 8810 format ('  accel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine accel.        (+1 line)
      end
