      subroutine area

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  area     Updated     1995 April 18 14:40.
c               area     Originated  1994 March 4 13:50.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find an area specified by a polygon with 3-36 vertex points,
c                 projected perpendicularly onto a specified plane.
c                 The polygon vertex points need not be coplanar.
c
c     Logic:    Entry points:  area.
c area          Called by:  helpall, main.
c
c area          Calls:  aptcsys, aptmopv, aptrotv, aptvusz, clout, pout,
c               ptfind, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /larea/ xp(npairm)
      common /larea/ yp(npairm)
      common /larea/ zp(npairm)
      common /larea/ rotm(3,3)

c.... Input line:  area [help]

 8110 format (
     &  '  Command "area":  acceptable input lines include:' /
     &  '  "area [help]"  Display this message.' //
     &  '  "area VNORM point PNAME1 PNAME2 PNAME3 ..."' /
     &  '     Find the area projected onto a plane with normal',
     &  ' vector VNORM' /
     &  '     by the polygon with vertex points PNAME1, PNAME2,',
     &  ' PNAME3, ...' /
     &  '  "area VNORM cluster CLNAME"' /
     &  '     Find the area projected onto a plane with normal',
     &  ' vector VNORM' /
     &  '     by the polygon formed by the points in cluster CLNAME.' //
     &  '  Synonyms:  [cluster, cl], [help, h], [point, pnt, pt, p].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Find the normal vector, and the corresponding unit vector.

 8130 format ('  area ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Direction vector has zero length:  ',a)

      nerr = 0

      call vectfind (2, nv)

      if (nv .eq. 0) then
        nerr = 1
        go to 130
      endif

      call aptvusz (xcom(nv), ycom(nv), zcom(nv), tol,
     &              ux, uy, uz, vlength)

      if (vlength .le. 0.0) then
        nerr = 1
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
      endif

c.... Find the points defining the polygon, and the average point.

  130 npver = 0

      puavg = 0.0
      pvavg = 0.0
      pwavg = 0.0

c=======================================================================********

c.... Input line:  area VNORM point PNAME1 PNAME2 PNAME3 ...

      if ((aword(3) .eq. 'point') .or.
     &    (aword(3) .eq. 'pnt')   .or.
     &    (aword(3) .eq. 'pt')    .or.
     &    (aword(3) .eq. 'p'))    then

 8135 format ('  area ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of vertex points less than 3:  ',i3)

      if (nword .lt. 6) then
        nerr = 1
        write (   6, 8135) ainput, ninput, nword
        write (iout, 8135) ainput, ninput, nword
      endif

      do 150 nw = 4, nword

        call ptfind (nw, np)

        if (np .eq. 0) then
          nerr = 1
          go to 150
        endif

        npver     = npver + 1
        xp(npver) = x(np)
        yp(npver) = y(np)
        zp(npver) = z(np)
        puavg     = puavg + x(np)
        pvavg     = pvavg + y(np)
        pwavg     = pwavg + z(np)

  150 continue

c=======================================================================********

c.... Input line:  area VNORM cluster CLNAME

      elseif ((aword(3) .eq. 'cluster') .or.
     &        (aword(3) .eq. 'cl'))     then

 8155 format ('  area ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of vertex points not 4:  ',i3)

      if (nword .ne. 4) then
        nerr = 1
        write (   6, 8155) ainput, ninput, nword
        write (iout, 8155) ainput, ninput, nword
      endif

 8160 format ('  area ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no clusters.')

      if (nclusts .eq. 0) then
        write (   6, 8160) ainput, ninput
        write (iout, 8160) ainput, ninput
        go to 999
      endif

      do 190 n = 1, nclusts           ! Loop over clusters.
        if (aword(4) .ne. aclust(n)) go to 190

        call clout (n, n, 'min')      ! Found specified cluster.

        if (aclust(n) .eq. ' ') go to 200

        do 180 npr = 1, npairs        ! Loop over cluster pairs.
          if (apaircl(npr) .ne. aclust(n)) go to 180

          do 170 np = 1, npoints      ! Loop over points.
            if (apoint(np) .ne. apairpt(npr)) go to 170

            call pout (np, np, 'min') ! Found a point in cluster.

            npver     = npver + 1
            xp(npver) = x(np)
            yp(npver) = y(np)
            zp(npver) = z(np)
            puavg     = puavg + x(np)
            pvavg     = pvavg + y(np)
            pwavg     = pwavg + z(np)

            go to 180                 ! Find next point in cluster.
  170     continue                    ! End of loop over points.

 8170     format ('  area ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named cluster point does not exist:  ',a)

          write (   6, 8170) ainput, ninput, apairpt(npr)
          write (iout, 8170) ainput, ninput, apairpt(npr)
          nerr = 1

  180   continue                      ! End of loop over cluster pairs.

        go to 210
  190 continue                        ! End of loop over clusters.

 8200 format ('  area ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cluster does not exist:  ',a)

  200 write (   6, 8200) ainput, ninput, aword(4)
      write (iout, 8200) ainput, ninput, aword(4)
      go to 999

      else                            ! Not recognized.

 8210   format ('  area ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Input line is not recognized.')

        write (   6, 8210) ainput, ninput
        write (iout, 8210) ainput, ninput
        go to 999

      endif                           ! Tested for "cluster" or "cl".

c=======================================================================********

  210 if (nerr .ne. 0) go to 999

      puavg = puavg / npver
      pvavg = pvavg / npver
      pwavg = pwavg / npver

c.... Convert the average point into the current coordinate system.

      if     (asys .eq. 'rect') then
        ksys = 0
      elseif (asys .eq. 'cyl')  then
        ksys = 1
      elseif (asys .eq. 'sph')  then
        ksys = 2
      endif

      if (ksys .ne. 0) then

        if     (angunit .eq. 'degrees') then
          iunit = 0
        elseif (angunit .eq. 'radians') then
          iunit = 1
        endif

        call aptcsys (0, ksys, iunit, puavg, pvavg, pwavg, 1, tol,
     &                nerra)

      endif

 8250 format ('  Average vertex  ',1p3e20.12)

      write (iout, 8250) puavg, pvavg, pwavg
      write (   6, 8250) puavg, pvavg, pwavg

c=======================================================================********

c.... Find the average edge.

      edgem = 0.0
  
      do 260 np = 1, npver
        next = mod (np, npver) + 1
        edgem = edgem + sqrt ((xp(np) - xp(next))**2 +
     &                        (yp(np) - yp(next))**2 +
     &                        (zp(np) - zp(next))**2)
  260 continue

      edgem = edgem / npver

 8185 format ('  Average edge    ',1pe20.12)

      write (   6, 8185) edgem
      write (iout, 8185) edgem

c=======================================================================********

c.... Find the rotation operator to make vector VNORM parallel to the z axis.

      call aptrotv (ux, uy, uz, 0.0, 0.0, 1.0, tol, rotm, nerrpa)

c.... Rotate the points to make vector VNORM parallel to the z axis.

      call aptmopv (rotm, 0, 0.0, 0.0, 0.0, xp, yp, zp, npver, tol,
     &              nerra)

c=======================================================================********

c.... Find the area of the polygon.

      parea = 0.0
      errpa = 0.0
  
      do 270 np = 1, npver
        next = mod (np, npver) + 1

        parea = parea + 0.5 * (xp(np) - xp(next)) * (yp(next) + yp(np))

        errpa = errpa + 0.5 * tol * (abs (xp(np)) + abs (xp(next))) *
     &                              (abs (yp(np)) + abs (yp(next)))

  270 continue

      parea = abs (parea)
      if (parea .le. errpa) parea = 0.0

 8190 format ('  Projected area  ',1pe20.12)

      write (   6, 8190) parea
      write (iout, 8190) parea

c=======================================================================********

  999 return

c.... End of subroutine area.         (+1 line)
      end
