      subroutine circle

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  circle   Updated     2004 July 27 15:20.
c               circle   Originated  1994 June 20 15:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find a circle by specifying three points on the
c               perimeter.
c
c     Logic:    Entry points:  circle.
c circle        Called by:  helpall, main.
c
c circle        Calls:  aptcirp, aptcsyv, ptfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

c.... Input line:  circle [help]

 8110 format (
     &  '  Command "circle":  acceptable input lines include:' /
     &  '  "circle [help]"  Display this message.' //
     &  '  "circle point PNAME1 PNAME2 PNAME3"' /
     &  '     Find the circle through points PNAME1, PNAME2 and',
     &  ' PNAME3,' //
     &  '  Synonyms:  [circle, circ], [help, h], [point, pnt, pt, p].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... Input line:  circle point PNAME1 PNAME2 PNAME3

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))    then

      nerr = 0

c.... Test number of words on input line.

 8135 format ('  circle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "circle" line not 5:  ',i3)

      if (nword .ne. 5) then
        write (   6, 8135) ainput, ninput, nword
        write (iout, 8135) ainput, ninput, nword
        go to 999
      endif

c.... Find the three points defining the circle.

      call ptfind (3, np)

      if (np .eq. 0) then
        nerr = 1
        go to 150
      endif

      xp1 = x(np)
      yp1 = y(np)
      zp1 = z(np)

  150 call ptfind (4, np)

      if (np .eq. 0) then
        nerr = 1
        go to 170
      endif

      xp2 = x(np)
      yp2 = y(np)
      zp2 = z(np)

  170 call ptfind (5, np)

      if (np .eq. 0) then
        nerr = 1
        go to 190
      endif

      xp3 = x(np)
      yp3 = y(np)
      zp3 = z(np)

  190 if (nerr .ne. 0) go to 999

c.... Find the vertex points and other circle data.

 8190 format ('  circle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are colinear or coincide.')

      call aptcirp (xp1, yp1, zp1, xp2, yp2, zp2, xp3, yp3, zp3, 1,
     &              tol, u, v, w, un, vn, wn, rcirc, nerra)
  
      if ((nerra .ne. 0) .or. (rcirc .gt. 1.e99)) then
        write (   6, 8190) ainput, ninput
        write (iout, 8190) ainput, ninput
        go to 999
      endif

      circum = 2.0 * pi * rcirc
      area   = pi * rcirc**2

c.... Transform coordinates, if necessary.

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

      vpos = 0.0
      if (asys .eq. 'cyl') then   ! Convert to cylindrical coordinates.
        call aptcsyv (0, 1, iunit, u, v, w, un, vn, wn, 1, tol, nerra)
        if (v .lt. 0.0) then
          if (iunit .eq. 0) then
            vpos = v + 360.0
          else
            vpos = v + 2.0 * pi
          endif
        endif
      elseif (asys .eq. 'sph') then   ! Convert to spherical coordinates.
        call aptcsyv (0, 2, iunit, u, v, w, un, vn, wn, 1, tol, nerra)
        if (v .lt. 0.0) then
          if (iunit .eq. 0) then
            vpos = v + 360.0
          else
            vpos = v + 2.0 * pi
          endif
        endif
      endif

c.... Print the circle specifications.

 8210 format ('Rad, circum, area ',1p3e20.12 )
 8220 format ('Center            ',1p3e20.12 )
 8230 format ('Normal vector     ',1p3e20.12 )

      write (   6, 8210) rcirc, circum, area
      write (iout, 8210) rcirc, circum, area
      write (   6, 8220) u, v, w
      write (iout, 8220) u, v, w
      if (vpos .gt. 0.0) then
        write (   6, 8220) u, vpos, w
        write (iout, 8220) u, vpos, w
      endif
      write (   6, 8230) un, vn, wn
      write (iout, 8230) un, vn, wn

      go to 999

      endif                           ! Specify circle by three points.

c=======================================================================********

 8810 format ('  circle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine circle.      (+1 line)
      end
