      subroutine closey (ius, aname, atype)
 
c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  closey   Updated     1993 October 28 12:10.
c               closey   Originated  1991 March 12 10:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To disconnect I/O unit ius from the file aname, of type atype.
c
c     Logic:    Entry points:  closey.
c closey        Called by:  geomend, output.
c
c closey        Calls:  geomend.
c               System calls:  close, izclose (cbase).
c
c     Biblio:   FORTLIB-THEORY and FORTLIB-SUBS (Category 9).
c
c     Glossary:
c
c     aname     Input    Name of file to be closed.  A character expression,
c                          which may have meaningless trailing blanks.
c
c     atype     Input    File type:  'text', 'hardcopy' or 'absolute'.
c                          'text':      ASCII input file.
c                          'hardcopy':  ASCII output file
c                            (same as 'text' on UNICOS).
c                          'absolute':  word-addressable, random-access file.
c
c     ius       Input    Number of I/O unit to be disconnected from file aname.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c
 
c.... Input argument specifications.
 
      character*80 aname              ! Name of file to close.
      character*8 atype               ! File type specification.

c.... Local variables.
 
      common /aclosey/ ierr           ! Error flag from system.
      common /aclosey/ atermess       ! Termination message.
      character*80     atermess       ! Termination message.
 
c.... Close the file.
 
      if ((atype .eq. 'text    ') .or. (atype .eq. 'hardcopy')) then
 
        close (ius)
 
      elseif (atype .eq. 'absolute') then
 
cuni        ierr  = izclose (ius, 3)
        ierr  = 1
 
      else                            ! Error in file type specification.
 
        ierr = 1
 
      endif                           ! Tested atype.
 
 8110 format (/ '  closey ERROR.  Closing ',a,' file ',a,
     &  ' on I/O unit=',i5,'.  ierr=',i5)
 
      if (ierr .ne. 0) then           ! Error closing file.
        write (   6, 8110) atype, aname, ius, ierr
        atermess = 'closey ERROR.  Closing a file.'
        call geomend (atermess)
      endif
 
c.... File has been closed.
 
cr 8120 format ('  Closed   ',a,' file ',a,' on I/O unit ',i5,'.')
 
cr      write (   6, 8120) atype, aname, ius
 
      return
 
c.... End of subroutine closey.       (+1 line).
      end
