      subroutine clout (n1, n2, amount)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  clout    Updated     2007 January 23 14:30.
c               clout    Originated  1994 March 7 15:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the names and coordinates of the points in clusters
c                 n1 to n2, with repacking, eliminating blanks and duplicates.
c
c     Logic:    Entry points:  clout.
c clout         Called by:  area, clfind, cluster, copy, delete, distance,
c               matrix, mesh, movex, project, renamea, renamex, search, twist,
c               where.
c
c clout         Calls:  pout, ptcoord.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'


c.... Local variables.

      character*8  amount             ! Amount of output to write.
      character*24 anamex(6)          ! Names of points.
      character*1  alab               ! Label "=" or "&".

c.... Initialize.

      umin  =  1.e99
      umax  = -1.e99
      vmin  =  1.e99
      vmax  = -1.e99
      wmin  =  1.e99
      wmax  = -1.e99
      uavg  = -1.e99
      vavg  = -1.e99
      wavg  = -1.e99
      u2avg = -1.e99
      v2avg = -1.e99
      w2avg = -1.e99
      usig  = -1.e99
      vsig  = -1.e99
      wsig  = -1.e99

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. nclusts) nn2 = nclusts

c.... Repack pair list.  Eliminate blanks and duplicates.

      npairx = 0
cbugc___DEBUG begins.
cbug 9911 format ('clout DEBUG X1.  npairs='i3,' npairx=',i3)
cbug 9912 format ('clout DEBUG X2.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9911) npairs, npairx
cbug      write (iout, 9911) npairs, npairx
cbug      write (   6, 9912) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9912) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.

      do 105 npr = 1, npairs          ! Loop over pairs.

        if ((apaircl(npr) .eq. ' ') .and.
     &      (apairpt(npr) .eq. ' ')) go to 105

        npairx = npairx + 1
        apaircl(npairx) = apaircl(npr)
        apairpt(npairx) = apairpt(npr)

        do 102 next = npr + 1, npairs
          if ((apaircl(next) .eq. apaircl(npr))  .and.
     &        (apairpt(next) .eq. apairpt(npr))) then
            apaircl(next) = ' '
            apairpt(next) = ' '
          endif
  102   continue
  105 continue                        ! End of loop over pairs.

cbugc___DEBUG begins.
cbug 9914 format ('clout DEBUG Y1.  npairs='i3,' npairx=',i3)
cbug 9913 format ('clout DEBUG Y2.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9914) npairs, npairx
cbug      write (iout, 9914) npairs, npairx
cbug      write (   6, 9913) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9913) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.
      npairs = npairx

      do 150 n = nn1, nn2
cbugc___DEBUG begins.
cbug 9901 format ('clout DEBUG 1.  Cluster ',i3,' is ',a)
cbug 9902 format ('clout DEBUG 1.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9901) n, aclust(n)
cbug      write (iout, 9901) n, aclust(n)
cbug      write (   6, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9902) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.

        if (aclust(n) .eq. ' ') go to 150

 8110   format ('  clout ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named point does not exist:  ',a)
 8120   format ('cl ',a,1x,a1,1x,a,1x,a)

        nx   = 0                      ! Point name counter, 1 to 3.
        kx   = 0                      ! Total number of points in this cluster.
        alab = '='                    ! Option in list of points, "=" or "&".

        do 120 npr = 1, npairs        ! Loop over pairs.

          if ((apaircl(npr) .eq. ' ') .or.
     &        (apairpt(npr) .eq. ' ')) go to 120

          if (apaircl(npr) .ne. aclust(n)) go to 120

          nx = nx + 1
          kx = kx + 1

          do 110 np = 1, npoints
            if (apairpt(npr) .eq. apoint(np)) go to 115
  110     continue
          write (   6, 8110) ainput, ninput, apairpt(npr)
          write (iout, 8110) ainput, ninput, apairpt(npr)
          go to 999

  115     anamex(nx) = apairpt(npr)
          if (nx .eq. 2) then
            write (   6, 8120) aclust(n), alab, (anamex(nn), nn = 1, 2)
            write (iout, 8120) aclust(n), alab, (anamex(nn), nn = 1, 2)
            nx = 0
            alab = '&'
          endif
  120   continue                      ! End of loop over pairs.

        if (nx .ne. 0) then
          write (   6, 8120) aclust(n), alab, (anamex(nn), nn = 1, nx)
          write (iout, 8120) aclust(n), alab, (anamex(nn), nn = 1, nx)
        endif

 8122   format ('  WARNING:  deleted empty cluster:  ',a)
 8125   format ('  Number of points in cluster ',a,' = ',i5,'.')

        if (kx .eq. 0) then
          anamex(1) = '**NONE**'
          alab      = ' '
          write (   6, 8120) aclust(n), alab, anamex(1)
          write (iout, 8120) aclust(n), alab, anamex(1)
          write (   6, 8122) aclust(n)
          write (iout, 8122) aclust(n)
          aclust(n) = ' '
        else
          write (   6, 8125) aclust(n), kx
          write (iout, 8125) aclust(n), kx
        endif
cbugc___DEBUG begins.
cbug 9903 format ('clout DEBUG 2.  Cluster ',i3,' is ',a)
cbug 9904 format ('clout DEBUG 2.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9903) n, aclust(n)
cbug      write (iout, 9903) n, aclust(n)
cbug      write (   6, 9904) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9904) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.

        if ((amount .eq. 'max')      .or.
     &      (amount .eq. 'maximum')) then

 8130 format ('coord  ',a,5x,a,4x,a,4x,a)

          write (   6, 8130) asys, acoordu, acoordv, acoordw
          write (iout, 8130) asys, acoordu, acoordv, acoordw

          navg = 0
          uavg = 0.0
          vavg = 0.0
          wavg = 0.0
          u2avg = 0.0
          v2avg = 0.0
          w2avg = 0.0

          do 140 npr = 1, npairs
            if (apaircl(npr) .ne. aclust(n)) go to 140
            do 130 np = 1, npoints
              if (apairpt(npr) .eq. apoint(np)) then

                call pout (np, np, 'min')

c....           Find minimum and maximum coordinates.

                call ptcoord (asys, angunit, x(np), y(np), z(np),
     &                        u, v, w, vpos, nerr)

                if (u .lt. umin) umin = u
                if (u .gt. umax) umax = u
                if (v .lt. vmin) vmin = v
                if (v .gt. vmax) vmax = v
                if (w .lt. wmin) wmin = w
                if (w .gt. wmax) wmax = w

                navg = navg + 1
                uavg = uavg + u
                vavg = vavg + v
                wavg = wavg + w
                u2avg = u2avg + u**2
                v2avg = v2avg + v**2
                w2avg = w2avg + w**2

              endif
  130       continue
  140     continue

          if (navg .gt. 0) then
            uavg = uavg / navg
            vavg = vavg / navg
            wavg = wavg / navg
            u2avg = u2avg / navg
            v2avg = v2avg / navg
            w2avg = w2avg / navg
            argu    = u2avg - uavg**2
            arguerr = tol * (abs (u2avg) + uavg**2)
            if (argu .le. arguerr) argu = 0.0
            usig    = sqrt (argu)

            argv    = v2avg - vavg**2
            argverr = tol * (abs (v2avg) + vavg**2)
            if (argv .le. argverr) argv = 0.0
            vsig    = sqrt (argv)

            argw    = w2avg - wavg**2
            argwerr = tol * (abs (w2avg) + wavg**2)
            if (argw .le. argwerr) argw = 0.0
            wsig    = sqrt (argw)

          endif

c....     Display minimum, maximum and average coordinates.

 8150     format (/ 'Minimum         = ',1p3e20.12 /
     &              'Maximum         = ',1p3e20.12 /
     &              'Average (',i5,') = ',1p3e20.12 /
     &              'Std Dev (',i5,') = ',1p3e20.12 )

          write (   6, 8150) umin, vmin, wmin, umax, vmax, wmax,
     &      navg, uavg, vavg, wavg, navg, usig, vsig, wsig
          write (iout, 8150) umin, vmin, wmin, umax, vmax, wmax,
     &      navg, uavg, vavg, wavg, navg, usig, vsig, wsig

        endif                         ! Tested amount.

  150 continue                        ! End of loop over clusters.

  999 return

c.... End of subroutine clout.        (+1 line)
      end
