      subroutine coord

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  coord    Updated     1995 December 18 15:50.
c               coord    Originated  1993 November 2 10:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To interpret data on "coord" input lines, to specify the
c               coordinate system, rectangular, cylindrical or spherical.
c
c     Logic:    Entry point:  coord.
c coord         Called by:  brout, distance, helpall, main, track.
c               Calls:  (none).
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

cbugc____ DEBUG begins.
cbug 9801 format ('coord DEBUG.  Entered coord.')
cbug      write (   6, 9801)
cbug      write (iout, 9801)
cbugc____ DEBUG ends.

c.... Initialize.

      if (nword .eq. 0) then
        asys = 'rect'
        acoordu = 'x'
        acoordv = 'y'
        acoordw = 'z'
        go to 999
      endif

c=======================================================================********

c.... Input line:  coordinate

 8100 format ('coord  ',a,5x,a,4x,a,4x,a)

      if (nword .eq. 1) then
        write (   6, 8100) asys, acoordu, acoordv, acoordw
        write (iout, 8100) asys, acoordu, acoordv, acoordw
        go to 999
      endif

c=======================================================================********

c.... Input line:  coordinate help

 8110 format ('  Command "coord":  acceptable input lines include:' /
     &  '  "coord help" Display this message.' /
     &  '  "coord"      Display the current coordinate system.' //
     &  '  "coord rect" Change the coordinate system to rectangular',
     &  ' (x,y,z).' /
     &  '  "coord cyl"  Change the coordinate system to cylindrical',
     &  ' (rcyl,theta,z).' /
     &  '  "coord sph"  Change the coordinate system to spherical',
     &  ' (rsph,theta,phi).' //
     &  '  Synonyms:  [coordinate, coord, cs, system, sys],',
     &  ' [cyl, cylindrical, cy],' /
     &  '    [help, h], [cartesian, cart, rectangular, rect, xyz],',
     &  ' [spherical, sph, sp].')

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))  then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Test for too many words on "coord" line.

 8120 format ('  coord ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Too many words on "coord" line:  ',i3)
      if (nword .gt. 2) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
      endif

c=======================================================================********

c.... Input line:  coordinate [cartesian,cylindrical,spherical]

c.... Change coordinate system.

 8130 format ('  coord ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Second word not "rect", "cyl" or "sph".')

      if ((aword(2) .eq. 'rect')         .or.
     &    (aword(2) .eq. 'cart')         .or.
     &    (aword(2) .eq. 'cartesian')    .or.
     &    (aword(2) .eq. 'Cartesian')    .or.
     &    (aword(2) .eq. 'xyz')          .or.
     &    (aword(2) .eq. 'rectangular')) then
        asys = 'rect'
        acoordu = 'x'
        acoordv = 'y'
        acoordw = 'z'
      elseif ((aword(2) .eq. 'cyl')          .or.
     &        (aword(2) .eq. 'cylindrical')) then
        asys = 'cyl'
        acoordu = 'rcyl'
        acoordv = 'theta (deg)'
        if (angunit .eq. 'radians') acoordv = 'theta (rad)'
        acoordw = 'z'
      elseif ((aword(2) .eq. 'sph')        .or.
     &        (aword(2) .eq. 'spherical')) then
        asys = 'sph'
        acoordu = 'rsph'
        acoordv = 'theta (deg)'
        acoordw = 'phi (deg)'
        if (angunit .eq. 'radians') then
          acoordv = 'theta (rad)'
          acoordw = 'phi (rad)'
        endif
      else
        write (   6, 8130) ainput, ninput
        write (iout, 8130) ainput, ninput
        go to 999
      endif

c.... Inform user of coordinate system.

      write (   6, 8100) asys, acoordu, acoordv, acoordw
      write (iout, 8100) asys, acoordu, acoordv, acoordw
      go to 999

c=======================================================================********

  999 return

c.... End of subroutine coord.        (+1 line)
      end
