      subroutine delete

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  delete   Updated     2006 January 6 17:40.
c               delete   Originated  1993 November 8 14:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To delete aliases, markers, variables, vectors, operators,
c               points, clusters, lines, triangles, regular polygons,
c               annular disks, planes, spheres, circular cylinders,
c               circular cones, hyperbolic paraboloids, ellipsoids, axisymmetric
c               quadric surfaces, general quadric surfaces, regular polyhedrons,
c               tetrahedrons, bricks, zones, bins, probability distribution
c               functions (pdfs), and symbols.
c
c     Logic:    Entry points:  delete
c delete        Called by:  helpall, main.
c
c delete        Calls:  alout, aptchmv, aptsubr, bigout, bindep, binout,
c               brout, clout, compsubs, dkout, lout, markdep, markout, opout,
c               pdfout, pgout, phout, plout, pout, ptdep, ptfind, quout,
c               qutype, symbdisp, tetout, trout, varout, vectfind, vout, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*1  abigmn             ! Minimum big integer name to delete.
      character*1  abigmx             ! Maximum big integer name to delete.
      character*1  amarkmn            ! Minimum marker name to delete.
      character*1  amarkmx            ! Maximum marker name to delete.
      character*24 anamemn            ! Minimum 24-character name to delete.
      character*24 anamemx            ! Maximum 24-character name to delete.
      character*3  ans                ! Answer to verify "delete ... all".

c.... Input line:  delete [help]

 8110 format ('  Command "delete":  acceptable input lines include:' /
     &  '  "delete [help]"  Display this message.' //
     &  '  "delete OBJTYPE all"' /
     &  '     Delete all objects of type OBJTYPE, where OBJTYPE is',
     &  ' alias, big, marker,' /
     &  '     variable, vector, operator, point, cluster, line,',
     &  ' triangle, polygon, disk,' /
     &  '     plane, sphere, cylinder, cone, hyperb, ellipsoid,',
     &  ' axisym, quadric,' /
     &  '     tetrahedron, brick, zone, bin, pdf, symbol.' /
     &  '  "delete OBJTYPE OBJNAME"' /
     &  '     Delete the object of type OBJTYPE with name OBJNAME.' /
     &  '  "delete OBJTYPE OBJNAME1 thru OBJNAME2"' /
     &  '     Delete any objects of type OBJTYPE with names from',
     &  ' OBJNAME1 to OBJNAME2,' /
     &  '     where the order is ASCII, except for subscripts,' /
     &  '     which are in numerical order.' /
     &  '  "delete OBJTYPE OBJNAME all"' /
     &  '     Delete all objects of type OBJTYPE with the names',
     &  ' OBJNAME(NSUB),' /
     &  '     where NSUB is any subscript, or null.' )

 8112 format (
     &  '  "delete point cluster CLNAME"' /
     &  '     Delete all points in cluster CLNAME.' /
     &  '  "delete bin pdf PDFNAME"' /
     &  '     Delete all probability bins in pdf PDFNAME.' //
     &  '  Synonyms:  [alias, al, a], [brick, br], [cluster, cl],' /
     &  '    [cylinder, cyl], [delete, del, remove, rm],',
     &  ' [disk, disc, dk],' /
     &  '    [ellipsoid, ellipse, ell, el], [help, h],',
     &  ' [hyperb, saddle],' /
     &  '    [line, ln, l], [marker, mark, m],' /
     &  '    [operator, oper, op, o], [plane, pl],' /
     &  '    [point, pnt, pt, p], [polygon, polyg, pg],',
     &  ' [polyhedron, polyh, ph],' /
     &  '    [quadric, quad, q], [sphere, sph], [symbol, symb, sym],' /
     &  '    [tetrahedron, tetra, tet], [triangle, tri],',
     &  ' [variable, var, set, s],' /
     &  '    [vector, vect, v], [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        go to 999
      endif

c===============================================================================

c.... Test for the wrong number of words on "delete" line.

 8120 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Wrong number of words on "delete" line:  ', i3)

      if ((nword .lt. 3) .or. (nword .gt. 5)) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

      if ((nword .eq. 5) .and. (aword(4) .ne. 'thru')) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

      if ((nword .eq. 4)            .and.
     &    (aword(3) .ne. 'pdf')     .and.
     &    (aword(3) .ne. 'cluster') .and.
     &    (aword(3) .ne. 'cl')      .and.
     &    (aword(4) .ne. 'all'))    then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

      if ((nword .eq. 4) .and.
     &    (aword(3) .eq. 'pdf')  .and.
     &    (aword(2) .ne. 'bin')) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

      if ((nword .eq. 4) .and.
     &   ((aword(3) .eq. 'cluster') .or.
     &    (aword(3) .eq. 'cl'))     .and.
     &   ((aword(2) .ne. 'point')   .and.
     &    (aword(2) .ne. 'pt')      .and.
     &    (aword(2) .ne. 'p')))     then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c===============================================================================

c.... Find the maximum lengths of the object names.

      lenmax = 24

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'big')    .or.
     &    (aword(2) .eq. 'm'))     then
        lenmax = 1
      endif

      if ((aword(3) .ne. 'all')    .and.
     &   ((lword(3) .lt. 1)        .or.
     &    (lword(3) .gt. lenmax))) then
        write (   6, 8136) ainput, ninput, lenmax, aword(3)
        write (iout, 8136) ainput, ninput, lenmax, aword(3)
        go to 999
      endif

c===============================================================================

c.... Verify request to delete all objects of a specified type.

 8150 format ('  Are you SURE you want to delete ALL objects',
     &  ' of type ',a12,'?' /
     &  '  If so, type "yes":  ',$)
 8155 format (a3)
 8156 format ('>',3('DELETE PRECEDING COMMAND!  '))
 8158 format ('  NO objects were deleted.')

      if (ainput .eq. 'tty') then
        if (aword(3) .eq. 'all') then
          write (   6, 8150) aword(2)
          read  (   5, 8155) ans
          if (ans .ne. 'yes') then
            write (iout, 8156)
            write (   6, 8158)
            write (iout, 8158)
            go to 999
          endif
        endif
      endif

c===============================================================================
c ADD CODING TO CHECK FOR A SINGLE VALUE, CHANGE TO 5-WORD "THRU" COMMAND.
c===============================================================================

c.... See if option "thru" is used.

 8136 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of object not from 1 to ',i3,' characters:  ',a)
 8138 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Min name > max name:  'a24,2x,a24)
 8139 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word "thru" is duplicated.')

      amarkmn  = '!'
      amarkmx  = '~'
      abigmn   = 'A'
      abigmx   = 'Z'
      anamemn  = '               !'
      anamemx  = '~~~~~~~~~~~~~~~~'

      if ((nword .eq. 5) .and.
     &    (aword(4) .eq. 'thru')) then

        if ((lword(3) .lt. 1) .or. (lword(3) .gt. lenmax)) then
          write (   6, 8136) ainput, ninput, lenmax, aword(3)
          write (iout, 8136) ainput, ninput, lenmax, aword(3)
          go to 999
        endif

        if ((lword(5) .lt. 1) .or. (lword(5) .gt. lenmax)) then
          write (   6, 8136) ainput, ninput, lenmax, aword(5)
          write (iout, 8136) ainput, ninput, lenmax, aword(5)
          go to 999
        endif

        call compsubs (aword(3), aword(5), nsmall, nerrc)

        if (nsmall .eq. 2) then
          write (   6, 8138) ainput, ninput, aword(3), aword(5)
          write (iout, 8138) ainput, ninput, aword(3), aword(5)
          go to 999
        endif

        if ((aword(2) .eq. 'thru')  .or.
     &      (aword(3) .eq. 'thru')  .or.
     &      (aword(5) .eq. 'thru')) then
          write (   6, 8139) ainput, ninput
          write (iout, 8139) ainput, ninput
          go to 999
        endif

        if ((aword(2) .eq. 'marker') .or.
     &      (aword(2) .eq. 'mark')   .or.
     &      (aword(2) .eq. 'm'))     then
          amarkmn = aword(3)
          amarkmx = aword(5)
        elseif (aword(2) .eq. 'big') then
          abigmn = aword(3)
          abigmx = aword(5)
        else
          anamemn = aword(3)
          anamemx = aword(5)
        endif

        nword = 3
        aword(3) = 'all'

      endif                           ! Word 4 was "thru".

c===============================================================================

c.... See if command  "delete OBJTYPE OBJNAME all" is used.

 8050 format ('  Are you SURE you want to delete ALL objects',
     &  ' of type ',a12 /
     &  '  with the subscripted name ',a,'?' /
     &  '  If so, type "yes":  ',$)
 8055 format (a3)
 8056 format ('>',3('DELETE PRECEDING COMMAND!  '))
 8058 format ('  NO objects were deleted.')

      if ((nword .eq. 4) .and. (aword(4) .eq. 'all')) then  ! Delete OBJN(...).

        if (ainput .eq. 'tty') then
          write (   6, 8050) aword(2), aword(3)
          read  (   5, 8055) ans
          if (ans .ne. 'yes') then
            write (iout, 8056)
            write (   6, 8058)
            write (iout, 8058)
            go to 999
          endif
        endif

        call aptsubr (aword(3), 1, 24, 80, astem, lstem,
     &                asub, lsub, nerra)

        if (lsub(1) .eq. 0) then        ! No subscript.
          if (lstem .le. 14) then

            call aptchmv (astem, 1, lstem, anamemn, 1, nerr)
            call aptchmv (astem, 1, lstem, anamemx, 1, nerr)

            lnext = lstem + 1

            call aptchmv ('(', 1, 1, anamemn, lnext, nerr)
            call aptchmv ('(', 1, 1, anamemx, lnext, nerr)

            nword = 3
            aword(3) = 'all'

          endif
        endif                         ! Unsubscripted name.
      endif                           ! Delete same name with any subscript.

c===============================================================================

c.... Input line:  delete alias ANAME
c.... Input line:  delete alias ANAME1 thru ANAME2
c.... Input line:  delete alias all

c.... See if aliases are to be deleted.

      if ((aword(2) .eq. 'alias') .or.
     &    (aword(2) .eq. 'al')    .or.
     &    (aword(2) .eq. 'a'))    then

 9240 format ('  Deleted alias ',a)
 9250 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named alias does not exist:  ',a)
 9252 format (/ 'Number of aliases deleted = ',i5)

        if (aword(3) .eq. 'all') then

          naliasx = 0
          do 1240 n = 1, nalias
            if (alias1(n) .eq. ' ') go to 1240
            call compsubs (alias1(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 1240
            call compsubs (alias1(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 1240
            call alout (n, n)
            write (   6, 9240) alias1(n)
            write (iout, 9240) alias1(n)
            naliasx   = naliasx + 1
            alias1(n) = ' '
 1240     continue
          write (   6, 9252) naliasx
          write (iout, 9252) naliasx

        else                          ! Delete one alias.

          do 1250 n = 1, nalias
            if (aword(3) .eq. alias1(n)) then
              write (   6, 9240) alias1(n)
              write (iout, 9240) alias1(n)
              call alout (n, n)
              alias1(n) = ' '
              go to 999
            endif
 1250     continue
          write (   6, 9250) ainput, ninput, aword(3)
          write (iout, 9250) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted aliases.

c===============================================================================

c.... Input line:  delete bin BINAME
c.... Input line:  delete bin BINAME1 thru BINAME2
c.... Input line:  delete bin pdf PDFNAME
c.... Input line:  delete bin all

c.... See if bins are to be deleted.

      if (aword(2) .eq. 'bin') then

 9110 format ('  Deleted bin ',a)

        if (aword(3) .eq. 'all') then

          nbinx = 0
          do 1120 n = 1, nbins        ! Loop over bins.
            if (abin(n) .eq. ' ') go to 1120
            call compsubs (abin(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 1120
            call compsubs (abin(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 1120

            call bindep (n, ' ')

            write (   6, 9110) abin(n)
            write (iout, 9110) abin(n)
            call binout (n, n, 'min')

            do 1110 npx = 1, npdfxs
              if (abinx(npx) .eq. abin(n)) then
                apdfx(npx) = ' '
                abinx(npx) = ' '
              endif
 1110       continue

            nbinx   = nbinx + 1
            abin(n) = ' '

 1120     continue                    ! End of loop over bins.

 9120     format (/ 'Number of bins deleted = ',i5)

          write (   6, 9120) nbinx
          write (iout, 9120) nbinx

        elseif (aword(3) .eq. 'pdf') then

 9125     format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Not 4 words on "delete bin pdf PDFNAME" line:  ',i3)

          if (nword .ne. 4) then
            write (   6, 9125) ainput, ninput, nword
            write (iout, 9125) ainput, ninput, nword
            go to 999
          endif

 9130     format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named pdf does not exist:  ',a)

          do 1130 n = 1, npdfs
            if (aword(4) .eq. apdf(n)) then
              call pdfout (n, n, 'min')
              np = n
              go to 1140
            endif
 1130     continue
          write (   6, 9130) ainput, ninput, aword(4)
          write (iout, 9130) ainput, ninput, aword(4)
          go to 999

 1140     nbinx = 0
          do 1170 n = 1, nbins        ! Loop over bins.

            do 1160 npx = 1, npdfxs
              if ((apdfx(npx) .eq. apdf(np)) .and.
     &            (abinx(npx) .eq. abin(n)))  then

                call bindep (n, apdf(np))

                apdfx(npx) = ' '
                abinx(npx) = ' '

                write (   6, 9110) abin(n)
                write (iout, 9110) abin(n)
                call binout (n, n, 'min')

                do 1150 nx = 1, npdfxs
                  if (abinx(nx) .eq. abin(n)) then
                    apdfx(nx) = ' '
                    abinx(nx) = ' '
                  endif
 1150           continue

                nbinx   = nbinx + 1
                abin(n) = ' '

              endif
 1160       continue
 1170     continue                    ! End of loop over bins.

 9170 format ('  Deleted pdf ',a)

          write (   6, 9170) apdf(np)
          write (iout, 9170) apdf(np)
          apdf(np) = ' '
          write (   6, 9120) nbinx
          write (iout, 9120) nbinx

        else                          ! Delete one bin.

          do 1190 n = 1, nbins        ! Loop over bins.
            if (aword(3) .eq. abin(n)) then

              call bindep (n, ' ')

              write (   6, 9110) abin(n)
              write (iout, 9110) abin(n)
              call binout (n, n, 'min')

              do 1180 npx = 1, npdfxs
                if (abinx(npx) .eq. abin(n)) then
                  apdfx(npx) = ' '
                  abinx(npx) = ' '
                endif
 1180         continue

              abin(n) = ' '

              go to 999
            endif
 1190     continue                    ! End of loop over bins.

 9190     format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named bin does not exist:  ',a)

          write (   6, 9190) ainput, ninput, aword(3)
          write (iout, 9190) ainput, ninput, aword(3)

        endif                         ! Tested aword(3).

        go to 999

      endif                           ! Deleted bins.

c===============================================================================

c.... Input line:  delete brick BRNAME
c.... Input line:  delete brick BRNAME1 thru BRNAME2
c.... Input line:  delete brick all

c.... See if bricks are to be deleted.

      if ((aword(2) .eq. 'brick') .or.
     &    (aword(2) .eq. 'br'))   then

 8230 format ('  Deleted brick ',a)
 8240 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named brick does not exist:  ',a)
 8242 format (/ 'Number of bricks deleted = ',i5)

        if (aword(3) .eq. 'all') then
          nbrickx = 0
          do 230 n = 1, nbricks
            if (abrick(n) .eq. ' ') go to 230
            call compsubs (abrick(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 230
            call compsubs (abrick(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 230
            write (   6, 8230) abrick(n)
            write (iout, 8230) abrick(n)
            call brout (n, n, 'min')
            nbrickx   = nbrickx + 1
            abrick(n) = ' '
  230     continue
          write (   6, 8242) nbrickx
          write (iout, 8242) nbrickx
        else                          ! Delete one brick.
          do 240 n = 1, nbricks
            if (aword(3) .eq. abrick(n)) then
              write (   6, 8230) abrick(n)
              write (iout, 8230) abrick(n)
              call brout (n, n, 'min')
              abrick(n) = ' '
              go to 999
            endif
  240     continue
          write (   6, 8240) ainput, ninput, aword(3)
          write (iout, 8240) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted bricks.

c===============================================================================

c.... Input line:  delete cluster CLNAME
c.... Input line:  delete cluster CLNAME1 thru CLNAME2
c.... Input line:  delete cluster all

c.... See if clusters are to be deleted.

      if ((aword(2) .eq. 'cluster') .or.
     &    (aword(2) .eq. 'cl'))     then

 8250 format ('  Deleted cluster ',a)
 8270 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cluster does not exist:  ',a)
 8260 format (/ 'Number of clusters deleted = ',i5)

        if (aword(3) .eq. 'all') then

          nclustx = 0
          do 250 n = 1, nclusts
            if (aclust(n) .eq. ' ') go to 250
            call compsubs (aclust(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 250
            call compsubs (aclust(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 250
            write (   6, 8250) aclust(n)
            write (iout, 8250) aclust(n)
            call clout (n, n, 'min')
            nclustx   = nclustx + 1
            aclust(n) = ' '
            do 245 npr = 1, npairs
              if (apaircl(npr) .eq. aclust(n)) then
                apaircl(npr) = ' '
                apairpt(npr) = ' '
              endif
  245       continue
  250     continue
          write (   6, 8260) nclustx
          write (iout, 8260) nclustx

        else                          ! Delete one cluster.

          do 270 n = 1, nclusts
            if (aword(3) .eq. aclust(n)) then
              write (   6, 8250) aclust(n)
              write (iout, 8250) aclust(n)
              call clout (n, n, 'min')
              do 260 npr = 1, npairs
                if (apaircl(npr) .eq. aclust(n)) then
                  apaircl(npr) = ' '
                  apairpt(npr) = ' '
                endif
  260         continue
              aclust(n) = ' '
              go to 999
            endif
  270     continue
          write (   6, 8270) ainput, ninput, aword(3)
          write (iout, 8270) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted clusters.

c===============================================================================

c.... Input line:  delete disk DKNAME
c.... Input line:  delete disk DKNAME1 thru DKNAME2
c.... Input line:  delete disk all

c.... See if annular disks are to be deleted.

      if ((aword(2) .eq. 'disk') .or.
     &    (aword(2) .eq. 'disc') .or.
     &    (aword(2) .eq. 'dk'))  then

 8650 format ('  Deleted annular disk ',a)
 8660 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named annular disk does not exist:  ',a)
 8662 format (/ 'Number of annular disks deleted = ',i5)

        if (aword(3) .eq. 'all') then
          ndiskx = 0
          do 650 n = 1, ndisks
            if (adisk(n) .eq. ' ') go to 650
            call compsubs (adisk(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 650
            call compsubs (adisk(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 650
            write (   6, 8650) adisk(n)
            write (iout, 8650) adisk(n)
            call dkout (n, n)
            ndiskx   = ndiskx + 1
            adisk(n) = ' '
  650     continue
          write (   6, 8662) ndiskx
          write (iout, 8662) ndiskx
        else                          ! Delete one disk.
          do 660 n = 1, ndisks
            if (aword(3) .eq. adisk(n)) then
              write (   6, 8650) adisk(n)
              write (iout, 8650) adisk(n)
              call dkout (n, n)
              adisk(n) = ' '
              go to 999
            endif
  660     continue
          write (   6, 8660) ainput, ninput, aword(3)
          write (iout, 8660) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted disks.

c===============================================================================

c.... Input line:  delete line ALNAME
c.... Input line:  delete line ALNAME1 thru ALNAME2
c.... Input line:  delete line all

c.... See if lines are to be deleted.

      if ((aword(2) .eq. 'line') .or.
     &    (aword(2) .eq. 'ln')   .or.
     &    (aword(2) .eq. 'l'))   then

 8410 format ('  Deleted line ',a)
 8420 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named line does not exist:  ',a)
 8422 format (/ 'Number of lines deleted = ',i5)

        if (aword(3) .eq. 'all') then
          nlinex = 0
          do 410 n = 1, nlines
            if (aline(n) .eq. ' ') go to 410
            call compsubs (aline(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 410
            call compsubs (aline(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 410
            write (   6, 8410) aline(n)
            write (iout, 8410) aline(n)
            call lout (n, n)
            nlinex   = nlinex + 1
            aline(n) = ' '
  410     continue
          write (   6, 8422) nlinex
          write (iout, 8422) nlinex
        else                          ! Delete one line.
          do 420 n = 1, nlines
            if (aword(3) .eq. aline(n)) then
              write (   6, 8410) aline(n)
              write (iout, 8410) aline(n)
              call lout (n, n)
              aline(n) = ' '
              go to 999
            endif
  420     continue
          write (   6, 8420) ainput, ninput, aword(3)
          write (iout, 8420) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted lines.

c===============================================================================

c.... Input line:  delete marker AMNAME
c.... Input line:  delete marker AMNAME1 thru AMNAME2
c.... Input line:  delete marker all

c.... See if markers are to be deleted.

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'm'))     then

 9270 format ('  Deleted marker ',a1)
 9280 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named marker does not exist:  ',a1)
 9285 format (/ 'Number of markers deleted = ',i5)

        if (aword(3) .eq. 'all') then

          nmarkx = 0
          do 1270 n = 1, nmarks
            if (amark(n) .eq. ' ') go to 1270
            if ((amark(n) .lt. amarkmn) .or.
     &          (amark(n) .gt. amarkmx)) go to 1270

            call markdep (n, ndep)

            if (ndep .ne. 0) go to 1270

            write (   6, 9270) amark(n)
            write (iout, 9270) amark(n)
            call markout (n, n)
            nmarkx   = nmarkx + 1
            amark(n) = ' '
 1270     continue
          write (   6, 9285) nmarkx
          write (iout, 9285) nmarkx
        else                          ! Delete one marker.
          do 1280 n = 1, nmarks
            if (aword(3) .eq. amark(n)) then

              call markdep (n, ndep)

              if (ndep .ne. 0) go to 999

              write (   6, 9270) amark(n)
              write (iout, 9270) amark(n)
              call markout (n, n)
              amark(n) = ' '
              go to 999
            endif
 1280     continue
          write (   6, 9280) ainput, ninput, aword(3)
          write (iout, 9280) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted markers.

c===============================================================================

c.... Input line:  delete operator OPNAME
c.... Input line:  delete operator OPNAME1 thru OPNAME2
c.... Input line:  delete operator all

c.... See if operators are to be deleted.

      if ((aword(2) .eq. 'operator') .or.
     &    (aword(2) .eq. 'oper')     .or.
     &    (aword(2) .eq. 'op'))      then

 8330 format ('  Deleted operator ',a)
 8340 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named operator does not exist:  ',a)
 8342 format (/ 'Number of operators deleted = ',i5)

        if (aword(3) .eq. 'all') then

          noperx = 0
          do 330 n = 1, nopers
            if (aoper(n) .eq. ' ') go to 330
            call compsubs (aoper(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 330
            call compsubs (aoper(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 330
            write (   6, 8330) aoper(n)
            write (iout, 8330) aoper(n)
            call opout (n, n, 'min')
            noperx   = noperx + 1
            aoper(n) = ' '
  330     continue
          write (   6, 8342) noperx
          write (iout, 8342) noperx
        else                          ! Delete one operator.
          do 340 n = 1, nopers
            if (aword(3) .eq. aoper(n)) then
              write (   6, 8330) aoper(n)
              write (iout, 8330) aoper(n)
              call opout (n, n, 'min')
              aoper(n) = ' '
              go to 999
            endif
  340     continue
          write (   6, 8340) ainput, ninput, aword(3)
          write (iout, 8340) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted operators.

c===============================================================================

c.... Input line:  delete point PNAME
c.... Input line:  delete point PNAME1 thru PNAME2
c.... Input line:  delete point cluster CLNAME
c.... Input line:  delete point all

c.... See if points are to be deleted.

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))     then

 8160 format ('  Deleted point ',a)
 8195 format (/ 'Number of points deleted = ',i5)

        if (aword(3) .eq. 'all') then

          npointx = 0
          do 160 n = 1, npoints       ! Loop over points.
            if (apoint(n) .eq. ' ') go to 160
            call compsubs (apoint(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 160
            call compsubs (apoint(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 160

            call ptdep (n, ndep, ' ')

            if (ndep .gt. 0) go to 160

            write (   6, 8160) apoint(n)
            write (iout, 8160) apoint(n)
            call pout (n, n, 'min')

            do 159 npr = 1, npairs
              if (apairpt(npr) .eq. apoint(n)) then
                apaircl(npr) = ' '
                apairpt(npr) = ' '
              endif
  159       continue

            npointx   = npointx + 1
            apoint(n) = ' '
            kpoint(n) = 0
            lpoint(n) = 0
            mpoint(n) = 0

  160     continue                    ! End of loop over points.
          write (   6, 8195) npointx
          write (iout, 8195) npointx

        elseif ((aword(3) .eq. 'cluster') .or.
     &          (aword(3) .eq. 'cl'))     then

 8162     format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Not 4 words on "delete point cluster CLNAME" line:  ',i3)

          if (nword .ne. 4) then
            write (   6, 8162) ainput, ninput, nword
            write (iout, 8162) ainput, ninput, nword
            go to 999
          endif

 8170 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cluster does not exist:  ',a)

          do 170 n = 1, nclusts
            if (aword(4) .eq. aclust(n)) then
              call clout (n, n, 'min')
              ncl = n
              go to 175
            endif
  170     continue
          write (   6, 8170) ainput, ninput, aword(4)
          write (iout, 8170) ainput, ninput, aword(4)
          go to 999

  175     ndeptot = 0

          npointx = 0
          do 178 n = 1, npoints       ! Loop over points.

            do 177 npr = 1, npairs
              if ((apaircl(npr) .eq. aclust(ncl)) .and.
     &            (apairpt(npr) .eq. apoint(n)))  then

                call ptdep (n, ndep, aclust(ncl))

                ndeptot = ndeptot + ndep
                if (ndep .gt. 0) go to 178

                apaircl(npr) = ' '
                apairpt(npr) = ' '

                write (   6, 8160) apoint(n)
                write (iout, 8160) apoint(n)
                call pout (n, n, 'min')

                do 176 nx = 1, npairs
                  if (apairpt(nx) .eq. apoint(n)) then
                    apaircl(nx) = ' '
                    apairpt(nx) = ' '
                  endif
  176           continue

                npointx   = npointx + 1
                apoint(n) = ' '
                kpoint(n) = 0
                lpoint(n) = 0
                mpoint(n) = 0

              endif
  177       continue
  178     continue                    ! End of loop over points.

          if (ndeptot .eq. 0) then
            write (   6, 8250) aclust(ncl)
            write (iout, 8250) aclust(ncl)
            aclust(ncl) = ' '
          endif
          write (   6, 8195) npointx
          write (iout, 8195) npointx

        else                          ! Delete one point.

          call ptfind (3, n)

          if (n .eq. 0) go to 999

          call ptdep (n, ndep, ' ')

          if (ndep .gt. 0) go to 999

          write (   6, 8160) apoint(n)
          write (iout, 8160) apoint(n)

          do 179 npr = 1, npairs
            if (apairpt(npr) .eq. apoint(n)) then
              apaircl(npr) = ' '
              apairpt(npr) = ' '
            endif
  179     continue

          apoint(n) = ' '
          kpoint(n) = 0
          lpoint(n) = 0
          mpoint(n) = 0

        endif

        go to 999

      endif                           ! Deleted points.

c===============================================================================

c.... Input line:  delete polygon PGNAME
c.... Input line:  delete polygon PGNAME1 thru PGNAME2
c.... Input line:  delete polygon all

c.... See if polygons are to be deleted.

      if ((aword(2) .eq. 'polygon') .or.
     &    (aword(2) .eq. 'polyg')   .or.
     &    (aword(2) .eq. 'pg'))     then

 8550 format ('  Deleted polygon ',a)
 8560 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named regular polygon does not exist:  ',a)
 8562 format (/ 'Number of polygons deleted = ',i5)

        if (aword(3) .eq. 'all') then
          npolygx = 0
          do 550 n = 1, npolygs
            if (apolyg(n) .eq. ' ') go to 550
            call compsubs (apolyg(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 550
            call compsubs (apolyg(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 550
            write (   6, 8550) apolyg(n)
            write (iout, 8550) apolyg(n)
            call pgout (n, n, 'min')
            npolygx   = npolygx + 1
            apolyg(n) = ' '
  550     continue
          write (   6, 8562) npolygx
          write (iout, 8562) npolygx
        else                          ! Delete one polygon.
          do 560 n = 1, npolygs
            if (aword(3) .eq. apolyg(n)) then
              write (   6, 8550) apolyg(n)
              write (iout, 8550) apolyg(n)
              call pgout (n, n, 'min')
              apolyg(n) = ' '
              go to 999
            endif
  560     continue
          write (   6, 8560) ainput, ninput, aword(3)
          write (iout, 8560) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted polygons.

c===============================================================================

c.... Input line:  delete polyhedron PHNAME
c.... Input line:  delete polyhedron PHNAME1 thru PHNAME2
c.... Input line:  delete polyhedron all

c.... See if polyhedrons are to be deleted.

      if ((aword(2) .eq. 'polyhedron') .or.
     &    (aword(2) .eq. 'polyh')      .or.
     &    (aword(2) .eq. 'ph'))        then

 8570 format ('  Deleted polyhedron ',a)
 8580 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named regular polyhedron does not exist:  ',a)
 8582 format (/ 'Number of polyhedrons deleted = ',i5)

        if (aword(3) .eq. 'all') then
          npolyhx = 0
          do 570 n = 1, npolyhs
            if (apolyh(n) .eq. ' ') go to 570
            call compsubs (apolyh(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 570
            call compsubs (apolyh(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 570
            write (   6, 8570) apolyh(n)
            write (iout, 8570) apolyh(n)
            call phout (n, n, 'min')
            npolyhx   = npolyhx + 1
            apolyh(n) = ' '
  570     continue
          write (   6, 8582) npolyhx
          write (iout, 8582) npolyhx
        else                          ! Delete one polyhedron.
          do 580 n = 1, npolyhs
            if (aword(3) .eq. apolyh(n)) then
              write (   6, 8570) apolyh(n)
              write (iout, 8570) apolyh(n)
              call phout (n, n, 'min')
              apolyh(n) = ' '
              go to 999
            endif
  580     continue
          write (   6, 8580) ainput, ninput, aword(3)
          write (iout, 8580) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted polyhedrons.

c===============================================================================

c.... Input line:  delete quadric   QNAME
c.... Input line:  delete quadric   QNAME1 thru QNAME2
c.... Input line:  delete quadric   all

c.... Input line:  delete axisym    AXNAME
c.... Input line:  delete axisym    AXNAME1 thru AXNAME2
c.... Input line:  delete axisym    all
c.... Input line:  delete cone      CNNAME
c.... Input line:  delete cone      CNNAME1 thru CNNAME2
c.... Input line:  delete cone      all
c.... Input line:  delete cylinder  CYLNAME
c.... Input line:  delete cylinder  CYLNAME1 thru CYLNAME2
c.... Input line:  delete cylinder  all
c.... Input line:  delete ellipsoid ELNAME
c.... Input line:  delete ellipsoid ELNAME1 thru ELNAME2
c.... Input line:  delete ellipsoid all
c.... Input line:  delete hyperb    HPNAME
c.... Input line:  delete hyperb    HPNAME1 thru HPNAME2
c.... Input line:  delete hyperb    all
c.... Input line:  delete plane     PLNAME
c.... Input line:  delete plane     PLNAME1 thru PLNAME2
c.... Input line:  delete plane     all
c.... Input line:  delete sphere    SPHNAME
c.... Input line:  delete sphere    SPHNAME1 thru SPHNAME2
c.... Input line:  delete sphere    all

c.... See if quadric surfaces are to be deleted.

      if ((aword(2) .eq. 'quadric')   .or.
     &    (aword(2) .eq. 'quad')      .or.
     &    (aword(2) .eq. 'q')         .or.
     &    (aword(2) .eq. 'plane')     .or.
     &    (aword(2) .eq. 'pl')        .or.
     &    (aword(2) .eq. 'sphere')    .or.
     &    (aword(2) .eq. 'sph')       .or.
     &    (aword(2) .eq. 'cylinder')  .or.
     &    (aword(2) .eq. 'cyl')       .or.
     &    (aword(2) .eq. 'cone')      .or.
     &    (aword(2) .eq. 'hyperb')    .or.
     &    (aword(2) .eq. 'saddle')    .or.
     &    (aword(2) .eq. 'ellipsoid') .or.
     &    (aword(2) .eq. 'ellipse')   .or.
     &    (aword(2) .eq. 'ell')       .or.
     &    (aword(2) .eq. 'el')        .or.
     &    (aword(2) .eq. 'axisym'))   then

 8510 format ('  Deleted ',a9,' ',a)
 8515 format (/ 'Number of quadrics deleted = ',i5)
 8520 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)
 8525 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not delete quadric that defines a zone:  ' / a / a)

        if (aword(3) .eq. 'all') then

          nquadx = 0
          do 510 n = 1, nquads        ! Delete all quadric surfaces.
            if (aquad(n) .eq. ' ') go to 510
            call compsubs (aquad(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 510
            call compsubs (aquad(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 510

            call qutype (aword(2), n, 0, nerrq)

            if (nerrq .ne. 0) go to 510

            ndep = 0

            do 505 nz = 1, nzones
              if (azone(nz) .eq. ' ') go to 505
              do 502 nq = 1, nzquads(nz)
                if (azquad(nq,nz) .eq. aquad(n)) then
                  write (   6, 8525) ainput, ninput, aquad(n), azone(nz)
                  write (iout, 8525) ainput, ninput, aquad(n), azone(nz)
                  ndep = 1
                endif
  502         continue
  505       continue

            if (ndep .gt. 0) go to 510

            write (   6, 8510) aword(2), aquad(n)
            write (iout, 8510) aword(2), aquad(n)

            if ((aword(2) .eq. 'plane') .or.
     &          (aword(2) .eq. 'pl'))  then
              call plout (n, n)
            else
              call quout (n, n, 'min', nerrq)
            endif

            nquadx   = nquadx + 1
            aquad(n) = ' '
  510     continue

c....     Find last non-blank quadric name.

          ntot = nquads
          do 512 n = nquads, 1, -1
            if (aquad(n) .ne. ' ') go to 514
            ntot = n - 1
  512     continue
  514     nquads = ntot

          write (   6, 8515) nquadx
          write (iout, 8515) nquadx

        else                          ! Delete one quadric surface.

          do 520 n = 1, nquads
            if (aword(3) .eq. aquad(n)) then

              call qutype (aword(2), n, 1, nerrq)

              if (nerrq .ne. 0) go to 999

              ndep = 0

              do 525 nz = 1, nzones
                if (azone(nz) .eq. ' ') go to 525
                do 522 nq = 1, nzquads(nz)
                  if (azquad(nq,nz) .eq. aquad(n)) then
                    write (   6, 8525) ainput, ninput, aquad(n),
     &                                 azone(nz)
                    write (iout, 8525) ainput, ninput, aquad(n),
     &                                 azone(nz)
                    ndep = 1
                  endif
  522           continue
  525         continue

              if (ndep .gt. 0) go to 999

              write (   6, 8510) aword(2), aquad(n)
              write (iout, 8510) aword(2), aquad(n)

              if ((aword(2) .eq. 'plane') .or.
     &            (aword(2) .eq. 'pl'))  then
                call plout (n, n)
              else
                call quout (n, n, 'min', nerrq)
              endif

              aquad(n) = ' '
              go to 999
            endif
  520     continue
          write (   6, 8520) ainput, ninput, aword(3)
          write (iout, 8520) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted quadric surfaces.

c===============================================================================

c.... Input line:  delete pdf PDFNAME
c.... Input line:  delete pdf PDFNAME1 thru PDFNAME2
c.... Input line:  delete pdf all

c.... See if pdfs are to be deleted.

      if (aword(2) .eq. 'pdf') then

 9210 format ('  Deleted pdf ',a)
 9220 format (/ 'Number of pdfs deleted = ',i5)
 9230 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named pdf does not exist:  ',a)

        if (aword(3) .eq. 'all') then

          npdfxx = 0
          do 1210 n = 1, npdfs
            if (apdf(n) .eq. ' ') go to 1210
            call compsubs (apdf(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 1210
            call compsubs (apdf(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 1210
            write (   6, 9210) apdf(n)
            write (iout, 9210) apdf(n)
            call pdfout (n, n, 'min')
            npdfxx  = npdfxx + 1
            apdf(n) = ' '
 1210     continue
          write (   6, 9220) npdfxx
          write (iout, 9220) npdfxx

        else                          ! Delete one pdf.

          do 1230 n = 1, npdfs
            if (aword(3) .eq. apdf(n)) then
              write (   6, 9210) apdf(n)
              write (iout, 9210) apdf(n)
              call pdfout (n, n, 'min')
              do 1220 npr = 1, npdfxs
                if (apdfx(npr) .eq. apdf(n)) then
                  apdfx(npr) = ' '
                  abinx(npr) = ' '
                endif
 1220         continue
              apdf(n) = ' '
              go to 999
            endif
 1230     continue
          write (   6, 9230) ainput, ninput, aword(3)
          write (iout, 9230) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted pdfs.

c===============================================================================

c.... Input line:  delete symbol SYMBNAME
c.... Input line:  delete symbol SYMBNAME1 thru SYMBNAME2
c.... Input line:  delete symbol all

c.... See if symbols are to be deleted.

      if ((aword(2) .eq. 'symbol')     .or.
     &    (aword(2) .eq. 'symb')       .or.
     &    (aword(2) .eq. 'sym'))       then

 9330 format ('  Deleted symbol ',a)
 9340 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named symbol does not exist:  ',a)
 9342 format (/ 'Number of symbols deleted = ',i5)

        if (aword(3) .eq. 'all') then

          nsymbx = 0
          do 1335 n = 1, nsymbs
            if (asymbl(n) .eq. ' ') go to 1335

            call compsubs (asymbl(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 1335
            call compsubs (asymbl(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 1335

            write (   6, 9330) asymbl(n)
            write (iout, 9330) asymbl(n)

            call symbdisp (n)

            nsymbx    = nsymbx + 1
            asymbl(n) = ' '
            asymbr(n) = ' '
 1335     continue

          write (   6, 9342) nsymbx
          write (iout, 9342) nsymbx

        else                          ! Delete one symbol.

          do 1340 n = 1, nsymbs
            if (aword(3) .eq. asymbl(n)) then

              write (   6, 9330) asymbl(n)
              write (iout, 9330) asymbl(n)

              call symbdisp (n, n)

              asymbl(n) = ' '
              asymbr(n) = ' '
              go to 999
            endif
 1340     continue

          write (   6, 9340) ainput, ninput, aword(3)
          write (iout, 9340) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted symbols.

c===============================================================================

c.... Input line:  delete tetrahedron TRNAME
c.... Input line:  delete tetrahedron TRNAME1 thru TRNAME2
c.... Input line:  delete tetrahedron all

c.... See if tetrahedrons are to be deleted.

      if ((aword(2) .eq. 'tetrahedron') .or.
     &    (aword(2) .eq. 'tetra')       .or.
     &    (aword(2) .eq. 'tet'))        then

 8460 format ('  Deleted tetrahedron ',a)
 8470 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named tetrahedron does not exist:  ',a)
 8472 format (/ 'Number of tetrahedrons deleted = ',i5)

        if (aword(3) .eq. 'all') then
          ntetx = 0
          do 460 n = 1, ntets
            if (atetra(n) .eq. ' ') go to 460
            call compsubs (atetra(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 460
            call compsubs (atetra(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 460
            write (   6, 8460) atetra(n)
            write (iout, 8460) atetra(n)
            call tetout (n, n, 'min')
            ntetx     = ntetx + 1
            atetra(n) = ' '
  460     continue
          write (   6, 8472) ntetx
          write (iout, 8472) ntetx
        else                          ! Delete one tetrahedron.
          do 470 n = 1, ntets
            if (aword(3) .eq. atetra(n)) then
              write (   6, 8460) atetra(n)
              write (iout, 8460) atetra(n)
              call tetout (n, n, 'min')
              atetra(n) = ' '
              go to 999
            endif
  470     continue
          write (   6, 8470) ainput, ninput, aword(3)
          write (iout, 8470) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted tetrahedrons.

c===============================================================================

c.... Input line:  delete triangle TRNAME
c.... Input line:  delete triangle TRNAME1 thru TRNAME2
c.... Input line:  delete triangle all

c.... See if triangles are to be deleted.

      if ((aword(2) .eq. 'triangle') .or.
     &    (aword(2) .eq. 'tri'))     then

 8440 format ('  Deleted triangle ',a)
 8450 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named triangle does not exist:  ',a)
 8452 format (/ 'Number of triangles deleted = ',i5)

        if (aword(3) .eq. 'all') then
          ntrigx = 0
          do 440 n = 1, ntrigs
            if (atrig(n) .eq. ' ') go to 440
            call compsubs (atrig(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 440
            call compsubs (atrig(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 440
            write (   6, 8440) atrig(n)
            write (iout, 8440) atrig(n)
            call trout (n, n, 'min')
            ntrigx   = ntrigx + 1
            atrig(n) = ' '
  440     continue
          write (   6, 8452) ntrigx
          write (iout, 8452) ntrigx
        else                          ! Delete one triangle.
          do 450 n = 1, ntrigs
            if (aword(3) .eq. atrig(n)) then
              write (   6, 8440) atrig(n)
              write (iout, 8440) atrig(n)
              call trout (n, n, 'min')
              atrig(n) = ' '
              go to 999
            endif
  450     continue
          write (   6, 8450) ainput, ninput, aword(3)
          write (iout, 8450) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted triangles.

c===============================================================================

c.... Input line:  delete variable VARNAME
c.... Input line:  delete variable VARNAME1 thru VARNAME2
c.... Input line:  delete variable all

c.... See if variables are to be deleted.

      if ((aword(2) .eq. 'variable') .or.
     &    (aword(2) .eq. 'var')      .or.
     &    (aword(2) .eq. 'set')      .or.
     &    (aword(2) .eq. 's'))       then

 8130 format ('  Deleted variable ',a)
 8140 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named variable does not exist:  ',a)
 8142 format (/ 'Number of variables deleted = ',i5)

        if (aword(3) .eq. 'all') then

          nvarx = 0
          do 130 n = 1, nvars
            if (avarbl(n) .eq. ' ') go to 130
            call compsubs (avarbl(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 130
            call compsubs (avarbl(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 130
            write (   6, 8130) avarbl(n)
            write (iout, 8130) avarbl(n)
            call varout (n, n)
            nvarx     = nvarx + 1
            avarbl(n) = ' '
            avar(n)   = ' '
            avartag(n) = ' '
  130     continue
          write (   6, 8142) nvarx
          write (iout, 8142) nvarx
        else                          ! Delete one variable.
          do 140 n = 1, nvars
            if (aword(3) .eq. avarbl(n)) then
              write (   6, 8130) avarbl(n)
              write (iout, 8130) avarbl(n)
              call varout (n, n)
              avarbl(n) = ' '
              avar(n)   = ' '
              avartag(n) = ' '
              go to 999
            endif
  140     continue
          write (   6, 8140) ainput, ninput, aword(3)
          write (iout, 8140) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted variables.

c===============================================================================

c.... Input line:  delete big BIGNAME
c.... Input line:  delete big BIGNAME1 thru BIGNAME2
c.... Input line:  delete big all

c.... See if big integers are to be deleted.

      if (aword(2) .eq. 'big') then

 8134 format ('  Deleted big integer ',a)
 8144 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named big integer does not exist:  ',a)
 8141 format (/ 'Number of big integers deleted = ',i5)

        if (aword(3) .eq. 'all') then

          nbigx = 0
          do 134 n = 1, nbigm
            if ((ndig(n) .eq. ' ') .or.
     &         ((ndig(n) .eq. 1) .and. (idig(1,n) .eq. 0))) go to 134
            if ((abig(n) .lt. abigmn) .or.
     &          (abig(n) .gt. abigmx)) go to 134
            write (   6, 8134) abig(n)
            write (iout, 8134) abig(n)
            call bigout (n, n, 'min')
            nbigx     = nbigx + 1
            do i = 1, ndig(n)
              idig(i,n) = 0
            enddo
            ndig(n) = 1
            abigtag(n) = ' '
  134     continue
          write (   6, 8141) nbigx
          write (iout, 8141) nbigx
        else                          ! Delete one big integer.
          do 144 n = 1, nbigm
            if (aword(3) .eq. abig(n)) then
              if ((ndig(n) .eq. 0) .or.
     &           ((ndig(n) .eq. 1) .and. (idig(1,n) .eq. 0))) then
                write (   6, 8144) ainput, ninput, aword(3)
                write (iout, 8144) ainput, ninput, aword(3)
                go to 999
              endif
              write (   6, 8134) abig(n)
              write (iout, 8134) abig(n)
              call bigout (n, n, 'min')
              do i = 1, ndig(n)
                idig(i,n) = 0
              enddo
              ndig(n) = 1
              abigtag(n) = ' '
              go to 999
            endif
  144     continue
          write (   6, 8144) ainput, ninput, aword(3)
          write (iout, 8144) ainput, ninput, aword(3)

        endif                         ! Tested for all or one big integer.

        go to 999

      endif                           ! Deleted big integers.

c===============================================================================

c.... Input line:  delete vector VNAME
c.... Input line:  delete vector VNAME1 thru VNAME2
c.... Input line:  delete vector all

c.... See if vectors are to be deleted.

      if ((aword(2) .eq. 'vector') .or.
     &    (aword(2) .eq. 'vect')   .or.
     &    (aword(2) .eq. 'v'))     then

 8210 format ('  Deleted vector ',a)
 8220 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named vector does not exist:  ',a)
 8222 format (/ 'Number of vectors deleted = ',i5)

        if (aword(3) .eq. 'all') then
          nvectorx = 0
          do 210 n = 1, nvectors
            if (avector(n) .eq. ' ') go to 210
            call compsubs (avector(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 210
            call compsubs (avector(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 210
            write (   6, 8210) avector(n)
            write (iout, 8210) avector(n)
            call vout (n, n)
            nvectorx   = nvectorx + 1
            avector(n) = ' '
  210     continue
          write (   6, 8222) nvectorx
          write (iout, 8222) nvectorx
        else                          ! Delete one vector.

          call vectfind (3, n)

          if (n .eq. 0) go to 999

          write (   6, 8210) avector(n)
          write (iout, 8210) avector(n)
          avector(n) = ' '

        endif

        go to 999

      endif                           ! Deleted vectors.

c===============================================================================

c.... Input line:  delete zone ZNAME
c.... Input line:  delete zone ZNAME1 thru ZNAME2
c.... Input line:  delete zone all

c.... See if zones are to be deleted.

      if ((aword(2) .eq. 'zone') .or.
     &    (aword(2) .eq. 'zn')   .or.
     &    (aword(2) .eq. 'z'))   then

 8610 format ('  Deleted ',a9,' ',a)
 8615 format (/ 'Number of zones deleted = ',i5)
 8620 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named zone does not exist:  ',a)

        if (aword(3) .eq. 'all') then

          nzonex = 0
          do 610 n = 1, nzones        ! Delete all zones.
            if (azone(n) .eq. ' ') go to 610
            call compsubs (azone(n), anamemn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 610
            call compsubs (azone(n), anamemx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 610

            write (   6, 8610) aword(2), azone(n)
            write (iout, 8610) aword(2), azone(n)

            call zout (n, n, 'min')

            nzonex     = nzonex + 1
            azone(n)   = ' '
            nzquads(n) = 0
  610     continue

c....     Find last non-blank zone name.

          ntot = nzones
          do 612 n = nzones, 1, -1
            if (azone(n) .ne. ' ') go to 614
            ntot = n - 1
  612     continue
  614     nzones = ntot

          write (   6, 8615) nzonex
          write (iout, 8615) nzonex

        else                          ! Delete one zone.

          do 620 n = 1, nzones
            if (aword(3) .eq. azone(n)) then

              write (   6, 8610) aword(2), azone(n)
              write (iout, 8610) aword(2), azone(n)

              call zout (n, n, 'min')

              azone(n)   = ' '
              nzquads(n) = 0
              go to 999
            endif
  620     continue
          write (   6, 8620) ainput, ninput, aword(3)
          write (iout, 8620) ainput, ninput, aword(3)

        endif

        go to 999

      endif                           ! Deleted zones.

c===============================================================================

 9990 format ('  delete ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Object type does not exist:  ',a24)

      write (   6, 9990) ainput, ninput, aword(2)
      write (iout, 9990) ainput, ninput, aword(2)

  999 return

c.... End of subroutine delete.       (+1 line)
      end
