      subroutine dkout (n1, n2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  dkout    Updated     2004 April 9 15:20.
c               dkout    Originated  1994 April 1 11:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the names, inner and outer radii, central points and
c               normal vectors of annular disks.
c
c     Logic:    Entry points:  dkout.
c dkout         Called by:  copy, delete, disk, plane, point, renamea,
c               renamex, search.
c
c dkout         Calls:  aptcsyv, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Initialize.

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. ndisks) nn2 = ndisks

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

 8110 format ('coord     ',a,5x,a,4x,a,4x,a)

      write (   6, 8110) asys, acoordu, acoordv, acoordw
      write (iout, 8110) asys, acoordu, acoordv, acoordw

      do 150 n = nn1, nn2
        if (adisk(n) .eq. ' ') go to 150

 8120   format ('disk      ',a,' centered at point ',a )

        write (   6, 8120) adisk(n), acendk(n)
        write (iout, 8120) adisk(n), acendk(n)

c....   Find the center point of the disk.

 8130   format ('  dkout ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Center point does not exist:  ',a)

        do 130 np = 1, npoints
          if (acendk(n) .eq. apoint(np)) then
            call pout (np, np, 'min')
            u = x(np)
            v = y(np)
            w = z(np)
            go to 140
          endif
  130   continue
        write (   6, 8130) ainput, ninput, acendk(n)
        write (iout, 8130) ainput, ninput, acendk(n)
        go to 150

  140   du = xnormdk(n)
        dv = ynormdk(n)
        dw = znormdk(n)

        if     (asys .eq. 'cyl') then
          call aptcsyv (0, 1, iunit, u, v, w, du, dv, dw, 1,
     &                  tol, nerr)
        elseif (asys .eq. 'sph') then
          call aptcsyv (0, 2, iunit, u, v, w, du, dv, dw, 1,
     &                  tol, nerr)
        endif

 8140   format ('  Normal vector   ',1p3e20.12 )
 8145   format ('  Radii  (in, out)',1p2e20.12 /
     &          '  Circum (in, out)',1p2e20.12 )
 8150   format ('  Area (annul, in)',1p2e20.12 )

        write (   6, 8140) du, dv, dw
        write (iout, 8140) du, dv, dw

        circin  = 2.0 * pi * radkin(n)
        circout = 2.0 * pi * radkout(n)

        write (   6, 8145) radkin(n), radkout(n), circin, circout
        write (iout, 8145) radkin(n), radkout(n), circin, circout

        areadk = pi * (radkout(n) - radkin(n)) *
     &                (radkout(n) + radkin(n))
        areah  = pi * radkin(n)**2

        write (   6, 8150) areadk, areah
        write (iout, 8150) areadk, areah

  150 continue

      return

c.... End of subroutine dkout.        (+1 line)
      end
