      subroutine dot

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  dot      Updated     2000 August 2 17:00.
c               dot      Originated  1993 November 10 15:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To interpret data on "dot" input lines, to find the dot (inner)
c               product and angle between two vectors in 3-D space.
c
c     Logic:    Entry points:  dot.
c dot           Called by:  helpall, main.
c
c dot           Calls:  aptvang, aptvdot, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input line:  dot [help]

 8110 format ('  Command "dot":  acceptable input lines include:' /
     &  '  "dot [help]"         Display this message.' //
     &  '  "dot VNAME1 VNAME2"  Find the dot product' /
     &  '    of vectors VNAME1 and VNAME2, and the angle',
     &  ' between them.' //
     &  '  "variable VARNAME angle VNAME1 VNAME2"' /
     &  '    Create variable VARNAME, the angle between vectors',
     &  ' VNAME1 and VNAME2,' /
     &  '    in the current angle units.' //
     &  '  Synonyms:  [dot, inner], [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c.... Test for wrong number of words on "dot" line.

 8120 format ('  dot ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 3 words on "dot" line:  ', i3)
      if (nword .ne. 3) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c=======================================================================********

c.... Input line:  dot VNAME1 VNAME2

c.... Find the first vector.

      call vectfind (2, n1)

      if (n1 .eq. 0) go to 999

c.... Find the second vector.

      call vectfind (3, n2)

      if (n2 .eq. 0) go to 999

c.... Find the dot product of the two vectors.

      call aptvdot (xcom(n1), ycom(n1), zcom(n1),
     &              xcom(n2), ycom(n2), zcom(n2), 1, tol, vdot, nerra)

c.... Find the cosine of the angle between the two vectors.

      call aptvang (xcom(n1), ycom(n1), zcom(n1),
     &              xcom(n2), ycom(n2), zcom(n2), 1, tol, vcos, nerra)

c.... Find the angle between the two vectors.

      vang = acos (vcos)
      if (abs (vang) .lt. 1.e-5) vang = 0.0
      if (abs (vang - pi) .lt. 1.e-5) vang = pi

      if (angunit .eq. 'degrees') then
        vang = vang * 180.0 / pi
      endif

 8160 format ('dot product       ',1pe20.12 )
 8165 format ('  Can not find the angle if one vector has zero length.')
 8170 format ('cosine of angle   ',1pe20.12 /
     &        'angle in ',a,1x,1pe20.12)

      write (   6, 8160) vdot
      write (iout, 8160) vdot

      if ((vlen(n1) .eq. 0.0) .or.
     &    (vlen(n2) .eq. 0.0)) then
        write (   6, 8165)
        write (iout, 8165)
        go to 999
      endif

      write (   6, 8170) vcos, angunit, vang
      write (iout, 8170) vcos, angunit, vang

c=======================================================================********

  999 return

c.... End of subroutine dot.          (+1 line)
      end
