      subroutine intcyl (ncyl1, ncyl2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  intcyl   Updated     1996 March 13 16:50.
c               intcyl   Originated  1994 December 21 17:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the distance of separation or overlap between two
c               circular cylinders.
c
c     Logic:    Entry points:  intcyl.
c intcyl        Called by:  distance.
c
c intcyl        Calls:  aptcycy, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Find the distances between the axes, the distance of separation or
c....   overlap, the corresponding points on the two circular cylinders,
c....   the vector between the points, and whether the axes are parallel.
 
      call aptcycy (qc(ncyl1), qx(ncyl1), qy(ncyl1), qz(ncyl1),
     &              qxy(ncyl1), qyz(ncyl1), qzx(ncyl1), 
     &              qxx(ncyl1), qyy(ncyl1), qzz(ncyl1),
     &              qc(ncyl2), qx(ncyl2), qy(ncyl2), qz(ncyl2),
     &              qxy(ncyl2), qyz(ncyl2), qzx(ncyl2),
     &              qxx(ncyl2), qyy(ncyl2), qzz(ncyl2), tol,
     &              rcyl1, rcyl2, ax1, ay1, az1, ax2, ay2, az2,
     &              bx, by, bz, daxis,
     &              cx1, cy1, cz1, cx2, cy2, cz2,
     &              dx, dy, dz, dsurf, ipar, nerr)
 
      pdist = dsurf

      proxptx = cx2
      proxpty = cy2
      proxptz = cz2

      proxvx = dx
      proxvy = dy
      proxvz = dz

      if (daxis .eq. 0.0) then        ! Cylinder axes intersect or coincide.

        dmm = -rcyl1 - rcyl2
        dpp = rcyl1 + rcyl2
        errd = tol * dpp
        dmp = -rcyl1 + rcyl2
        if (abs (dmp) .le. errd) dmp = 0.0
        dpm = rcyl1 - rcyl2
        if (abs (dpm) .le. errd) dpm = 0.0

 8140   format ('Axes coincide.' )
 8150   format ('Cylinders coincide.' )
 8235   format ('  Min dist   (ext)',1pe20.12,'  d12 - r1 - r2, 1| |2' /
     &          '  Separation (ext)',1pe20.12,'  d12 + r1 + r2, |1 2|' /
     &          '  Separation (int)',1pe20.12,'  d12 - r1 + r2, |1 |2' /
     &          '  Separation (int)',1pe20.12,'  d12 + r1 - r2, 1| 2|' )

        if (ipar .ne. 0) then         ! Axes coincide.
          if (ipar .eq. 2) then       ! Cylinders coincide.
            write (   6, 8150)
            write (iout, 8150)
          else
            write (   6, 8140)
            write (iout, 8140)
          endif

          write (   6, 8235) dmm, dpp, dmp, dpm
          write (iout, 8235) dmm, dpp, dmp, dpm

          go to 999
        endif

 8160   format ('Axes intersect at ',1p3e20.12)
 8200   format ('Cyl 1  prox x,y,z ',1p3e20.12)
 8210   format ('Cyl 2  prox x,y,z ',1p3e20.12)
 8230   format ('Cyl prox dx,dy,dz ',1p3e20.12)

        write (   6, 8160) ax1, ay1, az1
        write (iout, 8160) ax1, ay1, az1
        call plotpt (ax1, ay1, az1, nerrg)

        write (   6, 8200) cx1, cy1, cz1
        write (iout, 8200) cx1, cy1, cz1
        call plotpt (cx1, cy1, cz1, nerrg)

        write (   6, 8210) cx2, cy2, cz2
        write (iout, 8210) cx2, cy2, cz2
        call plotpt (cx2, cy2, cz2, nerrg)

        write (   6, 8230) dx, dy, dz
        write (iout, 8230) dx, dy, dz

        write (   6, 8235) dmm, dpp, dmp, dpm
        write (iout, 8235) dmm, dpp, dmp, dpm
        go to 999

      else                            ! Axes do not intersect.

 8180   format ('Axes are parallel.')
 8182   format ('Axis 1 prox x,y,z ',1p3e20.12 /
     &          'Axis 2 prox x,y,z ',1p3e20.12 )
 8184   format ('Axis prox dx,dy,dz',1p3e20.12 /
     &          'Axis distance     ',1pe20.12 )
 8190   format ('Tangent at x,y,z  ',1p3e20.12 )

c....   Display the axis proximal points and their distance.

        if (ipar .eq. 1) then         ! The cylinders are parallel.
          write (   6, 8180)
          write (iout, 8180)
        else
          write (   6, 8182) ax1, ay1, az1, ax2, ay2, az2
          write (iout, 8182) ax1, ay1, az1, ax2, ay2, az2
          call plotpt (ax1, ay1, az1, nerrg)
          call plotpt (ax2, ay2, az2, nerrg)
        endif

        write (   6, 8184) bx, by, bz, daxis
        write (iout, 8184) bx, by, bz, daxis

c....   Find all of the separation distances.

        dmm = dsurf
        dpp = daxis + rcyl1 + rcyl2
        dmp = daxis - rcyl1 + rcyl2
        errd = tol * dpp
        if (abs (dmp) .le. errd) dmp = 0.0
        dpm = daxis + rcyl1 - rcyl2
        if (abs (dpm) .le. errd) dpm = 0.0

c....   Display the surface proximal points and their distance.

        if (dsurf .eq. 0.0) then      ! The cylinders are tangent.
          write (   6, 8190) cx1, cy1, cz1
          write (iout, 8190) cx1, cy1, cz1
          call plotpt (cx1, cy1, cz1, nerrg)

          write (   6, 8235) dmm, dpp, dmp, dpm
          write (iout, 8235) dmm, dpp, dmp, dpm
          go to 999
        endif

        write (   6, 8200) cx1, cy1, cz1
        write (iout, 8200) cx1, cy1, cz1
        call plotpt (cx1, cy1, cz1, nerrg)

        write (   6, 8210) cx2, cy2, cz2
        write (iout, 8210) cx2, cy2, cz2
        call plotpt (cx2, cy2, cz2, nerrg)

        write (   6, 8230) dx, dy, dz
        write (iout, 8230) dx, dy, dz

        write (   6, 8235) dmm, dpp, dmp, dpm
        write (iout, 8235) dmm, dpp, dmp, dpm

      endif                           ! Tested axis intersection or not.

  999 return

c.... End of subroutine intcyl.       (+1 line)
      end
