      subroutine intplqu (npl, nq)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  intplqu  Updated     2005 February 1 16:30.
c               intplqu  Originated  1995 January 23 14:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the distance of separation or overlap between a plane
c               and a quadric surface, any proximal points on the plane and
c               quadric surface, and points on any curve of intersection.
c
c     Logic:    Entry points:  intplqu
c intplqu       Called by:  distance.
c
c intplqu       Calls:  aptplqu, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      common /lintplqu/ ex(2)         ! Coordinate x of an extreme point "e".
      common /lintplqu/ ey(2)         ! Coordinate y of an extreme point "e".
      common /lintplqu/ ez(2)         ! Coordinate z of an extreme point "e".

      common /lintplqu/ pex(2)        ! Coordinate x of planar pt prox to "e".
      common /lintplqu/ pey(2)        ! Coordinate y of planar pt prox to "e".
      common /lintplqu/ pez(2)        ! Coordinate z of planar pt prox to "e".

      common /lintplqu/ dpex(2)       ! Component x of distance "pe" to "e".
      common /lintplqu/ dpey(2)       ! Component y of distance "pe" to "e".
      common /lintplqu/ dpez(2)       ! Component z of distance "pe" to "e".

      common /lintplqu/ dplex(2)      ! Distance from point "pe" to point "e".

      common /lintplqu/ actype        ! Type of intersection curve.
      character*24      actype        ! Type of intersection curve.

      common /lintplqu/ nlen          ! Number of int radii, lengths, distances.
      common /lintplqu/ alen(6)       ! Type of int radius, length or distance.
      character*16      alen          ! Type of int radius, length or distance.
      common /lintplqu/ dlen(6)       ! Radius, length or distance.

      common /lintplqu/ npin          ! Number of points of intersection.
      common /lintplqu/ apin(11)      ! Type of intersection point.
      character*16      apin          ! Type of intersection point.
      common /lintplqu/ pinx(11)      ! Coordinate x of an intersection point.
      common /lintplqu/ piny(11)      ! Coordinate y of an intersection point.
      common /lintplqu/ pinz(11)      ! Coordinate z of an intersection point.

      common /lintplqu/ nvin          ! Number of vectors of intersection.
      common /lintplqu/ avin(2)       ! Type of intersection vector.
      character*16      avin          ! Type of intersection vector.
      common /lintplqu/ vinx(2)       ! Component x of an intersection vector.
      common /lintplqu/ viny(2)       ! Component y of an intersection vector.
      common /lintplqu/ vinz(2)       ! Component z of an intersection vector.

c-----------------------------------------------------------------------********

c.... Initialize.

      pdist = -1.e99

      proxptx = -1.e99
      proxpty = -1.e99
      proxptz = -1.e99

      proxvx = -1.e99
      proxvy = -1.e99
      proxvz = -1.e99

c-----------------------------------------------------------------------********

c.... For plane npl and quadric surface nq, find the distances of separation
c       or overlap, any proximal points or lines, and the nature of and
c....   properties of any point or quadric curve of intersection.

      call aptplqu (xcen(npl), ycen(npl), zcen(npl),
     &              qx(npl), qy(npl), qz(npl),
     &              qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), tol,
     &              nqtype, px, py, pz, qcenx, qceny, qcenz,
     &              dplqu, dpx, dpy, dpz, nextr, ex, ey, ez,
     &              dplex, pex, pey, pez, dpex, dpey, dpez,
     &              actype, nlen, alen, dlen,
     &              npin, apin, pinx, piny, pinz,
     &              nvin, avin, vinx, viny, vinz,
     &              sc, sx, sy, sz, sxy, syz, szx, sxx, syy, szz,
     &              nerra)

c-----------------------------------------------------------------------********

c.... Display results.

 8110 format ('  intplqu ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Plane or quadric surface is bad.  aptplqu nerr =',i3)

      if (nerra .ne. 0) then
        write (   6, 8110) ainput, ninput, nerr
        write (iout, 8110) ainput, ninput, nerr
        go to 999
      endif

c.... Display data about the center of the quadric.

 8120 format ('The plane passes through the center of the',
     &  ' quadric surface.' /
     &  '  Center     (xyz)',1p3e20.12 )
 8130 format ('The plane intersects the vertex of a cone.' )
 8140 format ('The plane has a point proximal to the center of the',
     &  ' quadric surface:' /
     &  '  Point pl   (xyz)',1p3e20.12 /
     &  '  Point q cen(xyz)',1p3e20.12 /
     &  '  Distance pl to q',1pe20.12 /
     &  '  Vector     (xyz)',1p3e20.12 )

      if (dplqu .eq. 0.0) then        ! Plane passes through center of quadric.
        write (   6, 8120) qcenx, qceny, qcenz
        write (iout, 8120) qcenx, qceny, qcenz
        call plotpt (qcenx, qceny, qcenz, nerrg)
        if ((nqtype .eq. 11) .or. (nqtype .eq. 12)) then
          write (   6, 8130)
          write (iout, 8130)
        endif
      else                            ! Plane misses center of quadric.
        write (   6, 8140) px, py, pz, qcenx, qceny, qcenz,
     &    dplqu, dpx, dpy, dpz
        write (iout, 8140) px, py, pz, qcenx, qceny, qcenz,
     &    dplqu, dpx, dpy, dpz
        call plotpt (px, py, pz, nerrg)
        call plotpt (qcenx, qceny, qcenz, nerrg)
      endif                           ! Tested distance plane to quadric center.

c.... Display data about proximal and distal points.

 8150 format (/ 'Number of extrema on the quadric, in the direction',
     &  ' normal to the plane, is',i2,'.')
 8160 format ('Extremum',i2,' is tangent to the plane:' /
     &  '  Tangent pt (xyz)',1p3e20.12 )
 8170 format ('Extremum',i2,' may be proximal or distal to the plane:' )
 8180 format ('Extremum',i2,' is proximal to the plane:' )
 8190 format ('Extremum',i2,' is distal   to the plane:' )
 8200 format ('  Point pl   (xyz)',1p3e20.12 /
     &        '  Point q ex (xyz)',1p3e20.12 /
     &        '  Distance pl to q',1pe20.12  /
     &        '  Vector     (xyz)',1p3e20.12 )

      write (   6, 8150) nextr
      write (iout, 8150) nextr

      do 200 n = 1, nextr             ! Loop over extreme points.

        if (dplex(n) .eq. 0.0) then   ! Tangent.         

          pdist = 0.0

          write (   6, 8160) n, ex(n), ey(n), ez(n)
          write (iout, 8160) n, ex(n), ey(n), ez(n)
          call plotpt (ex(n), ey(n), ez(n), nerrg)

          proxptx = ex(n)
          proxpty = ey(n)
          proxptz = ez(n)

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

        else                          ! Not tangent.  See if proximal or distal.

          if (nextr .eq. 1) then      ! Indeterminate.
            pdist = dplex(n)
            write (   6, 8170) n
            write (iout, 8170) n

            proxptx = ex(n)
            proxpty = ey(n)
            proxptz = ez(n)

            proxvx = dpex(n)
            proxvy = dpey(n)
            proxvz = dpez(n)

          elseif (abs (dplex(n)) .lt. abs (dplex(3-n))) then  ! Proximal.
            pdist = dplex(n)
            write (   6, 8180) n
            write (iout, 8180) n

            proxptx = ex(n)
            proxpty = ey(n)
            proxptz = ez(n)

            proxvx = dpex(n)
            proxvy = dpey(n)
            proxvz = dpez(n)

          else                        ! Distal.
            write (   6, 8190) n
            write (iout, 8190) n
          endif

          write (   6, 8200) pex(n), pey(n), pez(n),
     &      ex(n), ey(n), ez(n), dplex(n), dpex(n), dpey(n), dpez(n)
          write (iout, 8200) pex(n), pey(n), pez(n),
     &      ex(n), ey(n), ez(n), dplex(n), dpex(n), dpey(n), dpez(n)

          call plotpt (pex(n), pey(n), pez(n), nerrg)
          call plotpt (ex(n), ey(n), ez(n), nerrg)

        endif                         ! Tested for tangency.

  200 continue                        ! End of loop over extreme points.

c-----------------------------------------------------------------------********

c.... Display data about any intersection curve.

 8210 format ('There is no intersection curve.')

      if (nqtype .gt. 16) then
        write (   6, 8210)
        write (iout, 8210)
      endif

 8220 format ('Type of intersection:  ',a24)
 8230 format (2x,a16,1pe20.12)
 8240 format (2x,a16,1p3e20.12 )
 8250 format ('Quadric cylinder perpendicular to intersection curve:' /
     &  '  Quadric  sc=    ',1pe20.12 /
     &  '  sx,sy,sz=       ',1p3e20.12 /
     &  '  sxy,syz,szx=    ',1p3e20.12 /
     &  '  sxx,syy,szz=    ',1p3e20.12)

      if (actype .ne. ' ') then       ! An intersection exists.

        if (pdist .eq. -1.e99) pdist = 0.0

        write (   6, 8220) actype
        write (iout, 8220) actype

        if (nlen .gt. 0) then
          write (   6, 8230) (alen(n), dlen(n), n = 1, nlen)
          write (iout, 8230) (alen(n), dlen(n), n = 1, nlen)
        endif

        if (npin .gt. 0) then
          write (   6, 8240) (apin(n), pinx(n), piny(n), pinz(n),
     &      n = 1, npin)
          write (iout, 8240) (apin(n), pinx(n), piny(n), pinz(n),
     &      n = 1, npin)
          call plotpt (pinx(n), piny(n), pinz(n), nerrg)
        endif

        if (nvin .gt. 0) then
          write (   6, 8240) (avin(n), vinx(n), viny(n), vinz(n),
     &      n = 1, nvin)
          write (iout, 8240) (avin(n), vinx(n), viny(n), vinz(n),
     &      n = 1, nvin)
        endif

        write (   6, 8250) sc, sx, sy, sz, sxy, syz, szx, sxx, syy, szz
        write (iout, 8250) sc, sx, sy, sz, sxy, syz, szx, sxx, syy, szz

      endif                           ! Tested for intersection.

c-----------------------------------------------------------------------********

  999 return

c.... End of subroutine intplqu.      (+1 line)
      end
