      subroutine iris

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  iris     Updated     2002 June 11 14:20.
c               iris     Originated  2002 June 11 14:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the coefficients of the series equivalent of a number
c               expressed as an integer plus a sum of reciprocal integers.
c
c     Logic:    Entry points:  iris.
c iris          Called by:  helpall, main.
c
c iris          Calls:  aptiris, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      dimension kc(64)                ! Coefficients of series.
      dimension xc(64)                ! Partial sum of series.

c.... Input line:  iris [help]

 8110 format (
     &  '  Command "iris":  acceptable input lines include:' //
     &  '  "iris [help]"  Display this message.' //
     &  '  "iris = VALUE"' /
     &  '     Find the coefficients kc(1), kc(2), kc(3), kc(4), ...'
     &  ' of the' /
     &  '     series equivalent of VALUE:' /
     &  '     VALUE = kc(1) + sum (n = 2, 3, 4, ...) 1 / kc(n). ' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... Input line:  iris = VALUE

      if (aword(2) .eq. '=') then

c.... Test number of words on input line.

 8135 format ('  iris ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "iris" line not 3:  ',i3)

      if (nword .ne. 3) then
        write (   6, 8135) ainput, ninput, nword
        write (iout, 8135) ainput, ninput, nword
        go to 999
      endif

c.... Find the variable.

 8276 format ('  iris ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 3 not integer or floating point value or variable:  ',a)
 8278 format ('  iris ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 3 negative:  ',a)

      call varfind (3, 3)
  
      if ((mtype(3) .ne. 2) .and. (mtype(3) .ne. 3)) then
        write (   6, 8276) ainput, ninput, aword(3)
        write (iout, 8276) ainput, ninput, aword(3)
        go to 999
      endif

c.... Find the coefficients of the series.

      call aptiris (fword(3), 64, tol, kc, xc, nc, nerra)

 8203 format ('  iris ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  VALUE exceeds largest possible integer.' )
 8204 format ('  iris ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A coefficient exceeds largest possible integer.' )

      if (nerra .ne. 0) then
        if (nerra .eq. 2) then
          write (   6, 8203) ainput, ninput
          write (iout, 8203) ainput, ninput
        elseif (nerra .eq. 3) then
          write (   6, 8204) ainput, ninput
          write (iout, 8204) ainput, ninput
        endif
        go to 999
      endif
  
c.... Display the coefficients and partial sums.

 8210 format ('  n, kc(n), xc(n) =',i3,i22,1pe23.15)
 8215 format ('  n, kc(n), xc(n) =',i3,i22,1pe20.12)

      if (tol .lt. 1.e-15) then
        write (   6, 8210) (n, kc(n), xc(n), n = 1, nc)
        write (iout, 8210) (n, kc(n), xc(n), n = 1, nc)
      else
        write (   6, 8215) (n, kc(n), xc(n), n = 1, nc)
        write (iout, 8215) (n, kc(n), xc(n), n = 1, nc)
      endif

 8220 format (/ '  xc(n) = kc(1) + 1 / kc(2) + ... + 1 / kc(n).' )
      write (   6, 8220)
      write (iout, 8220)

      go to 999

      endif                           ! Find coefficients of series.

c=======================================================================********

 8810 format ('  iris ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine iris.        (+1 line)
      end
