      subroutine kiss

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  kiss     Updated     2001 May 29 17:00.
c               kiss     Originated  1998 November 10 17:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find two circles, each tangent to three tangent circles,
c               or two spheres, each tangent to four tangent spheres.
c
c     Logic:    Entry points:  kiss.
c kiss          Called by:  helpall, main.
c
c kiss          Calls:  aptcirk, aptsphk, aptvdis, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

c.... Input line:  kiss [help]

 8110 format (
     &  '  Command "kiss":  acceptable input lines include:' /
     &  '  "kiss [help]"  Display this message.' //
     &  '  "kiss RAD1 RAD2 RAD3"' /
     &  '     Find the radii of two circles tangent to the three'
     & ' tangent circles' /
     &  '     with radii RAD1, RAD2 and RAD3.' //
     &  '  "kiss RAD1 RAD2 RAD3 RAD4"' /
     &  '     Find the radii of two spheres tangent to the four'
     & ' tangent spheres' /
     &  '     with radii RAD1, RAD2, RAD3 and RAD4.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... Input line:  kiss RAD1 RAD2 RAD3

      if (nword .eq. 4) then

c....   Find the radii of the three tangent circles.

        call varfind (2, 4)

 8310   format ('  kiss ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word not integer or floating point value or variable:  ',a)

        do 310 nw = 2, 4
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8310) ainput, ninput, aword(nw)
            write (iout, 8310) ainput, ninput, aword(nw)
            go to 999
          endif
  310   continue

        r1  = fword(2)
        r2  = fword(3)
        r3  = fword(4)

        if (r1 .ne. 0.0) then
          c1 = 1.0 / r1
        else
          c1 = 1.e99
        endif

        if (r2 .ne. 0.0) then
          c2 = 1.0 / r2
        else
          c2 = 1.e99
        endif

        if (r3 .ne. 0.0) then
          c3 = 1.0 / r3
        else
          c3 = 1.e99
        endif

 8322   format ('  Radii R1, R2, R3',1p3e20.12 )

        write (   6, 8322) r1, r2, r3
        write (iout, 8322) r1, r2, r3

 8323   format ('  1/R1, 1/R2, 1/R3',1p3e20.12 )

        write (   6, 8323) c1, c2, c3
        write (iout, 8323) c1, c2, c3

 8329   format ('  kiss ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Impossible radii.')

        call aptcirk (r1, r2, r3, tol, r4, r5, xp1, yp1, zp1,
     &                xp2, yp2, zp2, xp3, yp3, zp3,
     &                xp4, yp4, zp4, xp5, yp5, zp5, nerra)

        if (nerra .gt. 0) then
          write (   6, 8329) ainput, ninput
          write (iout, 8329) ainput, ninput
          go to 999
        endif

 8340   format (/ '  Circle 4 and circle 5 are each tangent to',
     &            ' circles 1, 2 and 3.' /
     &            '    Radius is negative if tangency is internal.' /
     &            '  Radii R4, R5    ',1p2e20.12)

        write (   6, 8340) r4, r5
        write (iout, 8340) r4, r5

        if (r4 .ne. 0.0) then
          c4 = 1.0 / r4
        else
          c4 = 1.e99
        endif

        if (r5 .ne. 0.0) then
          c5 = 1.0 / r5
        else
          c5 = 1.e99
        endif

 8342   format ('  1/R4, 1/R5      ',1p3e20.12 )

        write (   6, 8342) c4, c5
        write (iout, 8342) c4, c5

 8330   format (/ '  Centers of circles:' /
     &    '  Center 1 (xyz)  ',1p3e20.12 /
     &    '  Center 2 (xyz)  ',1p3e20.12 /
     &    '  Center 3 (xyz)  ',1p3e20.12 )

        write (   6, 8330) xp1, yp1, zp1, xp2, yp2, zp2, xp3, yp3, zp3
        write (iout, 8330) xp1, yp1, zp1, xp2, yp2, zp2, xp3, yp3, zp3

 8348   format ('    Note:  sign of y below is uncertain.' )

        if ((nerra .eq. -1) .or. (nerra .eq. -3)) then
          write (   6, 8348)
          write (iout, 8348)
        endif

 8350   format ('  Center 4 (xyz)  ',1p3e20.12 )

        write (   6, 8350) xp4, yp4, zp4
        write (iout, 8350) xp4, yp4, zp4

        if ((nerra .eq. -2) .or. (nerra .eq. -3)) then
          write (   6, 8348)
          write (iout, 8348)
        endif

 8355   format ('  Center 5 (xyz)  ',1p3e20.12 )

        write (   6, 8355) xp5, yp5, zp5
        write (iout, 8355) xp5, yp5, zp5

c....   Find coordinates / radius at centers.

        xc1 = xp1 * c1
        yc1 = yp1 * c1
        zc1 = zp1 * c1

        xc2 = xp2 * c2
        yc2 = yp2 * c2
        zc2 = zp2 * c2

        xc3 = xp3 * c3
        yc3 = yp3 * c3
        zc3 = zp3 * c3

        xc4 = xp4 * c4
        yc4 = yp4 * c4
        zc4 = zp4 * c4

        xc5 = xp5 * c5
        yc5 = yp5 * c5
        zc5 = zp5 * c5

 8332   format (/ '  Centers / R:' /
     &    '  Center 1 / R1   ',1p3e20.12 /
     &    '  Center 2 / R2   ',1p3e20.12 /
     &    '  Center 3 / R3   ',1p3e20.12 /
     &    '  Center 4 / R4   ',1p3e20.12 /
     &    '  Center 5 / R5   ',1p3e20.12 )

        write (   6, 8332) xc1, yc1, zc1, xc2, yc2, zc2, xc3, yc3, zc3,
     &    xc4, yc4, zc4, xc5, yc5, zc5
        write (iout, 8332) xc1, yc1, zc1, xc2, yc2, zc2, xc3, yc3, zc3,
     &    xc4, yc4, zc4, xc5, yc5, zc5

c....   Find the distances between the centers of the circles.

        d12 = abs (r1 + r2)
        err = tol * (abs (r1) + abs (r2))
        if (d12 .le. err) d12 = 0.0

        d23 = abs (r2 + r3)
        err = tol * (abs (r2) + abs (r3))
        if (d23 .le. err) d23 = 0.0

        d31 = abs (r3 + r1)
        err = tol * (abs (r3) + abs (r1))
        if (d31 .le. err) d31 = 0.0

        d14   = abs (r1 + r4)
        err14 = tol * (abs (r1) + abs (r4))
        if (d14 .le. err14) d14 = 0.0

        d24   = abs (r2 + r4)
        err24 = tol * (abs (r2) + abs (r4))
        if (d24 .le. err24) d24 = 0.0

        d34   = abs (r3 + r4)
        err34 = tol * (abs (r3) + abs (r4))
        if (d34 .le. err34) d34 = 0.0

        d15   = abs (r1 + r5)
        err15 = tol * (abs (r1) + abs (r5))
        if (d15 .le. err15) d15 = 0.0

        d25   = abs (r2 + r5)
        err25 = tol * (abs (r2) + abs (r5))
        if (d25 .le. err25) d25 = 0.0

        d35   = abs (r3 + r5)
        err35 = tol * (abs (r3) + abs (r5))
        if (d35 .le. err35) d35 = 0.0

c....   Find the tangent points of circles 1, 2 and 3.

        if (d12 .eq. 0.0) then
          tx12 = -r1
        else
          tx12 = r1
        endif
        ty12 = 0.0
        tz12 = 0.0

        if (d23 .eq. 0.0) then
          tx23 = 0.5 * (xp2 + xp3) - r2
          ty23 = 0.0
        else
          tx23 = (r3 * xp2 + r2 * xp3) / (r2 + r3)
          ty23 = r2 * yp3 / (r2 + r3)
        endif
        tz23 = 0.0

        if (d31 .eq. 0.0) then
          tx31 = -r1
          ty31 = 0.0
        else
          tx31 = r1 * xp3 / (r3 + r1)
          ty31 = r1 * yp3 / (r3 + r1)
        endif
        tz31 = 0.0

 8326   format (/ '  Tangent points:' /
     &    '  pt12 (xyz)      ',1p3e20.12 /
     &    '  pt23 (xyz)      ',1p3e20.12 /
     &    '  pt31 (xyz)      ',1p3e20.12 )

        write (   6, 8326) tx12, ty12, tz12, tx23, ty23, tz23,
     &                     tx31, ty31, tz31
        write (iout, 8326) tx12, ty12, tz12, tx23, ty23, tz23,
     &                     tx31, ty31, tz31

 8325   format (/ '  Distances between centers:' /
     &  '  d12, d23,  d31 =',1p3e20.12 )

        write (   6, 8325) d12, d23, d31
        write (iout, 8325) d12, d23, d31

c....   Find the distances to the center of circle 4, and their errors.

        call aptvdis (xp1, yp1, zp1, xp4, yp4, zp4, 1, tol,
     &                dx14, dy14, dz14, dp14, nerra)
        e14   = dp14 - d14
        err14 = tol * (dp14 + d14)
        if (abs (e14) .le. err14) e14 = 0.0

        call aptvdis (xp2, yp2, zp2, xp4, yp4, zp4, 1, tol,
     &                dx24, dy24, dz24, dp24, nerra)
        e24   = dp24 - d24
        err24 = tol * (dp24 + d24)
        if (abs (e24) .le. err24) e24 = 0.0

        call aptvdis (xp3, yp3, zp3, xp4, yp4, zp4, 1, tol,
     &                dx34, dy34, dz34, dp34, nerra)
        e34   = dp34 - d34
        err34 = tol * (dp34 + d34)
        if (abs (e34) .le. err34) e34 = 0.0

 8520   format ('  real calc  error' /
     &          '  d14, dp14, e14 =',1p3e20.12 )
 8522   format ('  d24, dp24, e24 =',1p3e20.12 )
 8524   format ('  d34, dp34, e34 =',1p3e20.12 )

        write (   6, 8520) d14, dp14, e14
        write (iout, 8520) d14, dp14, e14
        write (   6, 8522) d24, dp24, e24
        write (iout, 8522) d24, dp24, e24
        write (   6, 8524) d34, dp34, e34
        write (iout, 8524) d34, dp34, e34

c....   Find the distances to the center of circle 5, and their errors.

        call aptvdis (xp1, yp1, zp1, xp5, yp5, zp5, 1, tol,
     &                dx15, dy15, dz15, dp15, nerra)
        e15   = dp15 - d15
        err15 = tol * (dp15 + d15)
        if (abs (e15) .le. err15) e15 = 0.0

        call aptvdis (xp2, yp2, zp2, xp5, yp5, zp5, 1, tol,
     &                dx25, dy25, dz25, dp25, nerra)
        e25   = dp25 - d25
        err25 = tol * (dp25 + d25)
        if (abs (e25) .le. err25) e25 = 0.0

        call aptvdis (xp3, yp3, zp3, xp5, yp5, zp5, 1, tol,
     &                dx35, dy35, dz35, dp35, nerra)
        e35   = dp35 - d35
        err35 = tol * (dp35 + d35)
        if (abs (e35) .le. err35) e35 = 0.0

 8530   format ('  d15, dp15, e15 =',1p3e20.12 )
 8532   format ('  d25, dp25, e25 =',1p3e20.12 )
 8534   format ('  d35, dp35, e35 =',1p3e20.12 )

        write (   6, 8530) d15, dp15, e15
        write (iout, 8530) d15, dp15, e15
        write (   6, 8532) d25, dp25, e25
        write (iout, 8532) d25, dp25, e25
        write (   6, 8534) d35, dp35, e35
        write (iout, 8534) d35, dp35, e35

        go to 999

      endif                           ! Find three tangent circles.

c=======================================================================********

c.... Input line:  kiss RAD1 RAD2 RAD3 RAD4

c.... Find two spheres, each tangent to four tangent spheres.

      if (nword .eq. 5) then

        nerr = 0

c....   Find the four radii.

        call varfind (2, 5)

 8410 format ('  kiss ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

        do 410 nw = 2, 5
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8410) ainput, ninput, aword(nw)
            write (iout, 8410) ainput, ninput, aword(nw)
            nerr = 1
          endif
  410   continue

        r1  = fword(2)
        r2  = fword(3)
        r3  = fword(4)
        r4  = fword(5)

        if (r1 .ne. 0.0) then
          c1 = 1.0 / r1
        else
          c1 = 1.e99
        endif

        if (r2 .ne. 0.0) then
          c2 = 1.0 / r2
        else
          c2 = 1.e99
        endif

        if (r3 .ne. 0.0) then
          c3 = 1.0 / r3
        else
          c3 = 1.e99
        endif

        if (r4 .ne. 0.0) then
          c4 = 1.0 / r4
        else
          c4 = 1.e99
        endif

 8422   format ('  Radius R1, 1/R1 ',1p2e20.12 /
     &          '  Radius R2, 1/R2 ',1p2e20.12 /
     &          '  Radius R3, 1/R3 ',1p2e20.12 /
     &          '  Radius R4, 1/R4 ',1p2e20.12 )

        write (   6, 8422) r1, c1, r2, c2, r3, c3, r4, c4
        write (iout, 8422) r1, c1, r2, c2, r3, c3, r4, c4

        if (nerr .ne. 0) go to 999

        call aptsphk (r1, r2, r3, r4, tol, r5, r6,
     &                xp1, yp1, zp1, xp2, yp2, zp2, xp3, yp3, zp3,
     &                xp4, yp4, zp4, xp5, yp5, zp5, xp6, yp6, zp6,
     &                nerra)

        if (nerra .gt. 0) then
          write (   6, 8329) ainput, ninput
          write (iout, 8329) ainput, ninput
          go to 999
        endif

        if (r5 .ne. 0.0) then
          c5 = 1.0 / r5
        else
          c5 = 1.e99
        endif

        if (r6 .ne. 0.0) then
          c6 = 1.0 / r6
        else
          c6 = 1.e99
        endif

 8440   format (/ '  Sphere 5 and sphere 6 are each tangent to',
     &            ' spheres 1, 2, 3 and 4.' /
     &            '    Radius is negative if tangency is internal.' /
     &            '  Radius R5, 1/R5 ',1p2e20.12 /
     &            '  Radius R6, 1/R6 ',1p2e20.12 )

        write (   6, 8440) r5, c5, r6, c6
        write (iout, 8440) r5, c5, r6, c6

 8430   format (/ '  Centers of spheres:' /
     &    '  Center 1 (xyz)  ',1p3e20.12 /
     &    '  Center 2 (xyz)  ',1p3e20.12 /
     &    '  Center 3 (xyz)  ',1p3e20.12 )

        write (   6, 8430) xp1, yp1, zp1, xp2, yp2, zp2,
     &                     xp3, yp3, zp3
        write (iout, 8430) xp1, yp1, zp1, xp2, yp2, zp2,
     &                     xp3, yp3, zp3

 8412   format ('    Note:  sign of small y or z may be uncertain.')

        if (nerra .eq. -1) then
          write (   6, 8412)
          write (iout, 8412)
        endif

        write (   6, 8350) xp4, yp4, zp4
        write (iout, 8350) xp4, yp4, zp4

 8450   format ('  Center 5 (xyz)  ',1p3e20.12 )

        if (nerra .eq. -1) then
          write (   6, 8412)
          write (iout, 8412)
        endif

        write (   6, 8450) xp5, yp5, zp5
        write (iout, 8450) xp5, yp5, zp5

 8455   format ('  Center 6 (xyz)  ',1p3e20.12 )

        if (nerra .eq. -1) then
          write (   6, 8412)
          write (iout, 8412)
        endif

        write (   6, 8455) xp6, yp6, zp6
        write (iout, 8455) xp6, yp6, zp6

c....   Find coordinates / radius at centers.

        xc1 = xp1 * c1
        yc1 = yp1 * c1
        zc1 = zp1 * c1

        xc2 = xp2 * c2
        yc2 = yp2 * c2
        zc2 = zp2 * c2

        xc3 = xp3 * c3
        yc3 = yp3 * c3
        zc3 = zp3 * c3

        xc4 = xp4 * c4
        yc4 = yp4 * c4
        zc4 = zp4 * c4

        xc5 = xp5 * c5
        yc5 = yp5 * c5
        zc5 = zp5 * c5

        xc6 = xp6 * c6
        yc6 = yp6 * c6
        zc6 = zp6 * c6

 8343   format (/ '  Centers / R:' /
     &    '  Center 1 / R1   ',1p3e20.12 /
     &    '  Center 2 / R2   ',1p3e20.12 /
     &    '  Center 3 / R3   ',1p3e20.12 /
     &    '  Center 4 / R4   ',1p3e20.12 /
     &    '  Center 5 / R5   ',1p3e20.12 /
     &    '  Center 6 / R6   ',1p3e20.12 )

        write (   6, 8343) xc1, yc1, zc1, xc2, yc2, zc2, xc3, yc3, zc3,
     &    xc4, yc4, zc4, xc5, yc5, zc5, xc6, yc6, zc6
        write (iout, 8343) xc1, yc1, zc1, xc2, yc2, zc2, xc3, yc3, zc3,
     &    xc4, yc4, zc4, xc5, yc5, zc5, xc6, yc6, zc6

c....   Find the distances between the centers of the spheres.

        d12 = abs (r1 + r2)
        err = tol * (abs (r1) + abs (r2))
        if (d12 .le. err) d12 = 0.0

        d23 = abs (r2 + r3)
        err = tol * (abs (r2) + abs (r3))
        if (d23 .le. err) d23 = 0.0

        d31 = abs (r3 + r1)
        err = tol * (abs (r1) + abs (r3))
        if (d31 .le. err) d31 = 0.0

        d14 = abs (r1 + r4)
        err = tol * (abs (r1) + abs (r4))
        if (d14 .le. err) d14 = 0.0

        d24 = abs (r2 + r4)
        err = tol * (abs (r2) + abs (r4))
        if (d24 .le. err) d24 = 0.0

        d34 = abs (r3 + r4)
        err = tol * (abs (r3) + abs (r4))
        if (d34 .le. err) d34 = 0.0

        d15   = abs (r1 + r5)
        err15 = tol * (abs (r1) + abs (r5))
        if (d15 .le. err15) d15 = 0.0

        d25   = abs (r2 + r5)
        err25 = tol * (abs (r2) + abs (r5))
        if (d25 .le. err25) d25 = 0.0

        d35   = abs (r3 + r5)
        err35 = tol * (abs (r3) + abs (r5))
        if (d35 .le. err35) d35 = 0.0

        d45   = abs (r4 + r5)
        err45 = tol * (abs (r4) + abs (r5))
        if (d45 .le. err45) d45 = 0.0

        d16   = abs (r1 + r6)
        err16 = tol * (abs (r1) + abs (r6))
        if (d16 .le. err16) d16 = 0.0

        d26   = abs (r2 + r6)
        err26 = tol * (abs (r2) + abs (r6))
        if (d26 .le. err26) d26 = 0.0

        d36   = abs (r3 + r6)
        err36 = tol * (abs (r3) + abs (r6))
        if (d36 .le. err36) d36 = 0.0

        d46   = abs (r4 + r6)
        err46 = tol * (abs (r4) + abs (r6))
        if (d46 .le. err46) d46 = 0.0

c....   Find the tangent points of spheres 1, 2, 3 and 4.

        if (d12 .eq. 0.0) then
          tx12 = -r1
        else
          tx12 = r1
        endif
        ty12 = 0.0
        tz12 = 0.0

        if (d23 .eq. 0.0) then
          tx23 = 0.5 * (xp2 + xp3) - r2
          ty23 = 0.0
        else
          tx23 = (r3 * xp2 + r2 * xp3) / (r2 + r3)
          ty23 = r2 * yp3 / (r2 + r3)
        endif
        tz23 = 0.0

        if (d31 .eq. 0.0) then
          tx31 = -r1
          ty31 = 0.0
        else
          tx31 = r1 * xp3 / (r3 + r1)
          ty31 = r1 * yp3 / (r3 + r1)
        endif
        tz31 = 0.0

1       if (d14 .eq. 0.0) then
          tx14 = -r1
          ty14 = 0.0
          tz14 = 0.0
        else
          tx14 = r1 * xp4 / (r1 + r4)
          ty14 = r1 * yp4 / (r1 + r4)
          tz14 = r1 * zp4 / (r1 + r4)
        endif

        if (d24 .eq. 0.0) then
          tx24 = 0.5 * (xp2 + xp4) + r2
          ty24 = 0.0
          tz24 = 0.0
        else
          tx24 = (r4 * xp2 + r2 * xp4) / (r2 + r4)
          ty24 = r2 * yp4 / (r2 + r4)
          tz24 = r2 * zp4 / (r2 + r4)
        endif

        if (d34 .eq. 0.0) then
          tx34 = 0.5 * (xp3 + xp4)
          ty34 = 0.5 * (yp3 + yp4) + abs (r3)
          tz34 = 0.0
        else
          tx34 = (r4 * xp3 + r3 * xp4) / (r3 + r4)
          ty34 = (r4 * yp3 + r3 * yp4) / (r3 + r4)
          tz34 = r3 * zp4 / (r3 + r4)
        endif

 8426   format (/ '  Tangent points:' /
     &    '  pt12 (xyz)      ',1p3e20.12 /
     &    '  pt23 (xyz)      ',1p3e20.12 /
     &    '  pt31 (xyz)      ',1p3e20.12 /
     &    '  pt14 (xyz)      ',1p3e20.12 /
     &    '  pt24 (xyz)      ',1p3e20.12 /
     &    '  pt34 (xyz)      ',1p3e20.12 )

        write (   6, 8426) tx12, ty12, tz12, tx23, ty23, tz23,
     &                     tx31, ty31, tz31, tx14, ty14, tz14,
     &                     tx24, ty24, tz24, tx34, ty34, tz34
        write (iout, 8426) tx12, ty12, tz12, tx23, ty23, tz23,
     &                     tx31, ty31, tz31, tx14, ty14, tz14,
     &                     tx24, ty24, tz24, tx34, ty34, tz34

 8425   format (/ '  Distances between centers:' /
     &    '  d12, d23,  d31 =',1p3e20.12 /
     &    '  d14, d24,  d34 =',1p3e20.12 )

        write (   6, 8425) d12, d23, d31, d14, d24, d34
        write (iout, 8425) d12, d23, d31, d14, d24, d34

c....   Find the distances to the center of sphere 5, and their errors.

        call aptvdis (xp1, yp1, zp1, xp5, yp5, zp5, 1, tol,
     &                dx15, dy15, dz15, dp15, nerra)
        e15   = dp15 - d15
        err15 = tol * (dp15 + d15)
        if (abs (e15) .le. err15) e15 = 0.0

        call aptvdis (xp2, yp2, zp2, xp5, yp5, zp5, 1, tol,
     &                dx25, dy25, dz25, dp25, nerra)
        e25   = dp25 - d25
        err25 = tol * (dp25 + d25)
        if (abs (e25) .le. err25) e25 = 0.0

        call aptvdis (xp3, yp3, zp3, xp5, yp5, zp5, 1, tol,
     &                dx35, dy35, dz35, dp35, nerra)
        e35   = dp35 - d35
        err35 = tol * (dp35 + d35)
        if (abs (e35) .le. err35) e35 = 0.0

        call aptvdis (xp4, yp4, zp4, xp5, yp5, zp5, 1, tol,
     &                dx45, dy45, dz45, dp45, nerra)
        e45   = dp45 - d45
        err45 = tol * (dp45 + d45)
        if (abs (e45) .le. err45) e45 = 0.0

 8536   format ('  d45, dp45, e45 =',1p3e20.12 )

 8730   format ('  real calc  error' /
     &          '  d15, dp15, e15 =',1p3e20.12 )
        write (   6, 8730) d15, dp15, e15
        write (iout, 8730) d15, dp15, e15
        write (   6, 8532) d25, dp25, e25
        write (iout, 8532) d25, dp25, e25
        write (   6, 8534) d35, dp35, e35
        write (iout, 8534) d35, dp35, e35
        write (   6, 8536) d45, dp45, e45
        write (iout, 8536) d45, dp45, e45

c....   Find the distances to the center of sphere 6, and their errors.

        call aptvdis (xp1, yp1, zp1, xp6, yp6, zp6, 1, tol,
     &                dx16, dy16, dz16, dp16, nerra)
        e16   = dp16 - d16
        err16 = tol * (dp16 + d16)
        if (abs (e16) .le. err16) e16 = 0.0

        call aptvdis (xp2, yp2, zp2, xp6, yp6, zp6, 1, tol,
     &                dx26, dy26, dz26, dp26, nerra)
        e26   = dp26 - d26
        err26 = tol * (dp26 + d26)
        if (abs (e26) .le. err26) e26 = 0.0

        call aptvdis (xp3, yp3, zp3, xp6, yp6, zp6, 1, tol,
     &                dx36, dy36, dz36, dp36, nerra)
        e36   = dp36 - d36
        err36 = tol * (dp36 + d36)
        if (abs (e36) .le. err36) e36 = 0.0

        call aptvdis (xp4, yp4, zp4, xp6, yp6, zp6, 1, tol,
     &                dx46, dy46, dz46, dp46, nerra)
        e46   = dp46 - d46
        err46 = tol * (dp46 + d46)
        if (abs (e46) .le. err46) e46 = 0.0

 8540   format ('  d16, dp16, e16 =',1p3e20.12 )
 8542   format ('  d26, dp26, e26 =',1p3e20.12 )
 8544   format ('  d36, dp36, e36 =',1p3e20.12 )
 8546   format ('  d46, dp46, e46 =',1p3e20.12 )

        write (   6, 8540) d16, dp16, e16
        write (iout, 8540) d16, dp16, e16
        write (   6, 8542) d26, dp26, e26
        write (iout, 8542) d26, dp26, e26
        write (   6, 8544) d36, dp36, e36
        write (iout, 8544) d36, dp36, e36
        write (   6, 8546) d46, dp46, e46
        write (iout, 8546) d46, dp46, e46

        go to 999

      endif                           ! Find four tangent spheres.

c=======================================================================********

 8810 format ('  kiss ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine kiss.        (+1 line)
      end
