      subroutine list

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  list     Updated     2006 January 5 17:20.
c               list     Originated  1993 November 23 17:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To list aliases, markers, variables, vectors, operators, points,
c               clusters, lines, triangles, annular disks, regular polygons,
c               planes, spheres, circular cylinders, circular cones, hyperbolic
c               paraboloids, ellipsoids, axisymmetric quadric surfaces, general
c               quadric surfaces, regular polyhedrons, tetrahedrons, bricks,
c               zones, bins, probability distribution functions (pdfs), and
c               symbols.
c
c     Logic:    Entry points:  list.
c list          Called by:  helpall, main, sort.
c
c list          Calls:  aptblsq, repack.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      common /llist/ anamex(2)        ! Names of objects.
      character*24   anamex           ! Names of objects.
      common /llist/ nx               ! Index of an object.
      common /llist/ amarkx(26)       ! Names of markers
      character*1    amarkx

      character*32   awordx(32)       ! Temporary space for output line.
      character*1    asink(1024)       ! Output line with no extra blanks.

c.... Input line:  list [help]

 8110 format (
     &  '  Command "list":  acceptable input lines include:' /
     &  '  "list [help]"       Display this message.' /
     &  '  "list all"          Display all object names.' /
     &  '  "list alias"        Display all alias names.' /
     &  '  "list axisym"       Display all axisymmetric quadric',
     &  ' names.' /
     &  '  "list big"          Display all big integer names.' /
     &  '  "list bin"          Display all probability bin names.' /
     &  '  "list brick"        Display all brick names.' /
     &  '  "list cluster"      Display all cluster names.' /
     &  '  "list cone"         Display all circular cone names.' /
     &  '  "list cylinder"     Display all circular cylinder names.' /
     &  '  "list disk"         Display all annular disk names.' /
     &  '  "list ellipsoid"    Display all ellipsoid names.' /
     &  '  "list hyperb        Display all hyperbolic paraboloid',
     &  ' names.' /
     &  '  "list line"         Display all line names.' /
     &  '  "list marker"       Display all marker names.' /
     &  '  "list mesh"         Display all mesh point names.' /
     &  '  "list operator"     Display all operator names.' )

 8112 format (
     &  '  "list pdf"          Display all pdf names.' /
     &  '  "list plane"        Display all plane names.' /
     &  '  "list point"        Display all point names.' /
     &  '  "list polygon"      Display all regular polygon names.' /
     &  '  "list polyhedron"   Display all regular polyhedron names.' /
     &  '  "list quadric"      Display all quadric surface names.' /
     &  '  "list sphere"       Display all sphere names.' /
     &  '  "list symbol"       Display all symbol names.' /
     &  '  "list tetrahedron"  Display all tetrahedron names.' /
     &  '  "list triangle"     Display all triangle names.' /
     &  '  "list variable"     Display all variable names.' /
     &  '  "list vector"       Display all vector names.' /
     &  '  "list zone"         Display all zone names.' /
     &  '  "list OBJTYPE1 OBJTYPE2 OBJTYPE3 ..."' /
     &  '        Display the names of all objects of types' /
     &  '        OBJTYPE1, OBJTYPE2, OBJTYPE3, ...  which may be',
     &  ' any of the' /
     &  '        object types listed above.' )

 8114 format (
     &  '  Synonyms:  [alias, al, a], [brick, br], [cluster, cl],' /
     &  '    [cylinder, cyl], [disk, disc, dk],',
     &  ' [ellipsoid, ellipse, ell, el],' /
     &  '    [help, h], [hyperb, saddle], [line, ln, l],',
     &  ' [list, print, display],' /
     &  '    [marker, mark, m], [operator, oper, op, o],' /
     &  '    [plane, pl], [point, pnt, pt, p], [polygon, polyg, pg],',
     &  ' [polyhedron, polyh],' /
     &  '    [quadric, quad, q], [sphere, sph], [symbol, symb, sym],' /
     &  '    [tetrahedron, tetra, tet], [triangle, tri],' /
     &  '    [variable, var, set, s], [vector, vect, v],',
     &  ' [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        go to 999
      endif                           ! Displayed help message.

c===============================================================================

c.... Repack if command "list" was used, not command "sort".

      if ((aword(1) .eq. 'list')    .or.
     &    (aword(1) .eq. 'display') .or.
     &    (aword(1) .eq. 'print'))  then
        call repack
      endif

c===============================================================================

c.... See if object classes are to be listed.

      kprall = 0

      do 115 nw = 2, nword
        if (aword(nw) .eq. 'all') then
          kprall = 1
        endif
  115 continue

      kprquad = 0

      do 116 nw = 2, nword
        if ((aword(nw) .eq. 'quadric') .or.
     &      (aword(nw) .eq. 'quad')    .or.
     &      (aword(nw) .eq. 'q'))      then
          kprquad = 1
        endif
  116 continue

      kpraxi = 0

      do 117 nw = 2, nword
        if (aword(nw) .eq. 'axisym') then
          kpraxi = 1
        endif
  117 continue

c===============================================================================

c.... Check for unrecognizable input.

 8118 format ('  list ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Object type does not exist:  ',a24)

      do 118 nw = 2, nword
        if (aword(nw) .eq. 'a')           go to 118
        if (aword(nw) .eq. 'al')          go to 118
        if (aword(nw) .eq. 'alias')       go to 118
        if (aword(nw) .eq. 'all')         go to 118
        if (aword(nw) .eq. 'axisym')      go to 118
        if (aword(nw) .eq. 'big')         go to 118
        if (aword(nw) .eq. 'bin')         go to 118
        if (aword(nw) .eq. 'br')          go to 118
        if (aword(nw) .eq. 'brick')       go to 118
        if (aword(nw) .eq. 'cl')          go to 118
        if (aword(nw) .eq. 'cluster')     go to 118
        if (aword(nw) .eq. 'cone')        go to 118
        if (aword(nw) .eq. 'cyl')         go to 118
        if (aword(nw) .eq. 'cylinder')    go to 118
        if (aword(nw) .eq. 'disc')        go to 118
        if (aword(nw) .eq. 'disk')        go to 118
        if (aword(nw) .eq. 'dk')          go to 118
        if (aword(nw) .eq. 'el')          go to 118
        if (aword(nw) .eq. 'ell')         go to 118
        if (aword(nw) .eq. 'ellipse')     go to 118
        if (aword(nw) .eq. 'ellipsoid')   go to 118
        if (aword(nw) .eq. 'hyperb')      go to 118
        if (aword(nw) .eq. 'l')           go to 118
        if (aword(nw) .eq. 'line')        go to 118
        if (aword(nw) .eq. 'ln')          go to 118
        if (aword(nw) .eq. 'm')           go to 118
        if (aword(nw) .eq. 'mark')        go to 118
        if (aword(nw) .eq. 'marker')      go to 118
        if (aword(nw) .eq. 'mesh')        go to 118
        if (aword(nw) .eq. 'op')          go to 118
        if (aword(nw) .eq. 'oper')        go to 118
        if (aword(nw) .eq. 'operator')    go to 118
        if (aword(nw) .eq. 'p')           go to 118
        if (aword(nw) .eq. 'pdf')         go to 118
        if (aword(nw) .eq. 'pg')          go to 118
        if (aword(nw) .eq. 'ph')          go to 118
        if (aword(nw) .eq. 'pl')          go to 118
        if (aword(nw) .eq. 'plane')       go to 118
        if (aword(nw) .eq. 'pnt')         go to 118
        if (aword(nw) .eq. 'point')       go to 118
        if (aword(nw) .eq. 'polyg')       go to 118
        if (aword(nw) .eq. 'polygon')     go to 118
        if (aword(nw) .eq. 'polyh')       go to 118
        if (aword(nw) .eq. 'polyhedron')  go to 118
        if (aword(nw) .eq. 'pt')          go to 118
        if (aword(nw) .eq. 'q')           go to 118
        if (aword(nw) .eq. 'quad')        go to 118
        if (aword(nw) .eq. 'quadric')     go to 118
        if (aword(nw) .eq. 's')           go to 118
        if (aword(nw) .eq. 'saddle')      go to 118
        if (aword(nw) .eq. 'set')         go to 118
        if (aword(nw) .eq. 'sph')         go to 118
        if (aword(nw) .eq. 'sphere')      go to 118
        if (aword(nw) .eq. 'sym')         go to 118
        if (aword(nw) .eq. 'symb')        go to 118
        if (aword(nw) .eq. 'symbol')      go to 118
        if (aword(nw) .eq. 'tet')         go to 118
        if (aword(nw) .eq. 'tetra')       go to 118
        if (aword(nw) .eq. 'tetrahedron') go to 118
        if (aword(nw) .eq. 'tri')         go to 118
        if (aword(nw) .eq. 'triangle')    go to 118
        if (aword(nw) .eq. 'v')           go to 118
        if (aword(nw) .eq. 'var')         go to 118
        if (aword(nw) .eq. 'variable')    go to 118
        if (aword(nw) .eq. 'vect')        go to 118
        if (aword(nw) .eq. 'vector')      go to 118
        if (aword(nw) .eq. 'z')           go to 118
        if (aword(nw) .eq. 'zn')          go to 118
        if (aword(nw) .eq. 'zone')        go to 118
        write (   6, 8118) ainput, ninput, aword(nw)
        write (iout, 8118) ainput, ninput, aword(nw)
  118 continue

c===============================================================================

c.... Input line:  list all
c.... Input line:  list alias
c.... Input line:  list ... alias ...

      kprint = 0

      do 410 nw = 2, nword
        if ((aword(nw) .eq. 'alias') .or.
     &      (aword(nw) .eq. 'al')    .or.
     &      (aword(nw) .eq. 'a'))    then
          kprint = 1
        endif
  410 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8610 format ('Aliases:              ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do na = 1, nalias
  
          if (alias1(na) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = alias1(na)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8610) (asink(ns), ns = 1, lsinks)
              write (iout, 8610) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = alias1(na)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8610) (asink(ns), ns = 1, lsink)
          write (iout, 8610) (asink(ns), ns = 1, lsink)
        endif

 8410 format ('Aliases:              ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8410) anamex(1)
          write (iout, 8410) anamex(1)
        endif

 8525   format (/ 'Number of aliases             = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8525) kx
          write (iout, 8525) kx
        endif
      endif                           ! Listed aliases.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list big
c.... Input line:  list ... big ...

      kprint = 0

      do 268 nw = 2, nword
        if (aword(nw) .eq. 'big') then
          kprint = 1
        endif
  268 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8768 format ('Big integers:         ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nb = 1, nbigm
  
          if ((ndig(nb) .gt. 1) .or.
     &       ((ndig(nb) .eq. 1) .and. (idig(1,nb) .gt. 0))) then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = abig(nb)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8768) (asink(ns), ns = 1, lsinks)
              write (iout, 8768) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = abig(nb)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8768) (asink(ns), ns = 1, lsink)
          write (iout, 8768) (asink(ns), ns = 1, lsink)
        endif

 8464 format ('Big integers:         ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8464) anamex(1)
          write (iout, 8464) anamex(1)
        endif

 8462   format (/ 'Number of big integers        = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8462) kx
          write (iout, 8462) kx
        endif
      endif                           ! Listed big integers.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list bin
c.... Input line:  list ... bin ...

      kprint = 0

      do 270 nw = 2, nword
        if (aword(nw) .eq. 'bin') then
          kprint = 1
        endif
  270 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8485   format (/ 'Number of pdfs                = ',i5 /)

        if (npdfs .ne. 0) then
          write (   6, 8485) npdfs
          write (iout, 8485) npdfs
        endif

 8468   format (/ 'Number of pdf/bin pairs       = ',i5 /)

        if (npdfxs .ne. 0) then
          write (   6, 8468) npdfxs
          write (iout, 8468) npdfxs
        endif

 8766 format ('Probability bins:     ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nb = 1, nbins
  
          if (abin(nb) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = abin(nb)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8766) (asink(ns), ns = 1, lsinks)
              write (iout, 8766) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = abin(nb)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8766) (asink(ns), ns = 1, lsink)
          write (iout, 8766) (asink(ns), ns = 1, lsink)
        endif

 8266 format ('Probability bins:     ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8266) anamex(1)
          write (iout, 8266) anamex(1)
        endif

 8465   format (/ 'Number of probability bins    = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8465) kx
          write (iout, 8465) kx
        endif
      endif                           ! Listed probability bins.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list brick
c.... Input line:  list ... brick ...

      kprint = 0

      do 340 nw = 2, nword
        if ((aword(nw) .eq. 'brick') .or.
     &      (aword(nw) .eq. 'br'))   then
          kprint = 1
        endif
  340 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8750 format ('Bricks:               ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nb = 1, nbricks
  
          if (abrick(nb) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = abrick(nb)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8750) (asink(ns), ns = 1, lsinks)
              write (iout, 8750) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = abrick(nb)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8750) (asink(ns), ns = 1, lsink)
          write (iout, 8750) (asink(ns), ns = 1, lsink)
        endif

 8350 format ('Bricks:               ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8350) anamex(1)
          write (iout, 8350) anamex(1)
        endif

 8285   format (/ 'Number of bricks              = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8285) kx
          write (iout, 8285) kx
        endif
      endif                           ! Listed bricks.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list cluster
c.... Input line:  list ... cluster ...

      kprint = 0

      do 133 nw = 2, nword
        if ((aword(nw) .eq. 'cluster') .or.
     &      (aword(nw) .eq. 'cl'))     then
          kprint = 1
        endif
  133 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8155   format (/ 'Number of points              = ',i5 /)

        if (npoints .ne. 0) then
          write (   6, 8155) npoints
          write (iout, 8155) npoints
        endif

 8159   format (/ 'Number of point/cluster pairs = ',i5 /)

        if (npairs .ne. 0) then
          write (   6, 8159) npairs
          write (iout, 8159) npairs
        endif

 8730 format ('Clusters:             ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do ncl = 1, nclusts
  
          if (aclust(ncl) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aclust(ncl)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8730) (asink(ns), ns = 1, lsinks)
              write (iout, 8730) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aclust(ncl)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8730) (asink(ns), ns = 1, lsink)
          write (iout, 8730) (asink(ns), ns = 1, lsink)
        endif

 8130 format ('Clusters:             ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8130) anamex(1)
          write (iout, 8130) anamex(1)
        endif

 8175   format (/ 'Number of clusters            = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8175) kx
          write (iout, 8175) kx
        endif
      endif                           ! Listed clusters.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list disk
c.... Input line:  list ... disk ...

      kprint = 0

      do 360 nw = 2, nword
        if ((aword(nw) .eq. 'disk') .or.
     &      (aword(nw) .eq. 'disc') .or.
     &      (aword(nw) .eq. 'dk'))  then
          kprint = 1
        endif
  360 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8760 format ('Annular disks:        ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nd = 1, ndisks
  
          if (adisk(nd) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = adisk(nd)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8760) (asink(ns), ns = 1, lsinks)
              write (iout, 8760) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = adisk(nd)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8760) (asink(ns), ns = 1, lsink)
          write (iout, 8760) (asink(ns), ns = 1, lsink)
        endif

 8360 format ('Annular Disks:        ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8360) anamex(1)
          write (iout, 8360) anamex(1)
        endif

 8425   format (/ 'Number of disks               = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8425) kx
          write (iout, 8425) kx
        endif
      endif                           ! Listed disks.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list line
c.... Input line:  list ... line ...

      kprint = 0

      do 140 nw = 2, nword
        if ((aword(nw) .eq. 'line') .or.
     &      (aword(nw) .eq. 'ln')   .or.
     &      (aword(nw) .eq. 'l'))   then
          kprint = 1
        endif
  140 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8736 format ('Lines:                ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nl = 1, nlines
  
          if (aline(nl) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aline(nl)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8736) (asink(ns), ns = 1, lsinks)
              write (iout, 8736) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aline(nl)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8736) (asink(ns), ns = 1, lsink)
          write (iout, 8736) (asink(ns), ns = 1, lsink)
        endif

 8136 format ('Lines:                ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8136) anamex(1)
          write (iout, 8136) anamex(1)
        endif

 8195   format (/ 'Number of lines               = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8195) kx
          write (iout, 8195) kx
        endif
      endif                           ! Listed lines.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list marker
c.... Input line:  list ... marker ...

 8420 format ('Markers:              ',28(a1,1x))

      kprint = 0

      do 420 nw = 2, nword
        if ((aword(nw) .eq. 'marker') .or.
     &      (aword(nw) .eq. 'mark')   .or.
     &      (aword(nw) .eq. 'm'))     then
          kprint = 1
        endif
  420 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

        nx = 0
        kx = 0
        do 425 ns = 1, nmarks
          if (amark(ns) .eq. ' ') go to 425
          nx = nx + 1
          kx = kx + 1
          amarkx(nx) = amark(ns)
          if (nx .eq. 28) then
            write (   6, 8420) (amarkx(n), n = 1, 28)
            write (iout, 8420) (amarkx(n), n = 1, 28)
            nx = 0
          endif
  425   continue
        if (nx .ne. 0) then
          write (   6, 8420) (amarkx(nn), nn = 1, nx)
          write (iout, 8420) (amarkx(nn), nn = 1, nx)
        endif
        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          amarkx(1) = '**NONE**'
          write (   6, 8420) amarkx(1)
          write (iout, 8420) amarkx(1)
        endif

 8545   format (/ 'Number of markers             = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8545) kx
          write (iout, 8545) kx
        endif
      endif                           ! Listed markers.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list mesh
c.... Input line:  list ... mesh ...

      kprint = 0

      do 510 nw = 2, nword
        if (aword(nw) .eq. 'mesh') then
          kprint = 1
        endif
  510 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8715 format ('Mesh points:          ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do np = 1, npoints
  
          if ((apoint(np) .ne. ' ') .and.
     &        (kpoint(np) .ne. 0)   .and.
     &        (lpoint(np) .ne. 0)   .and.
     &        (mpoint(np) .ne. 0))  then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = apoint(np)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8715) (asink(ns), ns = 1, lsinks)
              write (iout, 8715) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = apoint(np)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8715) (asink(ns), ns = 1, lsink)
          write (iout, 8715) (asink(ns), ns = 1, lsink)
        endif

 8515 format ('Mesh points:          ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8515) anamex(1)
          write (iout, 8515) anamex(1)
        endif

 8154   format (/ 'Number of mesh points         = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8154) kx
          write (iout, 8154) kx
        endif
      endif                           ! Listed mesh points.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list operator
c.... Input line:  list ... operator ...

      kprint = 0

      do 170 nw = 2, nword
        if ((aword(nw) .eq. 'operator') .or.
     &      (aword(nw) .eq. 'oper')     .or.
     &      (aword(nw) .eq. 'op'))      then
          kprint = 1
        endif
  170 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8765 format ('Operators:            ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nop = 1, nopers
  
          if (aoper(nop) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aoper(nop)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8765) (asink(ns), ns = 1, lsinks)
              write (iout, 8765) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aoper(nop)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8765) (asink(ns), ns = 1, lsink)
          write (iout, 8765) (asink(ns), ns = 1, lsink)
        endif

 8165 format ('Operators:            ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8165) anamex(1)
          write (iout, 8165) anamex(1)
        endif

 8395   format (/ 'Number of operators           = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8395) kx
          write (iout, 8395) kx
        endif
      endif                           ! Listed operators.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list pdf
c.... Input line:  list ... pdf ...

      kprint = 0

      do 274 nw = 2, nword
        if (aword(nw) .eq. 'pdf') then
          kprint = 1
        endif
  274 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

        if (nbins .ne. 0) then
          write (   6, 8465) nbins
          write (iout, 8465) nbins
        endif

        if (npdfxs .ne. 0) then
          write (   6, 8468) npdfxs
          write (iout, 8468) npdfxs
        endif

 8775 format ('PDFs:                 ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do npx = 1, npdfs
  
          if (apdf(npx) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = apdf(npx)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8775) (asink(ns), ns = 1, lsinks)
              write (iout, 8775) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = apdf(npx)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8775) (asink(ns), ns = 1, lsink)
          write (iout, 8775) (asink(ns), ns = 1, lsink)
        endif

 8275 format ('PDFs:                 ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8275) anamex(1)
          write (iout, 8275) anamex(1)
        endif

        if (kx .ne. 0) then
          write (   6, 8485) kx
          write (iout, 8485) kx
        endif
      endif                           ! Listed probability dist functions.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list point
c.... Input line:  list ... point ...

      kprint = 0

      do 130 nw = 2, nword
        if ((aword(nw) .eq. 'point') .or.
     &      (aword(nw) .eq. 'pnt')   .or.
     &      (aword(nw) .eq. 'pt')    .or.
     &      (aword(nw) .eq. 'p'))    then
          kprint = 1
        endif
  130 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

        if (npairs .ne. 0) then
          write (   6, 8159) npairs
          write (iout, 8159) npairs
        endif

 8720 format ('Points:               ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do np = 1, npoints
  
          if (apoint(np) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = apoint(np)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8720) (asink(ns), ns = 1, lsinks)
              write (iout, 8720) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = apoint(np)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8720) (asink(ns), ns = 1, lsink)
          write (iout, 8720) (asink(ns), ns = 1, lsink)
        endif

 8125 format ('Points:               ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8125) anamex(1)
          write (iout, 8125) anamex(1)
        endif

        if (kx .ne. 0) then
          write (   6, 8155) kx
          write (iout, 8155) kx
        endif
c=======================================================================

      endif                           ! Listed points.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list polygon
c.... Input line:  list ... polygon ...

      kprint = 0

      do 320 nw = 2, nword
        if ((aword(nw) .eq. 'polygon') .or.
     &      (aword(nw) .eq. 'polyg')   .or.
     &      (aword(nw) .eq. 'pg'))     then
          kprint = 1
        endif
  320 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8710 format ('Regular polygons:     ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do npg = 1, npolygs
  
          if (apolyg(npg) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = apolyg(npg)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8710) (asink(ns), ns = 1, lsinks)
              write (iout, 8710) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = apolyg(npg)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8710) (asink(ns), ns = 1, lsink)
          write (iout, 8710) (asink(ns), ns = 1, lsink)
        endif

 8310 format ('Regular Polygons:     ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8310) anamex(1)
          write (iout, 8310) anamex(1)
        endif

 8265   format (/ 'Number of polygons            = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8265) kx
          write (iout, 8265) kx
        endif
      endif                           ! Listed polygons.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list polyhedron
c.... Input line:  list ... polyhedron ...

      kprint = 0

      do 290 nw = 2, nword
        if ((aword(nw) .eq. 'polyhedron') .or.
     &      (aword(nw) .eq. 'polyh')      .or.
     &      (aword(nw) .eq. 'ph'))        then
          kprint = 1
        endif
  290 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8786 format ('Regular polyhedrons:  ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nt = 1, npolyhs
  
          if (apolyh(nt) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = apolyh(nt)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8786) (asink(ns), ns = 1, lsinks)
              write (iout, 8786) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = apolyh(nt)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8786) (asink(ns), ns = 1, lsink)
          write (iout, 8786) (asink(ns), ns = 1, lsink)
        endif

 8286 format ('Regular polyhedrons:  ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8286) anamex(1)
          write (iout, 8286) anamex(1)
        endif

 8305   format (/ 'Number of polyhedrons         = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8305) kx
          write (iout, 8305) kx
        endif
      endif                           ! Listed polyhedrons.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list plane
c.... Input line:  list ... plane ...
c.... Input line:  list axisym
c.... Input line:  list ... axisym ...
c.... Input line:  list quadric
c.... Input line:  list ... quadric ...

      kprint = 0

      do 190 nw = 2, nword
        if ((aword(nw) .eq. 'plane')   .or.
     &      (aword(nw) .eq. 'pl'))     then
          kprint = 1
        endif
  190 continue

      if ((kprint .eq. 1)  .or.
     &    (kprall .eq. 1)  .or.
     &    (kpraxi .eq. 1)  .or.
     &    (kprquad .eq. 1)) then

 8685 format ('Planes:               ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nq = 1, nquads
  
          if ((aquad(nq) .ne. ' ')          .and.
     &        (aqtype(nq) .eq. aqtypes(0))) then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aquad(nq)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8685) (asink(ns), ns = 1, lsinks)
              write (iout, 8685) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aquad(nq)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8685) (asink(ns), ns = 1, lsink)
          write (iout, 8685) (asink(ns), ns = 1, lsink)
        endif

 8185 format ('Planes:               ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8185) anamex(1)
          write (iout, 8185) anamex(1)
        endif

 8801   format (/ 'Number of planes              = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8801) kx
          write (iout, 8801) kx
        endif
      endif                           ! Listed planes.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list sphere
c.... Input line:  list ... sphere ...
c.... Input line:  list axisym
c.... Input line:  list ... axisym ...
c.... Input line:  list quadric
c.... Input line:  list ... quadric ...

      kprint = 0

      do 200 nw = 2, nword
        if ((aword(nw) .eq. 'sphere')  .or.
     &      (aword(nw) .eq. 'sph'))    then
          kprint = 1
        endif
  200 continue

      if ((kprint .eq. 1)  .or.
     &    (kpraxi .eq. 1)  .or.
     &    (kprquad .eq. 1) .or.
     &    (kprall .eq. 1)) then

 8796 format ('Spheres:              ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nq = 1, nquads
  
          if ((aquad(nq) .ne. ' ')           .and.
     &        (aqtype(nq) .eq. aqtypes(16))) then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aquad(nq)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8796) (asink(ns), ns = 1, lsinks)
              write (iout, 8796) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aquad(nq)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8796) (asink(ns), ns = 1, lsink)
          write (iout, 8796) (asink(ns), ns = 1, lsink)
        endif

 8196 format ('Spheres:              ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8196) anamex(1)
          write (iout, 8196) anamex(1)
        endif

 8802   format (/ 'Number of spheres             = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8802) kx
          write (iout, 8802) kx
        endif
      endif                           ! Listed spheres.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list cylinder
c.... Input line:  list ... cylinder ...
c.... Input line:  list axisym
c.... Input line:  list ... axisym ...
c.... Input line:  list quadric
c.... Input line:  list ... quadric ...

      kprint = 0

      do 210 nw = 2, nword
        if ((aword(nw) .eq. 'cylinder') .or.
     &      (aword(nw) .eq. 'cyl'))     then
          kprint = 1
        endif
  210 continue

      if ((kprint .eq. 1)  .or.
     &    (kpraxi .eq. 1)  .or.
     &    (kprquad .eq. 1) .or.
     &    (kprall .eq. 1)) then

 8705 format ('Circular cylinders:   ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nq = 1, nquads
  
          if ((aquad(nq) .ne. ' ')          .and.
     &        (aqtype(nq) .eq. aqtypes(7))) then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aquad(nq)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8705) (asink(ns), ns = 1, lsinks)
              write (iout, 8705) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aquad(nq)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8705) (asink(ns), ns = 1, lsink)
          write (iout, 8705) (asink(ns), ns = 1, lsink)
        endif

 8205 format ('Circular Cylinders:   ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8205) anamex(1)
          write (iout, 8205) anamex(1)
        endif

 8803   format (/ 'Number of circular cylinders  = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8803) kx
          write (iout, 8803) kx
        endif
      endif                           ! Listed circular cylinders.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list cone
c.... Input line:  list ... cone ...
c.... Input line:  list axisym
c.... Input line:  list ... axisym ...
c.... Input line:  list quadric
c.... Input line:  list ... quadric ...

      kprint = 0

      do 220 nw = 2, nword
        if (aword(nw) .eq. 'cone') then
          kprint = 1
        endif
  220 continue

      if ((kprint .eq. 1)  .or.
     &    (kpraxi .eq. 1)  .or.
     &    (kprquad .eq. 1) .or.
     &    (kprall .eq. 1)) then

 8415 format ('Circular Cones:       ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nq = 1, nquads
  
          if ((aquad(nq) .ne. ' ')          .and.
     &        (aqtype(nq) .eq. aqtypes(12))) then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aquad(nq)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8415) (asink(ns), ns = 1, lsinks)
              write (iout, 8415) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aquad(nq)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8415) (asink(ns), ns = 1, lsink)
          write (iout, 8415) (asink(ns), ns = 1, lsink)
        endif

 8215 format ('Circular Cones:       ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8215) anamex(1)
          write (iout, 8215) anamex(1)
        endif

 8804   format (/ 'Number of circular cones      = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8804) kx
          write (iout, 8804) kx
        endif
      endif                           ! Listed circular cones.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list ellipsoid
c.... Input line:  list ... ellipsoid ...
c.... Input line:  list quadric
c.... Input line:  list ... quadric ...

      kprint = 0

      do 230 nw = 2, nword
        if ((aword(nw) .eq. 'ellipsoid').or.
     &      (aword(nw) .eq. 'ellipse')  .or.
     &      (aword(nw) .eq. 'ell')      .or.
     &      (aword(nw) .eq. 'el'))      then
          kprint = 1
        endif
  230 continue

      if ((kprint .eq. 1)  .or.
     &    (kprquad .eq. 1) .or.
     &    (kprall .eq. 1)) then

 8725 format ('Ellipsoids:           ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nq = 1, nquads
  
          if ((aquad(nq) .ne. ' ')           .and.
     &        (aqtype(nq) .eq. aqtypes(15))) then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aquad(nq)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8725) (asink(ns), ns = 1, lsinks)
              write (iout, 8725) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aquad(nq)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8725) (asink(ns), ns = 1, lsink)
          write (iout, 8725) (asink(ns), ns = 1, lsink)
        endif

 8225 format ('Ellipsoids:           ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8225) anamex(1)
          write (iout, 8225) anamex(1)
        endif

 8805   format (/ 'Number of ellipsoids          = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8805) kx
          write (iout, 8805) kx
        endif
      endif                           ! Listed ellipsoids.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list hyperb
c.... Input line:  list ... hyperb ...
c.... Input line:  list quadric
c.... Input line:  list ... quadric ...

      kprint = 0

      do 231 nw = 2, nword
        if ((aword(nw) .eq. 'hyperb')  .or.
     &      (aword(nw) .eq. 'saddle')) then
          kprint = 1
        endif
  231 continue

      if ((kprint .eq. 1)  .or.
     &    (kprquad .eq. 1) .or.
     &    (kprall .eq. 1)) then

 8726 format ('Hyperbolic parabs:    ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nq = 1, nquads
  
          if ((aquad(nq) .ne. ' ')          .and.
     &        (aqtype(nq) .eq. aqtypes(8))) then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aquad(nq)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8726) (asink(ns), ns = 1, lsinks)
              write (iout, 8726) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aquad(nq)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8726) (asink(ns), ns = 1, lsink)
          write (iout, 8726) (asink(ns), ns = 1, lsink)
        endif

 8226 format ('Hyperbolic parabs:    ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8226) anamex(1)
          write (iout, 8226) anamex(1)
        endif

 8806   format (/ 'Number of hyperbolic parabs   = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8806) kx
          write (iout, 8806) kx
        endif
      endif                           ! Listed hyperbolic paraboloids.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list axisym
c.... Input line:  list ... axisym ...
c.... Input line:  list quadric
c.... Input line:  list ... quadric ...

      kprint = 0

      do 240 nw = 2, nword
        if (aword(nw) .eq. 'axisym') then
          kprint = 1
        endif
  240 continue

      if ((kprint .eq. 1)  .or.
     &    (kprquad .eq. 1) .or.
     &    (kprall .eq. 1)) then

 8785 format ('Axisymmetrics:        ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nq = 1, nquads
  
          if ((aquad(nq) .ne. ' ')            .and.
     &        (asym(nq) .eq. 'axisymmetric')) then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aquad(nq)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8785) (asink(ns), ns = 1, lsinks)
              write (iout, 8785) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aquad(nq)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8785) (asink(ns), ns = 1, lsink)
          write (iout, 8785) (asink(ns), ns = 1, lsink)
        endif

 8235 format ('Axisymmetrics:        ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8235) anamex(1)
          write (iout, 8235) anamex(1)
        endif

 8807   format (/ 'Number of axisymmetrics       = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8807) kx
          write (iout, 8807) kx
        endif
      endif                           ! Listed Axisymmetric quadrics

c===============================================================================

c.... Input line:  list all
c.... Input line:  list quadric
c.... Input line:  list ... quadric ...

      kprint = 0

      do 180 nw = 2, nword
        if ((aword(nw) .eq. 'quadric') .or.
     &      (aword(nw) .eq. 'quad')    .or.
     &      (aword(nw) .eq. 'q'))      then
          kprint = 1
        endif
  180 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8776 format ('Quadrics:             ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nq = 1, nquads
  
          if (aquad(nq) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = aquad(nq)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8776) (asink(ns), ns = 1, lsinks)
              write (iout, 8776) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = aquad(nq)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8776) (asink(ns), ns = 1, lsink)
          write (iout, 8776) (asink(ns), ns = 1, lsink)
        endif

 8176 format ('Quadrics:             ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8176) anamex(1)
          write (iout, 8176) anamex(1)
        endif

 8328   format (/ 'Number of quadric surfaces    = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8328) kx
          write (iout, 8328) kx
        endif
      endif                           ! Listed quadric surfaces.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list symbol
c.... Input line:  list ... symbol ...

      kprint = 0

      do 520 nw = 2, nword
        if ((aword(nw) .eq. 'symbol') .or.
     &      (aword(nw) .eq. 'symb')   .or.
     &      (aword(nw) .eq. 'sym'))   then
          kprint = 1
        endif
  520 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8722 format ('Symbols:              ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nsymb = 1, nsymbs
  
          if (asymbl(nsymb) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = asymbl(nsymb)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8722) (asink(ns), ns = 1, lsinks)
              write (iout, 8722) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = asymbl(nsymb)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8722) (asink(ns), ns = 1, lsink)
          write (iout, 8722) (asink(ns), ns = 1, lsink)
        endif

 8522 format ('Symbols:              ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8522) anamex(1)
          write (iout, 8522) anamex(1)
        endif

 8565   format (/ 'Number of symbols             = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8565) kx
          write (iout, 8565) kx
        endif
      endif                           ! Listed symbols.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list tetrahedron
c.... Input line:  list ... tetrahedron ...

      kprint = 0

      do 280 nw = 2, nword
        if ((aword(nw) .eq. 'tetrahedron') .or.
     &      (aword(nw) .eq. 'tetra')       .or.
     &      (aword(nw) .eq. 'tet'))        then
          kprint = 1
        endif
  280 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8755 format ('Tetrahedrons:         ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nt = 1, ntets
  
          if (atetra(nt) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = atetra(nt)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8755) (asink(ns), ns = 1, lsinks)
              write (iout, 8755) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = atetra(nt)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8755) (asink(ns), ns = 1, lsink)
          write (iout, 8755) (asink(ns), ns = 1, lsink)
        endif

 8255 format ('Tetrahedrons:         ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8255) anamex(1)
          write (iout, 8255) anamex(1)
        endif

 8245   format (/ 'Number of tetrahedrons        = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8245) kx
          write (iout, 8245) kx
        endif
      endif                           ! Listed tetrahedrons.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list triangle
c.... Input line:  list ... triangle ...

      kprint = 0

      do 160 nw = 2, nword
        if ((aword(nw) .eq. 'triangle') .or.
     &      (aword(nw) .eq. 'tri'))     then
          kprint = 1
        endif
  160 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8756 format ('Triangles:            ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nt = 1, ntrigs
  
          if (atrig(nt) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = atrig(nt)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8756) (asink(ns), ns = 1, lsinks)
              write (iout, 8756) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = atrig(nt)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8756) (asink(ns), ns = 1, lsink)
          write (iout, 8756) (asink(ns), ns = 1, lsink)
        endif

 8156 format ('Triangles:            ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8156) anamex(1)
          write (iout, 8156) anamex(1)
        endif

 8222   format (/ 'Number of triangles           = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8222) kx
          write (iout, 8222) kx
        endif
      endif                           ! Listed triangles.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list variable
c.... Input line:  list ... variable ...

      kprint = 0

      do 120 nw = 2, nword
        if ((aword(nw) .eq. 'variable') .or.
     &      (aword(nw) .eq. 'var')      .or.
     &      (aword(nw) .eq. 'set')      .or.
     &      (aword(nw) .eq. 's'))       then
          kprint = 1
        endif
  120 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8315 format ('Variables:            ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nvar = 1, nvars
  
          if (avarbl(nvar) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = avarbl(nvar)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8315) (asink(ns), ns = 1, lsinks)
              write (iout, 8315) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = avarbl(nvar)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8315) (asink(ns), ns = 1, lsink)
          write (iout, 8315) (asink(ns), ns = 1, lsink)
        endif

 8115 format ('Variables:            ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8115) anamex(1)
          write (iout, 8115) anamex(1)
        endif

 8135   format (/ 'Number of variables           = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8135) kx
          write (iout, 8135) kx
        endif
      endif                           ! Listed variables.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list vector
c.... Input line:  list ... vector ...

      kprint = 0

      do 150 nw = 2, nword
        if ((aword(nw) .eq. 'vector') .or.
     &      (aword(nw) .eq. 'vect')   .or.
     &      (aword(nw) .eq. 'v'))     then
          kprint = 1
        endif
  150 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8745 format ('Vectors:              ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nv = 1, nvectors
  
          if (avector(nv) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = avector(nv)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8745) (asink(ns), ns = 1, lsinks)
              write (iout, 8745) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = avector(nv)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8745) (asink(ns), ns = 1, lsink)
          write (iout, 8745) (asink(ns), ns = 1, lsink)
        endif

 8145 format ('Vectors:              ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8145) anamex(1)
          write (iout, 8145) anamex(1)
        endif

 8368   format (/ 'Number of vectors             = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8368) kx
          write (iout, 8368) kx
        endif
      endif                           ! Listed vectors.

c===============================================================================

c.... Input line:  list all
c.... Input line:  list zone
c.... Input line:  list ... zone ...

      kprint = 0

      do 250 nw = 2, nword
        if ((aword(nw) .eq. 'zone') .or.
     &      (aword(nw) .eq. 'zn')   .or.
     &      (aword(nw) .eq. 'z'))    then
          kprint = 1
        endif
  250 continue

      if ((kprint .eq. 1) .or. (kprall .eq. 1)) then

 8746 format ('Zones:                ',58a1)

        kx    = 0
        nw    = 0
        lsink = 0

        do nz = 1, nzones
  
          if (azone(nz) .ne. ' ') then

            kx = kx + 1
            nw = nw + 1
            awordx(nw) = azone(nz)
            isrce = 32 * nw
  
            lsinks = lsink
            call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
    
            if (lsink .gt. 58) then
              write (   6, 8746) (asink(ns), ns = 1, lsinks)
              write (iout, 8746) (asink(ns), ns = 1, lsinks)
              nw    = 1
              awordx(nw) = azone(nz)
              isrce = 32 * nw
              lsinks = 0
              call aptblsq (awordx, 1, isrce, asink, 1, lsink, nerra)
            endif

          endif

        enddo

        if (lsink .gt. 0) then
          write (   6, 8746) (asink(ns), ns = 1, lsink)
          write (iout, 8746) (asink(ns), ns = 1, lsink)
        endif

 8246 format ('Zones:                ',a,4x,a)

        if ((kx .eq. 0) .and. (kprint .eq. 1)) then
          anamex(1) = '**NONE**'
          write (   6, 8246) anamex(1)
          write (iout, 8246) anamex(1)
        endif

 8345   format (/ 'Number of zones               = ',i5 /)

        if (kx .ne. 0) then
          write (   6, 8345) kx
          write (iout, 8345) kx
        endif
      endif                           ! Listed zones.

c===============================================================================

  999 return

c.... End of subroutine list.         (+1 line)
      end
