      program main

c.... subroutine main.

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  main     Updated     2007 February 12 15:00.
c               main     Originated  1993 August 24 17:45.
c
c     Author:   Arthur L. Edwards.  Phone (925) 422-4123.  L-298.
c               LLNL, P.O. Box 808, Livermore, CA 94550.
c
c     Purpose:  To serve as the main entry point of GEOM, initialize, and
c               call the subroutines to interpret the input lines.
c
c     Documentation:  GPS cluster:  /g/g17/edwards/work/geom/doc/geom.base
c                     Web page http://nuclear.llnl.gov/CNP/apt/
c
c     Logic:
c main          Calls:  accel, alias, aliasget, aloha, angles, aptchap,
c               aptffip, arc, area, axisym, base, bigint, bin, bisectx,
c               blankout, brick, circle, cluster, cone, contfr, coord, copy,
c               cross, crt, cute, cuts, cylinder, debugx, define, delete,
c               delim, disk, distance, doenddo, dot, ellipsoid, extrema,
c               factor, geomend, getio, helpall, helpalph, helpenv, helperr,
c               helpgen, helpmath, helpmisc, helpobj, helpran, hex, hyperb,
c               icalc, ifendif, incr, indo, input, intcirc, iris, kiss, last,
c               line, list, marker, matrix, mcvol, mean, mesh, movex, openx,
c               operator, output, parse, pdf, plane, plotx, point, polygon,
c               polyhed, project, prompt, prox, quadric, ratio, redo, renamex,
c               repack, root, rootf, roots, sample, search, side, slice,
c               solve, sort, sphere, spin, status, subsrepl, sudo, symbol,
c               symbrepl, synonym, system, tables, tbank, tetra, timex, title,
c               tolset, track, triangle, trig, triple, twist, undo, variable,
c               vector, volume, walk, when, where, zone.
c
c               System calls:  mgtime (commented out for now).
c
c     History:
c
c     Note:     main is required.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Handle IEEE exceptions.

cold  integer ieee_handler,SIGFPE_ABORT
cold  parameter (SIGFPE_ABORT=2)      ! Abort on exception.

c.... Local variables.

      common /amain/ i                ! Array index.

c.... Type character variables.

      common /amain/ atermess         ! Termination message.
      character*80   atermess         ! Termination message.
      common /amain/ aindate          ! Problem date and time file.
      character*80   aindate          ! Problem date and time file.
      common /amain/ afile            ! A file name.
      character*80   afile            ! A file name.
      common /amain/ atype            ! A file type.
      character*80   atype            ! A file type.
      common /amain/ awords           ! Previous first word of a command.
      character*80   awords           ! Previous first word of a command.

      common /amain/ afirst           ! First character of a word.
      character*1    afirst           ! First character of a word.

coldc.... Abort on underflow or invalid operator.

cold  iii = ieee_handler('set','underflow',SIGFPE_ABORT)
cold  iii = ieee_handler('set','invalid',SIGFPE_ABORT)

cold                       Then compile with f77, with option -g.
cold                       Then execute:
cold                         dbx codename
cold                         ...
cold                         run
cold                         ... (dbx will tell you where error occurs)
cold                         quit

c=======================================================================********

c.... Initialize all type character data to blank.

cbugc***DEBUG begins.
cbug 9801 format ('main DEBUG.  Calling blankout.')
cbug      write (   6, 9801)
cbug      write (iout, 9801)
cbugc***DEBUG ends.
      call blankout
cbugc***DEBUG begins.
cbug 9901 format ('main DEBUG 1.  aversion="',a24,'"' /
cbug     &  '  aloddate="',a10,'", alodtime="',a,'", alodmach="',a,'"')
cbug      write (   6, 9901) aversion, aloddate, alodtime, alodmach
cbugc***DEBUG ends.

c.... Name code version.

      aversion = 'GEOM 2007.02.12 YANA'  ! Date.                     DATE.
      alodmach = 'YANA'

c.... Set load time, date, machine, and write out.

      aloddate = '2007.02.12'           ! Date.                     DATE.
      alodtime = '15:00'
      amach    = alodmach
cbugc***DEBUG begins.
cbug 9902 format ('main DEBUG 2.  aversion="',a24,'"' /
cbug     &  '  aloddate="',a10,'", alodtime="',a,'", alodmach="',a,'"')
cbug      write (   6, 9902) aversion, aloddate, alodtime, alodmach
cbugc***DEBUG ends.

 8100 format (/ 'Version ',a24,' Loaded on ',a10,' at ',a,' on ',a)

      write (   6, 8100) aversion, aloddate, alodtime, alodmach

c=======================================================================********

c.... Call getio to get execution line message, open or create output file.

      afd      = ' '                  ! Field delimiter for input cracking.
cbugc***DEBUG begins.
cbug 9903 format ('main DEBUG 3.  aversion="',a24,'"' /
cbug     &  '  aloddate="',a10,'", alodtime="',a,'", alodmach="',a,'"')
cbug      write (   6, 9903) aversion, aloddate, alodtime, alodmach
cbugc***DEBUG ends.

      call getio

cbugcbugc***DEBUG begins.
cbug 9904 format ('main DEBUG 4.  aversion="',a24,'"' /
cbug     &  '  aloddate="',a10,'", alodtime="',a,'", alodmach="',a,'"')
cbug      write (   6, 9904) aversion, aloddate, alodtime, alodmach
cbugc***DEBUG ends.

c=======================================================================********

c.... Find run time, date.

c**** DEC:  need a substitute for mgtime.

cmeg      ierr  = mgtime (atime, adate)   ! Run time and date.  Need for user stat.
cray      call izm01 (arg)
cray      atime = arg(1)                  ! Run time.  Need for user stat.
cray      adate = arg(2)                  ! Run date.  Need for user stat.
cSUN  ierr  = mgtime (atime, adate)   ! Run time and date.  Need for user stat.

c.... Write date and time into output file.

cBUG      call system (
cBUG     &  "date +""  Run date and time:  ""'%Y %b %d %T %a' >> geom.hsp" )

      call system ( "date +'%Y %b %d %T %a' > aindate" )

      afile = 'aindate'
      atype = 'text'
      iopt  = -1

      call openx (21, afile, atype, iopt, isize)

      rewind (21)

 9121 format (2a)

      read  (21, 9121) adate, atime

 8110 format (/ '  GEOM run at ',a,1x,a,2x,a)

      call system ( "rm -f aindate" )

      write (   6, 8110) adate, atime, amach
      write (iout, 8110) adate, atime, amach

c=======================================================================********

c.... Find total time charged to problem.

      call tbank (ksecr)

 8120 format ('  GEOM time used, milliseconds =',i9,'.')

ctem      write (   6, 8120) ksecr
      write (iout, 8120) ksecr

c=======================================================================********

c.... Initialize conventions and geometric objects.

      aprompt  = "on"                 ! Turn on prompt sentinal.
      aprompts = "?"                  ! Prompt string.
      lprompts = 1                    ! Length of prompt string.

      nifnest  = 0                    ! Start with no nested "if-endif" blocks.
      niftots  = 0                    ! Start with no nested "if-endif" blocks.
      ncmds    = 0                    ! Start with no command lines in geom.hsp.
      nloop    = 0                    ! Start with no do loop blocks.
      nstats   = 0                    ! Start with no multiple-command lines.
      ntemps   = 0                    ! Start with no temporary tables.

      kbl1     = 0                    ! Start with no mesh block k-lines.
      kbl2     = 0                    ! Start with no mesh block k-lines.
      kmax     = 0                    ! Start with no mesh k-lines.
      lbl1     = 0                    ! Start with no mesh block l-lines.
      lbl2     = 0                    ! Start with no mesh block l-lines.
      lmax     = 0                    ! Start with no mesh l-lines.
      mbl1     = 0                    ! Start with no mesh block m-lines.
      mbl2     = 0                    ! Start with no mesh block m-lines.
      mmax     = 0                    ! Start with no mesh m-lines.

      nalias   = 0                    ! Start with no aliases.
      nbins    = 0                    ! Start with no probability bins.
      nbricks  = 0                    ! Start with no bricks.
      ncells   = 0                    ! Start with no sudoku cells filled.
      nclusts  = 0                    ! Start with no clusters.
      ndisks   = 0                    ! Start with no annular disks.
      nlines   = 0                    ! Start with no lines.
      nmarks   = 0                    ! Start with no markers.
      nopers   = 0                    ! Start with no operators.
      npairs   = 0                    ! Start with no cluster/point list.
      npdfs    = 0                    ! Start with no pdfs.
      npdfxs   = 0                    ! Start with no pdf/bin list.
      npoints  = 0                    ! Start with no points.
      npolygs  = 0                    ! Start with no polygons.
      npolyhs  = 0                    ! Start with no polyhedrons.
      nquads   = 0                    ! Start with no quadric surfaces.
      nsymbs   = 0                    ! Start with no symbols.
      nsyns    = 0                    ! Start with no synonyms.
      ntets    = 0                    ! Start with no tetrahedrons.
      ntrigs   = 0                    ! Start with no triangles.
      nvars    = 0                    ! Start with no variables.
      nvectors = 0                    ! Start with no vectors.
      nzones   = 0                    ! Start with no zones.

c.... Initialize input file data.

      nword    = 0                    ! Number of words on an input line.
      call input                      ! Get input from the user's terminal.

      if (ainputex .ne. ' ') then     ! Use input file from execution line.
        nword = 2
        aword(1) = 'input'
        aword(2) = ainputex
        call input
cold        ninput = ninput + 1
      endif

c.... Initialize miscellaneous data.

      nword    = 0                    ! Number of words on an input line.

      call coord                      ! Make coordinate system rectangular.
      call angles                     ! Make angle units degrees.
      call tolset                     ! Make initial tol = tolz.
      call icalc                      ! Make init amode='dec', abind='nobin'.
      call bigint                     ! Initialize big integers to zero.
      call spin                       ! Use first 10 random numbers.
      call symbol                     ! Set special symbols.
      call synonym                    ! Initialize synonyms.
      call quadric                    ! Initialize quadric types.
      call plotx                      ! Initialize plot parameters.
      call parse                      ! Initialize parsing flag.
      call status                     ! Display initial status.
      call sudo                       ! Initialize sudoku character set.
      call variable                   ! Initialize avg, std dev, min, max.

c=======================================================================********

c.... Prompt the user for the problem title (optional).

 8130 format (/ '  Please type "title TITLE"' /
     &  '  where TITLE is the problem title and/or user ID,'
     &  ' or type "end".')

      write (   6, 8130)

c=======================================================================********

c.... Start here for each additional input item.

  200 continue

 8377 format (80('#'))
      write (iout, 8377)
cbugc***DEBUG begins.
cbug 9876 format ('main DEBUG',i4)
cbug      nbug = 1
cbug      write (   6, 9876) nbug
cbug      write (iout, 9876) nbug
cbugc***DEBUG ends.

c....   See if last input line has been reached.

      if (ninput .eq. infend(nfiles)) then
cbugc***DEBUG begins.
cbug 9701 format ('main DEBUG.  Reached last line of input medium.' /
cbug     &        '  line     ',i8,' or "',a40,'"' )
cbug      write (   6, 9701) infend(nfiles), ainfend(nfiles)
cbug      write (iout, 9701) infend(nfiles), ainfend(nfiles)
cbugc***DEBUG ends.
        go to 320
      endif
cbugc***DEBUG begins.
cbug      nbug = 11
cbug      write (   6, 9876) nbug
cbug      write (iout, 9876) nbug
cbugc***DEBUG ends.

      if ((nword .ge. 1)                   .and.
     &    (aword(1) .eq. ainfend(nfiles))) then
cbugc***DEBUG begins.
cbug 9702 format ('main DEBUG.  Reached last line of input medium.' /
cbug     &        '  string   ',i8,' or "',a40,'"' )
cbug      write (   6, 9702) infend(nfiles), ainfend(nfiles)
cbug      write (iout, 9702) infend(nfiles), ainfend(nfiles)
cbugc***DEBUG ends.
        go to 320
      endif

c=======================================================================********

c.... Prompt user if input is from user's terminal, and ready for more input.

cbugc***DEBUG begins.
cbug      nbug = 12
cbug      write (   6, 9876) nbug
cbug      write (iout, 9876) nbug
cbugc***DEBUG ends.
      if ((nstats .gt. 1) .and. (nstat .le. nstats)) go to 230
cbugc***DEBUG begins.
cbug      nbug = 13
cbug      write (   6, 9876) nbug
cbug      write (iout, 9876) nbug
cbugc***DEBUG ends.
      if ((iin .ne. 5) .or. (ainput .eq. 'command')) go to 230
cbugc***DEBUG begins.
cbug      nbug = 2
cbug      write (   6, 9876) nbug
cbug      write (iout, 9876) nbug
cbugc***DEBUG ends.

 8230 format (/ '  Input (up to 80 characters), or "help" or "end"?')
 8231 format (a1,' ',$)
 8232 format (a2,' ',$)
 8234 format (a4,' ',$)
 8238 format (a8,' ',$)
 8240 format ('  title   ?')

      if (aprompt .eq. 'on') then     ! Long prompt.
        write (   6, 8230)
      else                            ! Short prompt.
        if (lprompts .eq. 1) then
          write (   6, 8231) aprompts
        elseif (lprompts .eq. 2) then
          write (   6, 8232) aprompts
        elseif (lprompts .le. 4) then
          write (   6, 8234) aprompts
        else
          write (   6, 8238) aprompts
        endif
      endif

 8197 format ('  Parsing flag is on.  Type "parse off" to turn off.')

      if (aparse .eq. 'on') then
        write (   6, 8197)
      endif

      if ((atitle(1) .eq. 'NOT SPEC')  .and.
     &    (atitle(2) .eq. 'IFIED.  ')) then
        write (   6, 8240)
      endif

c=======================================================================********

c.... Count input lines.

  230 ninput   = ninput + 1
cbugc***DEBUG begins.
cbug      nbug = 3
cbug      write (   6, 9876) nbug
cbug      write (iout, 9876) nbug
cbugc***DEBUG ends.

c=======================================================================********

c.... See if next command is from a multiple command line.

  232 if (nstats .gt. 1) then

        if (nstat .gt. nstats) then
          nstat  = 0
          nstats = 0
          go to 238
        endif

        n1 = istat(nstat)   + 1
        n2 = istat(nstat+1) - 1
        nn = n2 - n1 + 1
cbugc***DEBUG begins.
cbug 9984 format ('nstats=',i3,' nstat=',i3,' n1=',i3,' n2=',i3,' nn=',i3)
cbug        write (   6, 9984) nstats, nstat, n1, n2, nn
cbug        write (iout, 9984) nstats, nstat, n1, n2, nn
cbugc***DEBUG ends.

        awords = aword(1)

        call aptchap (astat, n1, nn, nwordm, afd,
     &                iamax, idmax, iemax, aword, iword, fword,
     &                mtype, lword, nword, nerrch)
cbugc***DEBUG begins.
cbug 9101 format ('nbug=',i5,'  iin=',i3)
cbug      nbug = 101
cbug      write (   6, 9101) nbug, iin
cbug      write (iout, 9101) nbug, iin
cbugc***DEBUG ends.

        if (nword .eq. 0) then
          nstat = nstat + 1
          go to 232
        endif

        do 234 n = 1, nn
          achar(n) = astat(n+n1-1)
  234   continue

        do 236 n = nn + 1, 80
          achar(n) = ' '
  236   continue
cbugc***DEBUG begins.
cbug 9982 format (80a1)
cbug        write (   6, 9982) (achar(n), n = 1, ncharm)
cbug        write (iout, 9982) (achar(n), n = 1, ncharm)
cbugc***DEBUG ends.

c....   See if statement separator is allowed in this command sequence.
cbugc***DEBUG begins.
cbug 9666 format ('DEBUG nstat=',i3,' nstats=',i3,' aword= ',a)
cbug      write (   6, 9666) nstat, nstats, aword(1)
cbug      write (iout, 9666) nstat, nstats, aword(1)
cbugc***DEBUG ends.

        if ((nstat .gt. 1)         .and.
     &     ((awords .eq. '.')      .or.
     &      (awords .eq. 'call')   .or.
     &      (awords .eq. 'do')     .or.
     &      (awords .eq. 'enddo')  .or.
     &      (awords .eq. 'goto')   .or.
     &      (awords .eq. 'i')      .or.
     &      (awords .eq. 'in')     .or.
     &      (awords .eq. 'indo')   .or.
     &      (awords .eq. 'input')  .or.
     &      (awords .eq. 'r')      .or.
     &      (awords .eq. 'rd')     .or.
     &      (awords .eq. 'read')   .or.
     &      (awords .eq. 'redo')   .or.
     &      (awords .eq. 'return') .or.
     &      (awords .eq. 'rtn')    .or.
     &      (awords .eq. 'u')      .or.
     &      (awords .eq. 'undo'))) then

 8344     format ('  main ERROR.  Reading ',a40,' line',i6,'.' /
     &      '  Cannot use statement separator ";" following commands:' /
     &      '  ".", "do", "enddo", "goto", "indo", "input",' /
     &      '  "read", "redo", "return", "undo", or their synonyms.' /
     &      '  Cannot execute commands:' / 80a1)

          write (   6, 8344) ainput, ninput, (astat(n), n = n1, 80)
          write (iout, 8344) ainput, ninput, (astat(n), n = n1, 80)
 9344     format ('DEBUG 9344')       ! DEBUG.
          write (iout, 9344)          ! DEBUG.

          nstat  = 0
          nstats = 0
          go to 238
        endif                         ! Tested for valid use of ";",

        nstat = nstat + 1
        go to 270

      endif                           ! Get input from last input line.

c=======================================================================********

c.... See if next input line should come from the list of saved input lines.

  238 if ((iin .eq. 5)             .and.
     &    (ainput .eq. 'command')) then

        nn = 1 + mod (ninput - 1, ncmdm)

        call aptchap (acmd(1,nn), 1, ncharm, nwordm, afd,
     &                iamax, idmax, iemax, aword, iword, fword,
     &                mtype, lword, nword, nerrch)
cbugc***DEBUG begins.
cbug      nbug = 102
cbug      write (   6, 9101) nbug, iin
cbug      write (iout, 9101) nbug, iin
cbugc***DEBUG ends.

        do 240 n = 1, ncharm
          achar(n) = acmd(n,nn)
  240   continue

        go to 270

      endif                           ! Read input from tty or input file.

c=======================================================================********

c.... Input line:  COMMAND
c.... Input line:  COMMENT

c.... Read the next input line from the user's terminal or the input file.

  245 call aptffip (iin, afd, iamax, idmax, iemax, aword, iword, fword,
     &              mtype, lword, nword, achar, nerrf)
cbugc***DEBUG begins.
cbug      nbug = 103
cbug      write (   6, 9101) nbug, iin
cbug      write (iout, 9101) nbug, iin
cbugc***DEBUG ends.
cbugc***DEBUG begins.
cbug 9777 format ('main DEBUG.  ninput =',i5,'  iin=',i5,'  achar:' / 80a1)
cbug 9778 format ('main DEBUG.  nerrf  =',i5,'  aword(1):' / a80)
cbug      write (   6, 9777) ninput, iin, (achar(n), n = 1, 80)
cbug      write (iout, 9777) ninput, iin, (achar(n), n = 1, 80)
cbug      write (   6, 9778) nerrf, aword(1)
cbug      write (iout, 9778) nerrf, aword(1)
cbugc***DEBUG ends.

      if (nerrf .eq. -1) go to 290    ! Reached end-of-file.
cbugc***DEBUG begins.
cbug      nbug = 4
cbug      write (   6, 9876) nbug
cbug      write (iout, 9876) nbug
cbugc***DEBUG ends.

      if (nword .eq. 0) then          ! Null or blank input line.
        ninput = ninput + 1
cbugc***DEBUG begins.
cbug      nbug = 5
cbug      write (   6, 9876) nbug
cbug      write (iout, 9876) nbug
cbugc***DEBUG ends.
        if ((iin .eq. 5)             .and.
     &      (ainput .ne. 'command')) then
          if (aprompt .eq. 'off') then
            if (lprompts .eq. 1) then
              write (   6, 8231) aprompts
              write (iout, 8231) aprompts
            elseif (lprompts .eq. 2) then
              write (   6, 8232) aprompts
              write (iout, 8232) aprompts
            elseif (lprompts .le. 4) then
              write (   6, 8234) aprompts
              write (iout, 8234) aprompts
            else
              write (   6, 8238) aprompts
              write (iout, 8238) aprompts
            endif
          endif                         ! Short prompt.
        endif                           ! Prompt user's terminal.
cbugc***DEBUG begins.
cbug      nbug = 555
cbug      write (   6, 9876) nbug
cbug      write (iout, 9876) nbug
cbugc***DEBUG ends.
        go to 245
      endif

c=======================================================================********

c.... Check for aliases, and make replacements.

      call aliasget (aword(1), kalias)
cbugc***DEBUG begins.
cbug 9301 format ('MAIN DEBUG 1.  achar (80 char on next line):' / 80a1)
cbug 9302 format ('MAIN DEBUG 1.  NWORD = ',i5 )
cbug 9303 format (
cbug     &  '  I     = ',i5,' LWORD = ',i5,' MTYPE = ',i5,' IWORD = ',i5 /
cbug     &  '  AWORD = ',a60 /
cbug     &  '  FWORD = ',1pe22.14)
cbug      write (   6, 9301) (achar(i), i = 1, ncharm)
cbug      write (iout, 9301) (achar(i), i = 1, ncharm)
cbug      write (   6, 9302) nword
cbug      write (iout, 9302) nword
cbug      do 9991 i = 1, nword
cbug      write (   6, 9303) i, lword(i), mtype(i), iword(i),
cbug     &  aword(i), fword(i)
cbug      write (iout, 9303) i, lword(i), mtype(i), iword(i),
cbug     &  aword(i), fword(i)
cbug 9991 continue
cbugc***DEBUG ends.

      if (kalias .lt. 0) go to 200

c=======================================================================********

c.... Test for multiple statements.  Separator is character ";",
c....   not bracketed by single or double quotes.

      nstats   = 1
      nstat    = 1
      istat(1) = 0

      nsq = 0
      ndq = 0

      if (aword(1) .ne. 'hex') then
      do 242 n = 1, ncharm
        astat(n) = achar(n)
        if (achar(n) .eq. '''') nsq = nsq + 1
        if (achar(n) .eq. '"') ndq = ndq + 1
        if (mod (nsq, 2) .eq. 1) go to 242
        if (mod (ndq, 2) .eq. 1) go to 242
        if (achar(n) .eq. ';') then
          nstats = nstats + 1
          istat(nstats) = n
cnew----------------------------------------------------------------------------

        if ((nstats .gt. 1)          .and.
     &     ((aword(1) .eq. '.')      .or.
     &      (aword(1) .eq. 'call')   .or.
     &      (aword(1) .eq. 'do')     .or.
     &      (aword(1) .eq. 'enddo')  .or.
     &      (aword(1) .eq. 'goto')   .or.
     &      (aword(1) .eq. 'i')      .or.
     &      (aword(1) .eq. 'in')     .or.
     &      (aword(1) .eq. 'indo')   .or.
     &      (aword(1) .eq. 'input')  .or.
     &      (aword(1) .eq. 'r')      .or.
     &      (aword(1) .eq. 'rd')     .or.
     &      (aword(1) .eq. 'read')   .or.
     &      (aword(1) .eq. 'redo')   .or.
     &      (aword(1) .eq. 'return') .or.
     &      (aword(1) .eq. 'rtn')    .or.
     &      (aword(1) .eq. 'u')      .or.
     &      (aword(1) .eq. 'undo'))) then

          write (   6, 8344) ainput, ninput, (achar(nn), nn = 1, 80)
          write (iout, 8344) ainput, ninput, (achar(nn), nn = 1, 80)
 9345     format ('DEBUG 9345')       ! DEBUG.
          write (iout, 9345)          ! DEBUG.

          nstat  = 0
          nstats = 0
          go to 238
        endif                         ! Tested for valid use of ";",
cnew----------------------------------------------------------------------------
        endif
  242 continue
      endif

      istat(nstats+1) = ncharm + 1
cbugc***DEBUG begins.
cbug 9981 format ('nstats=',i2 / 25i3 / 25i3 / 25i3)
cbug      write (   6, 9981) nstats, (istat(n), n = 1, nstats)
cbug      write (iout, 9981) nstats, (istat(n), n = 1, nstats)
cbug      write (   6, 9982) (astat(n), n = 1, ncharm)
cbug      write (iout, 9982) (astat(n), n = 1, ncharm)
cbugc***DEBUG ends.

      if (nstats .gt. 1) then
        go to 232
      endif

c=======================================================================********

  270 continue

c=======================================================================********

c.... Test for end-of-file.

 8290 format ('  main WARNING.  Reading ',a40,' line',i6,'.' /
     &  '  Reached end-of-file.')

  290 if (nerrf .eq. -1) then       ! Reached end-of-file in input file.
        if (ainput .ne. 'tty') then
          write (   6, 8290) ainput, ninput
          write (iout, 8290) ainput, ninput
          go to 320
        else
          go to 999
        endif
      endif

      nends = 0                       ! # of sequential end-of-files.

      go to 330

c=======================================================================********

c.... Reached last specified line or end-of-file.  Return to last input medium.

 8310 format ('  main ERROR.  Reading ',a40,' line',i6,'.' /
     &  '  Reached last line of file 10 times in a row.')

  320 nends = nends + 1               ! # of sequential end-of-files.
      if (nends .ge. 10) then
        write (   6, 8310) ainput, ninput
        write (iout, 8310) ainput, ninput
        go to 999
      endif
      nword    = 2
      mtype(1) = 1
      aword(1) = 'input'
      mtype(2) = 1
      aword(2) = '#'

      call input

      go to 200

c=======================================================================********

c.... Display the new input line.

  330 nchars = ncharm
      do 325 i = ncharm,1,-1
        if (achar(i) .ne. ' ') then
          nchars = i
          go to 326
        endif
  325 continue
  326 continue

 8320 format (/ 'Reading     ',a40,' line',i9,'.' )

      if (ainput .ne. 'tty') then
        write (   6, 8320) ainput, ninput
        write (iout, 8320) ainput, ninput
      endif
cbugc***DEBUG begins.
cbug 8322 format ('main DEBUG.  nloop =',i3,'  ilbeg =',i8,'  ncmds =',i8)
cbug      ncmdx = ncmds + 1
cbug      write (   6, 8322) nloop, ilbeg(nloop), ncmdx
cbug      write (iout, 8322) nloop, ilbeg(nloop), ncmdx
cbugc***DEBUG ends.

 8325 format ('>',79a1)
 8330 format ('>',80a1)
 8339 format (/ '<',79a1)
 8340 format (/ '<',80a1)
 8341 format (/ '>',79a1)
 8342 format (/ '>',80a1)
 8343 format (      80a1)
 8345 format ('! ', 80a1)

c.... See what type of command this is.

      if ((aword(1) .eq. 'undo')   .or.
     &    (aword(1) .eq. 'u')      .or.
     &    (aword(1) .eq. 'indo')   .or.
     &    (aword(1) .eq. 'redo')   .or.
     &    (aword(1) .eq. '.')      .or.
     &    (aword(1) .eq. 'return') .or.
     &    (aword(1) .eq. 'rtn')    .or.
     &    (aword(1) .eq. 'input')  .or.
     &    (aword(1) .eq. 'in')     .or.
     &    (aword(1) .eq. 'i')      .or.
     &    (aword(1) .eq. 'goto')   .or.
     &    (aword(1) .eq. 'call')   .or.
     &    (aword(1) .eq. 'read')   .or.
     &    (aword(1) .eq. 'rd')     .or.
     &    (aword(1) .eq. 'r'))     then

c....   Display the input line on the user's terminal and the output file,
c....     preceded by "<".

        if (ainput .ne. 'tty') then
          if (nchars .le. 79) then
            write (   6, 8339) (achar(i), i = 1, nchars)
          else
            write (   6, 8340) (achar(i), i = 1, nchars)
          endif
        endif

        if (nchars .le. 79) then
          write (iout, 8339) (achar(i), i = 1, nchars)
        else
          write (iout, 8340) (achar(i), i = 1, nchars)
        endif

        if ((nloop .eq. 0) .or. (ainput .ne. "geom.cmd")) then

c....     Add the input line, as a comment, preceded by "!", to the list of
c....       saved input lines, and to the command file geom.cmd.

          ncmds = ncmds + 1

          nn = 1 + mod (ncmds - 1, ncmdm)

          acmd(1,nn) = '!'
          acmd(2,nn) = ' '
          ncharmax = nchars
          if (ncharmax .gt. (ncharm - 2)) ncharmax = ncharm - 2
          do 278 n = 1, ncharmax
            acmd(n+2,nn) = achar(n)
  278     continue

          write (icmd, 8345) (achar(i), i = 1, nchars)
cbugc***DEBUG begins.
cbug 7741 format ('ncmds',i5,' = ! ',80a1)
cbug      nbug = 10
cbug      write (   6, 7741) ncmds, (achar(i), i = 1, nchars)
cbug      write (iout, 7741) ncmds, (achar(i), i = 1, nchars)
cbugc***DEBUG ends.

        endif                         ! Not repeating a do loop.

        if (nloop .ne. 0) then        ! In a do loop.
          if (aword(1) .eq. 'goto') then
 8278       format ('  main ERROR.  Reading ',a40,' line',i6,'.' /
     &        '  Can not use "goto" command line in a do loop.')
            write (   6, 8278) ainput, ninput
            write (iout, 8278) ainput, ninput
          endif
        endif                         ! In a do loop.

      else                            ! Display command, add to command file.

c....   Display the input line on the user's terminal and the output file,
c....     preceded by ">".

        if (ainput .ne. 'tty') then
          if (nchars .le. 79) then
            write (   6, 8325) (achar(i), i = 1, nchars)
          else
            write (   6, 8330) (achar(i), i = 1, nchars)
          endif
        endif

        if (nchars .le. 79) then
          write (iout, 8341) (achar(i), i = 1, nchars)
        else
          write (iout, 8342) (achar(i), i = 1, nchars)
        endif

        if ((nloop .eq. 0) .or. (ainput .ne. "geom.cmd")) then

c....   Add the input line, as a command, to the list of saved input lines,
c....     and to the command file geom.cmd.

        ncmds = ncmds + 1

        nn = 1 + mod (ncmds - 1, ncmdm)

        do 280 n = 1, ncharm
          acmd(n,nn) = achar(n)
  280   continue

          write (icmd, 8343) (achar(i), i = 1, nchars)
cbugc***DEBUG begins.
cbug 7742 format ('ncmds',i5,' =   ',80a1)
cbug      write (   6, 7742) ncmds, (achar(i), i = 1, nchars)
cbug      write (iout, 7742) ncmds, (achar(i), i = 1, nchars)
cbugc***DEBUG ends.

        endif                         ! Not repeating a loop.

      endif                           ! Tested command type.

 8245 format (/ '  main WARNING.  Reading ',a40,' line',i6,'.' /
     &  '  Big integer.  OK for big.  Stored as floating point and',
     &  ' character string.')
 8248 format (/ '  main WARNING   Reading ',a40,' line',i6,'.' /
     &  '  Floating point too big.  Stored as character string.')

      if (nerrf .eq. 4) then
        write (   6, 8245) ainput, ninput
        write (iout, 8245) ainput, ninput
      elseif (nerrf .eq. 5) then
        write (   6, 8248) ainput, ninput
        write (iout, 8248) ainput, ninput
      endif

c=======================================================================********

c.... Skip the input line if inside an "if-endif" block that failed condition.

 8350 format ('  Skip this command.',
     &  '  Inside an unexecuted "if-endif" block',i3,'.')

      if (nifnest .gt. 0) then        ! Inside a failed "if-endif" block.

        if (aword(1) .eq. 'if') then
          call ifendif
        endif

        if (aword(1) .eq. 'endif') then
          call ifendif
        endif

        if (nifnest .gt. 0) then
          write (   6, 8350) nifnest
          write (iout, 8350) nifnest
        endif

        go to 200

      endif                           ! Tested for a failed "if-endif" block.

c=======================================================================********

c.... Blank line or null line.

  370 if (nword .eq. 0) then
        go to 200
      endif

 8370 format ('  main ERROR.  Reading ',a40,' line',i6,'.' /
     &  '  First field of input line is null.')

      if (aword(1) .eq. ' ') then
        write (   6, 8370) ainput, ninput
        write (iout, 8370) ainput, ninput
        go to 200
      endif

      if (aword(nword) .eq. ' ') then
        nword = nword - 1
        go to 370
      endif

c=======================================================================********

c.... Test for a comment line.

c.... Input line:  # COMMENT
c.... Input line:  * COMMENT
c.... Input line:  C COMMENT
c.... Input line:  c COMMENT
c.... Input line:  C++++ COMMENT
c.... Input line:  c++++ COMMENT
c.... Input line:  C---- COMMENT
c.... Input line:  c---- COMMENT
c.... Input line:  C.... COMMENT
c.... Input line:  c.... COMMENT
c.... Input line:  CCCOMMENT (starting in column 1)
c.... Input line:  ccCOMMENT (starting in column 1)
c.... Input line:  COMMAND !COMMENT
c.... Input line:  C____ COMMENT
c.... Input line:  c____ COMMENT
c.... Input line:  \ COMMENT

      if ((aword(1) .eq. 'c')      .or.
     &    (aword(1) .eq. 'C')      .or.
     &    (aword(1) .eq. 'c....')  .or.
     &    (aword(1) .eq. 'C....')  .or.
     &    (aword(1) .eq. 'c++++')  .or.
     &    (aword(1) .eq. 'C++++')  .or.
     &    (aword(1) .eq. 'c----')  .or.
     &    (aword(1) .eq. 'C----')  .or.
     &    (aword(1) .eq. 'c____')  .or.
     &    (aword(1) .eq. 'C____')) then
        go to 200
      endif

      if ((achar(1) .eq. 'c')  .and.
     &    (achar(2) .eq. 'c')) then
        go to 200
      endif

      if ((achar(1) .eq. 'C')  .and.
     &    (achar(2) .eq. 'C')) then
        go to 200
      endif

      if (((achar(1) .eq. '#')   .or.
     &     (achar(1) .eq. '*')   .or.
     &     (achar(1) .eq. '/'))  .and.
     &     (achar(1) .ne. afd))  then
        go to 200
      endif

c=======================================================================********

c.... Ignore any appended comment, beginning with the character "!".

      if (aword(1) .ne. 'hex') then
      do 376 nw = 1, nword
        afirst = aword(nw)
        if (afirst .eq. '!') then
          nword = nw - 1
          go to 377
        endif
  376 continue
      endif

  377 if (nword .eq. 0) go to 200

c=======================================================================********

c.... Make substitutions for symbols, when allowed.

      call symbrepl

c=======================================================================********

c.... Make substitutions for any variable subscripts.

cbugc***DEBUG begins.
cbug      do 661 nw = 1, nword
cbug 9661   format ('DEBUG main. 1.  nw =',i3,'  lword=',i3,'  aword = ',a)
cbug        write (   6, 9661) nw, lword(nw), aword(nw)
cbug        write (iout, 9661) nw, lword(nw), aword(nw)
cbug  661 continue
cbugc***DEBUG ends.
      call subsrepl
cbugc***DEBUG begins.
cbug      do 662 nw = 1, nword
cbug 9662   format ('DEBUG main. 2.  nw =',i3,'  lword=',i3,'  aword = ',a)
cbug        write (   6, 9662) nw, lword(nw), aword(nw)
cbug        write (iout, 9662) nw, lword(nw), aword(nw)
cbug  662 continue
cbugc***DEBUG ends.

c=======================================================================********

c.... New page line.

c.... Input line:  page

      if ((nword .eq. 1) .and. (aword(1) .eq. 'page')) then
        write (iout, "('')")
        go to 200
      endif

c=======================================================================********

c.... Termination line.

c.... Input line:  end
c.... Input line:  quit
c.... Input line:  exit
c.... Input line:  split

      if ((aword(1) .eq. 'end')    .or.
     &    (aword(1) .eq. 'quit')   .or.
     &    (aword(1) .eq. 'exit')   .or.
     &    (aword(1) .eq. 'split')) then
        go to 999
      endif

c=======================================================================********

c.... Interpret command "help".  Change "help COMMAND" to "COMMAND help".

      if ((nword .eq. 1)         .and.
     &   ((aword(1) .eq. 'help') .or.
     &    (aword(1) .eq. 'h')))  then
        call helpgen
        go to 200
      endif

      if ((nword .eq. 2)         .and.
     &   ((aword(1) .eq. 'help') .or.
     &    (aword(1) .eq. 'h')))  then

        if (aword(2) .eq. 'all') then
          call helpall
          go to 200
        endif

        aword(1) = aword(2)
        aword(2) = 'help'
      endif

c=======================================================================********

c.... See if parsing data should be displayed.

      if (aparse .eq. 'on') then
        if (aword(1) .ne. 'parse') then
          call parse
        endif
      endif

c=======================================================================********

      if ((aword(1) .eq. 'accelerate') .or.
     &    (aword(1) .eq. 'accel'))     then
        call accel
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'alias') .or.
     &    (aword(1) .eq. 'al')    .or.
     &    (aword(1) .eq. 'a'))    then
        call alias
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'alph')     .or.
     &    (aword(1) .eq. 'commands') .or.
     &    (aword(1) .eq. 'comm'))    then
        call helpalph
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'angles') .or.
     &    (aword(1) .eq. 'angle')  .or.
     &    (aword(1) .eq. 'ang'))   then
        call angles
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'arc') then
        call arc
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'area') then
        call area
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'axisym') then
        call axisym
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'base') then
        call base
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'big')  then
        call bigint
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'bin') then
        call bin
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'bisect') then
        call bisectx
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'brick') .or.
     &    (aword(1) .eq. 'br'))   then
        call brick
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'circle') .or.
     &    (aword(1) .eq. 'circ'))  then
        call circle
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'cluster') .or.
     &    (aword(1) .eq. 'cl'))     then
        call cluster
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'cone') then
        call cone
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'contfr') .or.
     &    (aword(1) .eq. 'cf'))    then
        call contfr
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'coordinate') .or.
     &    (aword(1) .eq. 'coord')      .or.
     &    (aword(1) .eq. 'cs')         .or.
     &    (aword(1) .eq. 'system')     .or.
     &    (aword(1) .eq. 'sys'))       then
        call coord
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'copy')      .or.
     &    (aword(1) .eq. 'cp')        .or.
     &    (aword(1) .eq. 'duplicate') .or.
     &    (aword(1) .eq. 'dup'))      then
        call copy
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'cross')  .or.
     &    (aword(1) .eq. 'outer')) then
        call cross
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'crt') then
        call crt
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'cute') then
        call cute
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'cuts') then
        call cuts
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'cylinder') .or.
     &    (aword(1) .eq. 'cyl'))     then
        call cylinder
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'debug') then
        call debugx
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'define') .or.
     &    (aword(1) .eq. 'def'))   then
        call define
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'delete') .or.
     &    (aword(1) .eq. 'del')    .or.
     &    (aword(1) .eq. 'remove') .or.
     &    (aword(1) .eq. 'rm'))    then
        call delete
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'delimiter') .or.
     &    (aword(1) .eq. 'delimit')   .or.
     &    (aword(1) .eq. 'delim'))    then
        call delim
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'disk')  .or.
     &    (aword(1) .eq. 'disc')  .or.
     &    (aword(1) .eq. 'dk'))   then
        call disk
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'distance')  .or.
     &    (aword(1) .eq. 'dist')      .or.
     &    (aword(1) .eq. 'intersect') .or.
     &    (aword(1) .eq. 'inter')     .or.
     &    (aword(1) .eq. 'int'))      then
        call distance
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'do')     .or.
     &    (aword(1) .eq. 'enddo')) then
        call doenddo
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'dot')    .or.
     &    (aword(1) .eq. 'inner')) then
        call dot
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'ellipsoid') .or.
     &    (aword(1) .eq. 'ellipse')   .or.
     &    (aword(1) .eq. 'ell')       .or.
     &    (aword(1) .eq. 'el'))       then
        call ellipsoid
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'error') .or.
     &    (aword(1) .eq. 'err'))   then
        call helperr
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'extrema') .or.
     &    (aword(1) .eq. 'extr'))   then
        call extrema
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'factor')   .or.
     &    (aword(1) .eq. 'totient')) then
        call factor
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'hex') then
        call hex
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'hyperb')  .or.
     &    (aword(1) .eq. 'saddle')) then
        call hyperb
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'icalc') then
        call icalc
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'if')     .or.
     &    (aword(1) .eq. 'endif')) then
        call ifendif
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'increment') .or.
     &    (aword(1) .eq. 'incr'))     then
        call incr
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'indo') then
        call indo
        if (nword .ge. 1) go to 330
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'input')  .or.
     &    (aword(1) .eq. 'in')     .or.
     &    (aword(1) .eq. 'i')      .or.
     &    (aword(1) .eq. '.')      .or.
     &    (aword(1) .eq. 'call')   .or.
     &    (aword(1) .eq. 'goto')   .or.
     &    (aword(1) .eq. 'read')   .or.
     &    (aword(1) .eq. 'rd')     .or.
     &    (aword(1) .eq. 'r')      .or.
     &    (aword(1) .eq. 'return') .or.
     &    (aword(1) .eq. 'rtn')    .or.
     &    (aword(1) .eq. 'trace')) then

        call input
        nword = 0                     ! Erase evidence of read in input.
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'intcirc') then
        call intcirc
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'invert')  .or.
     &    (aword(1) .eq. 'inv')     .or.
     &    (aword(1) .eq. 'reflect') .or.
     &    (aword(1) .eq. 'refl')    .or.
     &    (aword(1) .eq. 'rotate')  .or.
     &    (aword(1) .eq. 'rot')     .or.
     &    (aword(1) .eq. 'scale')   .or.
     &    (aword(1) .eq. 'sc'))     then
        call matrix
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'iris') then
        call iris
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'kiss') then
        call kiss
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'last') then
        call last
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'line') .or.
     &    (aword(1) .eq. 'ln')   .or.
     &    (aword(1) .eq. 'l'))     then
        call line
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'list')    .or.
     &    (aword(1) .eq. 'display') .or.
     &    (aword(1) .eq. 'print'))  then
        call list
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'lock') .or.
     &    (aword(1) .eq. 'zzz')) then
        if (ainput .ne. 'tty') go to 200
        call aloha
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'marker') .or.
     &    (aword(1) .eq. 'mark')   .or.
     &    (aword(1) .eq. 'm'))     then
        call marker
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'math') then
        call helpmath
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'mcvol') then
        call mcvol
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'mean') then
        call mean
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'mesh') then
        call mesh
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'misc') then
        call helpmisc
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'move')      .or.
     &    (aword(1) .eq. 'mv')        .or.
     &    (aword(1) .eq. 'translate') .or.
     &    (aword(1) .eq. 'trans'))     then
        call movex
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'object') .or.
     &    (aword(1) .eq. 'obj'))   then
        call helpobj
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'operator') .or.
     &    (aword(1) .eq. 'oper')     .or.
     &    (aword(1) .eq. 'op'))      then
        call operator
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'output') .or.
     &    (aword(1) .eq. 'out')    .or.
     &    (aword(1) .eq. 'o')      .or.
     &    (aword(1) .eq. 'write')  .or.
     &    (aword(1) .eq. 'wr')     .or.
     &    (aword(1) .eq. 'w'))     then
        call output
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'parse') then
        call parse
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'pdf') then
        call pdf
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'plane') .or.
     &    (aword(1) .eq. 'pl'))   then
        call plane
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'plot') then
        call plotx
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'point') .or.
     &    (aword(1) .eq. 'pnt')   .or.
     &    (aword(1) .eq. 'pt')    .or.
     &    (aword(1) .eq. 'p'))    then
        call point
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'polygon') .or.
     &    (aword(1) .eq. 'polyg')   .or.
     &    (aword(1) .eq. 'pg'))     then
        call polygon
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'polyhedron') .or.
     &    (aword(1) .eq. 'polyh')   .or.
     &    (aword(1) .eq. 'ph'))     then
        call polyhed
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'project') .or.
     &    (aword(1) .eq. 'proj'))   then
        call project
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'prompt') then
        call prompt
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'proximal') .or.
     &    (aword(1) .eq. 'prox'))    then
        call prox
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'quadric') .or.
     &    (aword(1) .eq. 'quad')    .or.
     &    (aword(1) .eq. 'q'))      then
        call quadric
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'random') .or.
     &    (aword(1) .eq. '?'))     then
        call helpran
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'ratio') .or.
     &    (aword(1) .eq. 'rat'))  then
        call ratio
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'redo') then
        call redo
        if (nword .ge. 1) go to 330
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'rename') .or.
     &    (aword(1) .eq. 'switch') .or.
     &    (aword(1) .eq. 'sw'))    then
        call renamex
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'repack')  then
        call repack
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'root') then
        call root
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'rootf') then
        call rootf
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'roots') then
        call roots
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'sample') then
        call sample
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'search')  .or.
     &    (aword(1) .eq. 'find'))  then
        call search
        go to 200
      endif
c=======================================================================********

      if ((aword(1) .eq. 'setup')       .or.
     &    (aword(1) .eq. 'environment') .or.
     &    (aword(1) .eq. 'env'))        then
        call helpenv
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'side') then
        call side
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'slice') .or.
     &    (aword(1) .eq. 'cut'))  then
        call slice
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'solve') then
        call solve
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'sort') then
        call sort
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'sphere') .or.
     &    (aword(1) .eq. 'sph'))   then
        call sphere
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'spin') then
        call spin
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'status') .or.
     &    (aword(1) .eq. 'stat'))  then
        call status
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'sudoku') .or.
     &    (aword(1) .eq. 'su'))    then
        call sudo
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'steiner') .or.
     &    (aword(1) .eq. 'stein'))  then
        call triangle
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'symbol') .or.
     &    (aword(1) .eq. 'symb')   .or.
     &    (aword(1) .eq. 'sym'))    then
        call symbol
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'synonym') .or.
     &    (aword(1) .eq. 'syn'))    then
        call synonym
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'tables') .or.
     &    (aword(1) .eq. 'sizes')) then
        call tables
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'tetrahedron') .or.
     &    (aword(1) .eq. 'tetra')       .or.
     &    (aword(1) .eq. 'tet'))        then
        call tetra
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'time') .or.
     &    (aword(1) .eq. 't'))   then
        call timex
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'id')  .or.
     &    (aword(1) .eq. 'title')) then
        call title
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'tol') then
        call tolset
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'track') .or.
     &    (aword(1) .eq. 'trk')) then
        call track
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'triangle') .or.
     &    (aword(1) .eq. 'tri'))   then
        call triangle
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'trig') then
        call trig
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'triple') then
        call triple
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'twist') .or.
     &    (aword(1) .eq. 'tw'))   then
        call twist
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'undo') .or.
     &    (aword(1) .eq. 'u'))    then
        call undo
        nword = 0                     ! Erase evidence of read in input.
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'variable') .or.
     &    (aword(1) .eq. 'var')      .or.
     &    (aword(1) .eq. 'set')      .or.
     &    (aword(1) .eq. 's'))       then
        call variable
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'vector') .or.
     &    (aword(1) .eq. 'vect')   .or.
     &    (aword(1) .eq. 'v'))     then
        call vector
        go to 200
      endif

c=======================================================================********

c.... Input line:  vi

 8550 format ('  "vi"                Open a shell window, execute vi.')

      if (aword(1) .eq. 'vi') then

        if (aword(2) .eq. 'help') then
          write (   6, 8550)
          write (iout, 8550)
          go to 200
        endif

        call system (
     &    'xterm -geom 80x48+055-010 -ls -sb -sl 512
     &     -fn 8x13bold -T "DEC-"$machine" : vi "$cwd"/""\!*" -e vi ' )
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'volume') .or.
     &    (aword(1) .eq. 'vol'))   then
        call volume
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'walk') then
        call walk
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'when') then
        call when
        go to 200
      endif

c=======================================================================********

      if (aword(1) .eq. 'where') then
        call where
        go to 200
      endif

c=======================================================================********

      if ((aword(1) .eq. 'zone') .or.
     &    (aword(1) .eq. 'zn')   .or.
     &    (aword(1) .eq. 'z'))     then
        call zone
        go to 200
      endif

c=======================================================================********

 8135 format (/ '  THIS IS NOT VI!  Type "end", "quit", "exit" or',
     & ' "split" to end.' )

      if (aword(1) .eq. 'ZZ') then
        write (   6, 8135)
        write (iout, 8135)
        go to 200
      endif

c=======================================================================********

 8500 format ('  main ERROR.  Reading ',a40,' line',i6,'.' /
     &  '  Input line is not recognized:' / 80a1 /
     &  '  Type "help" to display help messages.' /
     &  '  Type "commands" or "alph" to display a list of commands.' /
     &  '  Type "define ''SYMBOL''" to display a definition of any'
     &  ' SYMBOL.' )
 8510 format ('  The field delimiter is now "',a1,'".' /
     &        '  Type "delim" to restore it to a blank character.' )

      write (   6, 8500) ainput, ninput, (achar(n), n = 1, ncharm)
      write (iout, 8500) ainput, ninput, (achar(n), n = 1, ncharm)

      if (afd .ne. ' ') then
        write (   6, 8510) afd
        write (iout, 8510) afd
      endif
cbugc***DEBUG begins.
cbug 9911 format ('MAIN DEBUG 2.  achar (80 char on next line):' / 80a1)
cbug 9912 format ('MAIN DEBUG 2.  NWORD = ',i5 )
cbug 9913 format (
cbug     &  '  I     = ',i5,' LWORD = ',i5,' MTYPE = ',i5,' IWORD = ',i5 /
cbug     &  '  AWORD = ',a60 /
cbug     &  '  FWORD = ',1pe22.14)
cbug      write (   6, 9911) (achar(i), i = 1, ncharm)
cbug      write (iout, 9911) (achar(i), i = 1, ncharm)
cbug      write (   6, 9912) nword
cbug      write (iout, 9912) nword
cbug      do 9992 i = 1, nword
cbug      write (   6, 9913) i, lword(i), mtype(i), iword(i),
cbug     &  aword(i), fword(i)
cbug      write (iout, 9913) i, lword(i), mtype(i), iword(i),
cbug     &  aword(i), fword(i)
cbug 9992 continue
cbugc***DEBUG ends.

      go to 200                       ! Look for next input line.

c=======================================================================********

c.... End run.  Not restartable.  Result of input line "split" or "*split", or
c....   controller message "split" or "end", or no name input line.

 8610 format ('  main ERROR.  Reading ',a40,' line',i6,'.' /
     &  '  Incomplete do loops:  ',i5)

 8620 format ('  main ERROR.  Reading ',a40,' line',i6,'.' /
     &  '  Incomplete if/endif blocks:  ',i5)

  999 if (nloop .ne. 0) then
        write (   6, 8610) ainput, ninput, nloop
        write (iout, 8610) ainput, ninput, nloop
      endif

      if (niftots .ne. 0) then
        write (   6, 8620) ainput, ninput, niftots
        write (iout, 8620) ainput, ninput, niftots
      endif

      write (   6, 8100) aversion, aloddate, alodtime, alodmach
      write (iout, 8100) aversion, aloddate, alodtime, alodmach

c.... Write date and time into standard output file geom.hsp.

      write (   6, 8110) adate, atime, amach
      write (iout, 8110) adate, atime, amach

 8150 format (/ 'Problem title:  ',9a8)

      write (   6, 8150) (atitle(i), i = 1, 9)
      write (iout, 8150) (atitle(i), i = 1, 9)

 8160 format ('  Current output file is          ',a /
     &        '  Current command summary file is ',a )

      write (   6, 8160) afilout, afilcmd
      write (iout, 8160) afilout, afilcmd

      atermess = 'main.  User typed end, quit, exit or split.'
      call geomend (atermess)

c.... End of subroutine main.         (+1 line).
      end
