      subroutine mcvol

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  mcvol    Updated     1994 August 25 16:30.
c               mcvol    Originated  1994 February 25 14:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To make a Monte Carlo estimate of the volume of a zone,
c               and an estimate of its coordinate limits.
c
c     Logic:    Entry points:  mcvol
c mcvol         Called by:  helpall, main.
c
c mcvol         Calls:  aptbrks, aptcsys, brout, varfind, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      dimension dside(nzquadm)        ! Value of F(x,y,z) at point.
      dimension nuse(-180:180)        ! Indicates angle sampled, if 1.

c.... Input line:  mcvol [help]

 8110 format ('  Command "mcvol":  acceptable input lines include:' /
     &  '  "mcvol [help]"  Display this message.' //
     &  '  "mcvol ZNAME BRNAME NSAMP"' /
     &  '     Estimate the volume of the part of zone ZNAME that',
     &  ' is inside' /
     &  '     brick BRNAME, by using NSAMP random points in BRNAME.' //
     &  '  Synonyms:  [brick, br], [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  mcvol ZNAME BRNAME NSAMP

c.... Test input for validity.

 8120 format ('  mcvol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 words on input line:  ',i3)

      if (nword .ne. 4) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c=======================================================================********

c.... Find the brick.

 8125 format ('  mcvol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named brick does not exist:  ',a)

      do 120 nbr = 1, nbricks
        if (aword(3) .eq. abrick(nbr)) then
          nb = nbr
          call brout (nb, nb, 'min')
          go to 130
        endif
  120 continue
      write (   6, 8125) ainput, ninput, aword(3)
      write (iout, 8125) ainput, ninput, aword(3)
      go to 999

c=======================================================================********

c.... Find the zone.

 8130 format ('  mcvol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named zone does not exist:  ',a)

  130 do 140 nzn = 1, nzones
        if (aword(2) .eq. azone(nzn)) then
          nz = nzn
          call zout (nz, nz, 'min')
          go to 150
        endif
  140 continue
      write (   6, 8130) ainput, ninput, aword(2)
      write (iout, 8130) ainput, ninput, aword(2)
      go to 999

c=======================================================================********

c.... Find the number of random points.

 8155 format ('  mcvol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  NSAMP not an integer or integer variable:  ',a)

  150 call varfind (4, 4)

      if (mtype(4) .ne. 2) then
        write (   6, 8155) ainput, ninput, aword(4)
        write (iout, 8155) ainput, ninput, aword(4)
        go to 999
      endif

 8160 format ('  mcvol ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Less than two random points:  ',i5)

  160 if (iword(4) .lt. 2) then
        write (   6, 8160) ainput, ninput, iword(4)
        write (iout, 8160) ainput, ninput, iword(4)
        go to 999
      endif

      nsamp = iword(4)

c=======================================================================********

c.... Find the brick coordinate system.

 8162   format ('  mcvol ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Brick has unrecognized coordinate system:  ',a)

      nerr = 0

      if     (asysbr(nb) .eq. 'rect') then
        ksys = 0
      elseif (asysbr(nb) .eq. 'cyl')  then
        ksys = 1
      elseif (asysbr(nb) .eq. 'sph')  then
        ksys = 2
      else
        write (   6, 8162) ainput, ninput, asysbr(nb)
        write (iout, 8162) ainput, ninput, asysbr(nb)
        nerr =  1
        go to 999
      endif

c.... Find the angle units.

      if (angbr(nb) .eq. 'radians') then
        iunit = 1
        fact = 1.0
      elseif (angbr(nb) .eq. 'degrees') then
        iunit = 0
        fact = pi / 180.0
      endif

c.... Initialize.

      npin  = 0

      xmin    = bignum
      xmax    = -bignum
      ymin    = bignum
      ymax    = -bignum
      zmin    = bignum
      zmax    = -bignum
      rcylmin = bignum
      rcylmax = -bignum
      thmin   = bignum
      thmax   = -bignum
      rsphmin = bignum
      rsphmax = -bignum
      phimin  = bignum
      phimax  = -bignum

      do 165 n = -180,180
        nuse(n) = 0
  165 continue

c.... Loop over trial points.

      do 220 ntest = 1, nsamp

c....   Randomly sample a point in the brick.

        call aptbrks (ksys, iunit, ubr(nb), dubr(nb), vbr(nb), dvbr(nb),
     &                wbr(nb), dwbr(nb), 1, tol,
     &                u, v, w, fracu, fracv, fracw, nerra)
  
        call aptcsys (ksys, 0, iunit, u, v, w, 1, tol, nerra)

        xp = u
        yp = v
        zp = w

c....   See if point is in zone.

        nside = 1                   ! Assume point is in zone.

c....   Test each boundary surface of this zone.

        do 210 nzq = 1, nzquads(nz) ! Loop over boundary surfaces.

 8170    format ('  mcvol ERROR.  Reading ',a40,' line ',i5,'.' /
     &     '  Named zone boundary surface does not exist:  ',a)

          do 170 nq = 1, nquads     ! Loop over all quadrics.
            if (aquad(nq) .eq. azquad(nzq,nz)) then
              n = nq
              go to 180
            endif
  170     continue                  ! End of loop over all quadrics
          write (   6, 8170) ainput, ninput, azquad(nzq,nz)
          write (iout, 8170) ainput, ninput, azquad(nzq,nz)
          go to 999

c....     Find the side of the quadric surface the point is on.

  180     dside(nzq) = qc(n) + qx(n) * xp + qy(n) * yp + qz(n) * zp +

     &            qxy(n) * xp * yp +
     &            qyz(n) * yp * zp +
     &            qzx(n) * zp * xp +
     &            qxx(n) * xp**2 + qyy(n) * yp**2 + qzz(n) * zp**2

          derr  = tol * (abs (qc(n)) +
     &            abs (qx(n) * xp) +
     &            abs (qy(n) * yp) +
     &            abs (qz(n) * zp) +
     &            abs (qxy(n) * xp * yp) +
     &            abs (qyz(n) * yp * zp) +
     &            abs (qzx(n) * zp * xp) +
     &            abs (qxx(n)) * xp**2 +
     &            abs (qyy(n)) * yp**2 +
     &            abs (qzz(n)) * zp**2)

          if (abs (dside(nzq)) .lt. derr) then
            dside(nzq) = 0.0
          endif

c....     See if the point is inside (dside(nzq) has opposite sign of azsign)
c....       or outside the zone.

          if ((dside(nzq) .gt. 0.0)      .and.
     &        (azsign(nzq,nz) .eq. '+')) then
            nside = 0
          endif

          if ((dside(nzq) .lt. 0.0)      .and.
     &        (azsign(nzq,nz) .eq. '-')) then
            nside = 0
          endif

  210   continue                      ! End of loop over boundary surfaces.

c....   Find the extreme coordinates at which the point is found in the zone.

        if (nside .eq. 1) then
          npin = npin + 1             ! Point is in zone.
          if (xp .lt. xmin) xmin = xp
          if (xp .gt. xmax) xmax = xp
          if (yp .lt. ymin) ymin = yp
          if (yp .gt. ymax) ymax = yp
          if (zp .lt. zmin) zmin = zp
          if (zp .gt. zmax) zmax = zp
          rcyl = xp**2 + yp**2
          if (rcyl .lt. rcylmin) rcylmin = rcyl
          if (rcyl .gt. rcylmax) rcylmax = rcyl
          theta = atan2 (yp, xp)      ! Radians.    
          theta = 180.0 * theta / pi  ! Degrees.
          nuse (nint (theta)) = 1
          if (theta .lt. thmin) thmin = theta  ! Degrees.
          if (theta .gt. thmax) thmax = theta  ! Degrees.
          rsph = sqrt (xp**2 + yp**2 + zp**2)
          if (rsph .lt. rsphmin) rsphmin = rsph
          if (rsph .gt. rsphmax) rsphmax = rsph
          phi = acos (zp / (rsph + small))
          if (phi .lt. phimin) phimin = phi  ! Radians.
          if (phi .gt. phimax) phimax = phi  ! Radians.
        endif

  220 continue                        ! End of loop over test points.

c=======================================================================********

c.... Find the estimated volume of the zone.

      volz(nz) = npin * volbr(nb) / nsamp
      if (npin .gt. 0) then
        ferr = sqrt ((nsamp - npin) / (npin * (nsamp - 1.0)))
      else
        ferr = 1.0
      endif
      errz        = 100.0 * ferr
      volzerr(nz) = ferr * volz(nz)

      if (npin .eq. 0) then
        go to 280
      endif

 8240 format ('  x, y, z     min ',1p3e20.12 /   
     &        '  x, y, z     max ',1p3e20.12 )

      write (   6, 8240) xmin, ymin, zmin, xmax, ymax, zmax
      write (iout, 8240) xmin, ymin, zmin, xmax, ymax, zmax

 8250 format ('  rcyl, th, z min ',1p3e20.12 /   
     &        '  rcyl, th, z max ',1p3e20.12 )

      if (angunit .eq. 'radians') then
        thmin = pi * thmin / 180.0
        thmax = pi * thmax / 180.0
      endif

      write (   6, 8250) rcylmin, thmin, zmin, rcylmax, thmax, zmax
      write (iout, 8250) rcylmin, thmin, zmin, rcylmax, thmax, zmax

      if (angunit .eq. 'degrees') then
        phimin = 180.0 * phimin / pi
        phimax = 180.0 * phimax / pi
      endif

 8260 format ('  rsph,th,phi min ',1p3e20.12 /   
     &        '  rsph,th,phi max ',1p3e20.12 )

      write (   6, 8260) rsphmin, thmin, phimin, rsphmax, thmax, phimax
      write (iout, 8260) rsphmin, thmin, phimin, rsphmax, thmax, phimax

 8270 format ('  theta range, deg')
 8272 format (2x,i4,' to ',i4,2x,i1,1x,30i1,1x,30i1)

      write (   6, 8270)
      write (iout, 8270)
      do 270 nn = 1, 6
        n1 = -240 + 60 * nn
        n2 = n1 + 60
        write (   6, 8272) n1, n2, (nuse(n), n = n1, n2)
        write (iout, 8272) n1, n2, (nuse(n), n = n1, n2)
  270 continue

 8222 format ('  nsamp =',i8,'   npin =',i8,11x,
     &  'fraction in =',1pe20.12)
 8225 format ('  Estimated volume',1pe20.12,'  with ',0pf7.3,
     &  ' % estimated error.' )      

  280 fracin = npin / fword(4)
      write (   6, 8222) nsamp, npin, fracin
      write (iout, 8222) nsamp, npin, fracin
      write (   6, 8225) volz(nz), errz
      write (iout, 8225) volz(nz), errz

c=======================================================================********

  999 return

c.... End of subroutine mcvol.        (+1 line)
      end
