      subroutine mesh

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  mesh     Updated     2007 January 18 15:30.
c               mesh     Originated  1995 July 17 17:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create a logical mesh in k, l, m space.
c
c     Logic:    Entry points:  mesh.
c mesh          Called by:  helpall, main.
c
c mesh          Calls:  aptchmv, aptsubi, aptsubm, clout, meshchk, meshchk1,
c               meshdel, meshexch, meshfind, meshins, meshinv, meshline,
c               meshmove, meshout, meshrel, meshrlxb, meshrlxm, meshtest,
c               meshvout, pout, ptfind, ptstore, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*24 anamex             ! Temporary name of a point.
      character*1  afirst             ! First character of a point name.

c=======================================================================********

c.... Input line:  mesh [help]

 8110 format (
     &  '  Command "mesh":  acceptable input lines include:' //
     &  '  "mesh [help]"  Display this message.' //
     &  '  "mesh size"    Display the mesh size (initially zero).' /
     &  '  "mesh size [=] KMAX LMAX MMAX"' /
     &  '     Change the mesh size to k = 1 to KMAX, l = 1 to LMAX',
     &  ' and m = 1, MMAX,' /
     &  '     and release any mesh points with mesh indices k > KMAX,',
     &  '  l > LMAX' /
     &  '     or m > MMAX.' //
     &  '  "mesh block"' /
     &  '     Display the mesh block indices (initially zero).' /
     &  '  "mesh block all"' /
     &  '     Change the mesh block to the entire mesh.' /
     &  '  "mesh block K1 K2  L1 L2  M1 M2"' /
     &  '     Change the mesh block indices to the ranges' /
     &  '     k = K1 to K2, l = L1 to L2 and m = M1 to M2.' /
     &  '     Any missing final arguments after K1 default to 1.' )

 8112 format (/
     &  '  "mesh [all,list [all]]"' /
     &  '     Display the mesh size, block indices, and all',
     &  ' mesh points.' /
     &  '  "mesh list void"' /
     &  '     Display the mesh size, block indices, and all',
     &  ' unassigned mesh indices.' /
     &  '  "mesh list block [void]"' /
     &  '     Display all mesh points [or unassigned mesh indices]',
     &  ' in the mesh block.' /
     &  '  "mesh list K L M"' /
     &  '     Display any mesh point with mesh indices K, L and M.' //
     &  '  "mesh point PMESH"  Create mesh points PMESH(k,l,m),' /
     &  '     with their initial coordinates, if any, or' /
     &  '     with coordinates x = -1.e99, y = -1,e99,z = -1.e99,'
     &  ' for all' /
     &  '     mesh indices k = 1, KMAX, l = 1, LMAX, m = 1, MMAX',
     &  ' not currently assigned.' /
     &  '  "mesh point PNAME K L M"' /
     &  '     Assign mesh indices K, L and M to existing point PNAME,' /
     &  '     after releasing any other mesh point with those',
     &  ' indices.' /
     &  '     If PNAME has three subscripts, they must be K, L, M.' /
     &  '     Any missing final arguments after PNAME default to 1.' /
     &  '  "mesh point PMESH(K,L,M)"' /
     &  '     Assign mesh indices K, L and M to existing point',
     &  ' PMESH(K,L,M),' /
     &  '     after releasing any other mesh point with those',
     &  ' indices.' //
     &  '  Type <return> for more help.' )

 8115 format (a)

 8114 format (/
     &  '  "mesh point PMESH block"  Create mesh points PMESH(k,l,m),' /
     &  '     with their initial coordinates, if any, or' /
     &  '     with coordinates x = -1.e99, y = -1,e99,z = -1.e99,'
     &  ' for all' /
     &  '     mesh indices in the mesh block not currently assigned.' )

 8116 format (/
     &  '  "mesh - all"' /
     &  '     Release all mesh points.' /
     &  '  "mesh - block"' /
     &  '     Release all mesh points in the mesh block.' /
     &  '  "mesh - K L M"' /
     &  '     Release any mesh point with mesh indices K, L and M.' /
     &  '  "mesh - point PNAME"' /
     &  '     Release mesh point PNAME (make its mesh indices zero).' /
     &  '  "mesh - cluster CLNAME"' /
     &  '     Release all mesh points in cluster CLNAME.' )

 8118 format (/
     &  '  "mesh exchange block [k,l,m] [l,m,k]"' /
     &  '     Exchange the [k, l, m]-layers in the mesh block with' /
     &  '     the [l, m, k]-layers in the mesh block,' /
     &  '     and exchange the relative indices of all mesh points',
     &  ' in the mesh block.' //
     &  '  "mesh invert block [k,l,m]"' /
     &  '     Invert the [k, l, m]-layer indices of all mesh points',
     &  ' in the mesh block.' //
     &  '  "mesh move block KMOVE LMOVE MMOVE"' /
     &  '     Move the mesh block and any of its mesh points' /
     &  '     distances KMOVE, LMOVE and MMOVE in the k, l and m',
     &  ' directions,' /
     &  '     after releasing any mesh points not in the mesh block,' /
     &  '     but in the new mesh block location.' )

 8119 format (/
     &  '  "mesh insert [k,l,m] N1 [N2|N1]"' /
     &  '     Insert new [k, l, m]-layers with indices from N1 to N2',
     &  ' into the mesh.' /
     &  '     Shift mesh point [k, l, m] indices of N1 or more up',
     &  ' to make room' /
     &  '     for new mesh layers with [k, l, m] indices from',
     &  ' N1 to N2.' //
     &  '  "mesh delete [k,l,m] N1 [N2|N1]"' /
     &  '     Delete [k, l, m]-layers with indices from N1 to N2',
     &  ' from the mesh.' /
     &  '     Release all points with [k, l, m] indices from',
     &  ' N1 to N2,' /
     &  '     and shift all higher mesh point [k, l, m] indices down',
     &  ' to fill the gap.' //
     &  '  Type <return> for more help.' )

 8120 format (/
     &  '  "mesh linear block"' /
     &  '     Linearly interpolate the x, y and z coordinates of all',
     &  ' mesh points' /
     &  '     in the mesh block, between the x, y and z coordinates of',
     &  ' the mesh points' /
     &  '     at the vertices of the mesh block.' //
     &  '  "mesh relax [-] block"' /
     &  '     Find the x, y and z coordinates of all mesh points [not]',
     &  ' in the mesh block,' /
     &  '     except for exterior points of [the mesh and] the mesh',
     &  ' block,' /
     &  '     by a relaxation method, which moves each mesh point' /
     &  '     to the average coordinates of its nearest neighbors.' /
     &  '     Exterior points have exactly one neighbor in one or more',
     &  ' of the' /
     &  '     k, l and m directions.' )

 8121 format (/
     &  '  "mesh test block"' /
     &  '     See if any mesh indices in the mesh block have not been',
     &  ' assigned' /
     &  '     to mesh points, and if any line, surface or volume',
     &  ' elements' /
     &  '     of the mesh block violate certain geometric',
     &  ' criteria.' //
     &  '  Synonyms:  [block, bl], [cluster, cl],',
     &  ' [delete, del, remove, rm],' /
     &  '    [insert, ins], [invert, inv],',
     &  ' [move, mv, m, translate, trans],' /
     &  '    [point, pnt, pt, p].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8115) ans
        endif
        write (   6, 8114)
        write (iout, 8114)
        write (   6, 8116)
        write (iout, 8116)
        write (   6, 8118)
        write (iout, 8118)
        write (   6, 8119)
        write (iout, 8119)
        if (ainput .eq. 'tty') then
          read  (   5, 8115) ans
        endif
        write (   6, 8120)
        write (iout, 8120)
        write (   6, 8121)
        write (iout, 8121)
        go to 999
      endif

c=======================================================================********

c.... Input line:  mesh size
c.... Input line:  mesh size [=] KMAX LMAX MMAX

      if (aword(2) .eq. 'size') then

        if (nword .eq. 2) then

 8122   format (
     &    'Mesh size:   kmax      =',i6,'   lmax      =',i6,
     &    '   mmax      =',i6 /
     &    '             kmax*lmax =',i6,'   lmax*mmax =',i6,
     &    '   mmax*kmax =',i6 /
     &    '             kmax*lmax*mmax                =',i6)

          klmax  = kmax * lmax
          lmmax  = lmax * mmax
          mkmax  = mmax * kmax
          klmmax = kmax * lmax * mmax
  
          write (   6, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &      klmmax
          write (iout, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &      klmmax
  
          go to 999

        endif                         ! If nword = 2.  

 8125   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh size [=] KMAX ..." command not ',
     &    i1,' to ',i1,':  ',i2)

        if (aword(3) .eq. '=') then
          nwk = 4
          nwl = 5
          nwm = 6
        else
          nwk = 3
          nwl = 4
          nwm = 5
        endif
cbugc***DEBUG begins.
cbug 7301 format ('MESH nbugg=',i3,'  nwk, nwl, nwm=',3i3 /
cbug     &  'iword(nwk), iword(nwl), iword(nwm) =',3i3)
cbug      nbugg = 1
cbug      write (   6, 7301) nbugg, nwk, nwl, nwm,
cbug     &                   iword(nwk), iword(nwl), iword(nwm)
cbug      write (iout, 7301) nbugg, nwk, nwl, nwm,
cbug     &                   iword(nwk), iword(nwl), iword(nwm)
cbugc***DEBUG ends.

        if ((nword .lt. nwk)  .or.
     &      (nword .gt. nwm)) then
          write (   6, 8125) ainput, ninput, nwk, nwm, nword
          write (iout, 8125) ainput, ninput, nwk, nwm, nword
          go to 999
        endif

        nerr = 0

 8130   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Mesh size is not a positive integer:  ',a)

        do 140 nw = nwk, nwm

          if (nw .gt. nword) then
            mtype(nw) = 2
            iword(nw) = 1
            nword     = nw
          endif

          call varfind (nw, nw)

          if ((mtype(nw) .ne. 2) .or.
     &        (iword(nw) .le. 0)) then
            nerr = 1
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif

  140   continue

        if (nerr .ne. 0) go to 999

c....   See if mesh size has changed.  If not, display old size.
cbugc***DEBUG begins.
cbug      nbugg = 2
cbug      write (   6, 7301) nbugg, nwk, nwl, nwm,
cbug     &                   iword(nwk), iword(nwl), iword(nwm)
cbug      write (iout, 7301) nbugg, nwk, nwl, nwm,
cbug     &                   iword(nwk), iword(nwl), iword(nwm)
cbugc***DEBUG ends.

        if ((kmax .eq. iword(nwk))  .and.
     &      (lmax .eq. iword(nwl))  .and.
     &      (mmax .eq. iword(nwm))) then
          klmax   = iword(nwk) * iword(nwl)
          lmmax   = iword(nwl) * iword(nwm)
          mkmax   = iword(nwm) * iword(nwk)
          klmmax  = iword(nwk) * iword(nwl) * iword(nwm)
          write (   6, 8122) iword(nwk), iword(nwl), iword(nwm),
     &      klmax, lmmax, mkmax, klmmax
          write (iout, 8122) iword(nwk), iword(nwl), iword(nwm),
     &      klmax, lmmax, mkmax, klmmax
          go to 999
        endif

c....   Display old and new sizes.

 8140   format ('  WARNING:  changing old mesh size:' /
     &    'Mesh size:   kmax old  =',i6,'   lmax old  =',i6,
     &    '   mmax old  =',i6 )

        if ((kmax .ne. 0)  .or.
     &      (lmax .ne. 0)  .or.
     &      (mmax .ne. 0)) then
          write (   6, 8140) kmax, lmax, mmax
          write (iout, 8140) kmax, lmax, mmax
        endif

        klmax   = iword(nwk) * iword(nwl)
        lmmax   = iword(nwl) * iword(nwm)
        mkmax   = iword(nwm) * iword(nwk)
        klmmax  = iword(nwk) * iword(nwl) * iword(nwm)
cbugc***DEBUG begins.
cbug      nbugg = 3
cbug      write (   6, 7301) nbugg, nwk, nwl, nwm,
cbug     &                   iword(nwk), iword(nwl), iword(nwm)
cbug      write (iout, 7301) nbugg, nwk, nwl, nwm,
cbug     &                   iword(nwk), iword(nwl), iword(nwm)
cbugc***DEBUG ends.

        write (   6, 8122) iword(nwk), iword(nwl), iword(nwm),
     &    klmax, lmmax, mkmax, klmmax
        write (iout, 8122) iword(nwk), iword(nwl), iword(nwm),
     &    klmax, lmmax, mkmax, klmmax

c....   Reduce the mesh block, if needed, to fit in the new mesh.

        newbl   = 0
        kblold1 = kbl1
        kblold2 = kbl2
        lblold1 = lbl1
        lblold2 = lbl2
        mblold1 = mbl1
        mblold2 = mbl2

        if (kbl1 .gt. iword(nwk)) then
          kbl1 = iword(nwk)
          newbl = 1
        endif
        if (kbl2 .gt. iword(nwk)) then
          kbl2 = iword(nwk)
        endif
        if (lbl1 .gt. iword(nwl)) then
          lbl1 = iword(nwl)
          newbl = 1
        endif
        if (lbl2 .gt. iword(nwl)) then
          lbl2 = iword(nwl)
          newbl = 1
        endif
        if (mbl1 .gt. iword(nwm)) then
          mbl1 = iword(nwm)
          newbl = 1
        endif
        if (mbl2 .gt. iword(nwm)) then
          mbl2 = iword(nwm)
          newbl = 1
        endif

        if (newbl .eq. 1) then        ! Changed mesh block size.
          write (   6, 8144) kblold1, lblold1, mblold1,
     &                       kblold2, lblold2, mblold2
          write (iout, 8144) kblold1, lblold1, mblold1,
     &                       kblold2, lblold2, mblold2
          write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
          write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

c....     Display indices at vertices of mesh block.

          write (   6, 8146) kbl1, lbl1, mbl1
          write (iout, 8146) kbl1, lbl1, mbl1

          if (kbl2 .gt. kbl1) then
            write (   6, 8146) kbl2, lbl1, mbl1
            write (iout, 8146) kbl2, lbl1, mbl1
          endif                       ! kbl2 > kbl1.

          if (lbl2 .gt. lbl1) then

            write (   6, 8146) kbl1, lbl2, mbl1
            write (iout, 8146) kbl1, lbl2, mbl1

            if (kbl2 .gt. kbl1) then
              write (   6, 8146) kbl2, lbl2, mbl1
              write (iout, 8146) kbl2, lbl2, mbl1
            endif                     ! kbl2 > kbl1.

          endif                       ! lbl2 > lbl1.

          if (mbl2 .gt. mbl1) then

            write (   6, 8146) kbl1, lbl1, mbl2
            write (iout, 8146) kbl1, lbl1, mbl2

            if (kbl2 .gt. kbl1) then
              write (   6, 8146) kbl2, lbl1, mbl2
              write (iout, 8146) kbl2, lbl1, mbl2
            endif                     ! kbl2 > kbl1.

            if (lbl2 .gt. lbl1) then
  
              write (   6, 8146) kbl1, lbl2, mbl2
              write (iout, 8146) kbl1, lbl2, mbl2

              if (kbl2 .gt. kbl1) then
                write (   6, 8146) kbl2, lbl2, mbl2
                write (iout, 8146) kbl2, lbl2, mbl2
              endif                   ! kbl2 > kbl1.

            endif                     ! lbl2 > lbl1.

          endif                       ! mbl2 > mbl1.

        endif                         ! Changed mesh block size.

c....   Release mesh points with any index larger than new kmax, lmax or mmax.

        krel = iword(nwk) + 1
        lrel = iword(nwl) + 1
        mrel = iword(nwm) + 1

        nprel    = 0
        aword(3) = ' '

        if (krel .le. kmax) then
          call meshrel (krel, kmax, 1, lmax, 1, mmax, npadd)
          if (npadd .lt. 0) go to 999
          nprel = nprel + npadd
        endif

        if (lrel .le. lmax) then
          call meshrel (1, kmax, lrel, lmax, 1, mmax, npadd)
          if (npadd .lt. 0) go to 999
          nprel = nprel + npadd
        endif

        if (mrel .le. mmax) then
          call meshrel (1, kmax, 1, lmax, mrel, mmax, npadd)
          if (npadd .lt. 0) go to 999
          nprel = nprel + npadd
        endif

 8430   format ('  Number of mesh points released =',i7)

        if (nprel .ne. 0) then
          write (   6, 8430) nprel
          write (iout, 8430) nprel
        endif

c....   Store new mesh size.

        kmax = iword(nwk)
        lmax = iword(nwl)
        mmax = iword(nwm)
cbugc***DEBUG begins.
cbug 7302 format ('MESH nbugg=',i3,'  kmax, lmax, mmax=',3i3)
cbug      nbugg = 4
cbug      write (   6, 7301) nbugg, nwk, nwl, nwm,
cbug     &                   iword(nwk), iword(nwl), iword(nwm)
cbug      write (iout, 7301) nbugg, nwk, nwl, nwm,
cbug     &                   iword(nwk), iword(nwl), iword(nwm)
cbug      write (   6, 7302) nbugg, kmax, lmax, mmax
cbug      write (iout, 7302) nbugg, kmax, lmax, mmax
cbugc***DEBUG ends.

        go to 999

      endif                           ! Input line "mesh size ...".

c=======================================================================********

c.... Input line:  mesh block
c.... Input line:  mesh block all
c.... Input line:  mesh block K1 K2  L1 L2  M1 M2

      nerr = 0

      if ((aword(2) .eq. 'block')    .or.
     &    (aword(2) .eq. 'bl'))      then

 8144   format ('  WARNING:  changing old mesh block indices:' /
     &    'Mesh block:  kbl1 old  =',i6,'   lbl1 old  =',i6,
     &    '   mbl1 old  =',i6 /
     &    '             kbl2 old  =',i6,'   lbl2 old  =',i6,
     &    '   mbl2 old  =',i6 )
 8145   format (
     &    'Mesh block:  kbl1      =',i6,'   lbl1      =',i6,
     &    '   mbl1      =',i6 /
     &    '             kbl2      =',i6,'   lbl2      =',i6,
     &    '   mbl2      =',i6 )
 8146   format (
     &    '  Vertex:    k         =',i6,'   l         =',i6,
     &    '   m         =',i6 )

        if (nword .eq. 2) then

c....     Display ranges of indices of mesh block.

          write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
          write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

          if (kbl1 .le. 0) go to 999

c....     Display indices at vertices of mesh block.

          write (   6, 8146) kbl1, lbl1, mbl1
          write (iout, 8146) kbl1, lbl1, mbl1
          call meshfind (kbl1, lbl1, mbl1, np)
          if (np .gt. 0) call pout (np, np, 'minimum')

          if (kbl2 .gt. kbl1) then
            write (   6, 8146) kbl2, lbl1, mbl1
            write (iout, 8146) kbl2, lbl1, mbl1
            call meshfind (kbl2, lbl1, mbl1, np)
            if (np .gt. 0) call pout (np, np, 'minimum')
          endif                       ! kbl2 > kbl1.

          if (lbl2 .gt. lbl1) then

            write (   6, 8146) kbl1, lbl2, mbl1
            write (iout, 8146) kbl1, lbl2, mbl1
            call meshfind (kbl1, lbl2, mbl1, np)
            if (np .gt. 0) call pout (np, np, 'minimum')

            if (kbl2 .gt. kbl1) then
              write (   6, 8146) kbl2, lbl2, mbl1
              write (iout, 8146) kbl2, lbl2, mbl1
              call meshfind (kbl2, lbl2, mbl1, np)
              if (np .gt. 0) call pout (np, np, 'minimum')
            endif                     ! kbl2 > kbl1.

          endif                       ! lbl2 > lbl1.

          if (mbl2 .gt. mbl1) then

            write (   6, 8146) kbl1, lbl1, mbl2
            write (iout, 8146) kbl1, lbl1, mbl2
            call meshfind (kbl1, lbl1, mbl2, np)
            if (np .gt. 0) call pout (np, np, 'minimum')

            if (kbl2 .gt. kbl1) then
              write (   6, 8146) kbl2, lbl1, mbl2
              write (iout, 8146) kbl2, lbl1, mbl2
              call meshfind (kbl2, lbl1, mbl2, np)
              if (np .gt. 0) call pout (np, np, 'minimum')
            endif                     ! kbl2 > kbl1.

            if (lbl2 .gt. lbl1) then
  
              write (   6, 8146) kbl1, lbl2, mbl2
              write (iout, 8146) kbl1, lbl2, mbl2
              call meshfind (kbl1, lbl2, mbl2, np)
              if (np .gt. 0) call pout (np, np, 'minimum')

              if (kbl2 .gt. kbl1) then
                write (   6, 8146) kbl2, lbl2, mbl2
                write (iout, 8146) kbl2, lbl2, mbl2
                call meshfind (kbl2, lbl2, mbl2, np)
                if (np .gt. 0) call pout (np, np, 'minimum')
              endif                   ! kbl2 > kbl1.

            endif                     ! lbl2 > lbl1.

          endif                       ! mbl2 > mbl1.

          go to 999
        endif                         ! nword = 2.

        if (aword(3) .eq. 'all') then

 8150     format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Number of words in "mesh block all" command not',
     &      ' 3:  ',i2)

          if (nword .ne. 3) then
            write (   6, 8150) ainput, ninput, nword
            write (iout, 8150) ainput, ninput, nword
            go to 999
          endif

          if ((kbl1 .ne. 0)  .or.
     &        (kbl2 .ne. 0)  .or.
     &        (lbl1 .ne. 0)  .or.
     &        (lbl2 .ne. 0)  .or.
     &        (mbl1 .ne. 0)  .or.
     &        (mbl2 .ne. 0)) then
            write (   6, 8144) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
            write (iout, 8144) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
          endif

          kbl1 = 1
          kbl2 = kmax
          lbl1 = 1
          lbl2 = lmax
          mbl1 = 1
          mbl2 = mmax

          write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
          write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
          
          go to 999
            
        elseif ((nword .lt. 3)  .or.
     &          (nword .gt. 8)) then

 8190     format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Number of words in "mesh ',a4,' ..." command not',
     &      ' 3 to 8:  ',i2)

          write (   6, 8190) ainput, ninput, aword(2), nword
          write (iout, 8190) ainput, ninput, aword(2), nword
          go to 999
        endif

        if (nword .eq. 3) then
          nword = 4
          mtype(4) = 2
          iword(4) = 1
        endif

        if (nword .eq. 4) then
          nword = 5
          mtype(5) = 2
          iword(5) = 1
        endif

        if (nword .eq. 5) then
          nword = 6
          mtype(6) = 2
          iword(6) = 1
        endif

        if (nword .eq. 6) then
          nword = 7
          mtype(7) = 2
          iword(7) = 1
        endif

        if (nword .eq. 7) then
          nword = 8
          mtype(8) = 2
          iword(8) = 1
        endif

        do 160 nw = 3, 8

          call varfind (nw, nw)

          if ((mtype(nw) .ne. 2) .or.
     &        (iword(nw) .le. 0)) then
            nerr = 1
            write (   6, 8170) ainput, ninput, aword(nw)
            write (iout, 8170) ainput, ninput, aword(nw)
          endif

  160   continue

        call meshchk (iword(3), iword(4),
     &                iword(5), iword(6),
     &                iword(7), iword(8), nerrchk)

        if (nerr    .ne. 0) go to 999
        if (nerrchk .ne. 0) go to 999

        if ((kbl1 .ne. 0)  .or.
     &      (kbl2 .ne. 0)  .or.
     &      (lbl1 .ne. 0)  .or.
     &      (lbl2 .ne. 0)  .or.
     &      (mbl1 .ne. 0)  .or.
     &      (mbl2 .ne. 0)) then
          write (   6, 8144) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
          write (iout, 8144) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        endif

        kbl1 = iword(3)
        kbl2 = iword(4)
        lbl1 = iword(5)
        lbl2 = iword(6)
        mbl1 = iword(7)
        mbl2 = iword(8)

        write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

        go to 999

      endif                           ! Input line "mesh block ...".

c=======================================================================********

c.... Input line:  mesh [all,list]

      if (((aword(2) .eq. 'all')   .or.
     &     (aword(2) .eq. 'list')) .and.
     &     (nword .eq. 2))         then

        klmax  = kmax * lmax
        lmmax  = lmax * mmax
        mkmax  = mmax * kmax
        klmmax = kmax * lmax * mmax

        write (   6, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax, klmmax
        write (iout, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax, klmmax

        if (klmmax .eq. 0) go to 999

        write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

c....   Display all mesh points.

        call meshout (1, kmax, 1, lmax, 1, mmax, npmesh)

 8165   format ('  Number of mesh points =',i7)

        write (   6, 8165) npmesh
        write (iout, 8165) npmesh

        go to 999

      endif                           ! Input line "mesh [all,list]".

c=======================================================================********

c.... Input line:  mesh list [all,void]

 8170 format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh index is not a positive integer:  ',a)

      if ((aword(2) .eq. 'list')   .and.
     &   ((aword(3) .eq. 'all')    .or.
     &    (aword(3) .eq. 'void'))) then

 8180   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh list [all,void]" command',
     &    ' not 3:  ',i2)

        if (nword .gt. 3) then
          write (   6, 8180) ainput, ninput, nword
          write (iout, 8180) ainput, ninput, nword
          go to 999
        endif

        klmax  = kmax * lmax
        lmmax  = lmax * mmax
        mkmax  = mmax * kmax
        klmmax = kmax * lmax * mmax

        write (   6, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &    klmmax
        write (iout, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &    klmmax

        if (klmmax .eq. 0) go to 999

        write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

        if (aword(3) .ne. 'void') then  ! Display all mesh points.

          call meshout (1, kmax, 1, lmax, 1, mmax, npmesh)

 8240     format ('  Number of mesh points listed   =',i7)

          write (   6, 8240) npmesh
          write (iout, 8240) npmesh

        else                          ! Display all unassigned mesh indices.

          call meshvout (1, kmax, 1, lmax, 1, mmax, nvoidt)

          write (   6, 8188) nvoidt
          write (iout, 8188) nvoidt

        endif                         !

        go to 999

      endif                           ! Input line "mesh list all".

c=======================================================================********

c.... Input line:  mesh list block [void]

      if ((aword(2) .eq. 'list')  .and.
     &   ((aword(3) .eq. 'block') .or.
     &    (aword(3) .eq. 'bl')))  then

 8185   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh list block" command',
     &    ' not 3 or 4:  ',i2)

        if (nword .gt. 4) then
          write (   6, 8185) ainput, ninput, nword
          write (iout, 8185) ainput, ninput, nword
          go to 999
        endif

        if ((nword .eq. 4)          .and.
     &      (aword(4) .ne. 'void')) then
            write (   6, 8810) ainput, ninput
            write (iout, 8810) ainput, ninput
            go to 999
        endif

        klmmax = kmax * lmax * mmax

        if (klmmax .eq. 0) go to 999

        write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

        if (nword .eq. 3) then        ! Display mesh points.

          call meshout (kbl1, kbl2, lbl1, lbl2, mbl1, mbl2, npmesh)

          write (   6, 8240) npmesh
          write (iout, 8240) npmesh

        else                          ! Display unassigned mesh indices.

          call meshvout (kbl1, kbl2, lbl1, lbl2, mbl1, mbl2, nvoidt)

 8188     format ('  Number of unassigned mesh indices listed =',i7)

          write (   6, 8188) nvoidt
          write (iout, 8188) nvoidt

        endif

        go to 999

      endif                           ! Input line "mesh list block [void]".

c=======================================================================********

c.... Input line:  mesh list K L M

      if (aword(2) .eq. 'list') then

 8172   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh list K L M" command',
     &    ' not 3 to 5:  ',i2)

        if ((nword .lt. 3) .or. (nword .gt. 5)) then
          write (   6, 8172) ainput, ninput, nword
          write (iout, 8172) ainput, ninput, nword
          go to 999
        endif

        do 174 nw = 4, 5

          if (nw .gt. nword) then
            mtype(nw) = 2
            iword(nw) = 1
            nword     = nw
          endif

          call varfind (nw, nw)

          if ((mtype(nw) .ne. 2) .or.
     &        (iword(nw) .le. 0)) then
            nerr = 1
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif

  174  continue

        if (nerr .ne. 0) go to 999

c....   Display ranges of indices of mesh block.

        write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

c....   Display mesh point.

        call meshout (iword(3), iword(3), iword(4), iword(4),
     &    iword(5), iword(5), npmesh)

        write (   6, 8240) npmesh
        write (iout, 8240) npmesh

        go to 999

      endif                           ! Input line "mesh list K L M".

c=======================================================================********

c.... Input line:  mesh point PMESH

      if (((aword(2) .eq. 'point')  .or.
     &     (aword(2) .eq. 'pnt'  )  .or.
     &     (aword(2) .eq. 'pt'   )  .or.
     &     (aword(2) .eq. 'p'    )) .and.
     &    (nword .eq. 3))           then

        nerr   = 0
        npass  = 0
        npvac  = 0
        npmesh = 0

 8889   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  No mesh has been defined.')

        if (kmax * lmax * mmax .eq. 0) then
          write (   6, 8889) ainput, ninput
          write (iout, 8889) ainput, ninput
          go to 999
        else
          klmax  = kmax * lmax
          lmmax  = lmax * mmax
          mkmax  = mmax * kmax
          klmmax = kmax * lmax * mmax
  
          write (   6, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &      klmmax
          write (iout, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &      klmmax
        endif

c....   See if point name is allowed.

 8888   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Point name begins with "-":  ',a)

        afirst = aword(3)
        if (afirst .eq. '-') then
          write (   6, 8888) ainput, ninput, aword(3)
          write (iout, 8888) ainput, ninput, aword(3), aword(3)
          go to 999
        endif

c....   See if point name is already subscripted.

        call aptsubm (aword(3), 1, lword(3), iamax, idmax, iemax,
     &    '(', ',', ')', 16,
     &          astem, istem, fstem, mstem, lstem,
     &    nsub, asub,  isub,  fsub,  msub,  lsub, nerra)

 8891   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Point name has bad subscripts:  ',a)

        if (nerra .ne. 0) then
          write (   6, 8891) ainput, ninput, aword(3)
          write (iout, 8891) ainput, ninput, aword(3)
          go to 999
        endif

 8892   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Point name does not have 0 or 3 subscripts:  ',a)

        if (nsub .ne. 0) then
          if (nsub .eq. 3) go to 410
          write (   6, 8892) ainput, ninput, aword(3)
          write (iout, 8892) ainput, ninput, aword(3)
          go to 999
        endif

c....   Store the mesh points.

cbugc***DEBUG begins.
cbug 9771 format ('mesh DEBUG.  nbug =',i3,
cbug     &  ' lword(3) =',i5,' aword(3) = ',a)
cbug      nbug = 1
cbug      write (   6, 9771) nbug, lword(3), aword(3)
cbug      write (iout, 9771) nbug, lword(3), aword(3)
cbugc***DEBUG ends.
 8902     format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Name of point not from 1 to 24 characters:  ',a)

        do 170 k = 1, kmax            ! Loop over index k.
          do 168 l = 1, lmax          ! Loop over index l.
            do 166 m = 1, mmax        ! Loop over index m.

c....         Find any point currently assigned to these mesh indices.

              call meshfind (k, l, m, np)

              if (np .ne. 0) go to 166

c....         Generate the subscripted point name.

              isub(1) = k
              isub(2) = l
              isub(3) = m

              call aptsubi (aword(3), lword(3), 3, isub, 24,
     &                      '(', ',', ')', anamex, lanamex, nerra)

              if (nerra .ne. 0) then
                write (   6, 8902) ainput, ninput, aword(2)
                write (iout, 8902) ainput, ninput, aword(2)
                go to 999
              endif
cbugc***DEBUG begins.
cbug 9772 format ('mesh DEBUG.  nbug =',i3,
cbug     &  ' lanamex =',i5,' anamex = ',a)
cbug      nbug = 2
cbug      write (   6, 9772) nbug, lanamex, anamex
cbug      write (iout, 9772) nbug, lanamex, anamex
cbugc***DEBUG ends.

c....         Find the index in the point list for the new point.

              aword(2) = anamex
              lword(2) = lanamex
cbugc***DEBUG begins.
cbug      nbug = 3
cbug      write (   6, 9772) nbug, lanamex, anamex
cbug      write (iout, 9772) nbug, lanamex, anamex
cbugc***DEBUG ends.

              call ptstore (nrep, n)

              if (n .eq. 0) go to 999

              apoint(n)  = anamex
              lapoint(n) = lanamex
cbugc***DEBUG begins.
cbug      nbug = 4
cbug      write (   6, 9771) nbug, lanamex, anamex
cbug      write (iout, 9771) nbug, lanamex, anamex
cbugc***DEBUG ends.

              apoints    = anamex
              lapoints   = lanamex
cbugc***DEBUG begins.
cbug      nbug = 5
cbug      write (   6, 9772) nbug, lanamex, anamex
cbug      write (iout, 9772) nbug, lanamex, anamex
cbugc***DEBUG ends.

              npmesh     = npmesh + 1

              if (nrep .eq. 0) then
                npvac    = npvac + 1
                x(n)     = -1.e99
                y(n)     = -1.e99
                z(n)     = -1.e99
              else
                npass    = npass + 1
              endif

              kpoint(n)  = k
              lpoint(n)  = l
              mpoint(n)  = m

              amount = 'maximum'
              call pout (n, n, amount)

  166       continue                  ! End of loop over index k.
  168     continue                    ! End of loop over index l.
  170   continue                      ! End of loop over index m.

 8161   format ('  Number of mesh points assigned       =',i7)
 8163   format ('  Number of vacuum mesh points created =',i7)
 8164   format ('  Number of existing points assigned   =',i7)

        if (npvac .gt. 0) then
          write (   6, 8163) npvac
          write (iout, 8163) npvac
        endif

        if (npass .gt. 0) then
          write (   6, 8164) npass
          write (iout, 8164) npass
        endif

        if (npmesh .gt. 0) then
          write (   6, 8161) npmesh
          write (iout, 8161) npmesh
        endif

        go to 999

      endif                           ! Input line "mesh point PMESH".

c=======================================================================********

c.... Input line:  mesh point PMESH block

      if (((aword(2) .eq. 'point')   .or.
     &     (aword(2) .eq. 'pnt'  )   .or.
     &     (aword(2) .eq. 'pt'   )   .or.
     &     (aword(2) .eq. 'p'    ))  .and.
     &    ((aword(4) .eq. 'block')   .or.
     &     (aword(4) .eq. 'bl'   ))) then

 8352   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh point PMESH block"',
     &    ' command not 4:  ',i2)

        if (nword .ne. 4) then
          write (   6, 8352) ainput, ninput, nword
          write (iout, 8352) ainput, ninput, nword
          go to 999
        endif

        nerr   = 0
        npass  = 0
        npvac  = 0
        npmesh = 0

        if (kmax * lmax * mmax .eq. 0) then
          write (   6, 8889) ainput, ninput
          write (iout, 8889) ainput, ninput
          go to 999
        else
          klmax  = kmax * lmax
          lmmax  = lmax * mmax
          mkmax  = mmax * kmax
          klmmax = kmax * lmax * mmax
  
          write (   6, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &      klmmax
          write (iout, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &      klmmax
        endif

c....   See if point name is allowed.

        afirst = aword(3)
        if (afirst .eq. '-') then
          write (   6, 8888) ainput, ninput, aword(3)
          write (iout, 8888) ainput, ninput, aword(3), aword(3)
          go to 999
        endif

c....   See if point name is already subscripted.

        call aptsubm (aword(3), 1, lword(3), iamax, idmax, iemax,
     &    '(', ',', ')', 16,
     &          astem, istem, fstem, mstem, lstem,
     &    nsub, asub,  isub,  fsub,  msub,  lsub, nerra)

        if (nerra .ne. 0) then
          write (   6, 8891) ainput, ninput, aword(3)
          write (iout, 8891) ainput, ninput, aword(3)
          go to 999
        endif

        if (nsub .ne. 0) then
          if (nsub .eq. 3) go to 410
          write (   6, 8892) ainput, ninput, aword(3)
          write (iout, 8892) ainput, ninput, aword(3)
          go to 999
        endif

c....   See if a mesh block exists.

 8355   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  No mesh block has been defined.' )

        if ((kbl1 .eq. 0)  .or.
     &      (kbl2 .eq. 0)  .or.
     &      (lbl1 .eq. 0)  .or.
     &      (lbl2 .eq. 0)  .or.
     &      (mbl1 .eq. 0)  .or.
     &      (mbl2 .eq. 0)) then
          nerr = 1
          write (   6, 8355) ainput, ninput
          write (iout, 8355) ainput, ninput
        else
          write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
          write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        endif
 
        if (nerr .ne. 0) go to 999

c....   Store the mesh points.

        do 270 k = kbl1, kbl2         ! Loop over index k.
          do 268 l = lbl1, lbl2       ! Loop over index l.
            do 266 m = mbl1, mbl2     ! Loop over index m.

c....         Find any point currently assigned to these mesh indices.

              call meshfind (k, l, m, np)

              if (np .ne. 0) go to 266

c....         Generate the subscripted point name.

              isub(1) = k
              isub(2) = l
              isub(3) = m

              call aptsubi (aword(3), lword(3), 3, isub, 24,
     &                      '(', ',', ')', anamex, lanamex, nerra)

              if (nerra .ne. 0) then
                write (   6, 8902) ainput, ninput, aword(2)
                write (iout, 8902) ainput, ninput, aword(2)
                go to 999
              endif

c....         Find the index in the point list for the new point.

              aword(2) = anamex
              lword(2) = lanamex

              call ptstore (nrep, n)

              if (n .eq. 0) go to 999

              apoint(n)  = anamex
              lapoint(n) = lanamex

              apoints    = anamex
              lapoints   = lanamex

              npmesh     = npmesh + 1

              if (nrep .eq. 0) then
                npvac    = npvac + 1
                x(n)     = -1.e99
                y(n)     = -1.e99
                z(n)     = -1.e99
              else
                npass    = npass + 1
              endif

              kpoint(n)  = k
              lpoint(n)  = l
              mpoint(n)  = m

              amount = 'maximum'
              call pout (n, n, amount)

  266       continue                  ! End of loop over index k.
  268     continue                    ! End of loop over index l.
  270   continue                      ! End of loop over index m.

        if (npvac .gt. 0) then
          write (   6, 8163) npvac
          write (iout, 8163) npvac
        endif

        if (npass .gt. 0) then
          write (   6, 8164) npass
          write (iout, 8164) npass
        endif

        if (npmesh .gt. 0) then
          write (   6, 8161) npmesh
          write (iout, 8161) npmesh
        endif

        go to 999

      endif                           ! Input line "mesh point PMESH block".

c=======================================================================********

c.... Input line:  mesh point PNAME K L M
c.... Input line:  mesh point PMESH(K,L,M)

c.... Assign mesh indices k, l and m to point PNAME or PMESH(K,L,M).
c....   If PNAME has 3 subscripts, they must be K, L, M.

  410 if (((aword(2) .eq. 'point') .or.
     &     (aword(2) .eq. 'pnt')   .or.
     &     (aword(2) .eq. 'pt')    .or.
     &     (aword(2) .eq. 'p'))    .and.
     &    ((aword(4) .ne. 'block') .and.
     &     (aword(4) .ne. 'bl')))  then
 8260   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh point ..." command not',
     &    ' 3 to 6:  ',i2)

        if ((nword .lt. 3)  .or.
     &      (nword .gt. 6)) then
          write (   6, 8260) ainput, ninput, nword
          write (iout, 8260) ainput, ninput, nword
          go to 999
        endif

        nerr = 0

c....   Find the point.

        call ptfind (3, np)

        if (np .eq. 0) then
          go to 999
        endif

c....   See if point name is allowed.

        afirst = aword(3)
        if (afirst .eq. '-') then
          write (   6, 8888) ainput, ninput, aword(3)
          write (iout, 8888) ainput, ninput, aword(3), aword(3)
          go to 999
        endif

c....   Find the mesh indices of name, new assignment, or old assignment.

        kpt = 0
        lpt = 0
        mpt = 0

c....   See if point name is already subscripted.

        call aptsubm (aword(3), 1, lword(3), iamax, idmax, iemax,
     &    '(', ',', ')', 16,
     &          astem, istem, fstem, mstem, lstem,
     &    nsub, asub,  isub,  fsub,  msub,  lsub, nerra)

        if (nerra .ne.  0) nerr = 1

        if (nsub .eq. 3) then
          kpt = isub(1)
          lpt = isub(2)
          mpt = isub(3)
        endif

 8893   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Point name does not have 3 subscripts:  ',a)

        if ((nword .eq. 3) .and. (nsub .ne. 3)) then
          nerr = 1
          write (   6, 8893) ainput, ninput, aword(3)
          write (iout, 8893) ainput, ninput, aword(3)
        endif

        if (nword .gt. 3) then        ! Input line has >3 words.

          do 310 nw = 4, 6

            if (nw .gt. nword) then
              mtype(nw) = 2
              iword(nw) = 1
              nword     = nw
            endif

            call varfind (nw, nw)

            if ((mtype(nw) .ne. 2) .or.
     &          (iword(nw) .le. 0)) then
              nerr = 1
              write (   6, 8170) ainput, ninput, aword(nw)
              write (iout, 8170) ainput, ninput, aword(nw)
            endif

  310     continue                    ! Tested input words 4, 5 and 6.

c...      See if point name subscripts, if any, match new mesh indices.

 8339     format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Point name subscripts mismatch mesh indices:  ',a24)

          if (((kpt .ne. 0) .and. (kpt .ne. iword(4)))  .or.
     &        ((lpt .ne. 0) .and. (lpt .ne. iword(5)))  .or.
     &        ((mpt .ne. 0) .and. (mpt .ne. iword(6)))) then
            nerr = 1
            write (   6, 8339) ainput, ninput, aword(3)
            write (iout, 8339) ainput, ninput, aword(3)
          endif

          kpt = iword(4)
          lpt = iword(5)
          mpt = iword(6)

        endif                         ! Specifying new mesh indices.

c....   See if point name subscripts, if any,  match new mesh indices, if any.

        if (nsub .eq. 3) then
        if (((kpoint(np) .ne. 0) .and. (kpoint(np) .ne. kpt))  .or.
     &      ((lpoint(np) .ne. 0) .and. (lpoint(np) .ne. lpt))  .or.
     &      ((mpoint(np) .ne. 0) .and. (mpoint(np) .ne. mpt))) then
          nerr = 1
          write (   6, 8339) ainput, ninput, aword(3)
          write (iout, 8339) ainput, ninput, aword(3)
        endif
        endif

        call meshchk1 (kpt, lpt, mpt, nerrchk)

        if (nerr    .ne. 0) go to 999
        if (nerrchk .ne. 0) go to 999

c....   Release any other existing point assigned the same mesh indices.

        call meshrel (kpt, kpt, lpt, lpt, mpt, mpt, nprel)
        if (nprel .lt. 0) go to 999

        if (nprel .gt. 0) then
          write (   6, 8430) nprel
          write (iout, 8430) nprel
        endif

c....   See if the point is already correctly assigned.

        if ((kpoint(np) .eq. kpt)  .and.
     &      (lpoint(np) .eq. lpt)  .and.
     &      (mpoint(np) .eq. mpt)) go to 999

c....   Assign the new mesh indices to the point, and display.

 8340   format ('  mesh point ',a,2x,3i6,7x,'! ASSIGNED')

        kpoint(np) = kpt
        lpoint(np) = lpt
        mpoint(np) = mpt

        amount = 'maximum'
        call pout (np, np, amount)
        go to 999

      endif  ! Input line "mesh point PNAME K L M" or "mesh point PMESH(K,L,M)".

c===============================================================================

c.... Input line:  mesh - all

      nerr = 0

      if ((aword(2) .eq. '-')    .and.
     &    (aword(3) .eq. 'all')) then

 8365   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh - all" command not 3:  ',i2)

        if (nword .gt. 3) then
          write (   6, 8365) ainput, ninput, nword
          write (iout, 8365) ainput, ninput, nword
          go to 999
        endif

c....   Display ranges of indices of mesh.

        klmax  = kmax * lmax
        lmmax  = lmax * mmax
        mkmax  = mmax * kmax
        klmmax = kmax * lmax * mmax
        write (   6, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &    klmmax
        write (iout, 8122) kmax, lmax, mmax, klmax, lmmax, mkmax,
     &    klmmax

c....   Release all mesh points.

        aword(3) = ' '

        call meshrel (1, kmax, 1, lmax, 1, mmax, nprel)
        if (nprel .lt. 0) go to 999

        write (   6, 8430) nprel
        write (iout, 8430) nprel

        go to 999

      endif                           ! Input line "mesh - all".

c===============================================================================

c.... Input line:  mesh - block

      if ((aword(2) .eq. '-')     .and.
     &   ((aword(3) .eq. 'block') .or.
     &    (aword(3) .eq. 'bl')))  then

 8370   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh - block" command not 3:  ',i2)

        if (nword .gt. 3) then
          write (   6, 8370) ainput, ninput, nword
          write (iout, 8370) ainput, ninput, nword
          go to 999
        endif

c....   Display ranges of indices of mesh block.

        write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

c....   Release mesh points in mesh block.

        aword(3) = ' '

        call meshrel (kbl1, kbl2, lbl1, lbl2, mbl1, mbl2, nprel)
        if (nprel .lt. 0) go to 999

        write (   6, 8430) nprel
        write (iout, 8430) nprel

        go to 999

      endif                           ! Input line "mesh - block".

c=======================================================================********

c.... Input line:  mesh - point PNAME

c.... Release point PNAME from the mesh.

      if ((aword(2) .eq. '-')     .and.
     &   ((aword(3) .eq. 'point') .or.
     &    (aword(3) .eq. 'pnt')   .or.
     &    (aword(3) .eq. 'pt')    .or.
     &    (aword(3) .eq. 'p')))   then

 8440   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh - point PNAME" command',
     &    ' not 4:  ',i2)

        if (nword .ne. 4) then
          write (   6, 8440) ainput, ninput, nword
          write (iout, 8440) ainput, ninput, nword
          go to 999
        endif

c....   Find the point.

        call ptfind (4, np)

        if (np .eq. 0) go to 999

c....   See if the point is a mesh point.

 8450   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Point is not a mesh point:  ',a)

        if ((kpoint(np) .le. 0)  .or.
     &      (lpoint(np) .le. 0)  .or.
     &      (mpoint(np) .le. 0)) then
          write (   6, 8450) ainput, ninput, aword(3)
          write (iout, 8450) ainput, ninput, aword(3)
          go to 999
        endif

c....   Release and rename the point.

        call aptchmv ('-', 1, 1, apoint(np),1, nerra)

        nprel      = 1
        kpoint(np) = 0
        lpoint(np) = 0
        mpoint(np) = 0
        klm        = 0

 8460   format ('  mesh point ',a,2x,3i6,7x,'! RENAMED')

        write (   6, 8460) apoint(np), kpoint(np), lpoint(np),
     &    mpoint(np)
        write (iout, 8460) apoint(np), kpoint(np), lpoint(np),
     &    mpoint(np)

        write (   6, 8430) nprel
        write (iout, 8430) nprel
 
        go to 999

      endif                           ! Input line "mesh - point PNAME".

c===============================================================================

c.... Input line:  mesh - cluster CLNAME

      if ((aword(2) .eq. '-')       .and.
     &   ((aword(3) .eq. 'cluster') .or.
     &    (aword(3) .eq. 'cl')))    then

 8470   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh - cluster" command not 4:  ',i2)

        if (nword .ne. 4) then
          write (   6, 8470) ainput, ninput, nword
          write (iout, 8470) ainput, ninput, nword
          go to 999
        endif

        nprel = 0

c....   Find the specified cluster.

        do 530 n = 1, nclusts         ! Loop over clusters.
          if (aword(4) .ne. aclust(n)) go to 530

          call clout (n, n, 'min')

c....     Find all of the points in the specified cluster.

          do 520 npr = 1, npairs      ! Loop over cluster pairs.
            if (apaircl(npr) .ne. aclust(n)) go to 520

            do 510 np = 1, npoints    ! Loop over points.
              if (apoint(np) .ne. apairpt(npr)) go to 510

              if ((kpoint(np) .le. 0)  .or.
     &            (lpoint(np) .le. 0)  .or.
     &            (mpoint(np) .le. 0)) go to 520

              call pout (np, np, 'min')

c....         Release and rename the point.

              call aptchmv ('-', 1, 1, apoint(np),1, nerra)

              nprel      = nprel + 1
              kpoint(np) = 0
              lpoint(np) = 0
              mpoint(np) = 0

              write (   6, 8460) apoint(np), kpoint(np), lpoint(np),
     &          mpoint(np)
              write (iout, 8460) apoint(np), kpoint(np), lpoint(np),
     &          mpoint(np)

              go to 520               ! Find next point in cluster.
  510       continue                  ! End of loop over points.

 8510       format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named cluster point does not exist:  ',a)

            write (   6, 8510) ainput, ninput, apairpt(npr)
            write (iout, 8510) ainput, ninput, apairpt(npr)
            go to 999

  520     continue                    ! End of loop over cluster pairs.

          write (   6, 8430) nprel
          write (iout, 8430) nprel

          go to 999
  530   continue                      ! End of loop over clusters.

 8530   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named cluster does not exist:  ',a)

        write (   6, 8530) ainput, ninput, aword(3)
        write (iout, 8530) ainput, ninput, aword(3)
        go to 999

      endif                           ! Input line "mesh - cluster CLNAME".

c===============================================================================

c.... Input line:  mesh - K L M

      if (aword(2) .eq. '-') then

 8176   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh - K L M" command',
     &    ' not 3 to 5:  ',i2)

        if ((nword .lt. 3) .or. (nword .gt. 5)) then
          write (   6, 8176) ainput, ninput, nword
          write (iout, 8176) ainput, ninput, nword
          go to 999
        endif

        do 176 nw = 4, 5

          if (nw .gt. nword) then
            mtype(nw) = 2
            iword(nw) = 1
            nword     = nw
          endif

          call varfind (nw, nw)

          if ((mtype(nw) .ne. 2) .or.
     &        (iword(nw) .le. 0)) then
            nerr = 1
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif

  176  continue

        if (nerr .ne. 0) go to 999

        call meshrel (iword(3), iword(3), iword(4), iword(4),
     &    iword(5), iword(5), nprel)
        if (nprel .lt. 0) go to 999

        write (   6, 8430) nprel
        write (iout, 8430) nprel

        go to 999

      endif                           ! Input line "mesh - K L M".

c=======================================================================********

c.... Input line:  mesh exchange block [k,l,m] [l,m,k]

      if (((aword(2) .eq. 'exchange') .or.
     &     (aword(2) .eq. 'exch'))    .and.
     &    ((aword(3) .eq. 'block')    .or.
     &     (aword(3) .eq. 'bl')))     then

        nerr = 0

 8630   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh exchange ..." command not',
     &    ' 5:  ',i2)

        if (nword .ne. 5) then
          nerr = 1
          write (   6, 8630) ainput, ninput, nword
          write (iout, 8630) ainput, ninput, nword
        endif

 8632   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  First mesh layer type is not k, l or m:  ',a)

        if ((aword(4) .ne. 'k')  .and.
     &      (aword(4) .ne. 'l')  .and.
     &      (aword(4) .ne. 'm')) then
          nerr = 1
          write (   6, 8632) ainput, ninput, aword(4)
          write (iout, 8632) ainput, ninput, aword(4)
        endif

 8634   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Second mesh layer type is not k, l or m:  ',a)

        if ((aword(5) .ne. 'k')  .and.
     &      (aword(5) .ne. 'l')  .and.
     &      (aword(5) .ne. 'm')) then
          nerr = 1
          write (   6, 8634) ainput, ninput, aword(5)
          write (iout, 8634) ainput, ninput, aword(5)
        endif

 8636   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  First and second layer types are the same:  ',a)

        if (aword(4) .eq. aword(5)) then
          nerr = 1
          write (   6, 8636) ainput, ninput, aword(5)
          write (iout, 8636) ainput, ninput, aword(5)
        endif

c....   See if a mesh block exists.

        if ((kbl1 .eq. 0)  .or.
     &      (kbl2 .eq. 0)  .or.
     &      (lbl1 .eq. 0)  .or.
     &      (lbl2 .eq. 0)  .or.
     &      (mbl1 .eq. 0)  .or.
     &      (mbl2 .eq. 0)) then
          nerr = 1
          write (   6, 8355) ainput, ninput
          write (iout, 8355) ainput, ninput
        endif

        write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
 
        if (nerr .ne. 0) go to 999

        call meshexch (aword(4), aword(5))

        go to 999

      endif                           ! Input line "mesh exchange ...".

c===============================================================================

c.... Input line:  mesh invert block [k,l,m]

      if (((aword(2) .eq. 'invert') .or.
     &     (aword(2) .eq. 'inv'))   .and.
     &    ((aword(3) .eq. 'block')    .or.
     &     (aword(3) .eq. 'bl')))     then

        nerr = 0

 8615   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh invert ..." command not',
     &    ' 4:  ',i2)

        if (nword .ne. 4) then
          nerr = 1
          write (   6, 8615) ainput, ninput, nword
          write (iout, 8615) ainput, ninput, nword
        endif

 8616   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Mesh layer type is not k, l or m:  ',a)

        if ((aword(4) .ne. 'k')  .and.
     &      (aword(4) .ne. 'l')  .and.
     &      (aword(4) .ne. 'm')) then
          nerr = 1
          write (   6, 8616) ainput, ninput, aword(4)
          write (iout, 8616) ainput, ninput, aword(4)
        endif

c....   See if a mesh block exists.

        if ((kbl1 .eq. 0)  .or.
     &      (kbl2 .eq. 0)  .or.
     &      (lbl1 .eq. 0)  .or.
     &      (lbl2 .eq. 0)  .or.
     &      (mbl1 .eq. 0)  .or.
     &      (mbl2 .eq. 0)) then
          nerr = 1
          write (   6, 8355) ainput, ninput
          write (iout, 8355) ainput, ninput
        endif

        write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
 
        if (nerr .ne. 0) go to 999

        call meshinv (aword(4))

        go to 999

      endif                           ! Input line "mesh invert ...".

c===============================================================================

c.... Input line:  mesh move block KMOVE LMOVE MMOVE

      if (((aword(2) .eq. 'move')      .or.
     &     (aword(2) .eq. 'mv')        .or.
     &     (aword(2) .eq. 'm')         .or.
     &     (aword(2) .eq. 'translate') .or.
     &     (aword(2) .eq. 'trans'))    .and.
     &    ((aword(3) .eq. 'block')     .or.
     &     (aword(3) .eq. 'bl')))      then

        nerr = 0

 8540   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh move ..." command not',
     &    ' 6:  ',i2)

        if (nword .ne. 6) then
          nerr = 1
          write (   6, 8540) ainput, ninput, nword
          write (iout, 8540) ainput, ninput, nword
        endif

 8550   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Mesh index increment is not an integer:  ',a)

        do 540 nw = 4, 6

          call varfind (nw, nw)

          if (mtype(nw) .ne. 2) then
            nerr = 1
            write (   6, 8550) ainput, ninput, aword(nw)
            write (iout, 8550) ainput, ninput, aword(nw)
          endif

  540   continue

c....   See if a mesh block exists.

        if ((kbl1 .eq. 0)  .or.
     &      (kbl2 .eq. 0)  .or.
     &      (lbl1 .eq. 0)  .or.
     &      (lbl2 .eq. 0)  .or.
     &      (mbl1 .eq. 0)  .or.
     &      (mbl2 .eq. 0)) then
          nerr = 1
          write (   6, 8355) ainput, ninput
          write (iout, 8355) ainput, ninput
        endif

        write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

 8555   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Mesh index increments are all zero.')

        if ((iword(4) .eq. 0)  .and.
     &      (iword(5) .eq. 0)  .and.
     &      (iword(6) .eq. 0)) then
          nerr = 1
          write (   6, 8555) ainput, ninput
          write (iout, 8555) ainput, ninput
        endif
 
        if (nerr .ne. 0) go to 999

        call meshmove (iword(4), iword(5), iword(6))

        go to 999

      endif                           ! Input line "mesh move ...".

c===============================================================================

c.... Input line:  mesh linear block

      if ((aword(2) .eq. 'linear') .and.
     &   ((aword(3) .eq. 'block')  .or.
     &    (aword(3) .eq. 'bl')))   then

        nerr = 0

 8558   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh linear block" command not',
     &    ' 3:  ',i2)

        if (nword .ne. 3) then
          nerr = 1
          write (   6, 8558) ainput, ninput, nword
          write (iout, 8558) ainput, ninput, nword
        endif

        if ((kbl1 .eq. 0)  .or.
     &      (kbl2 .eq. 0)  .or.
     &      (lbl1 .eq. 0)  .or.
     &      (lbl2 .eq. 0)  .or.
     &      (mbl1 .eq. 0)  .or.
     &      (mbl2 .eq. 0)) then
          nerr = 1
          write (   6, 8355) ainput, ninput
          write (iout, 8355) ainput, ninput
        endif

        if (nerr .ne. 0) go to 999

        call meshline

        go to 999

      endif                           ! Input line "mesh linear block".

c===============================================================================

c.... Input line:  mesh relax block

      if ((aword(2) .eq. 'relax')  .and.
     &   ((aword(3) .eq. 'block')  .or.
     &    (aword(3) .eq. 'bl')))   then

 8562   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh relax block" command not',
     &    ' 3:  ',i2)

        if (nword .ne. 3) then
          write (   6, 8562) ainput, ninput, nword
          write (iout, 8562) ainput, ninput, nword
          go to 999
        endif

        call meshchk (kbl1, kbl2, lbl1, lbl2, mbl1, mbl2, nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8355) ainput, ninput
          write (iout, 8355) ainput, ninput
          go to 999
        endif

        call meshrlxb

        go to 999

      endif                           ! Input line "mesh relax block".

c===============================================================================

c.... Input line:  mesh relax - block


      if ((aword(2) .eq. 'relax')  .and.
     &    (aword(3) .eq. '-')      .and.
     &   ((aword(4) .eq. 'block')  .or.
     &    (aword(4) .eq. 'bl')))   then

 8563   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh relax - block" command not',
     &    ' 4:  ',i2)

        if (nword .ne. 4) then
          write (   6, 8563) ainput, ninput, nword
          write (iout, 8563) ainput, ninput, nword
          go to 999
        endif

        call meshchk (kbl1, kbl2, lbl1, lbl2, mbl1, mbl2, nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8355) ainput, ninput
          write (iout, 8355) ainput, ninput
          go to 999
        endif

        call meshrlxm

        go to 999

      endif                           ! Input line "mesh relax - block".

c===============================================================================

c.... Input line:  mesh test block

      if ((aword(2) .eq. 'test')  .and.
     &   ((aword(3) .eq. 'block') .or.
     &    (aword(3) .eq. 'bl')))  then

 8564   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh check block" command not',
     &    ' 3:  ',i2)

        if (nword .ne. 3) then
          write (   6, 8564) ainput, ninput, nword
          write (iout, 8564) ainput, ninput, nword
          go to 999
        endif

        call meshchk (kbl1, kbl2, lbl1, lbl2, mbl1, mbl2, nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8355) ainput, ninput
          write (iout, 8355) ainput, ninput
          go to 999
        endif

        call meshtest

        go to 999

      endif

c===============================================================================

c.... Input line:  mesh insert [k,l,m] N1 [N2|N1]

      if ((aword(2) .eq. 'insert') .or.
     &    (aword(2) .eq. 'ins'))   then

        nerr = 0

        if (nword .eq. 4) then
          nword    = 5
          mtype(5) = 2
          aword(5) = aword(4)
          iword(5) = iword(4)
        endif

 8620   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh insert ..." command not',
     &    ' 4 or 5:  ',i2)

        if (nword .ne. 5) then
          nerr = 1
          write (   6, 8620) ainput, ninput, nword
          write (iout, 8620) ainput, ninput, nword
        endif

        if ((aword(3) .ne. 'k')  .and.
     &      (aword(3) .ne. 'l')  .and.
     &      (aword(3) .ne. 'm')) then
          nerr = 1
          write (   6, 8616) ainput, ninput, aword(3)
          write (iout, 8616) ainput, ninput, aword(3)
        endif

        if (nerr .ne. 0) go to 999

        call meshins (aword(3), iword(4), iword(5))

        go to 999

      endif                           ! Input line "mesh insert ...".

c===============================================================================

c.... Input line:  mesh delete [k,l,m] N1 [N2|N1]

      if ((aword(2) .eq. 'delete') .or.
     &    (aword(2) .eq. 'del')    .or.
     &    (aword(2) .eq. 'remove') .or.
     &    (aword(2) .eq. 'rm'))    then

        nerr = 0

        if (nword .eq. 4) then
          nword    = 5
          mtype(5) = 2
          aword(5) = aword(4)
          iword(5) = iword(4)
        endif

 8610   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words in "mesh delete ..." command not',
     &    ' 4 or 5:  ',i2)

        if (nword .ne. 5) then
          nerr = 1
          write (   6, 8610) ainput, ninput, nword
          write (iout, 8610) ainput, ninput, nword
        endif

        if ((aword(3) .ne. 'k')  .and.
     &      (aword(3) .ne. 'l')  .and.
     &      (aword(3) .ne. 'm')) then
          nerr = 1
          write (   6, 8616) ainput, ninput, aword(3)
          write (iout, 8616) ainput, ninput, aword(3)
        endif

        if (nerr .ne. 0) go to 999

        call meshdel (aword(3), iword(4), iword(5))

        go to 999

      endif                           ! Input line "mesh delete ...".

c===============================================================================

 8810 format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine mesh.         (+1 line)
      end
