      subroutine meshchk (k1, k2, l1, l2, m1, m2, nerrchk)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshchk  Updated     2005 March 9 14:40.
c               meshchk  Originated  1995 July 21 11:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To check the validity of mesh indices k from k1 to k2,
c               l from l1 to l2, and m from m1 to m2.
c
c     Logic:    Entry points:  meshchk.
c meshchk       Called by:  cluster, mesh, meshmove, meshout, meshrel, meshvout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     k1, k2    Input    Minimum and maximum k indices.
c
c     l1, l2    Input    Minimum and maximum l indices.
c
c     m1, m2    Input    Minimum and maximum m indices.
c
c     nerrchk   Output   Error indicator, 1 if input errors were found.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

c.... Initialize.

      nerrchk    = 0

c.... Test input for errors.

 8110 format ('  meshchk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh index K1 =',i6,' is <  1 or > K2 =',i6,'.')
 8210 format ('  meshchk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh index L1 =',i6,' is <  1 or > L2 =',i6,'.')
 8130 format ('  meshchk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh index M1 =',i6,' is <  1 or > M2 =',i6,'.')
 8140 format ('  meshchk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh index K2 =',i6,' is < K1 =',i6,' or > KMAX =',i6,'.')
 8150 format ('  meshchk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh index L2 =',i6,' is < L1 =',i6,' or > LMAX =',i6,'.')
 8160 format ('  meshchk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh index M2 =',i6,' is < M1 =',i6,' or > MMAX =',i6,'.')

      if ((k1 .le. 0)   .or.
     &    (k1 .gt. k2)) then
        write (   6, 8110) ainput, ninput, k1, k2
        write (iout, 8110) ainput, ninput, k1, k2
        nerrchk = 1
      endif

      if ((l1 .le. 0)   .or.
     &    (l1 .gt. l2)) then
        write (   6, 8210) ainput, ninput, l1, l2
        write (iout, 8210) ainput, ninput, l1, l2
        nerrchk = 2
      endif

      if ((m1 .le. 0)   .or.
     &    (m1 .gt. m2)) then
        write (   6, 8130) ainput, ninput, m1, m2
        write (iout, 8130) ainput, ninput, m1, m2
        nerrchk = 3
      endif

      if ((k2 .lt. k1)     .or.
     &    (k2 .gt. kmax))  then
        write (   6, 8140) ainput, ninput, k2, k1, kmax
        write (iout, 8140) ainput, ninput, k2, k1, kmax
        nerrchk = 4
      endif

      if ((l2 .lt. l1)     .or.
     &    (l2 .gt. lmax))  then
        write (   6, 8150) ainput, ninput, l2, l1, lmax
        write (iout, 8150) ainput, ninput, l2, l1, lmax
        nerrchk = 5
      endif

      if ((m2 .lt. m1)     .or.
     &    (m2 .gt. mmax))  then
        write (   6, 8160) ainput, ninput, m2, m1, mmax
        write (iout, 8160) ainput, ninput, m2, m1, mmax
        nerrchk = 6
      endif

      return

c.... End of subroutine meshchk.      (+1 line)
      end
