      subroutine meshins (alayer, n1, n2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshins  Updated     2005 March 21 15:45.
c               meshins  Originated  1995 August 2 12:50.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To insert into the mesh layers of index type "alayer"
c               ("k", "l" or "m") with index values from n1 to n2, preceding
c               the current layer of index type "alayer" with index n1.
c               All mesh layers of index type "alayer" with initial indices
c               of n1 or higher will have their index increased by the number
c               of layers inserted, and the mesh size will be adjusted
c               accordingly.
c               The index range of any mesh block may be changed.
c
c     Logic:    Entry points:  meshins.
c meshins       Called by:  mesh.
c
c meshins       Calls:  meshsub, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     alayer    Input    Type of layer to insert into mesh, "k", "l" or "m".
c
c     n1        Input    Index of first layer of type "alayer" to insert,
c                          preceding the current layer with index n1.
c
c     n2        Input    Index of last layer of type "alayer" to insert,
c                          preceding the current layer with index n1.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input variables.

      character*1       alayer        ! Type of layer inserted, "k", "l" or "m".

c.... Local variables.

      character*12      adone         ! "REASSIGNED".

c.... Initialize.

      npinc  = 0
      nerr   = 0
      nins   = n2 - n1 + 1

 8110 format ('  meshins ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh index type is not k, l or m:  ',a)

      nmax = 0
      if     (alayer .eq. 'k') then
        nmax = kmax + nins
      elseif (alayer .eq. 'l') then
        nmax = lmax + nins
      elseif (alayer .eq. 'm') then
        nmax = mmax + nins
      else
        nerr  = 1
        write (   6, 8110) ainput, ninput, alayer
        write (iout, 8110) ainput, ninput, alayer
      endif


 8120 format ('  meshins ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh ',a1,' index N1 =',i6,' is <  1 or > N2 =',i6,'.')
 8130 format ('  meshins ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Mesh ',a1,' index N2 =',i6,' is < N1 =',i6,
     &  ' or > NMAX =',i6,'.')

      if ((n1 .le. 0)   .or.
     &    (n1 .gt. n2)) then
        nerr = 2
        write (   6, 8120) ainput, ninput, alayer, n1, n2
        write (iout, 8120) ainput, ninput, alayer, n1, n2
      endif

      if ((n2 .lt. n1)     .or.
     &    (n2 .gt. nmax))  then
        nerr = 3
        write (   6, 8130) ainput, ninput, alayer, n2, n1, nmax
        write (iout, 8130) ainput, ninput, alayer, n2, n1, nmax
      endif

      if (nerr .ne. 0) go to 999

      newbl = 0                       ! Change to 1 if mesh block redefined.

c.... Save the old mesh size.

      kmaxold = kmax
      lmaxold = lmax
      mmaxold = mmax

      kbl1old = kbl1
      kbl2old = kbl2
      lbl1old = lbl1
      lbl2old = lbl2
      mbl1old = mbl1
      mbl2old = mbl2

c.... Increase the indices of all points with "alayer" indices from n1 to nmax.

 8150 format ('  mesh point ',a,2x,3i6,7x,'! ',a)

      if (alayer .eq. 'k') then

        kmax = kmax + nins

        if (n1 .le. kbl2) newbl = 1

        if (n1 .le. kbl1) kbl1 = kbl1 + nins
        if (n1 .le. kbl2) kbl2 = kbl2 + nins

        do 160 np = 1, npoints    ! Loop over points.
          if (apoint(np) .eq. ' ') go to 160
          if (kpoint(np) .lt. n1)  go to 160

          call pout (np, np, 'min')

          npinc      = npinc + 1
          kpoint(np) = kpoint(np) + nins

          call meshsub (np, nrep, nerr)

 8160     format ('  meshins ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Can not make point subscripts match mesh indices.')

          if (nerr .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

          adone = "REASSIGNED"

          write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np), adone
          write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np), adone

  160   continue                  ! End of loop over points.

      elseif (alayer .eq. 'l') then

        lmax = lmax + nins

        if (n1 .le. lbl2) newbl = 1

        if (n1 .le. lbl1) lbl1 = lbl1 + nins
        if (n1 .le. lbl2) lbl2 = lbl2 + nins

        do 170 np = 1, npoints    ! Loop over points.
          if (apoint(np) .eq. ' ') go to 170
          if (lpoint(np) .lt. n1)  go to 170

          call pout (np, np, 'min')

          npinc      = npinc + 1
          lpoint(np) = lpoint(np) + nins

          call meshsub (np, nrep, nerr)

          if (nerr .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

          adone = "REASSIGNED"

          write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np), adone
          write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np), adone

  170   continue                  ! End of loop over points.

      elseif (alayer .eq. 'm') then

        mmax = mmax + nins

        if (n1 .le. mbl2) newbl = 1

        if (n1 .le. mbl1) mbl1 = mbl1 + nins
        if (n1 .le. mbl2) mbl2 = mbl2 + nins

        do 180 np = 1, npoints    ! Loop over points.
          if (apoint(np) .eq. ' ') go to 180
          if (mpoint(np) .lt. n1)  go to 180

          call pout (np, np, 'min')

          npinc      = npinc + 1
          mpoint(np) = mpoint(np) + nins

          call meshsub (np, nrep, nerr)

          if (nerr .ne. 0) then
            write (   6, 8160) ainput, ninput
            write (iout, 8160) ainput, ninput
            go to 999
          endif

          adone = "REASSIGNED"

          write (   6, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np), adone
          write (iout, 8150) apoint(np), kpoint(np), lpoint(np),
     &      mpoint(np), adone

  180   continue                  ! End of loop over points.

      endif

c.... Display the number of points with mesh indices increased.

 8210 format ('  Number of mesh points with ',a1,
     &  ' indices increased =',i7 )

      if (npinc .gt. 0) then
        write (   6, 8210) alayer, npinc
        write (iout, 8210) alayer, npinc
      endif

c.... Display the old and new mesh sizes.

 8140 format ('  WARNING:  changing old mesh size.' /
     &  'Mesh size:   kmax old  =',i6,'   lmax old  =',i6,
     &  '   mmax old  =',i6 )
 8220 format (
     &  'Mesh size:   kmax      =',i6,'   lmax      =',i6,
     &  '   mmax      =',i6 /
     &  '             kmax*lmax =',i6,'   lmax*mmax =',i6,
     &  '   mmax*kmax =',i6 /
     &  '             kmax*lmax*mmax                =',i6)

      write (   6, 8140) kmaxold, lmaxold, mmaxold
      write (iout, 8140) kmaxold, lmaxold, mmaxold

      klmax   = kmax * lmax
      lmmax   = lmax * mmax
      mkmax   = mmax * lmax
      klmmax  = kmax * lmax * mmax

      write (   6, 8220) kmax, lmax, mmax,
     &  klmax, lmmax, mkmax, klmmax
      write (iout, 8220) kmax, lmax, mmax,
     &  klmax, lmmax, mkmax, klmmax

c.... Display the new mesh block, if it was changed.

 8230 format ('  WARNING:  changing old mesh block indices:' /
     &  'Mesh block:  kbl1 old  =',i6,'   lbl1 old  =',i6,
     &  '   mbl1 old  =',i6 /
     &  '             kbl2 old  =',i6,'   lbl2 old  =',i6,
     &  '   mbl2 old  =',i6 )

 8240 format (
     &  'Mesh block:  kbl1      =',i6,'   lbl1      =',i6,
     &  '   mbl1      =',i6 /
     &  '             kbl2      =',i6,'   lbl2      =',i6,
     &  '   mbl2      =',i6 )

      if (newbl .eq. 1) then
        write (   6, 8230) kbl1old, lbl1old, mbl1old,
     &                     kbl2old, lbl2old, mbl2old
        write (iout, 8230) kbl1old, lbl1old, mbl1old,
     &                     kbl2old, lbl2old, mbl2old
        write (   6, 8240) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        write (iout, 8240) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
      endif

  999 return

c.... End of subroutine meshins.      (+1 line)
      end
