      subroutine meshmove (kt, lt, mt)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshmove Updated     2005 March 18 18:20.
c               meshmove Originated  1995 September 5 15:50.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To move the mesh block and all mesh points in the mesh block,
c               by kt, lt and mt units in the k, l and m directions,
c               respectively, after releasing any mesh points not in the
c               mesh block, that are in the new mesh block location.
c
c     Logic:    Entry points:  meshmove.
c meshmove      Called by:  mesh.
c
c meshmove      Calls:  aptchmv, meshchk, meshsub, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     aword(3)  store.h  Name of point being assigned indices.
c
c     kt        Input    Number of k units to move the mesh block.
c
c     lt        Input    Number of l units to move the mesh block.
c
c     mt        Input    Number of m units to move the mesh block.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

c.... Initialize.

      npmove = 0
      nprel  = 0

c.... See if the new mesh block location is completely in the mesh.

      k1 = kbl1 + kt
      k2 = kbl2 + kt
      l1 = lbl1 + lt
      l2 = lbl2 + lt
      m1 = mbl1 + mt
      m2 = mbl2 + mt

      call meshchk (k1, k2, l1, l2, m1, m2, nerrchk)

      if (nerrchk .ne. 0) then
        npmove = -1
        go to 999
      endif

c.... Release all mesh points that are in the new mesh block location,
c....   but not in the original mesh block.

 8210 format ('  mesh point ',a,2x,3i6,7x,'! RENAMED')

      do 210 np = 1, npoints          ! Loop over points.
        if (apoint(np) .eq. ' ')      go to 210

        if ((kpoint(np) .lt. k1) .or.
     &      (kpoint(np) .gt. k2)) go to 210
        if ((lpoint(np) .lt. l1) .or.
     &      (lpoint(np) .gt. l2)) go to 210
        if ((mpoint(np) .lt. m1) .or.
     &      (mpoint(np) .gt. m2)) go to 210

        if ((kpoint(np) .ge. kbl1)  .and.
     &      (kpoint(np) .le. kbl2)  .and.
     &      (lpoint(np) .ge. lbl1)  .and.
     &      (lpoint(np) .le. lbl2)  .and.
     &      (mpoint(np) .ge. mbl1)  .and.
     &      (mpoint(np) .le. mbl2)) go to 210

        call pout (np, np, 'min')

        call aptchmv ('-', 1, 1, apoint(np),1, nerra)

        nprel      = nprel + 1
        kpoint(np) = 0
        lpoint(np) = 0
        mpoint(np) = 0

        write (   6, 8210) apoint(np), kpoint(np), lpoint(np),
     &    mpoint(np)
        write (iout, 8210) apoint(np), kpoint(np), lpoint(np),
     &    mpoint(np)

  210 continue                        ! End of loop over points.

 8215 format ('  Number of mesh points released =',i7)

      if (nprel .gt. 0) then
        write (   6, 8215) nprel
        write (iout, 8215) nprel
      endif

c.... Move all mesh points in the original mesh block location.

 8220 format ('  mesh point ',a,2x,3i6,7x,'! MOVED')

 8160 format ('  meshdel ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not make point subscripts match mesh indices.')

      do 220 np = 1, npoints          ! Loop over points.
        if (apoint(np) .eq. ' ')      go to 220

        if ((kpoint(np) .lt. kbl1) .or.
     &      (kpoint(np) .gt. kbl2)) go to 220
        if ((lpoint(np) .lt. lbl1) .or.
     &      (lpoint(np) .gt. lbl2)) go to 220
        if ((mpoint(np) .lt. mbl1) .or.
     &      (mpoint(np) .gt. mbl2)) go to 220

        call pout (np, np, 'min')

        npmove     = npmove + 1
        kpoint(np) = kpoint(np) + kt
        lpoint(np) = lpoint(np) + lt
        mpoint(np) = mpoint(np) + mt

        call meshsub (np, nrep, nerr)

        if (nerr .ne. 0) then
          write (   6, 8160) ainput, ninput
          write (iout, 8160) ainput, ninput
          go to 999
        endif

        write (   6, 8220) apoint(np), kpoint(np), lpoint(np),
     &    mpoint(np)
        write (iout, 8220) apoint(np), kpoint(np), lpoint(np),
     &    mpoint(np)

  220 continue                        ! End of loop over points.

 8225 format ('  Number of mesh points moved =',i7)

      if (npmove .gt. 0) then
        write (   6, 8225) npmove
        write (iout, 8225) npmove
      endif

 8144 format ('  WARNING:  changing old mesh block indices:' /
     &  'Mesh block:  kbl1 old  =',i6,'   lbl1 old  =',i6,
     &  '   mbl1 old  =',i6 /
     &  '             kbl2 old  =',i6,'   lbl2 old  =',i6,
     &  '   mbl2 old  =',i6 )
 8145 format (
     &  'Mesh block:  kbl1      =',i6,'   lbl1      =',i6,
     &  '   mbl1      =',i6 /
     &  '             kbl2      =',i6,'   lbl2      =',i6,
     &  '   mbl2      =',i6 )

      write (   6, 8144) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
      write (iout, 8144) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

      kbl1 = k1
      kbl2 = k2
      lbl1 = l1
      lbl2 = l2
      mbl1 = m1
      mbl2 = m2
          
      write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
      write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2

  999 return

c.... End of subroutine meshmove.     (+1 line)
      end
