      subroutine meshrel (k1, k2, l1, l2, m1, m2, nprel)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshrel  Updated     2005 April 5 17:45.
c               meshrel  Originated  1995 July 21 11:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To release and rename all mesh points, except aword(3), with
c               mesh indices k from k1 to k2, l from l1 to l2, and
c               m from m1 to m2, and count the number of such points.
c
c     Logic:    Entry points:  meshrel.
c meshrel       Called by:  mesh, point.
c
c meshrel       Calls:  aptchmv, meshchk, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     aword(3)  store.h  Name of point being assigned indices.
c
c     k1, k2    Input    Minimum and maximum k indices.
c                          1 <= k1 <= k2 <= kmax.
c
c     l1, l2    Input    Minimum and maximum l indices.
c                          1 <= l1 <= l2 <= lmax.
c
c     m1, m2    Input    Minimum and maximum m indices.
c                          1 <= m1 <= m2 <= mmax.
c
c     nprel    Output   Number of mesh points released.
c                          -1 if input indices are bad.
c                          -2 if can not rename released points.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*24 anamex

c.... Initialize.

      nprel  = 0

      call meshchk (k1, k2, l1, l2, m1, m2, nerrchk)

      if (nerrchk .ne. 0) then
        nprel = -1
        go to 999
      endif

c.... Release all mesh points except aword(3) with the specified indices.
c....   Rename released points by replacing first character with "-".

 8210 format ('  mesh point ',a,2x,3i6,7x,'! RENAMED')
 8205   format ('  meshrel ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Can not duplicate point name:  ',a)

      do 210 np = 1, npoints          ! Loop over points.
        if (apoint(np) .eq. aword(3)) go to 210
        if (apoint(np) .eq. ' ')      go to 210


        if ((kpoint(np) .lt. k1) .or. (kpoint(np) .gt. k2)) go to 210
        if ((lpoint(np) .lt. l1) .or. (lpoint(np) .gt. l2)) go to 210
        if ((mpoint(np) .lt. m1) .or. (mpoint(np) .gt. m2)) go to 210

        call pout (np, np, 'min')

        anamex = apoint(np)

        call aptchmv ('-', 1, 1, anamex, 1, nerra)

        do 205 npp = 1, npoints       ! Loop over point names.
          if (anamex .eq. apoint(npp)) then
            write (   6, 8205) ainput, ninput, anamex
            write (iout, 8205) ainput, ninput, anamex
            nprel = -2
            go to 999
          endif
  205   continue

        nprel      = nprel + 1
        apoint(np) = anamex
        kpoint(np) = 0
        lpoint(np) = 0
        mpoint(np) = 0

        write (   6, 8210) apoint(np), kpoint(np), lpoint(np),
     &    mpoint(np)
        write (iout, 8210) apoint(np), kpoint(np), lpoint(np),
     &    mpoint(np)

  210 continue                        ! End of loop over points.

  999 return

c.... End of subroutine meshrel.      (+1 line)
      end
