      subroutine meshsub (np, nrep, nerrsub)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshsub  Updated     2005 March 18 16:20.
c               meshsub  Originated  2005 March 14 15:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To replace the subscripts in the name of point np with
c               the mesh indices assigned to the point, if different.
c               of points n1 to n2.
c
c     Logic:    Entry points:  meshsub.
c meshsub       Called by:  meshblk, meshdel, meshexch, meshins, meshinv,
c               meshmove.
c
c meshsub       Calls:  aptsubi, aptsubm.
c
c     Biblio:   ?
c
c     Glossary:
c
c     nerrsub   Error flag:
c                 0 if no error.
c                 1 if np < 1.
c                 2 if np > npoints.
c                 3 if apoint(np) is blank.
c                 4 if the point name subscripts can not be found.
c                 5 if the point name does not have 3 subscripts.
c                 6 if no mesh has been defined.
c                 7 if the assigned mesh indices are outside the allowed range.
c                 8 if the point name can not be given the new subscripts.
c
c     np        Index of the point.
c
c     nrep      Zero if no change, 1 if the name of the point changed.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*8  amount
      character*24 anamex             ! Temporary name of a point.
      character*24 anamez             ! Temporary name of a point.

c.... Initialize.

      nrep    = 0
      nerrsub = 0
     
      if (np .lt. 1) then
        nerrsub = 1
        go to 999
      endif

      if (np .gt. npoints) then
        nerrsub = 2
        go to 999
      endif

      if (apoint(np) .eq. ' ') then
        nerrsub = 3
        go to 999
      endif

c.... See if the point name has three subscripts.

        call aptsubm (apoint(np), 1, lapoint(np), iamax, idmax, iemax,
     &    '(', ',', ')', 16,
     &          astem, istem, fstem, mstem, lstem,
     &    nsub, asub,  isub,  fsub,  msub,  lsub,  nerra)

      if (nerra .ne. 0) then
        nerrsub = 4
        go to 999
      endif

      if (nsub .ne. 3) then
        nerrsub = 5
        go to 999
      endif

c.... See if a mesh is defined.

      if ((kmax .le. 0)  .or.
     &    (lmax .le. 0)  .or.
     &    (mmax .le. 0)) then
        nerrsub = 6
        go to 999
      endif

c.... See if the point has assigned mesh indices.

      if ((kpoint(np) .eq. 0)  .or.
     &    (lpoint(np) .eq. 0)  .or.
     &    (mpoint(np) .eq. 0)) then
        go to 999
      endif

c.... See if the assigned mesh indices are in the allowed range.

      if ((kpoint(np) .lt. 0) .or. (kpoint(np) .gt. kmax)  .or.
     &    (lpoint(np) .lt. 0) .or. (lpoint(np) .gt. lmax)  .or.
     &    (mpoint(np) .lt. 0) .or. (mpoint(np) .gt. mmax)) then
        nerrsub = 7
        go to 999
      endif

c.... See if the point name suscripts match the mesh indices.

      if ((isub(1) .eq. kpoint(np)) .and.
     &    (isub(2) .eq. lpoint(np)) .and.
     &    (isub(3) .eq. mpoint(np))) then
        go to 999
      endif

c.... Replace the point name subscripts with the mesh indices.

      anamex  = apoint(np)
      lanamex = lapoint(np)

      isub(1) = kpoint(np)
      isub(2) = lpoint(np)
      isub(3) = mpoint(np)

      call aptsubi (astem, lstem, 3, isub, 24,
     &              '(', ',', ')', anamez, lanamez, nerra)

      if (nerra .ne. 0) then
        nerrsub = 8
        go to 999
      endif

      nrep = 1

      apoint(np)  = anamez
      lapoint(np) = lanamez

c.... Rename same point in definitions of clusters, lines, triangles,
c....   annular disks, regular polygons, tetrahedrons.

      do 182 npr = 1, npairs
        if (apairpt(npr) .eq. anamex) apairpt(npr) = apoint(np)
  182 continue

      do 185 nn = 1, nlines
        if (alipt1(nn)  .eq. anamex) alipt1(nn)   = apoint(np)
        if (alipt2(nn)  .eq. anamex) alipt2(nn)   = apoint(np)
        if (alipt1(nn)  .eq. anamex) lalipt1(nn)  = lpoint(np)
        if (alipt2(nn)  .eq. anamex) lalipt2(nn)  = lpoint(np)
  185 continue

      do 190 nt = 1, ntrigs
        if (atript1(nt) .eq. anamex) atript1(nt)  = apoint(np)
        if (atript2(nt) .eq. anamex) atript2(nt)  = apoint(np)
        if (atript3(nt) .eq. anamex) atript3(nt)  = apoint(np)
        if (atript1(nt) .eq. anamex) latript1(nt) = lpoint(np)
        if (atript2(nt) .eq. anamex) latript2(nt) = lpoint(np)
        if (atript3(nt) .eq. anamex) latript3(nt) = lpoint(np)
  190 continue

      do 192 npg = 1, npolygs
        if (acenpg(npg) .eq. anamex) acenpg(npg) = apoint(np)
        if (averpg(npg) .eq. anamex) averpg(npg) = apoint(np)
        if (acoppg(npg) .eq. anamex) acoppg(npg) = apoint(np)
  192 continue

      do 193 nd = 1, ndisks
        if (acendk(nd)  .eq. anamex) acendk(nd)  = apoint(np)
  193 continue

      do 194 nt = 1, ntets
        if (atetpt(1,nt) .eq. anamex) atetpt(1,nt) = apoint(np)
        if (atetpt(2,nt) .eq. anamex) atetpt(2,nt) = apoint(np)
        if (atetpt(3,nt) .eq. anamex) atetpt(3,nt) = apoint(np)
        if (atetpt(4,nt) .eq. anamex) atetpt(4,nt) = apoint(np)
  194 continue

      do 195 nt = 1, ntets
        if (atetpt(1,nt) .eq. anamex) latetpt(1,nt) = lpoint(np)
        if (atetpt(2,nt) .eq. anamex) latetpt(2,nt) = lpoint(np)
        if (atetpt(3,nt) .eq. anamex) latetpt(3,nt) = lpoint(np)
        if (atetpt(4,nt) .eq. anamex) latetpt(4,nt) = lpoint(np)
  195 continue

      do 196 nph = 1, npolyhs
        if (acenph(nph) .eq. anamex) acenph(nph) = apoint(np)
        if (averph(nph) .eq. anamex) averph(nph) = apoint(np)
        if (acopph(nph) .eq. anamex) acopph(nph) = apoint(np)
  196 continue


  999 return

c.... End of subroutine meshsub.      (+1 line)
      end
