      subroutine namechk (anamex, lanamex, nerrchk)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  namechk  Updated     2006 January 6 14:50.
c               namechk  Originated  2005 April 4 16:20
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To check the validity of object names.
c
c     Logic:    Entry points:  namechk.
c namechk       Called by:  alias, axisym, bin, brick, cluster, cone, copy,
c               cylinder, disk, ellipsoid, hyperb, line, operator, pdf, plane,
c               point, polygon, polyhed, ptoper, quadric, qumove, quoper,
c               renamex, sphere, symbol, tetra, triangle, variable, vector,
c               voper, zone.
c
c namechk       Calls:  aptsubm.
c
c     Biblio:   ?
c
c     Glossary:
c
c     anamex    Input    Name to be checked.
c
c     lanamex   Input    Number of characters in anamex.
c
c     nerrchk   Output   Error indicator:
c                          0 if name is allowed.
c                          1 if name is blank.
c                          2 if name has zero length.
c                          3 if name is not allowed.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*24 astemx

c.... Initialize.

      nerrchk    = 0

c.... Test input for errors.

 8110     format ('  namechk ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Name is blank.')
 8120     format ('  namechk ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Name has zero length.')

       if (anamex .eq. ' ') then
         nerrchk = 1
         write (   6, 8110) ainput, ninput
         write (iout, 8110) ainput, ninput
         go to 999
       endif

       if (lanamex .eq. 0) then
         nerrchk = 2
         write (   6, 8120) ainput, ninput
         write (iout, 8120) ainput, ninput
         go to 999
       endif

 8130   format ('  namechk ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Name too dangerous to use:  ',a)

        if ((anamex .eq. 'all' )   .or.
     &      (anamex .eq. 'comp.')  .or.
     &      (anamex .eq. 'h'   )   .or.
     &      (anamex .eq. 'help')   .or.
     &      (anamex .eq. 'root')   .or.
     &      (anamex .eq. 'mod' )   .or.
     &      (anamex .eq. 'fact')   .or.
     &      (anamex .eq. 'list')   .or.
     &      (anamex .eq. 'thru'))  then
          nerrchk = 3
          write (   6, 8130) ainput, ninput, anamex
          write (iout, 8130) ainput, ninput, anamex
          go to 999
        endif

        call aptsubm (anamex, 1, lanamex, iamax, idmax, iemax,
     &    '(', ',', ')', 16,
     &          astemx, istem, fstem, mstem, lstem,
     &    nsub, asub,  isub,  fsub,  msub,  lsub, nerra)

        if ((astemx .eq. 'all' )   .or.
     &      (anamex .eq. 'comp.')  .or.
     &      (astemx .eq. 'h'   )   .or.
     &      (astemx .eq. 'help')   .or.
     &      (astemx .eq. 'root')   .or.
     &      (astemx .eq. 'mod' )   .or.
     &      (anamex .eq. 'fact')   .or.
     &      (astemx .eq. 'list')   .or.
     &      (astemx .eq. 'thru'))  then
          nerrchk = 3
          write (   6, 8130) ainput, ninput, astemx
          write (iout, 8130) ainput, ninput, astemx
          go to 999
        endif

  999 return

c.... End of subroutine namechk.      (+1 line)
      end
