      subroutine pdf

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  pdf      Updated     2005 April 5 17:45.
c               pdf      Originated  1994 June 22 14:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create pdfs, consisting of a set of probability
c               bins.
c
c     Logic:    Entry point:  pdf
c pdf           Called by:  helpall, main.
c
c pdf           Calls:  aptsubr, binfind, compsubs, incrsubs, namechk, pdfout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8  amount             ! Amount of output to write in pdfout.
      character*24 abinmn             ! Lowest bin name to test.
      character*24 abinmx             ! Highest bin name to test.

c.... Input line:  pdf [help]

 8110 format (
     &  '  Command "pdf":  acceptable input lines include:' /
     &  '  "pdf [help]"   Display this message.' /
     &  '  "pdf all"      Display all pdfs.' /
     &  '  "pdf list [PDFNAME1 PDFNAME2 PDFNAME3 ...]"' /
     &  '     Display all listed pdfs (default is all',
     &  ' pdfs).' /
     &  '  "pdf PDFNAME"  Display pdf PDFNAME.' //
     &  '  "pdf PDFNAME all"' /
     &  '     Create new pdf PDFNAME, consisting of',
     &  ' all bins.' //
     &  '  "pdf PDFNAME = BINAME1 ... BINAME2 [thru] BINAME3 ..."' /
     &  '     Create new pdf PDFNAME, consisting of',
     &  ' the bins' /
     &  '     BINAME1, ..., BINAME2 [, through] BINAME3, ...' //
     &  '  "pdf PDFNAME & BINAME1 ... BINAME2 [thru] BINAME3 ..."' /
     &  '     Change pdf PDFNAME, to add the bins' /
     &  '     BINAME1, ..., BINAME2 [, through] BINAME3, ...' )

 8112 format (/
     &  '  "pdf PDFNAME - BINAME1 ... BINAME2 [thru] BINAME3 ..."' /
     &  '     Change pdf PDFNAME, to remove the bins' /
     &  '     BINAME1, ..., BINAME2 [, through] BINAME3, ...' //
     &  '  Note:  use +/- for PDFNAME to incr/decr the last pdf',
     &  ' name:  ',a //
     &  '  Note:  see commands "bin" and "sample".' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112) apdfs
        write (iout, 8112) apdfs
        go to 999
      endif

c===============================================================================

c.... Input line:  pdf all
c.... Input line:  pdf list

 8120 format (/ 'Number of pdfs = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        npdfxx = 0
        do 120 n = 1, npdfs           ! Loop over pdfs.
          if (apdf(n) .eq. ' ') go to 120
          npdfxx = npdfxx + 1
  120   continue                      ! End of loop over pdfs.
        if (npdfxx .eq. 1) then
          amount = 'maximum'
        else
          amount = 'minimum'
        endif
        call pdfout (1, npdfs, amount)
        write (   6, 8120) npdfxx
        write (iout, 8120) npdfxx
        go to 999
      endif

c===============================================================================

c.... Input line:  pdf list PDFNAME1 PDFNAME2 PDFNAME3 ...

 8130 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named pdf does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 140 nw = 3, nword          ! Loop over input line words.
          nfind = 0
          do 130 n = 1, npdfs         ! Loop over pdfs.
            call aptsubr (apdf(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. apdf(n)) .or.
     &          (aword(nw) .eq. astem))  then
              nfind = nfind + 1
              call pdfout (n, n, 'min')
            endif
  130     continue                    ! End of loop over pdfs.
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  140   continue                      ! End of loop over input line words.
        go to 999
      endif                           ! Word 2 is "list".

c===============================================================================

c.... Input line:  pdf PDFNAME

c.... See if user wants one pdf displayed.

      if (nword .eq. 2) then
        do 150 n = 1, npdfs           ! Loop over pdfs.
          if (aword(2) .eq. apdf(n)) then
            amount = 'maximum'
            call pdfout (n, n, amount)
            go to 999
          endif
  150   continue                      ! End of loop over pdfs.
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test input for validity.

 8160 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of pdf not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8160) ainput, ninput, aword(2)
        write (iout, 8160) ainput, ninput, aword(2)
        go to 999
      endif

 8165 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      if ((aword(3) .ne. '=')    .and.
     &    (aword(3) .ne. 'all')  .and.
     &    (aword(3) .ne. '&')    .and.
     &    (aword(3) .ne. '-'))   then
        write (   6, 8165) ainput, ninput
        write (iout, 8165) ainput, ninput
        go to 999
      endif

c===============================================================================

c.... See if last pdf name should be incremented and used.

 8128 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last pdf can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = apdfs
        lword(2) = lapdfs
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, apdfs
          write (iout, 8128) ainput, ninput, apdfs
          go to 999
        endif
      endif

 8129 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last pdf can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = apdfs
        lword(2) = lapdfs
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, apdfs
          write (iout, 8129) ainput, ninput, apdfs
          go to 999
        endif
      endif

 8905 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of pdf too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if pdf exists.

 8170 format ('  WARNING:  replacing existing pdf:  ',a)
 8180 format ('  Adding bins to existing pdf:  ',a)
 8185 format ('  Removing bins from existing pdf:  ',a)

      nrep = 0
      do 170 n = 1, npdfs             ! Loop over pdfs.
        if (aword(2) .eq. apdf(n)) then
          nrep = 1

          if (aword(3) .eq. '=') then
            write (   6, 8170) apdf(n)
            write (iout, 8170) apdf(n)
          elseif (aword(3) .eq. '&') then
            write (   6, 8180) apdf(n)
            write (iout, 8180) apdf(n)
          elseif (aword(3) .eq. '-') then
            write (   6, 8185) apdf(n)
            write (iout, 8185) apdf(n)
          endif

          call pdfout (n, n, 'min')

          go to 190
        endif
  170 continue                        ! End of loop over pdfs.

      do 180 n = 1, npdfs             ! Loop over pdfs.
        if (apdf(n) .eq. ' ') go to 190
  180 continue                        ! End of loop over pdfs.

      npdfs = npdfs + 1

 8190 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of pdfs > limit ',i5)

      if (npdfs .gt. npdfm) then
        npdfs = npdfm
        write (   6, 8190) ainput, ninput, npdfm
        write (iout, 8190) ainput, ninput, npdfm
        go to 999
      endif

      n = npdfs

  190 nerr = 0

 8200 format ('  WARNING:  assigning bins to new pdf:  ',a)

      if ((nrep .eq. 0) .and. (aword(3) .eq. '&')) then
        write (   6, 8200) aword(2)
        write (iout, 8200) aword(2)
      endif

c===============================================================================

c.... Input line:  pdf PDFNAME all

      if (aword(3) .eq. 'all') then

 8310   format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Extra words after "all":  ',i3)

        if (nword .ne. 3) then
          nwordx = nword - 3
          write (   6, 8310) ainput, ninput, nwordx
          write (iout, 8310) ainput, ninput, nwordx
          go to 999
        endif

c....   Add all bins to the pdf.

        npdfxx = npdfxs

        do 320 nb = 1, nbins          ! Loop over bins.

          npdfxs = npdfxs + 1

          if (npdfxs .gt. npdfxm) then
            nerr = 1
            npdfxs = npdfxm
            write (   6, 8230) ainput, ninput, npdfxm
            write (iout, 8230) ainput, ninput, npdfxm
            go to 320
          endif

          apdfx(npdfxs) = aword(2)
          abinx(npdfxs) = abin(nb)

  320   continue                      ! End of loop over bins.

        if (nerr .ne. 0) then
          npdfxs = npdfxx
          if (nrep .eq. 0) then
            apdf(n) = ' '
            if (n .eq. npdfs) npdfs = npdfs - 1
          endif
          go to 999
        endif

c....   Blank out old bins for a redefined pdf.

        if ((nrep .eq. 1) .and. (aword(3) .eq. '=')) then
          do 330 npx = 1, npdfxx      ! Loop over pdf/bin pairs.
            if (apdfx(npx) .eq. aword(2)) then
              apdfx(npx) = ' '
            endif
  330     continue                    ! End of loop over pdf/bin pairs.
        endif

c....   Store and display the pdf specifications.

        apdf(n)  = aword(2)
        apdfs    = aword(2)
        lapdfs   = lword(2)
        prpdf(n) = -1.e99

        amount = 'maximum'
        call pdfout (n, n, amount)

        go to 999

      endif                           ! Word 3 was "all".

c=======================================================================********

c.... Input line:  pdf PDFNAME [=,&] BINAME1 ... BINAME2 [thru] BINAME3 ...

      if ((aword(3) .eq. '=')  .or.
     &    (aword(3) .eq. '&')) then

c.... Find the bins to add to the pdf.

      npdfxx = npdfxs

 8210 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No bins specified.')

      if (nword .eq. 3) then
        nerr = 1
        write (   6, 8210)
        write (iout, 8210)
      endif

 8220 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named bin does not exist:  ',a)
 8230 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Exceeded maximum pdf pair table size:  ',i4)
 8234 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Key word "thru" duplicated or misplaced.')
 8138 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Min name > max name:  ',a24,2x,a24)

      do 230 nw = 4, nword            ! Loop over input line words.

c....   Test for option "thru".

        if (aword(nw) .eq. 'thru') then

          if ((nw          .eq. 4     )  .or.
     &        (aword(nw-2) .eq. 'thru')  .or.
     &        (aword(nw-1) .eq. 'thru')  .or.
     &        (aword(nw+1) .eq. 'thru')  .or.
     &        (aword(nw+2) .eq. 'thru')  .or.
     &        (nw          .eq. nword )) then
            nerr = 1
            write (   6, 8234) ainput, ninput
            write (iout, 8234) ainput, ninput
            go to 230
          endif

          abinmn = aword(nw-1)
          abinmx = aword(nw+1)


          call compsubs (aword(nw-1), aword(nw+1), nsmall, nerrc)
          if (nsmall .eq. 2) then
            nerr = 1
            write (   6, 8138) ainput, ninput, aword(nw-1), aword(nw+1)
            write (iout, 8138) ainput, ninput, aword(nw-1), aword(nw+1)
            go to 230
          endif

          do 225 nb = 1, nbins        ! Loop over bins.
            if (abin(nb) .eq. ' ') go to 225

            call compsubs (abin(nb), abinmn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 225
            call compsubs (abin(nb), abinmx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 225

              npdfxs = npdfxs + 1

              if (npdfxs .gt. npdfxm) then
                nerr = 1
                npdfxs = npdfxm
                write (   6, 8230) ainput, ninput, npdfxm
                write (iout, 8230) ainput, ninput, npdfxm
                go to 230
              endif

              apdfx(npdfxs) = aword(2)
              abinx(npdfxs) = abin(nb)

  225     continue                    ! End of loop over bins.

          go to 230

        endif                         ! Tested for option "thru".

c....   Test for bin name.

        if (aword(nw-1) .eq. 'thru') go to 230
        if (aword(nw+1) .eq. 'thru') go to 230

        do 220 nb = 1, nbins          ! Loop over bins.
          if (aword(nw) .eq. abin(nb)) then

            npdfxs = npdfxs + 1

            if (npdfxs .gt. npdfxm) then
              nerr = 1
              npdfxs = npdfxm
              write (   6, 8230) ainput, ninput, npdfxm
              write (iout, 8230) ainput, ninput, npdfxm
              go to 230
            endif

            apdfx(npdfxs) = aword(2)
            abinx(npdfxs) = abin(nb)

            go to 230

          endif
  220   continue                      ! End of loop over bins.

        nerr = 1
        write (   6, 8220) ainput, ninput, aword(nw)
        write (iout, 8220) ainput, ninput, aword(nw)

  230 continue                        ! End of loop over input line words.

      if (nerr .ne. 0) then
        npdfxs = npdfxx
        if (nrep .eq. 0) then
          apdf(n) = ' '
          if (n .eq. npdfs) npdfs = npdfs - 1
        endif
        go to 999
      endif

c.... Blank out old bins for a redefined pdf.

      if ((nrep .eq. 1) .and. (aword(3) .eq. '=')) then
        do 240 npx = 1, npdfxx        ! Loop over pdf/bin pairs.
          if (apdfx(npx) .eq. aword(2)) then
            apdfx(npx) = ' '
          endif
  240   continue                      ! End of loop over pdf/bin pairs.
      endif

c.... Store and display the pdf specifications.

      apdf(n)  = aword(2)
      apdfs    = aword(2)
      lapdfs   = lword(2)
      prpdf(n) = -1.e99
cbugc___DEBUG begins.
cbug 9901 format ('pdf DEBUG.  Bin ',i3,' is ',a)
cbug 9902 format ('pdf DEBUG.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9901) n, apdf(n)
cbug      write (iout, 9901) n, apdf(n)
cbug      write (   6, 9902) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbug      write (iout, 9902) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbugc___DEBUG ends.

      amount = 'maximum'
      call pdfout (n, n, amount)

      go to 999

      endif                           ! Word 3 was "=" or "&".

c=======================================================================********

c.... Input line:  pdf PDFNAME - BINAME1 ... BINAME2 [thru] BINAME3 ...

      if (aword(3) .eq. '-') then
cbugc***DEBUG begins.
cbug 9811 format ('pdf debug 21.  npdfs = ',i10,' apdf(1) = "',a,'"')
cbug      write (iout, 9811) npdfs, apdf(1)
cbug      write (   6, 9811) npdfs, apdf(1)
cbugc***DEBUG ends.

c.... Find the bins to remove from the pdf.

 8410 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No bins specified.')

      if (nword .eq. 3) then
        write (   6, 8410) ainput, ninput
        write (iout, 8410) ainput, ninput
        go to 999
      endif

      do 460 nw = 4, nword            ! Loop over input line words.

c....   Test for option "thru".

        if (aword(nw) .eq. 'thru') then

          if ((nw          .eq. 4     )  .or.
     &        (aword(nw-2) .eq. 'thru')  .or.
     &        (aword(nw-1) .eq. 'thru')  .or.
     &        (aword(nw+1) .eq. 'thru')  .or.
     &        (aword(nw+2) .eq. 'thru')  .or.
     &        (nw          .eq. nword )) then
            write (   6, 8234) ainput, ninput
            write (iout, 8234) ainput, ninput
            go to 999
          endif

          abinmn = aword(nw-1)
          abinmx = aword(nw+1)

          call compsubs (aword(nw-1), aword(nw+1), nsmall, nerrc)
          if (nsmall .eq. 2) then
            write (   6, 8138) ainput, ninput, aword(nw-1), aword(nw+1)
            write (iout, 8138) ainput, ninput, aword(nw-1), aword(nw+1)
            go to 999
          endif

 8430     format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  No bin name before option "thru".')
 8432     format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  No bin name after final option "thru".' /
     &      '  WARNING:  already removed bins displayed above from',
     &      ' pdf.' )

 8434     format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Word "thru" duplicated.  Will ignore.')

          if (abinmx .eq. 'thru') then
            aword(nw) = aword(nw-1)
            write (   6, 8434) ainput, ninput
            write (iout, 8434) ainput, ninput
            go to 460
          endif

          do 450 nb = 1, nbins        ! Loop over bins.
            if (abin(nb) .eq. ' ') go to 450

            call compsubs (abin(nb), abinmn, nsmall, nerrc)
            if (nsmall .eq. 1) go to 450
            call compsubs (abin(nb), abinmx, nsmall, nerrc)
            if (nsmall .eq. 2) go to 450

c....         Blank out specified bins.

              do 440 npx = 1, npdfxs  ! Loop over pdf/bin pairs.
                if ((apdfx(npx) .eq. aword(2))    .and.
     &              (abinx(npx) .eq. abin(nb))) then
                  apdfx(npx) = ' '
                  abinx(npx) = ' '
                endif
  440         continue                ! End of loop over pdf/bin pairs.

  450     continue                    ! End of loop over bins.

          go to 460

        endif                         ! Tested for option "thru".

c....   Test for a bin name.

        if (aword(nw-1) .eq. 'thru') go to 460
        if (aword(nw+1) .eq. 'thru') go to 460

        call binfind (nw, nb)

        if (nb .gt. 0) then           ! Argument is a bin name.

c....     Blank out specified bins.

          do 420 npx = 1, npdfxs      ! Loop over pdf/bin pairs.
            if ((apdfx(npx) .eq. aword(2))    .and.
     &          (abinx(npx) .eq. abin(nb))) then
              apdfx(npx) = ' '
              abinx(npx) = ' '
            endif
  420     continue                    ! End of loop over pdf/bin pairs.

          go to 460

        else                          ! Not a bin name or option "thru".

 8460     format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Not a bin name or option "thru":  ',a)

          write (   6, 8460) ainput, ninput, aword(nw)
          write (iout, 8460) ainput, ninput, aword(nw)
          go to 460

        endif                         !  Tested for a bin name.

  460 continue                        ! End of loop over input line words.

c.... Display the pdf specifications.

cbugc___DEBUG begins.
cbug      write (   6, 9901) n, apdf(n)
cbug      write (iout, 9901) n, apdf(n)
cbug      write (   6, 9902) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbug      write (iout, 9902) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbugc___DEBUG ends.

      amount = 'maximum'
      call pdfout (n, n, amount)

      go to 999

      endif                           ! Word 3 was "-".

c===============================================================================

 8900 format ('  pdf ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8900) ainput, ninput
      write (iout, 8900) ainput, ninput

c===============================================================================

  999 return

c.... End of subroutine pdf.          (+1 line)
      end
