      subroutine pdfout (n1, n2, amount)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  pdfout   Updated     2004 November 23 13:30.
c               pdfout   Originated  1994 June 23 10:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the names, data for the bins, and total probability
c               for probability distribution functions n1 to n2.
c
c     Logic:    Entry points:  pdfout.
c pdfout        Called by:  copy, delete, pdf, renamea, renamex, sample, search.
c
c pdfout        Calls:  binout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8 amount              ! Amount of output to write.

c.... Local variables.

      common /lpdfout/ anamex(3)      ! Names of bins.
      character*24 anamex             ! Names of bins.
      common /lpdfout/ alab           ! Label "=" or "&".
      character*1 alab                ! Label "=" or "&".

c.... Initialize.

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. npdfs) nn2 = npdfs

c.... Repack pdf/bin pair list.  Eliminate blanks and duplicates.
cbugc___DEBUG begins.
cbug 9911 format ('pdfout DEBUG X1.  npx='i3,' npairx=',i3)
cbug 9912 format ('pdfout DEBUG X2.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9911) npx, npairx
cbug      write (iout, 9911) npx, npairx
cbug      write (   6, 9912) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbug      write (iout, 9912) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbugc___DEBUG ends.

      npairx = 0

      do 105 npx = 1, npdfxs          ! Loop over pairs.

c....   Skip blank entries.

        if ((apdfx(npx) .eq. ' ') .or.
     &      (abinx(npx) .eq. ' ')) go to 105

        npairx = npairx + 1
        apdfx(npairx) = apdfx(npx)
        abinx(npairx) = abinx(npx)

c....   Blank out duplicate entries.

        do 102 next = npx + 1, npdfxs
          if ((apdfx(next) .eq. apdfx(npx))  .and.
     &        (abinx(next) .eq. abinx(npx))) then
            apdfx(next) = ' '
            abinx(next) = ' '
          endif
  102   continue
  105 continue                        ! End of loop over pairs.

      npdfxs = npairx
cbugc___DEBUG begins.
cbug 9914 format ('pdfout DEBUG Y1.  npx='i3,' npairx=',i3)
cbug 9913 format ('pdfout DEBUG Y2.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9914) npx, npairx
cbug      write (iout, 9914) npx, npairx
cbug      write (   6, 9913) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbug      write (iout, 9913) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbugc___DEBUG ends.

c.... Find data for each pdf.

      do 150 n = nn1, nn2             ! Loop over pdfs.
cbugc___DEBUG begins.
cbug 9901 format ('pdfout DEBUG 1.  pdf ',i3,' is ',a)
cbug 9902 format ('pdfout DEBUG 1.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9901) n, apdf(n)
cbug      write (iout, 9901) n, apdf(n)
cbug      write (   6, 9902) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbug      write (iout, 9902) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbugc___DEBUG ends.

        if (apdf(n) .eq. ' ') go to 150

 8110   format ('  pdfout ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named bin does not exist:  ',a)
 8120   format ('pdf ',a,1x,a1,4x,a,2x,a,2x,a)

        nx         = 0                ! Bin counter, 0 to 3 per output line.
        nbinpdf(n) = 0                ! Total number of bins in this pdf.
        prpdf(n)   = 0.0              ! Total relative probability of this pdf.
        prvar      = 0.0              ! Total rel prob of a random variable.
        prdis      = 0.0              ! Total rel prob of a discreet event.
        vexpdf(n)  = 0.0              ! Expected value for this pdf.
        alab       = '='              ! Option in list of bins, "=" or "&".

c....   Find bins in this pdf.

        do 120 npx = 1, npdfxs        ! Loop over pairs.

          if ((apdfx(npx) .eq. ' ') .or.
     &        (abinx(npx) .eq. ' ')) go to 120

          if (apdfx(npx) .ne. apdf(n)) go to 120

c....     Found a bin in this pdf.

          nx         = nx + 1
          nbinpdf(n) = nbinpdf(n) + 1

c....     Find the bin in the bin table.

          do 110 nb = 1, nbins
            if (abinx(npx) .eq. abin(nb)) go to 115
  110     continue
          write (   6, 8110) ainput, ninput, abinx(npx)
          write (iout, 8110) ainput, ninput, abinx(npx)
          go to 999

c....     Found the bin in the bin table.

  115     anamex(nx) = abinx(npx)
          prpdf(n)   = prpdf(n) + prbin(nb)
          if ((atypeb(nb) .ne. 'discrete event')  .and.
     &        (atypeb(nb) .ne. 'rel Maxwellian')) then
            prvar     = prvar     + prbin(nb)
            vexpdf(n) = vexpdf(n) + prbin(nb) * vexbin(nb)
          elseif (atypeb(nb) .eq. 'discrete event') then
            prdis = prdis + prbin(nb)
          endif

          if (nx .eq. 3) then
            write (   6, 8120) apdf(n), alab, (anamex(nn), nn = 1, 3)
            write (iout, 8120) apdf(n), alab, (anamex(nn), nn = 1, 3)
            nx = 0
            alab = '&'
          endif

  120   continue                      ! End of loop over pairs.

        if (nx .ne. 0) then
          write (   6, 8120) apdf(n), alab, (anamex(nn), nn = 1, nx)
          write (iout, 8120) apdf(n), alab, (anamex(nn), nn = 1, nx)
        endif

 8122   format ('  WARNING:  deleted empty pdf:  ',a)
 8124   format ('  Number of bins in pdf ',a,' = ',i5,'.')

        if (nbinpdf(n) .eq. 0) then
          anamex(1) = '**NONE**'
          alab      = ' '
          write (   6, 8120) apdf(n), alab, anamex(1)
          write (iout, 8120) apdf(n), alab, anamex(1)
          write (   6, 8122) apdf(n)
          write (iout, 8122) apdf(n)
          apdf(n) = ' '
        else
          write (   6, 8124) apdf(n), nbinpdf(n)
          write (iout, 8124) apdf(n), nbinpdf(n)
        endif
cbugc___DEBUG begins.
cbug 9903 format ('pdfout DEBUG 2.  pdf     ',i3,' is ',a)
cbug 9904 format ('pdfout DEBUG 2.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9903) n, apdf(n)
cbug      write (iout, 9903) n, apdf(n)
cbug      write (   6, 9904) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbug      write (iout, 9904) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbugc___DEBUG ends.

 8125   format ('  Total prob      ',1pe20.12)    

        write (   6, 8125) prpdf(n)
        write (iout, 8125) prpdf(n)

c....   Find the probabilities of a discrete event or a random variable,
c....     and the expected value of the latter, for this pdf.

 8126   format ('  WARNING:  this pdf has both a random variable and',
     &    ' discrete events.' /
     &    '  Random variable has  abs probability ',1pe20.12)
 8127   format ('  Expected value  ',1pe20.12)

        if (prvar .gt. 0.0) then
          prvart = prvar / prpdf(n)
          vexpdf(n) = vexpdf(n) / prvar
          if (prvar .ne. prpdf(n)) then
            write (   6, 8126) prvart
            write (iout, 8126) prvart
          endif
          write (   6, 8127) vexpdf(n)
          write (iout, 8127) vexpdf(n)
        else
          vexpdf(n) = -1.e99
        endif

 8129   format ('  Discrete events have abs probability ',1pe20.12)

        if (prdis .ne. 0.0) then
          prdist = prdis / prpdf(n)
          write (   6, 8129) prdist
          write (iout, 8129) prdist
        endif

        if ((amount .eq. 'max')      .or.
     &      (amount .eq. 'maximum')) then

c....     Display data for all bins in this pdf.

          do 140 npx = 1, npdfxs      ! Loop over pdf/bin pair table.
            if (apdfx(npx) .ne. apdf(n)) go to 140
            do 130 nb = 1, nbins      ! Loop over bins.
              if (abinx(npx) .eq. abin(nb)) then
                call binout (nb, nb, 'min')
              endif
  130       continue                  ! End of loop over bins.
  140     continue                    ! End of loop over pdf/bin pair table.

        endif                         ! Tested amount.

  150 continue                        ! End of loop over pdfs.

  999 return

c.... End of subroutine pdfout.       (+1 line)
      end
