      subroutine plotpt (xpt, ypt, zpt, nerrg)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  plotpt   Updated     1996 March 13 17:20.
c               plotpt   Originated  1996 January 29 17:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To store the coordinates of a point into the plot point array,
c               either in the first unused space, or at the end.
c
c     Logic:    Entry points:  plotpt.
c plotpt        Called by:  distance, extrema, intcirc, intcyl, intplcy,
c               intplqu, intpls, intquad, intspcy, intsph, pgout, phout, pout,
c               prox, ptdist, pttet, quout, slice, sliceq, track, triple.
c               Calls:  (none).
c
c     Biblio:   ?
c
c     Glossary:
c
c     nerrg     Output   0 if no error, 1 if point can not be stored in the
c                          plot point array.
c
c     nplpts    Output   Number of plot points saved.
c
c     xplot     Output   X coordinate of a plot point.
c
c     xpt       Input    X coordinate of a point.
c
c     yplot     Output   Y coordinate of a plot point.
c
c     ypt       Input    Y coordinate of a point.
c
c     zplot     Output   Z coordinate of a plot point.
c
c     zpt       Input    Z coordinate of a point.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      nerrg = 0

c.... See if plot status is "on".

      if (aplstat .eq. 'off') go to 999

c.... See if there is an unmarked plot point.

      do 110 n = 1, nplpts
        if (aplotm(n) .eq. ' ') then
          aplotm(n) = aptmarks
          xplot(n)  = xpt
          yplot(n)  = ypt
          zplot(n)  = zpt
          go to 999
        endif
  110 continue

c.... See if there is room to add another plot point.

 8110 format ('  plotpt ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of plot points exceeds limit ',i5)

      if (nplpts .eq. nplptm) then
        nerrg = 1
        write (   6, 8110) ainput, ninput, nplpts
        write (iout, 8110) ainput, ninput, nplpts
        go to 999
      endif

c.... Store the marker and coordinates of the point.

      nplpts          = nplpts + 1
      aplotm(nplpts)  = aptmarks
      xplot(nplpts)   = xpt
      yplot(nplpts)   = ypt
      zplot(nplpts)   = zpt

  999 return

c.... End of subroutine plotpt.       (+1 line)
      end
