      subroutine plotx

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  plotx    Updated     2005 February 2 17:10.
c               plotx    Originated  1995 December 15 16:50.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To define the plot parameters, and make plots.
c
c     Logic:    Entry points:  plotx.
c plotx         Called by:  helpall, main.
c
c plotx         Calls:  compsubs, markget, plotmap, plotvar, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c a   aplbuf             Plot character array.  Size nplinem by nplcolm.
c     aplmarkh           Marker with label of horizontal axis.
c     aplmarkt           Marker with title of plot.
c     aplmarkv           Marker with label of vertical axis.
c a   aplotm             Character marker for a plot point.  Size nplptm.
c     aplstat            Plotting status, "on" or "off".
c a   aptmark            Point markers used in current plot.  Size nptmarkm.
c     aptmarks           Marker currently being used to mark points.
c     axish              Name of coordinate of horizontal axis.
c     axisv              Name of coordinate of vertical axis.
c     nplcen             Indicates plot center specified, if 1.
c     nplcols            Number of plot array columns <= 166 = nplcolm - 2.
c     nplimh             Indicates horizontal axis limits specified, if 1.
c     nplimv             Indicates vertical   axis limits specified, if 1.
c     nplines            Number of plot array lines <= 102 = nplinem - 2.
c     nplpts             Number of plot points <= 1000 = nplptm.
c     nptmarks           Number of point markers in use <= nptmarkm.
c     splhcen            Horizontal axis center  on plot.
c     splvcen            Vertical   axis center  on plot.
c     splhmax            Horizontal axis maximum on plot.
c     splvmax            Vertical   axis maximum on plot.
c     spthmax            Horizontal axis maximum of plot points.
c     sptvmax            Vertical   axis maximum of plot points.
c     splhmin            Horizontal axis minimum on plot.
c     splvmin            Vertical   axis minimum on plot.
c     spthmin            Horizontal axis minimum of plot points.
c     sptvmin            Vertical   axis minimum of plot points.
c a   xplot              X coordinate of a plot point.  Size nplptm.
c a   yplot              Y coordinate of a plot point.  Size nplptm.
c a   zplot              Z coordinate of a plot point.  Size nplptm.
c     zoomplh            Zoom factor for horizontal axis.
c     zoomplv            Zoom factor for vertical axis.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*1 amarkmn             ! Lower limit of a range of markers.
      character*1 amarkmx             ! Upper limit of a range of markers.

c=======================================================================********

c.... Initialize.

      if (nword .eq. 0) then          ! Initialize at beginning of problem.

        nplcols    = 65               ! Number of plot array columns.
        nplines    = 41               ! Number of plot array lines.
        nplpts     = 0                ! Number of plot points.

        aplstat    = 'off'            ! Plotting status.

        axish      = 'x'              ! Name of coordinate of horizontal axis.
        nplimh     = 0                ! Indicates h axis limits specified, if 1.
        splhmin    = -1.e99           ! Horizontal axis minimum on plot.
        splhmax    =  1.e99           ! Horizontal axis maximum on plot.

        axisv      = 'y'              ! Name of coordinate of vertical axis.
        nplimv     = 0                ! Indicates v axis limits specified, if 1.
        splvmin    = -1.e99           ! Vertical axis minimum on plot.
        splvmax    =  1.e99           ! Vertical axis maximum on plot.

        nplcen     = 0                ! Indicates plot center specified, if 1.
        splhcen    = -1.e99           ! Horizontal axis center on plot.
        splvcen    = -1.e99           ! Vertical axis center on plot.

        nptmarks   = 1                ! Number of plot point markers in use.
        aptmark(1) = '*'              ! First plot point marker.
        aptmarks   = '*'              ! Current plot point marker.

        zoomplh    = 1.0              ! Zoom factor for horizontal axis.
        zoomplv    = 1.0              ! Zoom factor for vertical axis.

        go to 999

      endif

c=======================================================================********

c.... Input line:  plot [help]

 8110 format ('  Command "plot":  acceptable input lines include:' /
     &  '  "plot [help]"    Display this message.' //
     &  '  "plot status"' /
     &  '     Display the plot status (default is "off"),' /
     &  '     the plot title, the coordinates, labels and limits',
     &  ' of the axes,' /
     &  '     the zoom factors, the point marker, and the size of',
     &  ' the plot array.' //
     &  '  "plot size [NCOLUMNS NLINES]"' /
     &  '     Display [or specify] the number of plot array columns',
     &  ' [NCOLUMNS <= 166]' /
     &  '     and lines [NLINES <= 102].' /
     &  '     Good sizes are (65,41), (81,51), (101,61),',
     &  ' (129,81) and (161,101).' //
     &  '  "plot off"          Stop storing plot points.',
     &  '  This is the initial default.' /
     &  '  "plot on"           Start or resume storing plot points.' //
     &  '  "plot array"' /
     &  '     Map the plot points to the plot array and',
     &  ' display it,' /
     &  '     along with all of the plot parameters.' //
     &  '  "plot point [all]"   Display all plot points.' /
     &  '  "plot point index N1 ... N2 [thru] N3 ..."' /
     &  '     Display the plot points with indices N1, ...,',
     &  ' N2 [, through] N3, ...' /
     &  '  "plot point marker AMNAME1 ... AMNAME2 [thru] AMNAME3 ..."' /
     &  '     Display plot points with markers AMNAME1, ...,',
     &  ' AMNAME2 [, through] AMNAME3, ...' )

 8112 format (/
     &  '  "plot delete [all]"  Delete all plot points.' /
     &  '  "plot delete index N1 ... N2 [thru] N3 ..."' /
     &  '     Delete the plot points with indices N1, ...,',
     &  ' N2 [, through] N3, ...' /
     &  '  "plot delete marker AMNAME1 ... AMNAME2 [thru]',
     &  ' AMNAME3 ..."' /
     &  '     Delete plot points with markers AMNAME1, ...,',
     &  ' AMNAME2 [, through] AMNAME3, ...' //
     &  '  "plot title [AMNAME]"' /
     &  '     Display [or specify the marker AMNAME whose text is]',
     &  ' the plot title.' //
     &  '  "plot axis [[h,v] COORD AMNAME]"' /
     &  '     Display the plot axis coordinates and labels [or',
     &  ' specify' /
     &  '     the horizontal or vertical axis COORD and the marker',
     &  ' AMNAME' /
     &  '     whose text is the horizontal or vertical axis label].' /
     &  '     Argument COORDH must be x, y, z, rcyl, rsph, theta or',
     &  ' phi.' //
     &  '  Type <return> for more help.' )

 8113 format (a)

 8114 format (/
     &  '  "plot marker all"  Display all plot point markers.' //
     &  '  "plot marker [AMNAME]"' /
     &  '     Display [or specify AMNAME to be] the marker to be',
     &  ' the plot array' /
     &  '     character for subsequently stored plot points.' //
     &  '  "plot limits -"' /
     &  '     Specify that the plot axis limits will include all plot',
     &  ' points (default).' /
     &  '  "plot limits [[h,v] SMIN SMAX]"' /
     &  '     Display the plot axis limits [or specify the horizontal',
     &  ' or vertical' /
     &  '     plot axis limits to be SMIN and SMAX.]' //
     &  '  "plot center -"' /
     &  '     Default the center of the plot to the center of the plot',
     &  ' axis limits.' /
     &  '  "plot center [SCENH SCENV]"' /
     &  '     Display [or specify SCENH and SCENV to be] the',
     &  ' horizontal and vertical' /
     &  '     coordinates of the plot center.' //
     &  '  "plot zoom [[all,h,v] ZOOMULT]' /
     &  '     Display [or multiply by ZOOMULT] the zoom factor' /
     &  '     [for both axes, or the horizontal or vertical axis].' /
     &  '     Use ZOOMULT = 0 to make the zoom factor 1 (no zoom).' //
     &  '  Synonyms:  [delete, remove, del], [help, h],',
     &  ' [limits, lim],' /
     &  '   [marker, mark, m], [point, pnt, pt, p], [status, stat].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then

        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8113) ans
        endif
        write (   6, 8114)
        write (iout, 8114)

        go to 999

      endif

c=======================================================================********

c.... Input line:  plot status

 8200 format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Command "plot status" does not have 2 words:  ',i3)

      if ((aword(2) .eq. 'status') .or.
     &    (aword(2) .eq. 'stat'))  then

        if (nword .ne. 2) then
          write (   6, 8200) ainput, ninput, nword
          write (iout, 8200) ainput, ninput, nword
          go to 999
        endif

c....   Display the plot status.

 8205   format ('Plot status:                   ',a)

        write (   6, 8205) aplstat
        write (iout, 8205) aplstat

c....   Display the plot title.

 8210   format ('Plot title:' / 2x,a)
 8215   format ('Plot title:                    (blank)')

        call markget (aplmarkt, nmark)

        if (nmark .gt. 0) then
          write (   6, 8210) amarkt(nmark)
          write (iout, 8210) amarkt(nmark)
        else
          write (   6, 8215)
          write (iout, 8215)
        endif

c....   Display the horizontal axis coordinate, label, limits, zoom factor.

 8220   format ('Horizontal axis coordinate:    ',a)
 8230   format ('Horizontal axis label:' / 2x,a)
 8235   format ('Horizontal axis label:         (blank)')
 8224   format ('Horizontal axis limits:        (includes all',
     &    ' plot points)')
 8225   format ('Horizontal axis limits:      ',1p2e20.12)
 8273   format ('Horizontal axis zoom factor: ',1pe20.12)

        call markget (aplmarkh, nmark)

        if (nmark .gt. 0) then
          write (   6, 8230) amarkt(nmark)
          write (iout, 8230) amarkt(nmark)
        else
          write (   6, 8235)
          write (iout, 8235)
        endif

        write (   6, 8220) axish
        write (iout, 8220) axish

        if (nplimh .eq. 0) then
          write (   6, 8224)
          write (iout, 8224)
        else
          write (   6, 8225) splhmin, splhmax
          write (iout, 8225) splhmin, splhmax
        endif

        if (zoomplh .ne. 1.0) then
          write (   6, 8273) zoomplh
          write (iout, 8273) zoomplh
        endif

c....   Display the vertical axis coordinate, label, limits, zoom factor.

 8250   format ('Vertical   axis coordinate:    ',a)
 8260   format ('Vertical   axis label:' / 2x,a)
 8265   format ('Vertical   axis label:         (blank)')
 8254   format ('Vertical   axis limits:        (includes all',
     &    ' plot points)')
 8255   format ('Vertical   axis limits:      ',1p2e20.12)
 8274   format ('Vertical   axis zoom factor: ',1pe20.12)

        call markget (aplmarkv, nmark)

        if (nmark .gt. 0) then
          write (   6, 8260) amarkt(nmark)
          write (iout, 8260) amarkt(nmark)
        else
          write (   6, 8265)
          write (iout, 8265)
        endif

        write (   6, 8250) axisv
        write (iout, 8250) axisv

        if (nplimv .eq. 0) then
          write (   6, 8254)
          write (iout, 8254)
        else
          write (   6, 8255) splvmin, splvmax
          write (iout, 8255) splvmin, splvmax
        endif

        if (zoomplv .ne. 1.0) then
          write (   6, 8274) zoomplh
          write (iout, 8274) zoomplh
        endif

c....   Display the center point of plot.

 8270   format ('Center point:                ',1p2e20.12)
 8272   format ('Center point:                  (center of plot',
     &    ' points)')

        if (nplcen .ne. 0) then
          write (   6, 8270) splhcen, splvcen
          write (iout, 8270) splhcen, splvcen
        else
          write (   6, 8272)
          write (iout, 8272)
        endif

c....   Display the current plot point marker and label.

 8275   format ('Current plot point marker:     ',a )
 8280   format ('Current plot point label:' / 2x,a)
 8285   format ('Current plot point label:      (blank)')

        write (   6, 8275) aptmarks
        write (iout, 8275) aptmarks

        call markget (aptmarks, nmark)

        if (nmark .gt. 0) then
          write (   6, 8280) amarkt(nmark)
          write (iout, 8280) amarkt(nmark)
        else
          write (   6, 8285)
          write (iout, 8285)
        endif

c....   Display the size of the plot array.

 8290   format (/ 'Number of plot array columns:',i5 /
     &            'Number of plot array lines:  ',i5 )

        write (   6, 8290) nplcols, nplines
        write (iout, 8290) nplcols, nplines

c....   Display the number of plot points.

 8295   format (/ 'Number of plot points:       ',i5 )

        write (   6, 8295) nplpts
        write (iout, 8295) nplpts

        go to 999
      endif                           ! Input line "plot status".

c=======================================================================********

c.... Input line:  plot size [NCOLUMNS NLINES]

      if (aword(2) .eq. 'size') then

 8305   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Command "plot size" does not have 2 or 4 words:  ',i3)

        if ((nword .ne. 2) .and.
     &      (nword .ne. 4)) then
          write (   6, 8305) ainput, ninput, nword
          write (iout, 8305) ainput, ninput, nword
          go to 999
        endif

        if (nword .eq. 4) then

          nerr = 0

          call varfind (3, 4)

 8310     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Third argument is not an integer => 2:  ',a)

          if ((mtype(3) .ne. 2)  .or.
     &        (iword(3) .lt. 2)) then
            write (   6, 8310) ainput, ninput, aword(3)
            write (iout, 8310) ainput, ninput, aword(3)
            nerr = 1
          endif

 8340     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Fourth argument is not an integer => 2:  ',a)

          if ((mtype(4) .ne. 2)  .or.
     &        (iword(4) .lt. 2)) then
            write (   6, 8340) ainput, ninput, aword(4)
            write (iout, 8340) ainput, ninput, aword(4)
            nerr = 1
          endif

 8350     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  NCOLUMNS >',i4,':  ',a)

          if (iword(3) .gt. (nplcolm - 2)) then
            nplcolmx = nplcolm - 2
            write (   6, 8350) ainput, ninput, nplcolmx, aword(3)
            write (iout, 8350) ainput, ninput, nplcolmx, aword(3)
            nerr = 1
          endif

 8352     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  NLINES >  ',i4,':  ',a)

          if (iword(4) .gt. (nplinem - 2)) then
            nplinemx = nplinem - 2
            write (   6, 8352) ainput, ninput, nplinemx, aword(4)
            write (iout, 8352) ainput, ninput, nplinemx, aword(4)
            nerr = 1
          endif

          if (nerr .ne. 0) go to 999

          nplcols = iword(3)
          nplines = iword(4)

        endif                         ! Specified new plot array size.

c....   Display the plot array size.

        write (   6, 8290) nplcols, nplines
        write (iout, 8290) nplcols, nplines

        go to 999

      endif                          ! Input line "plot size [NCOLUMNS NLINES]".

c=======================================================================********

c.... Input line:  plot off

      if (aword(2) .eq. 'off') then   ! Turn storage of plot points off.

 8602   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Command "plot off" does not have 2 words:  ',i3)

        if (nword .ne. 2) then
          write (   6, 8602) ainput, ninput, nword
          write (iout, 8602) ainput, ninput, nword
          go to 999
        endif

        aplstat    = 'off'            ! Plotting status.

c....   Display the plot status.

        write (   6, 8205) aplstat
        write (iout, 8205) aplstat

c....   Display the number of plot points.

        write (   6, 8295) nplpts
        write (iout, 8295) nplpts

c....   Display the plot point mark and label.

        write (   6, 8275) aptmarks
        write (iout, 8275) aptmarks

        call markget (aptmarks, nmark)

        if (nmark .gt. 0) then
          write (   6, 8280) amarkt(nmark)
          write (iout, 8280) amarkt(nmark)
        else
          write (   6, 8285)
          write (iout, 8285)
        endif

        go to 999

      endif

c=======================================================================********

c.... Input line:  plot on

      if (aword(2) .eq. 'on') then    ! Turn storage of plot point on.

 8604   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Command "plot on" does not have 2 words:  ',i3)

        if (nword .ne. 2) then
          write (   6, 8604) ainput, ninput, nword
          write (iout, 8604) ainput, ninput, nword
          go to 999
        endif

        aplstat    = 'on'             ! Plotting status.

c....   Display the plot status.

        write (   6, 8205) aplstat
        write (iout, 8205) aplstat

c....   Display the number of plot points.

        write (   6, 8295) nplpts
        write (iout, 8295) nplpts

c....   Display the plot point mark and label.

        write (   6, 8275) aptmarks
        write (iout, 8275) aptmarks

        call markget (aptmarks, nmark)

        if (nmark .gt. 0) then
          write (   6, 8280) amarkt(nmark)
          write (iout, 8280) amarkt(nmark)
        else
          write (   6, 8285)
          write (iout, 8285)
        endif

        go to 999

      endif                           ! Input line "plot on".

c=======================================================================********

c.... Input line:  plot delete [all]
c.... Input line:  plot delete marker AMNAME1 ... AMNAME2 [thru] AMNAME3 ...
c.... Input line:  plot delete index N1 ... N2 [thru] N3 ...

      if ((aword(2) .eq. 'delete') .or.
     &    (aword(2) .eq. 'remove') .or.
     &    (aword(2) .eq. 'del'))   then

c....   Find the plot points to delete.

        if ((nword .eq. 2) .or.
     &     ((nword .eq. 3) .and.
     &      (aword(3) .eq. 'all'))) then  ! Delete all plot points.

          nplpts   = 0                ! Number of plot points.
          nplcen   = 0                ! No plot center is specified.
          zoomplh  = 1.0              ! Zoom factor for horizontal axis.
          zoomplv  = 1.0              ! Zoom factor for vertical axis.
          aptmarks = '*'              ! Current plot point marker.

c....     Display the plot status.

          write (   6, 8205) aplstat
          write (iout, 8205) aplstat

c....     Display the number of plot points.

          write (   6, 8295) nplpts
          write (iout, 8295) nplpts

c....     Display the plot point mark and label.

          write (   6, 8275) aptmarks
          write (iout, 8275) aptmarks

          call markget (aptmarks, nmark)

          if (nmark .gt. 0) then
            write (   6, 8280) amarkt(nmark)
            write (iout, 8280) amarkt(nmark)
          else
            write (   6, 8285)
            write (iout, 8285)
          endif

          go to 999

        endif                         ! Deleted all plot points.

c....   See if deleting plot points by marker.

        if ((aword(3) .eq. 'marker') .or.
     &      (aword(3) .eq. 'mark')   .or.
     &      (aword(3) .eq. 'm'))     then

 8606     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  No markers are specified:  ',i3)

          if (nword .le. 3) then
            write (   6, 8606) ainput, ninput
            write (iout, 8606) ainput, ninput
            go to 999
          endif

          nerr = 0

 8610     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Marker has more than one character:  ',a)

          do 610 nw = 4, nword
            if (aword(nw) .eq. 'thru') go to 610
            if (lword(nw) .ne. 1) then
              write (   6, 8610) ainput, ninput, aword(nw)
              write (iout, 8610) ainput, ninput, aword(nw)
              go to 999
            endif
  610     continue

c....     Test for "thru" errors.

 8616     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Word "thru" duplicated or misplaced.')
 8618     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Lower mark > upper mark:  ',a,2x,a)

          do 620 nw = 4, nword        ! Loop over input line words.

            if (aword(nw) .eq. 'thru') then  ! Found option "thru".

              if ((nw          .eq. 4     )  .or.
     &            (aword(nw-2) .eq. 'thru')  .or.
     &            (aword(nw-1) .eq. 'thru')  .or.
     &            (aword(nw+1) .eq. 'thru')  .or.
     &            (aword(nw+2) .eq. 'thru')  .or.
     &            (nw          .eq. nword )) then
                nerr = 1
                write (   6, 8616) ainput, ninput
                write (iout, 8616) ainput, ninput
                go to 620
              endif

              call compsubs (aword(nw-1), aword(nw+1), nsmall, nerrc)
              if (nsmall .eq. 2) then
                write (   6, 8618) ainput, ninput,
     &            aword(nw-1), aword(nw+1)
                write (iout, 8618) ainput, ninput,
     &            aword(nw-1), aword(nw+1)
                nerr = 1
                go to 620
              endif

            endif                     ! Found option "thru".

  620     continue                    ! End of loop over input line words.

          if (nerr .ne. 0) go to 999

          nplptx = nplpts

          do 625 nw = 4, nword        ! Loop over input line words.

c....       Test for option "thru".

            if (aword(nw) .eq. 'thru') then  ! Found option "thru".

              if ((nw          .eq. 4     )  .or.
     &            (aword(nw-2) .eq. 'thru')  .or.
     &            (aword(nw-1) .eq. 'thru')  .or.
     &            (aword(nw+1) .eq. 'thru')  .or.
     &            (aword(nw+2) .eq. 'thru')  .or.
     &            (nw          .eq. nword )) then
                write (   6, 8616) ainput, ninput
                write (iout, 8616) ainput, ninput
                go to 999
              endif

              amarkmn = aword(nw-1)
              amarkmx = aword(nw+1)

 8620         format ('Del plot pt ',i6,2x,a1,2x,a6,1pe20.12,
     &                                        2x,a6,1pe20.12)

              do 622 n = 1, nplpts    ! Loop over plot points.
                if (aplotm(n) .eq. ' ') go to 622

                call compsubs (aplotm(n), amarkmn, nsmall, nerrc)
                if (nsmall .eq. 1) go to 622
                call compsubs (aplotm(n), amarkmx, nsmall, nerrc)
                if (nsmall .eq. 2) go to 622

c....             Found an existing plot point to delete.

                  call plotvar (xplot(n), yplot(n), zplot(n),
     &                          spth, sptv, nerrg)

                  write (   6, 8620) n, aplotm(n), axish, spth,
     &              axisv, sptv
                  write (iout, 8620) n, aplotm(n), axish, spth,
     &              axisv, sptv

                  nplptx    = nplptx - 1
                  aplotm(n) = ' '

  622         continue                ! End of loop over plot points.

              go to 625

            endif                     ! Tested for option "thru".

            if (aword(nw-1) .eq. 'thru') go to 625
            if (aword(nw+1) .eq. 'thru') go to 625

c....       Test for a marker name.

            nplx = 0

            do 624 n = 1, nplpts      ! Loop over plot points.
              if (aplotm(n) .eq. aword(nw)) then  ! Delete this plot point.

c....           Found an existing plot point to delete.

                call plotvar (xplot(n), yplot(n), zplot(n),
     &                        spth, sptv, nerrg)

                write (   6, 8620) n, aplotm(n), axish, spth,
     &            axisv, sptv
                write (iout, 8620) n, aplotm(n), axish, spth,
     &            axisv, sptv

                nplx      = nplx + 1
                nplptx    = nplptx - 1
                aplotm(n) = ' '

              endif
  624       continue                  ! End of loop over plot points.

 8624       format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  No plot points have this marker:  ',a)

            if (nplx .eq. 0) then     ! No points were deleted.
              write (   6, 8624) ainput, ninput, aword(nw)
              write (iout, 8624) ainput, ninput, aword(nw)
              go to 999
            endif                     ! Tested for number of points deleted.

  625     continue                    ! End of loop over input line words.

c....     See how many plot points are left.

          if (nplptx .eq. 0) then     ! Deleted all plot points.

            nplpts   = 0              ! Number of plot points.
            nplcen   = 0              ! No plot center is specified.
            zoomplh  = 1.0            ! Zoom factor for horizontal axis.
            zoomplv  = 1.0            ! Zoom factor for vertical axis.
            aptmarks = '*'            ! Current plot point marker.

c....       Display the plot status.

            write (   6, 8205) aplstat
            write (iout, 8205) aplstat

c....       Display the number of plot points.

            write (   6, 8295) nplpts
            write (iout, 8295) nplpts

c....       Display the plot point mark and label.

            write (   6, 8275) aptmarks
            write (iout, 8275) aptmarks

            call markget (aptmarks, nmark)

            if (nmark .gt. 0) then
              write (   6, 8280) amarkt(nmark)
              write (iout, 8280) amarkt(nmark)
            else
              write (   6, 8285)
              write (iout, 8285)
            endif

            go to 999

          elseif (nplptx .lt. nplpts) then  ! Deleted some plot points.

c....       Repack the plot points.

            nn = 0
            do 630 n = 1, nplpts
              if (aplotm(n) .eq. ' ') go to 630
              nn = nn + 1
              aplotm(nn) = aplotm(n)
              xplot(nn)  = xplot(n)
              yplot(nn)  = yplot(n)
              zplot(nn)  = zplot(n)
  630       continue
            nplpts = nn

          endif                       ! Tested number of plot points.

c....     Display the number of plot points.

          write (   6, 8295) nplpts
          write (iout, 8295) nplpts

          go to 999

        endif                         ! Input line "plot delete marker ... ".

c....   See if deleting plot points by index.

        if (aword(3) .eq. 'index') then

 8632     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  No indices are specified.')

          if (nword .le. 3) then
            write (   6, 8632) ainput, ninput
            write (iout, 8632) ainput, ninput
            go to 999
          endif

          nerr = 0

 8635     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Index is not a positive integer:  ',a)

          do 640 nw = 4, nword
            if (aword(nw) .eq. 'thru') go to 640

            call varfind (nw, nw)

            if ((mtype(nw) .ne. 2)  .or.
     &          (iword(nw) .le. 0)) then
              write (   6, 8635) ainput, ninput, aword(nw)
              write (iout, 8635) ainput, ninput, aword(nw)
              go to 999
            endif
  640     continue

c....     Test for "thru" errors.

 8642     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Key word "thru" in bad position at word:  ',i4)
 8648     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Lower limit > upper limit:  ',2a)
 8649     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Lower limit > upper limit:  ',2i5)
 8650     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Upper limit > number of plot points:  ',i5)

          do 650 nw = 4, nword        ! Loop over input line words.

            if (aword(nw) .eq. 'thru') then  ! Found option "thru".


              if ((nw          .eq. 4     )  .or.
     &            (aword(nw-2) .eq. 'thru')  .or.
     &            (aword(nw-1) .eq. 'thru')  .or.
     &            (aword(nw+1) .eq. 'thru')  .or.
     &            (aword(nw+2) .eq. 'thru')  .or.
     &            (nw          .eq. nword )) then
                nerr = 1
                write (   6, 8616) ainput, ninput
                write (iout, 8616) ainput, ninput
                go to 650
              endif

              call compsubs (aword(nw-1), aword(nw+1), nsmall, nerrc)
              if (nsmall .eq. 2) then
                write (   6, 8648) ainput, ninput,
     &            aword(nw-1), aword(nw+1)
                write (iout, 8648) ainput, ninput,
     &            aword(nw-1), aword(nw+1)
                nerr = 1
                go to 650
              endif

              if (iword(nw-1) .gt. iword(nw+1)) then
                write (   6, 8649) ainput, ninput,
     &            iword(nw-1), iword(nw+1)
                write (iout, 8649) ainput, ninput,
     &            iword(nw-1), iword(nw+1)
                nerr = 1
                go to 650
              endif

              if (iword(nw+1) .gt. nplpts) then
                write (   6, 8650) ainput, ninput, iword(nw+1)
                write (iout, 8650) ainput, ninput, iword(nw+1)
                nerr = 1
                go to 650
              endif

            endif                     ! Found option "thru".

  650     continue                    ! End of loop over input line words.

          if (nerr .ne. 0) go to 999

          nplptx = nplpts

          do 670 nw = 4, nword        ! Loop over input line words.

c....       Test for option "thru".

            if (aword(nw) .eq. 'thru') then  ! Found option "thru".

 8655         format ('Del plot pt ',i6,2x,a1,2x,a6,1pe20.12,
     &                                        2x,a6,1pe20.12)

              do 660 n = iword(nw-1), iword(nw+1)  ! Loop over plot points.

                call plotvar (xplot(n), yplot(n), zplot(n),
     &                        spth, sptv, nerrg)

                write (   6, 8655) n, aplotm(n), axish, spth,
     &            axisv, sptv
                write (iout, 8655) n, aplotm(n), axish, spth,
     &            axisv, sptv

                nplptx    = nplptx - 1
                aplotm(n) = ' '

  660         continue                ! End of loop over plot point indices.

              go to 670

            endif                     ! Tested for option "thru".

            if (aword(nw-1) .eq. 'thru') go to 670
            if (aword(nw+1) .eq. 'thru') go to 670

c....       Test for an index.

 8665       format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Index is larger than number of plot points:  ',a)

            if (iword(nw) .gt. nplpts) then
              write (   6, 8665) ainput, ninput, aword(nw)
              write (iout, 8665) ainput, ninput, aword(nw)
              go to 670
            endif

            n = iword(nw)

            call plotvar (xplot(n), yplot(n), zplot(n),
     &                    spth, sptv, nerrg)

            write (   6, 8655) n, aplotm(n), axish, spth,
     &        axisv, sptv
            write (iout, 8655) n, aplotm(n), axish, spth,
     &        axisv, sptv

            nplptx    = nplptx - 1
            aplotm(n) = ' '

  670     continue                    ! End of loop over input line words.

c....     See how many points are left.

          if (nplptx .eq. 0) then     ! Deleted all plot points.

            nplpts   = 0              ! Number of plot points.
            nplcen   = 0              ! No plot center is specified.
            zoomplh  = 1.0            ! Zoom factor for horizontal axis.
            zoomplv  = 1.0            ! Zoom factor for vertical axis.
            aptmarks = '*'            ! Current plot point marker.

c....       Display the plot status.

            write (   6, 8205) aplstat
            write (iout, 8205) aplstat

c....       Display the number of plot points.

            write (   6, 8295) nplpts
            write (iout, 8295) nplpts

c....       Display the plot point mark and label.

            write (   6, 8275) aptmarks
            write (iout, 8275) aptmarks

            call markget (aptmarks, nmark)

            if (nmark .gt. 0) then
              write (   6, 8280) amarkt(nmark)
              write (iout, 8280) amarkt(nmark)
            else
              write (   6, 8285)
              write (iout, 8285)
            endif

            go to 999

          elseif (nplptx .lt. nplpts) then  ! Deleted some plot points.

c....       Repack the plot points.

            nn = 0
            do 680 n = 1, nplpts
              if (aplotm(n) .eq. ' ') go to 680
              nn = nn + 1
              aplotm(nn) = aplotm(n)
              xplot(nn)  = xplot(n)
              yplot(nn)  = yplot(n)
              zplot(nn)  = zplot(n)
  680       continue
            nplpts = nn

          endif                       ! Tested number of plot points.

c....     Display the number of plot points.

          write (   6, 8295) nplpts
          write (iout, 8295) nplpts

          go to 999

        endif                         ! Input line "plot delete index ... ".

c....   Input line "plot delete ..." not recognized.

        write (   6, 8810) ainput, ninput
        write (iout, 8810) ainput, ninput

        go to 999

      endif                           ! Input line "plot delete"

c=======================================================================********

c.... Input line:  plot point [all]
c.... Input line:  plot point marker AMNAME1 ... AMNAME2 [thru] AMNAME3 ...
c.... Input line:  plot point index N1 ... N2 [thru] N3 ...

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))    then

        if (nplpts .le. 0) then
          write (   6, 8295) nplpts
          write (iout, 8295) nplpts
          go to 999
        endif

        if ((nword .eq. 2) .or.
     &     ((nword .eq. 3) .and.
     &      (aword(3) .eq. 'all'))) then

 8685     format ('Plot point  ',i6,2x,a1,2x,a6,1pe20.12,2x,a6,1pe20.12)

          nplptx = 0

          spthmin =  1.e99
          spthmax = -1.e99
          sptvmin =  1.e99
          sptvmax = -1.e99

          do 690 n = 1, nplpts
            if (aplotm(n) .eq. ' ') go to 690

            call plotvar (xplot(n), yplot(n), zplot(n),
     &                    spth, sptv, nerrg)

            nplptx = nplptx + 1
            if (spth .lt. spthmin) spthmin = spth
            if (spth .gt. spthmax) spthmax = spth
            if (sptv .lt. sptvmin) sptvmin = sptv
            if (sptv .gt. sptvmax) sptvmax = sptv

            write (   6, 8685) n, aplotm(n), axish, spth, axisv, sptv
            write (iout, 8685) n, aplotm(n), axish, spth, axisv, sptv

  690     continue

 8690     format (/ 'Horizontal axis point range: ',1p2e20.12)

          write (   6, 8690) spthmin, spthmax
          write (iout, 8690) spthmin, spthmax

          if (nplimh .eq. 0) then
            write (   6, 8224)
            write (iout, 8224)
          else
            write (   6, 8225) splhmin, splhmax
            write (iout, 8225) splhmin, splhmax
          endif

 8695     format ('Vertical   axis point range: ',1p2e20.12)

          write (   6, 8695) sptvmin, sptvmax
          write (iout, 8695) sptvmin, sptvmax

          if (nplimv .eq. 0) then
            write (   6, 8254)
            write (iout, 8254)
          else
            write (   6, 8255) splvmin, splvmax
            write (iout, 8255) splvmin, splvmax
          endif

          write (   6, 8295) nplptx
          write (iout, 8295) nplptx

          go to 999

        endif                         ! Input line "plot point [all]".

c....   See if displaying plot points by marker.

        if ((aword(3) .eq. 'marker') .or.
     &      (aword(3) .eq. 'mark')   .or.
     &      (aword(3) .eq. 'm'))     then

          if (nword .le. 3) then
            write (   6, 8606) ainput, ninput
            write (iout, 8606) ainput, ninput
            go to 999
          endif

          nerr = 0

          do 710 nw = 4, nword
            if (aword(nw) .eq. 'thru') go to 710
            if (lword(nw) .ne. 1) then
              write (   6, 8710) ainput, ninput, aword(nw)
              write (iout, 8710) ainput, ninput, aword(nw)
              go to 999
            endif
  710     continue

c....     Test for "thru" errors.

          do 720 nw = 4, nword        ! Loop over input line words.

            if (aword(nw) .eq. 'thru') then  ! Found option "thru".


              if ((nw          .eq. 4     )  .or.
     &            (aword(nw-2) .eq. 'thru')  .or.
     &            (aword(nw-1) .eq. 'thru')  .or.
     &            (aword(nw+1) .eq. 'thru')  .or.
     &            (aword(nw+2) .eq. 'thru')  .or.
     &            (nw          .eq. nword )) then
                nerr = 1
                write (   6, 8616) ainput, ninput
                write (iout, 8616) ainput, ninput
                go to 720
              endif

              call compsubs (aword(nw-1), aword(nw+1), nsmall, nerrc)
              if (nsmall .eq. 2) then
                write (   6, 8618) ainput, ninput,
     &            aword(nw-1), aword(nw+1)
                write (iout, 8618) ainput, ninput,
     &            aword(nw-1), aword(nw+1)
                nerr = 1
                go to 720
              endif

            endif                     ! Found option "thru".

  720     continue                    ! End of loop over input line words.

          if (nerr .ne. 0) go to 999

          do 725 nw = 4, nword        ! Loop over input line words.

c....       Test for option "thru".

            if (aword(nw) .eq. 'thru') then  ! Found option "thru".

              amarkmn = aword(nw-1)
              amarkmx = aword(nw+1)

              do 722 n = 1, nplpts    ! Loop over plot points.
                if (aplotm(n) .eq. ' ') go to 722


                call compsubs (aplotm(n), amarkmn, nsmall, nerrc)
                if (nsmall .eq. 1) go to 722
                call compsubs (aplotm(n), amarkmx, nsmall, nerrc)
                if (nsmall .eq. 2) go to 722

                  call plotvar (xplot(n), yplot(n), zplot(n),
     &                          spth, sptv, nerrg)

                  write (   6, 8685) n, aplotm(n), axish, spth,
     &              axisv, sptv
                  write (iout, 8685) n, aplotm(n), axish, spth,
     &              axisv, sptv

  722         continue                ! End of loop over plot points.

              go to 725

            endif                     ! Tested for option "thru".

            if (aword(nw-1) .eq. 'thru') go to 725
            if (aword(nw+1) .eq. 'thru') go to 725

c....       Test for a marker name.

            nplx = 0

            do 724 n = 1, nplpts      ! Loop over plot points.
              if (aplotm(n) .eq. aword(nw)) then  ! Display this plot point.

c....           Found an existing plot point to display.

                nplx = nplx + 1

                call plotvar (xplot(n), yplot(n), zplot(n),
     &                        spth, sptv, nerrg)

                write (   6, 8685) n, aplotm(n), axish, spth,
     &            axisv, sptv
                write (iout, 8685) n, aplotm(n), axish, spth,
     &            axisv, sptv

              endif                   ! Displayed plot point.
  724       continue                  ! End of loop over plot points.

            if (nplx .eq. 0) then     ! No points were displayed.
              write (   6, 8624) ainput, ninput, aword(nw)
              write (iout, 8624) ainput, ninput, aword(nw)
              go to 999
            endif                     ! Tested for number of points displayed.

  725     continue                    ! End of loop over input line words.

c....     Display the number of plot points.

          write (   6, 8295) nplpts
          write (iout, 8295) nplpts

          go to 999

        endif                         ! Input line "plot point marker ... ".

c....   See if displaying plot points by index.

        if (aword(3) .eq. 'index') then

          if (nword .le. 3) then
            write (   6, 8632) ainput, ninput
            write (iout, 8632) ainput, ninput
            go to 999
          endif

          nerr = 0

          do 740 nw = 4, nword
            if (aword(nw) .eq. 'thru') go to 740

            call varfind (nw, nw)

            if ((mtype(nw) .ne. 2)  .or.
     &          (iword(nw) .le. 0)) then
              write (   6, 8635) ainput, ninput, aword(nw)
              write (iout, 8635) ainput, ninput, aword(nw)
              go to 999
            endif
  740     continue

c....     Test for "thru" errors.

          do 750 nw = 4, nword        ! Loop over input line words.

            if (aword(nw) .eq. 'thru') then  ! Found option "thru".

              if ((nw          .eq. 4     )  .or.
     &            (aword(nw-2) .eq. 'thru')  .or.
     &            (aword(nw-1) .eq. 'thru')  .or.
     &              (aword(nw+1) .eq. 'thru')  .or.
     &            (aword(nw+2) .eq. 'thru')  .or.
     &            (nw          .eq. nword )) then
                nerr = 1
                write (   6, 8616) ainput, ninput
                write (iout, 8616) ainput, ninput
                go to 750
              endif

              if (iword(nw-1) .gt. iword(nw+1)) then
                write (   6, 8649) ainput, ninput,
     &            iword(nw-1), iword(nw+1)
                write (iout, 8649) ainput, ninput,
     &            iword(nw-1), iword(nw+1)
                nerr = 1
                go to 750
              endif

              if (iword(nw+1) .gt. nplpts) then
                write (   6, 8650) ainput, ninput, iword(nw+1)
                write (iout, 8650) ainput, ninput, iword(nw+1)
                nerr = 1
                go to 750
              endif

            endif                     ! Found option "thru".

  750     continue                    ! End of loop over input line words.

          if (nerr .ne. 0) go to 999

          nplptx = nplpts

          do 770 nw = 4, nword        ! Loop over input line words.

c....       Test for option "thru".

            if (aword(nw) .eq. 'thru') then  ! Found option "thru".

              do 760 n = iword(nw-1), iword(nw+1)  ! Loop over plot points.

                call plotvar (xplot(n), yplot(n), zplot(n),
     &                        spth, sptv, nerrg)

                write (   6, 8685) n, aplotm(n), axish, spth,
     &            axisv, sptv
                write (iout, 8685) n, aplotm(n), axish, spth,
     &            axisv, sptv

  760         continue                ! End of loop over plot point indices.

              go to 770

            endif                     ! Tested for option "thru".

            if (aword(nw-1) .eq. 'thru') go to 770
            if (aword(nw+1) .eq. 'thru') go to 770

c....       Test for an index.

            if (iword(nw) .gt. nplpts) then
              write (   6, 8665) ainput, ninput, aword(nw)
              write (iout, 8665) ainput, ninput, aword(nw)
              go to 770
            endif

            n = iword(nw)

            call plotvar (xplot(n), yplot(n), zplot(n),
     &                    spth, sptv, nerrg)

            write (   6, 8685) n, aplotm(n), axish, spth,
     &        axisv, sptv
            write (iout, 8685) n, aplotm(n), axish, spth,
     &        axisv, sptv

  770     continue                    ! End of loop over input line words.

c....     Display the number of plot points.

          write (   6, 8295) nplpts
          write (iout, 8295) nplpts

          go to 999

        endif                         ! Input line "plot point index ... ".

c....   Input line "plot point ..." is not recognized.

        write (   6, 8810) ainput, ninput
        write (iout, 8810) ainput, ninput

        go to 999

      endif                           ! Input line "plot point".

c=======================================================================********

c.... Input line:  plot title [AMNAME]

      if (aword(2) .eq. 'title') then

 8355   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Command "plot title" does not have 2 or 3 words:  ',i3)

        if ((nword .ne. 2) .and.
     &      (nword .ne. 3)) then
          write (   6, 8355) ainput, ninput, nword
          write (iout, 8355) ainput, ninput, nword
          go to 999
        endif

        if (nword .eq. 3) then

 8360     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Word 3 is not a single character:  ',a)

          if (lword(3) .ne. 1) then
            write (   6, 8360) ainput, ninput, aword(3)
            write (iout, 8360) ainput, ninput, aword(3)
            go to 999
          endif

          call markget (aword(3), nmark)

 8365     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named marker does not exist:  ',a)

          if (nmark .eq. 0) then
            write (   6, 8365) ainput, ninput, aword(3)
            write (iout, 8365) ainput, ninput, aword(3)
            go to 999
          endif

          aplmarkt = aword(3)

        endif                         ! Specified new plot title marker.

c....   Display the plot title.

        call markget (aplmarkt, nmark)

        if (nmark .gt. 0) then
          write (   6, 8210) amarkt(nmark)
          write (iout, 8210) amarkt(nmark)
        else
          write (   6, 8215)
          write (iout, 8215)
        endif

        go to 999

      endif                           ! Input line "plot title [AMNAME]".

c=======================================================================********

c.... Input line:  plot axis [[h,v] COORD AMNAME]

      if (aword(2) .eq. 'axis') then

 8370   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Command "plot axis" does not have 2 or 5 words:  ',i3)

        if ((nword .ne. 2) .and.
     &      (nword .ne. 5)) then
          write (   6, 8370) ainput, ninput, nword
          write (iout, 8370) ainput, ninput, nword
          go to 999
        endif

        if (nword .eq. 5) then

          nerr = 0

 8375     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Word 4 is not x, y, z, rcyl, rsph, theta or phi:  ',a)

          if ((aword(4) .ne. 'x')     .and.
     &        (aword(4) .ne. 'y')     .and.
     &        (aword(4) .ne. 'z')     .and.
     &        (aword(4) .ne. 'rcyl')  .and.
     &        (aword(4) .ne. 'rsph')  .and.
     &        (aword(4) .ne. 'theta') .and.
     &        (aword(4) .ne. 'phi'))  then
            write (   6, 8375) ainput, ninput, aword(4)
            write (iout, 8375) ainput, ninput, aword(4)
            nerr = 1
          endif

          call markget (aword(5), nmark)

          if (nmark .eq. 0) then
            write (   6, 8365) ainput, ninput, aword(5)
            write (iout, 8365) ainput, ninput, aword(5)
            nerr = 1
          endif

          if (nerr .ne. 0) go to 999

 8374     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Word 3 is not h or v:  ',a)

          if (aword(3) .eq. 'h') then
            axish  = aword(4)
            aplmarkh = aword(5)
          elseif (aword(3) .eq. 'v') then
            axisv  = aword(4)
            aplmarkv = aword(5)
          else
            write (   6, 8374) ainput, ninput, aword(3)
            write (iout, 8374) ainput, ninput, aword(3)
            go to 999
          endif

        endif                         ! Specified new axis coordinate and label.

c....   Display the horizontal axis variable and label.

        write (   6, 8220) axish
        write (iout, 8220) axish

        call markget (aplmarkh, nmark)

        if (nmark .gt. 0) then
          write (   6, 8230) amarkt(nmark)
          write (iout, 8230) amarkt(nmark)
        else
          write (   6, 8235)
          write (iout, 8235)
        endif

c....   Display the vertical axis variable and label.

        write (   6, 8250) axisv
        write (iout, 8250) axisv

        call markget (aplmarkv, nmark)

        if (nmark .gt. 0) then
          write (   6, 8260) amarkt(nmark)
          write (iout, 8260) amarkt(nmark)
        else
          write (   6, 8265)
          write (iout, 8265)
        endif

        go to 999

      endif                           ! Input line "plot axis".

c=======================================================================********

c.... Input line:  plot marker [AMNAME]
c.... Input line:  plot marker all

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'm'))     then

 8390   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Command "plot mark" does not have 2 or 3 words:  ',i3)

        if ((nword .ne. 2)  .and.
     &      (nword .ne. 3)) then
          write (   6, 8390) ainput, ninput, nword
          write (iout, 8390) ainput, ninput, nword
          go to 999
        endif

        if (nword .eq. 3) then

 8400     format ('Plot point marks and labels:')
 8405     format ('mark ',a1,2x,'(blank)')
 8410     format ('mark ',a1,2x,a72)

          if (aword(3) .eq. 'all') then  ! Display all plot point marks.

            write (   6, 8400)
            write (iout, 8400)
            do 410 n = 1, nptmarks

              call markget (aptmark(n), nmark)

              if (nmark .eq. 0) then
                write (   6, 8405) aptmark(n)
                write (iout, 8405) aptmark(n)
              else
                write (   6, 8410) aptmark(n), amarkt(nmark)
                write (iout, 8410) aptmark(n), amarkt(nmark)
              endif
  410       continue

c....       Display the current plot point mark and label.

            write (   6, 8275) aptmarks
            write (iout, 8275) aptmarks

            call markget (aptmarks, nmark)

            if (nmark .gt. 0) then
              write (   6, 8280) amarkt(nmark)
              write (iout, 8280) amarkt(nmark)
            else
              write (   6, 8285)
              write (iout, 8285)
            endif

            go to 999

          endif                       ! Displayed all plot point marks.

c....     Specify the current plot point mark.

          if (lword(3) .ne. 1) then
            write (   6, 8360) ainput, ninput, aword(3)
            write (iout, 8360) ainput, ninput, aword(3)
            go to 999
          endif

          call markget (aword(3), nmark)

c???          if (nmark .eq. 0) then
c???            write (   6, 8365) ainput, ninput, aword(3)
c???            write (iout, 8365) ainput, ninput, aword(3)
c???            go to 999
c???          endif

          aptmarks = aword(3)

c....     See if plot point mark is new, and if so, add to list.

          do 420 n = 1, nptmarks
            if (aptmarks .eq. aptmark(n)) go to 430
  420     continue

 8420   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of plot point marks has reached maximum:  ',i3)

          if (nptmarks .eq. nptmarkm) then
            write (   6, 8420) ainput, ninput, nptmarks
            write (iout, 8420) ainput, ninput, nptmarks
            go to 999
          endif

          nptmarks = nptmarks + 1
          aptmark(nptmarks) = aptmarks

  430     continue

        endif                         ! Specified new plot point mark.

c....   Display the plot point mark and label.

        write (   6, 8275) aptmarks
        write (iout, 8275) aptmarks

        call markget (aptmarks, nmark)

        if (nmark .gt. 0) then
          write (   6, 8280) amarkt(nmark)
          write (iout, 8280) amarkt(nmark)
        else
          write (   6, 8285)
          write (iout, 8285)
        endif

        go to 999

      endif                           ! Input line "plot marker [AMNAME]".

c=======================================================================********

c.... Input line:  plot limits [-]
c.... Input line:  plot limits [[h,v] SMIN SMAX]

      if ((aword(2) .eq. 'limits') .or.
     &    (aword(2) .eq. 'lim'))   then

        if (aword(3) .eq. '-') then

 8510   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Command "plot limits -" does not have 3 words:  ',i3)

          if (nword .ne. 3) then
            write (   6, 8510) ainput, ninput, nword
            write (iout, 8510) ainput, ninput, nword
            go to 999
          endif

          nplimh = 0
          nplimv = 0

        elseif (nword .eq. 5) then

          nerr = 0

 8530     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  SMIN or SMAX not integer or floating point value',
     &      ' or variable:  ',a)

          call varfind (4, 5)

          if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
            write (   6, 8530) ainput, ninput, aword(4)
            write (iout, 8530) ainput, ninput, aword(4)
            nerr = 1
          endif

          if ((mtype(5) .ne. 2) .and. (mtype(5) .ne. 3)) then
            write (   6, 8530) ainput, ninput, aword(5)
            write (iout, 8530) ainput, ninput, aword(5)
            nerr = 1
          endif

 8540     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  SMIN > SMAX:               ',1p2e20.12)

          if (fword(4) .gt. fword(5)) then
            write (   6, 8540) ainput, ninput, fword(4), fword(5)
            write (iout, 8540) ainput, ninput, fword(4), fword(5)
            nerr = 1
          endif

          if (nerr .ne. 0) go to 999

 8520     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Word 3 is not "h" or "v":  ',a)

          if (aword(3) .eq. 'h') then
            nplimh = 1
            splhmin = fword(4)
            splhmax = fword(5)
          elseif (aword(3) .eq. 'v') then
            nplimv = 1
            splvmin = fword(4)
            splvmax = fword(5)
          else
            write (   6, 8520) ainput, ninput, aword(3)
            write (iout, 8520) ainput, ninput, aword(3)
            go to 999
          endif

        elseif (nword .ne. 2) then    ! Command not recognized.

 8570     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Command "plot limits" does not have 2 or 5 words:  ',i3)

          write (   6, 8570) ainput, ninput, nword
          write (iout, 8570) ainput, ninput, nword
          go to 999

        endif                         ! Tested word 3.

c....   Display the plot axis limits.

        if (nplimh .eq. 0) then
          write (   6, 8224)
          write (iout, 8224)
        else
          write (   6, 8225) splhmin, splhmax
          write (iout, 8225) splhmin, splhmax
        endif

        if (nplimv .eq. 0) then
          write (   6, 8254)
          write (iout, 8254)
        else
          write (   6, 8255) splvmin, splvmax
          write (iout, 8255) splvmin, splvmax
        endif

        go to 999

      endif                           ! Input line "plot limits".

c=======================================================================********

c.... Input line:  plot center [-]
c.... Input line:  plot center [SCENH SCENV]

      if (aword(2) .eq. 'center') then

 8712   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Command "plot center" does not have 2, 3 or 4 words:  ',i3)

        if ((nword .lt. 2)  .or.
     &      (nword .gt. 4)) then
          write (   6, 8712) ainput, ninput, nword
          write (iout, 8712) ainput, ninput, nword
          go to 999
        endif

        if (aword(3) .eq. '-') then   ! Input line "plot center -".

 8714     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Command "plot limits -" does not have 3 words:  ',i3)

          if (nword .ne. 3) then
            write (   6, 8714) ainput, ninput, nword
            write (iout, 8714) ainput, ninput, nword
            go to 999
          endif

          nplcen = 0
          splhcen = -1.e99
          splvcen = -1.e99

        elseif (nword .eq. 4) then

          nerr = 0

          call varfind (3, 4)

 8715     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  SCENH or SCENV not integer or floating point value',
     &      ' or variable:  ',a)

          if ((mtype(3) .ne. 2) .and. (mtype(3) .ne. 3)) then
            write (   6, 8715) ainput, ninput, aword(3)
            write (iout, 8715) ainput, ninput, aword(3)
            nerr = 1
          endif

          if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
            write (   6, 8715) ainput, ninput, aword(4)
            write (iout, 8715) ainput, ninput, aword(4)
            nerr = 1
          endif

          if (nerr .ne. 0) go to 999

          nplcen  = 1
          splhcen = fword(3)
          splvcen = fword(4)

        endif                         ! Input line "plot center point PCEN".

c....   Display the center point of plot.

        if (nplcen .ne. 0) then
          write (   6, 8270) splhcen, splvcen
          write (iout, 8270) splhcen, splvcen
        else
          write (   6, 8272)
          write (iout, 8272)
        endif

        go to 999

      endif                           ! Input line "plot center".

c=======================================================================********

c.... Input line:  plot zoom [[all,h,v] ZOOMFACT]

      if (aword(2) .eq. 'zoom') then

 8710   format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Command "plot zoom" does not have 2 or 4 words:  ',i3)

        if ((nword .ne. 2)  .and.
     &      (nword .ne. 4)) then
          write (   6, 8710) ainput, ninput, nword
          write (iout, 8710) ainput, ninput, nword
          go to 999
        endif

        if (nword .eq. 4) then

          nerr = 0

          call varfind (4, 4)

 8720     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  ZOOMFACT not integer or floating point value',
     &      ' or variable:  ',a)

          if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
            write (   6, 8720) ainput, ninput, aword(4)
            write (iout, 8720) ainput, ninput, aword(4)
            go to 999
          endif

 8725     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  ZOOMFACT is negative:  ',a)

          if (fword(4) .lt. 0.0) then
            write (   6, 8725) ainput, ninput, aword(4)
            write (iout, 8725) ainput, ninput, aword(4)
            go to 999
          endif

 8730     format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Word 3 is not "all", "h" or "v":  ',a)

          if (aword(3) .eq. 'all') then
            zoomplh = zoomplh * fword(4)
            zoomplv = zoomplv * fword(4)
          elseif (aword(3) .eq. 'h') then
            zoomplh = zoomplh * fword(4)
          elseif (aword(3) .eq. 'v') then
            zoomplv = zoomplv * fword(4)
          else
            write (   6, 8730) ainput, ninput, aword(3)
            write (iout, 8730) ainput, ninput, aword(3)
            go to 999
          endif

          if (zoomplh .eq. 0) zoomplh = 1.0
          if (zoomplv .eq. 0) zoomplv = 1.0

        endif                         ! Specified zoom factor.

c....   Display the zoom factor.

        write (   6, 8273) zoomplh
        write (iout, 8273) zoomplh
        write (   6, 8274) zoomplv
        write (iout, 8274) zoomplv

        go to 999

      endif                           ! Input line "plot zoom [ZOOMFACT]".

c=======================================================================********

c.... Input line:  plot array

      if (aword(2) .eq. 'array') then  ! Display 2-D array of plot characters.

 8805 format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Command "plot array" does not have 2 words:  ',i3)

        if (nword .ne. 2) then
          write (   6, 8805) ainput, ninput, nword
          write (iout, 8805) ainput, ninput, nword
          go to 999
        endif

        call plotmap

        go to 999

      endif                           ! Input line "plot array".

c=======================================================================********

c.... Input line is not recognized.

 8810 format ('  plotx ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine plotx.        (+1 line)
      end
