      subroutine point

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  point    Updated     2005 April 5 17:45.
c               point    Originated  1993 November 2 10:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create points in 3-D space.
c
c     Logic:    Entry points:  point.
c point         Called by:  helpall, main.
c
c point         Calls:  aptbrkp, aptbrks, aptcsys, aptetrp, aptetrw, aptlocd,
c               aptlocs, aptrlop, aptscat, aptsubi, aptsubr, apttlod, brout,
c               dkout, incrsubs, meshrel, namechk, pgout, phout, pout, ptfind,
c               ptoper, ptstore, quout, qutype, tetout, trout, varfind,
c               vectfind, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      dimension dside(nzquadm)        ! Value of F(x,y,z) at point.

      character*8  amount             ! Amount of output data.
      character*24 anamex             ! Temporary name of a point.
      character*8  ans                ! User's response.
      character*1  asrce              ! Character "=".
      character*1  afirst             ! First character of point name.

c.... Input line:  point [help]

 8110 format ('  Command "point":  acceptable input lines include:' /
     &  '  "point [help]"      Display this message.' /
     &  '  "point [all,list]"  Display all points.' /
     &  '  "point list PNAME1 PNAME2 PNAME3 ..."  Display all',
     &  ' listed points.' /
     &  '  "point PNAME"       Display point PNAME.' //
     &  '  "point PNAME [=] X    Y     Z"' /
     &  '     Create point PNAME with Cartesian coordinates X, Y, Z',
     &  ' (coord rect).' /
     &  '  "point PNAME [=] RCYL THETA Z"' /
     &  '     Create point PNAME with cylindrical coordinates RCYL,',
     &  ' THETA, Z (coord cyl).' /
     &  '  "point PNAME [=] RSPH THETA PHI"' /
     &  '     Create point PNAME with spherical coordinates RSPH,',
     &  ' THETA, PHI (coord sph).' /
     &  ' In the 3 commands above, "=" is optional if the',
     &  ' next argument is numerical.' )

 8112 format (/
     &  '  "point PNAME [option] QNAME"' /
     &  '     option:  plane, sphere, cylinder, cone, hyperb,',
     &  ' ellipsoid, axisym, quadric.' /
     &  '     Create the central point of quadric QNAME.' //
     &  '  "point PNAME01 polygon PGNAME"' /
     &  '     Create points PNAME01, PNAME02, PNAME03, ...,',
     &  ' at the vertices' /
     &  '     of regular polygon PGNAME.' /
     &  '  "point PNAME01 polyhedron PHNAME"' /
     &  '     Create points PNAME01, PNAME02, PNAME03, ...,',
     &  ' at the vertices' /
     &  '     of regular polyhedron PHNAME.' )

 8114 format (/
     &  '  "point PNAME triangle TRNAME W1 W2 W3"' /
     &  '     Create point PNAME in triangle TRNAME with' /
     &  '     vertex weights W1, W2 and W3.' /
     &  '  "point PNAME tetrahedron TETNAME W1 W2 W3 W4"' /
     &  '     Create point PNAME in tetrahedron TETNAME with' /
     &  '     vertex weights W1, W2, W3 and W4.' /
     &  '  "point PNAME brick BRNAME FU FV FW"' /
     &  '     Create point PNAME in brick BRNAME, at',
     &  ' fractional distances' /
     &  '     FU, FV and FW across the brick.' /
     &  '  "point PMESH brick BRNAME mesh block [volume]"' /
     &  '     Create a family of points PMESH(k,l,m) assigned to',
     &  ' the mesh, with' /
     &  '     indices k, l and m ranging over the indices of',
     &  ' the current mesh block,' /
     &  '     and representing equal intervals [or volumes] between',
     &  ' the minimum and' /
     &  '     maximum u, v and w coordinates of the brick, resp.,',
     &  ' where' /
     &  '     u, v and w are in the coordinate system used to',
     &  ' create the brick.' /
     &  '     Release and rename all points previously assigned to'
     &  ' the mesh block.' //
     &  '  Type <return> for more help.' )

 8115 format (a)

 8116 format (
     &  '  "point PNAME proximal"' /
     &  '     to create point PNAME, at the last proximal point found',
     &  ' by command' /
     &  '     "distance", "proximal", "side" or "track".' //
     &  '  "point PNAME random [RADIUS|1]" to create point PNAME',
     &  ' randomly sampled' /
     &  '     in a sphere of radius RADIUS (default = 1)',
     &  ' centered at the origin.' /
     &  '  "point PNAME OBJTYPE OBJNAME random" to create point PNAME',
     &  ' randomly sampled' /
     &  '     in OBJTYPE (sphere, triangle, disk, tetrahedron,',
     &  ' brick) OBJNAME.' /
     &  '  "point PNAME zone ZNAME random BRNAME"' /
     &  '     Create point PNAME randomly sampled in zone ZNAME,' /
     &  '     by trying points randomly sampled in brick BRNAME.' )

 8118 format (/
     &  '  "point PNAME02 move NUMPT INC PNAME01 VMOVE"' /
     &  '     Create a family of NUMPT points PNAME02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     and spaced at intervals of vector VMOVE,' /
     &  '     starting from point PNAME01.' //
     &  '  "point PNAME02 rotate NUMPT INC PNAME01 OPNAME PINV"' /
     &  '     Create a family of NUMPT points PNAME02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by rotating the preceding point with operator OPNAME' /
     &  '     and invariant point PINV, starting from point PNAME01.' //
     &  '  "point PNAME02 scale NUMPT INC PNAME01 OPNAME PINV"' /
     &  '     Create a family of NUMPT points PNAME02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by scaling the preceding point with operator OPNAME' /
     &  '     and invariant point PINV, starting from point PNAME01.' )

 8120 format (/
     &  '  Note:  PNAME or PNAME02 may not be "thru".' //
     &  '  Note:  use +/- for PNAME or PNAME02 to incr/decr the last',
     &  ' point name:  ',a )

 8122 format (/
     &  '  Synonyms:  [brick, br], [cylinder, cyl], [disk, disc, dk],' /
     &  '    [ellipsoid, ellipse, ell, el], [help, h],',
     &  ' [hyperb, saddle]' /
     &  '    [move, mv, translate, tran],' /
     &  '    [plane, pl], [point, pnt, pt, p], [polygon, polyg, pg],' /
     &  '    [polyhedron, polyh, ph], [proximal, prox],',
     &  ' [quadric, quad, q],' /
     &  '    [random, ?], [scale, sc], [sphere, sph],',
     &  ' [tetrahedron, tetra, tet],' /
     &  '    [triangle, tri], [volume, vol], [zone, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        if (ainput .eq. 'tty') then
          read  (   5, 8115) ans
        endif
        write (   6, 8116)
        write (iout, 8116)
        write (   6, 8118)
        write (iout, 8118)
        write (   6, 8120) apoints
        write (iout, 8120) apoints
        write (   6, 8122)
        write (iout, 8122)
        go to 999
      endif

c===============================================================================

c.... Input line:  point all
c.... Input line:  point list

 8119 format ('coord  ',a,5x,a,4x,a,4x,a)
 8125 format (/ 'Number of points  = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        write (   6, 8119) asys, acoordu, acoordv, acoordw
        write (iout, 8119) asys, acoordu, acoordv, acoordw
        npointx = 0
        amount = 'minimum'
        call pout (1, npoints, amount)
        do 120 n = 1, npoints
          if (apoint(n) .eq. ' ') go to 120
          npointx = npointx + 1
  120   continue
        write (   6, 8125) npointx
        write (iout, 8125) npointx
        go to 999
      endif

c===============================================================================

c.... Input line:  point list PNAME1 PNAME2 PNAME3 ...

      if (aword(2) .eq. 'list') then

        write (   6, 8119) asys, acoordu, acoordv, acoordw
        write (iout, 8119) asys, acoordu, acoordv, acoordw

 8135   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Named point does not exist:  ',a)

        do 128 nw = 3, nword
cold
cold          call ptfind (nw, n)

          nfind = 0
          do 125 n = 1, npoints
            call aptsubr (apoint(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. apoint(n)) .or.
     &          (aword(nw) .eq. astem))    then
              nfind = nfind + 1
              amount = 'minimum'
              call pout (n, n, amount)
            endif
  125     continue

          if (nfind .eq. 0) then
            write (   6, 8135) ainput, ninput, aword(nw)
            write (iout, 8135) ainput, ninput, aword(nw)
          endif

  128   continue

        go to 999

      endif

c===============================================================================

c.... Test for proper length and mode of point name.

 8902 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of point not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8902) ainput, ninput, aword(2)
        write (iout, 8902) ainput, ninput, aword(2)
        go to 999
      endif

 8905 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of point too dangerous to use:  ',a)

      afirst = aword(2)

      if (afirst .eq. '-'   ) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  point PNAME

c.... See if user wants one point displayed.

      if (nword .eq. 2) then

        do 140 n = 1, npoints
          if (aword(2) .eq. apoint(n)) then
            amount = 'maximum'
            call pout (n, n, amount)
            go to 999
          endif
  140   continue

 8140 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named point does not exist:  ',a)

        write (   6, 8140) ainput, ninput, aword(2)
        write (iout, 8140) ainput, ninput, aword(2)
        go to 999

      endif

c===============================================================================

c.... See if last point name should be incremented and used.

 8128 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last point can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = apoints
        lword(2) = lapoints
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, apoints
          write (iout, 8128) ainput, ninput, apoints
          go to 999
        endif
      endif

 8129 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last point can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = apoints
        lword(2) = lapoints
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, apoints
          write (iout, 8129) ainput, ninput, apoints
          go to 999
        endif
      endif

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  point PMESH brick BRNAME mesh block [volume]

      if (((aword(3) .eq. 'brick')  .or.
     &     (aword(3) .eq. 'br'   )) .and.
     &     (nword .ge. 5)           .and.
     &     (aword(5) .eq. "mesh" )) then

c....   Test for errors.

        nerr = 0

 8142   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 6 or 7 words on input line:  ',i3)
 8143   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word 6 is not "mesh":  ',a)
 8144   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word 7 is not "volume" or "vol":  ',a)

        if ((nword .ne. 6) .and. (nword .ne. 7)) then
          nerr = 1
          write (   6, 8142) ainput, ninput, nword
          write (iout, 8142) ainput, ninput, nword
        endif

        if ((aword(6) .ne. 'block')  .and.
     &      (aword(6) .ne. 'bl'   )) then
          nerr = 1
          write (   6, 8143) ainput, ninput, aword(6)
          write (iout, 8143) ainput, ninput, aword(6)
        endif

        if ((nword .eq. 7)            .and.
     &     ((aword(7) .ne. 'volume')  .and.
     &      (aword(7) .ne. 'vol'   ))) then
          nerr = 1
          write (   6, 8144) ainput, ninput, aword(7)
          write (iout, 8144) ainput, ninput, aword(7)
        endif

c....   See if the brick exists.

        do 750 nbr = 1, nbricks
          if (aword(4) .eq. abrick(nbr)) then
            nb = nbr
            amount = 'maximum'
            call brout (nb, nb, amount)
            go to 755
          endif
  750   continue
        nerr = 1
        write (   6, 8510) ainput, ninput, aword(4)
        write (iout, 8510) ainput, ninput, aword(4)
  755   continue

 8145   format (
     &    'Mesh block:  kbl1      =',i6,'   lbl1      =',i6,
     &    '   mbl1      =',i6 /
     &    '             kbl2      =',i6,'   lbl2      =',i6,
     &    '   mbl2      =',i6 )

c....   See if a mesh block exists.

 8355   format ('  mesh ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  No mesh block has been defined.' )

        if ((kbl1 .eq. 0) .or. (kbl2 .eq. 0)  .or.
     &      (lbl1 .eq. 0) .or. (lbl2 .eq. 0)  .or.
     &      (mbl1 .eq. 0) .or. (mbl2 .eq. 0)) then
          nerr = 1
          write (   6, 8355) ainput, ninput
          write (iout, 8355) ainput, ninput
        else
          write (   6, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
          write (iout, 8145) kbl1, lbl1, mbl1, kbl2, lbl2, mbl2
        endif

c....   See if any errors were detected in the input arguments.

        if (nerr .ne. 0) go to 999

c....   Release and rename any points currently assigned to the mesh block.

        aword(3) = ' '

        call meshrel (kbl1, kbl2, lbl1, lbl2, mbl1, mbl2, nprel)

        if (nprel .lt. 0) go to 999

c....   Find the coordinate system and angle units of the brick.

        if (asysbr(nb) .eq. 'rect') then
          ksys = 0
        elseif (asysbr(nb) .eq. 'cyl') then
          ksys = 1
        elseif (asysbr(nb) .eq. 'sph') then
          ksys = 2
        endif

        if     (angbr(nb) .eq. 'degrees') then
          iunit = 0
        elseif (angbr(nb) .eq. 'radians') then
          iunit = 1
        endif

c....   Find the fractional increments in each direction.

        if (kmax .gt. 1) then
          dfu = 1.0 / (kmax - 1)
        else
          dfu = 1.0
        endif

        if (lmax .gt. 1) then
          dfv = 1.0 / (lmax - 1)
        else
          dfv = 1.0
        endif

        if (mmax .gt. 1) then
          dfw = 1.0 / (mmax - 1)
        else
          dfw = 1.0
        endif

        anamex  = aword(2)
        lanamex = lword(2)

        do 790 m = mbl1, mbl2         ! Loop over w direction.
          fracw = (m - 1) * dfw

          do 785 l = lbl1, lbl2       ! Loop over v direction.
            fracv = (l - 1) * dfv

            do 780 k = kbl1, kbl2     ! Loop over u direction.
              fracu  = (k - 1) * dfu

c....         Find the coordinates of the point in the brick.

              if (nword .eq. 7) then  ! Use equal-volume intervals.

                call aptbrkp (ksys, iunit,
     &                        ubr(nb), dubr(nb),
     &                        vbr(nb), dvbr(nb),
     &                        wbr(nb), dwbr(nb),
     &                        fracu, fracv, fracw, 1, tol,
     &                        u, v, w, nerra)

              else                    ! Use equal intervals.

                u = ubr(nb) + fracu * dubr(nb)
                v = vbr(nb) + fracv * dvbr(nb)
                w = wbr(nb) + fracw * dwbr(nb)

              endif                   ! Find type of intervals.

c....         Convert into rectangular coordinates, if necessary.

              call aptcsys (ksys, 0, iunit, u, v, w, 1, tol, nerra)

c....         Find the next point name.

              isub(1) = k
              isub(2) = l
              isub(3) = m

              call aptsubi (anamex, lanamex, 3, isub, 24,
     &                      '(', ',', ')', aword(2), lword(2), nerr)

c....         Find where to store the point.

              call ptstore (nrep, n)

              if (n .eq. 0) go to 999

              apoint(n)  = aword(2)
              lapoint(n) = lword(2)

              apoints    = aword(2)
              lapoints   = lword(2)

              x(n)       = u
              y(n)       = v
              z(n)       = w

              kpoint(n)  = k
              lpoint(n)  = l
              mpoint(n)  = m

c....         Print the coordinates of the point.

              amount = 'maximum'
              call pout (n, n, amount)

  780       continue                  ! End of loop over u direction.
  785     continue                    ! End of loop over v direction.
  790   continue                      ! End of loop over w direction.

        go to 999

      endif                           ! Input line "point PMESH brick ...".

c===============================================================================

c.... Input line:  point PNAME02 move NUMPT INC PNAME01 VMOVE

      if ((aword(3) .eq. 'move')      .or.
     &    (aword(3) .eq. 'mv')        .or.
     &    (aword(3) .eq. 'translate') .or.
     &    (aword(3) .eq. 'trans'))    then

        nerr   = 0

 8805   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 7 words in input line:  ',i3)

        if (nword .ne. 7) then
          write (   6, 8805) ainput, ninput, nword
          write (iout, 8805) ainput, ninput, nword
          nerr = 1
        endif

 8816   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMPT is not a positive integer:  ',a)
 8818   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMPT is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numpt = iword(4)
          if (numpt .lt. 1) then
            write (   6, 8816) ainput, ninput, aword(4)
            write (iout, 8816) ainput, ninput, aword(4)
            nerr = 1
          endif
        else
          write (   6, 8818) ainput, ninput, aword(4)
          write (iout, 8818) ainput, ninput, aword(4)
          nerr = 1
        endif

 8810   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8812   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8810) ainput, ninput
            write (iout, 8810) ainput, ninput
            nerr = 1
          endif
        else
          write (   6, 8812) ainput, ninput, aword(5)
          write (iout, 8812) ainput, ninput, aword(5)
          nerr = 1
        endif

c....   Find the base point.

        call ptfind (6, nbase)

        if (nbase .eq. 0) nerr = 1

c....   Find the translation vector.

        call vectfind (7, nv)

        if (nv .eq. 0) nerr = 1

c....   See if the translation vector has nonzero length.

 8850   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Translation vector has zero length:  ',a)

        if (nv .gt. 0) then           ! Translation vector exists.
          if (vlen(nv) .eq. 0.0) then
            write (   6, 8850) ainput, ninput, aword(7)
            write (iout, 8850) ainput, ninput, aword(7)
            nerr = 1
          endif
        endif                         ! Translation vector exists.

c....   See if the point names can be generated uniquely.

 8860   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not generate',i5,' point names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numpt - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          write (   6, 8860) ainput, ninput, numpt, aword(2)
          write (iout, 8860) ainput, ninput, numpt, aword(2)
          nerr = 1
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

        if (nerr .ne. 0) go to 999

c....   Create the translated points.

        anamex  = aword(2)            ! Initial point name.
        lanamex = lword(2)            ! Initial point name length.

        do 870 nqt = 1, numpt         ! Loop over the new points.

c....     Find the index in the point list for the new point.

          aword(2) = anamex
          lword(2) = lanamex

          call ptstore (nrep, n)

          if (n .eq. 0) go to 999

c....     Translate the base point by vector vname.

          x(n)  = x(nbase) + xcom(nv)
          y(n)  = y(nbase) + ycom(nv)
          z(n)  = z(nbase) + zcom(nv)

          tolxx = tol * (abs (x(nbase)) + abs (xcom(nv)))
          tolyy = tol * (abs (y(nbase)) + abs (ycom(nv)))
          tolzz = tol * (abs (z(nbase)) + abs (zcom(nv)))

          if (abs (x(n)) .le. tolxx) x(n) = 0.0
          if (abs (y(n)) .le. tolyy) y(n) = 0.0
          if (abs (z(n)) .le. tolzz) z(n) = 0.0

          apoint(n)  = anamex
          lapoint(n) = lanamex

          apoints    = anamex
          lapoints   = lanamex

          amount = 'maximum'
          call pout (n, n, amount)

          nbase = n

c....     Find the next point name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            nerr = 1
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 580
          endif

  870   continue                      ! End of loop over the new points.

        go to 999

      endif                           ! Created translated point(s).

c===============================================================================

c.... Input line:  point PNAME02 rotate NUMPT INC PNAME01 OPNAME PINV
c.... Input line:  point PNAME02 scale  NUMPT INC PNAME01 OPNAME PINV

      if ((aword(3) .eq. 'scale')   .or.
     &    (aword(3) .eq. 'sc')      .or.
     &    (aword(3) .eq. 'rotate')  .or.
     &    (aword(3) .eq. 'rot'))    then

        call ptoper

        go to 999

      endif                           ! Created rotated or scaled point(s).

c===============================================================================

c.... Find the index in the point array at which to store this point.
c....   Set nrep = 1 if replacing an existing point.

      call ptstore (nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c===============================================================================

c.... Input line:  point PNAME proximal

      if ((aword(3) .eq. 'proximal') .or.
     &    (aword(3) .eq. 'prox'))    then

 8801   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 3 words in input line "point PNAME proximal":  ',i3)

        if (nword .ne. 3) then
          write (   6, 8801) ainput, ninput, nword
          write (iout, 8801) ainput, ninput, nword
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
          go to 999
        endif

        apoint(n)  = aword(2)
        lapoint(n) = lword(2)
        apoints    = aword(2)
        lapoints   = lword(2)

        x(n)      = proxptx
        y(n)      = proxpty
        z(n)      = proxptz

        amount    = 'maximum'
        call pout (n, n, amount)

        go to 999

      endif

c===============================================================================

c.... Input line:  point PNAME random [RADIUS|1]

 8175 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Too many words on "random" line:  ', i3)
 8176 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Radius is negative:  ', 1pe20.12)

      if ((aword(3) .eq. 'random') .or.
     &    (aword(3) .eq. '?'))     then

c....   Find the radius of the sphere containing the random point.

        if (nword .eq. 3) then
          fword(4) = 1.0
        elseif (nword .eq. 4) then

          call varfind (4, 4)

          if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
            write (   6, 8903) ainput, ninput, aword(4)
            write (iout, 8903) ainput, ninput, aword(4)
            nerr = 1
          endif
        else
          nerr = 1
          write (   6, 8175) ainput, ninput, nword
          write (iout, 8175) ainput, ninput, nword
        endif

        if (fword(4) .lt. 0.0) then
          nerr = 1
          write (   6, 8176) ainput, ninput, nword
          write (iout, 8176) ainput, ninput, nword
        endif

c....   Find a random direction from the origin.

        call aptscat (1, u, v, w, nerra)

c....   Find a random point uniformly distributed inside the sphere.

        call aptlocs (0., fword(4), 0., 0., 0., 1, tol, u, v, w, nerra)

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
          go to 999
        endif

        apoint(n)  = aword(2)
        lapoint(n) = lword(2)
        apoints    = aword(2)
        lapoints   = lword(2)

        x(n)     = u
        y(n)     = v
        z(n)     = w

c....   Print the coordinates of the point.

        amount = 'maximum'
        call pout (n, n, amount)

        go to 999

      endif

c===============================================================================

c.... Input line:  point PNAME sphere SPHNAME random

 8180 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric is not a sphere:  ',a)
 8185 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named sphere does not exist:  ',a)

      if (((aword(3) .eq. 'sphere')  .or.
     &     (aword(3) .eq. 'sph'))    .and.
     &    ((aword(5) .eq. 'random')  .or.
     &     (aword(5) .eq. '?'))      .and.
     &     (nword .eq. 5))           then

        do 185 nq = 1, nquads
          if (aquad(nq) .ne. aword(4)) go to 185

          call qutype (aword(3), nq, 1, nerrq)

          if (nerrq .ne. 0) then
            write (   6, 8180) ainput, ninput, aword(3)
            write (iout, 8180) ainput, ninput, aword(3)
            nerr =  1
            go to 190
          endif

          rsq = xcen(nq)**2 + ycen(nq)**2 + zcen(nq)**2 -
     &          qc(nq) / qxx(nq)
          rsphere = sqrt (rsq)

          go to 190
  185   continue
        write (   6, 8185) ainput, ninput, aword(4)
        write (iout, 8185) ainput, ninput, aword(4)
        nerr =  1

  190   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
          go to 999
        endif

c....   Find a random point uniformly distributed inside the sphere.

        call aptlocs (0., rsphere, xcen(nq), ycen(nq), zcen(nq), 1, tol,
     &                u, v, w, nerra)

        apoint(n)  = aword(2)
        lapoint(n) = lword(2)
        apoints    = aword(2)
        lapoints   = lword(2)

        x(n)     = u
        y(n)     = v
        z(n)     = w

c....   Print the coordinates of the point.

        amount = 'maximum'
        call pout (n, n, amount)

        go to 999

      endif

c===============================================================================

c.... Input line:  point PNAME plane     PLNAME
c.... Input line:  point PNAME sphere    SPHNAME
c.... Input line:  point PNAME cylinder  CYLNAME
c.... Input line:  point PNAME cone      CNNAME
c.... Input line:  point PNAME hyperb    HPNAME
c.... Input line:  point PNAME ellipsoid ELNAME
c.... Input line:  point PNAME axisym    AXNAME
c.... Input line:  point PNAME quadric   QNAME

      if ((aword(3) .eq. 'quadric')   .or.
     &    (aword(3) .eq. 'quad')      .or.
     &    (aword(3) .eq. 'q')         .or.
     &    (aword(3) .eq. 'plane')     .or.
     &    (aword(3) .eq. 'pl')        .or.
     &    (aword(3) .eq. 'sphere')    .or.
     &    (aword(3) .eq. 'sph')       .or.
     &    (aword(3) .eq. 'cylinder')  .or.
     &    (aword(3) .eq. 'cyl')       .or.
     &    (aword(3) .eq. 'cone')      .or.
     &    (aword(3) .eq. 'hyperb')    .or.
     &    (aword(3) .eq. 'saddle')    .or.
     &    (aword(3) .eq. 'ellipsoid') .or.
     &    (aword(3) .eq. 'ellipse')   .or.
     &    (aword(3) .eq. 'ell')       .or.
     &    (aword(3) .eq. 'el')        .or.
     &    (aword(3) .eq. 'axisym'))   then

c....   See if quadric exists.

 8210   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named quadric surface does not exist:  ',a)

        do 220 nq = 1, nquads
          if (aword(4) .eq. aquad(nq)) then

            call qutype (aword(3), nq, 1, nerrq)

            if (nerrq .ne. 0) go to 999

            n1 = nq

            call quout (n1, n1, 'min', nerrq)

            go to 230
          endif
  220   continue
        write (   6, 8210) ainput, ninput, aword(4)
        write (iout, 8210) ainput, ninput, aword(4)
        nerr =  1

c....   Find central point of quadric.

  230   u = xcen(n1)
        v = ycen(n1)
        w = zcen(n1)

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
          go to 999
        endif

        apoint(n)  = aword(2)
        lapoint(n) = lword(2)
        apoints    = aword(2)
        lapoints   = lword(2)

        x(n)     = u
        y(n)     = v
        z(n)     = w

c....   Print the coordinates of the point.

        amount = 'maximum'
        call pout (n, n, amount)

        go to 999

      endif

c===============================================================================

c.... Input line:  point PNAME triangle TRNAME W1 W2 W3
c.... Input line:  point PNAME triangle TRNAME random

      if ((aword(3) .eq. 'triangle') .or.
     &    (aword(3) .eq. 'tri'))     then

c....   See if triangle exists.

 8305   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 5 or 7 words on input line:  ',i3)

        if ((nword .ne. 5)  .and.
     &      (nword .ne. 7)) then
          write (   6, 8305) ainput, ninput, nword
          write (iout, 8305) ainput, ninput, nword
          nerr =  1
        endif

 8310   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named triangle does not exist:  ',a)

        do 310 nt = 1, ntrigs
          if (aword(4) .eq. atrig(nt)) then
            n1 = nt
            amount = 'maximum'
            call trout (n1, n1, amount)
            go to 320
          endif
  310   continue
        write (   6, 8310) ainput, ninput, aword(4)
        write (iout, 8310) ainput, ninput, aword(4)
        nerr =  1

c....   Find the points in the triangle.

 8325 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named point does not exist:  ',a)

  320   do 325 np = 1, npoints
          if (apoint(np) .eq. atript1(n1)) then
            xtra = x(np)
            ytra = y(np)
            ztra = z(np)
            go to 330
          endif
  325   continue
        nerr = 1
        write (   6, 8325) ainput, ninput, atript1(n1)
        write (iout, 8325) ainput, ninput, atript1(n1)

  330   do 335 np = 1, npoints
          if (apoint(np) .eq. atript2(n1)) then
            xtrb = x(np)
            ytrb = y(np)
            ztrb = z(np)
            go to 340
          endif
  335   continue
        nerr = 1
        write (   6, 8325) ainput, ninput, atript2(n1)
        write (iout, 8325) ainput, ninput, atript2(n1)

  340   do 345 np = 1, npoints
          if (apoint(np) .eq. atript3(n1)) then
            xtrc = x(np)
            ytrc = y(np)
            ztrc = z(np)
            go to 350
          endif
  345   continue
        nerr = 1
        write (   6, 8325) ainput, ninput, atript3(n1)
        write (iout, 8325) ainput, ninput, atript3(n1)

c....   Find the point selection option.

  350   if (nword .eq. 7) then

c....   Find the weight factors.

 8360 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

        call varfind (5, 7)

        do 370 nw = 5, 7
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8360) ainput, ninput, aword(nw)
            write (iout, 8360) ainput, ninput, aword(nw)
            nerr = 1
          endif
  370   continue

        wa   = fword(5)
        wb   = fword(6)
        wc   = fword(7)
        wsum = wa + wb + wc
        werr = tol * (abs (wa) + abs (wb) + abs (wc))

 8370 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Sum of weights is too small:  ',1pe20.12)

        if (wsum .le. werr) then
          write (   6, 8370) ainput, ninput, wsum
          write (iout, 8370) ainput, ninput, wsum
          nerr = 1
        endif

c....   Find the point.

        call aptrlop (wa, wb, wc, xtra, ytra, ztra, xtrb, ytrb, ztrb,
     &                xtrc, ytrc, ztrc, 1, tol, u, v, w, nerra)

        go to 380

        elseif ((nword .eq. 5) .and.
     &         ((aword(5) .eq. 'random')  .or.
     &          (aword(5) .eq. '?')))     then

c....   Find the point.

        call apttlod (xtra, ytra, ztra, xtrb, ytrb, ztrb,
     &                xtrc, ytrc, ztrc, 1, 1, tol,
     &                u, v, w, wa, wb, wc, nerra)

        go to 380

        else

 8375 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 5 is not "random":  ',a)
          write (   6, 8375) ainput, ninput, aword(5)
          write (iout, 8375) ainput, ninput, aword(5)
          nerr = 1

          go to 380

        endif

  380   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
          go to 999
        endif

        apoint(n)  = aword(2)
        lapoint(n) = lword(2)
        apoints    = aword(2)
        lapoints   = lword(2)

        x(n)     = u
        y(n)     = v
        z(n)     = w

 8380   format ('  Vertex weights  ',1p3e20.12)

        write (   6, 8380) wa, wb, wc
        write (iout, 8380) wa, wb, wc

c....   Print the coordinates of the point.

        amount = 'maximum'
        call pout (n, n, amount)

        go to 999

      endif

c===============================================================================

c.... Input line:  point PNAME disk DKNAME random

      if ((aword(3) .eq. 'disk') .or.
     &    (aword(3) .eq. 'disc') .or.
     &    (aword(3) .eq. 'dk'))  then

c....   See if disk exists.

 8700   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 5 words on input line:  ',i3)

        if (nword .ne. 5) then
          write (   6, 8700) ainput, ninput, nword
          write (iout, 8700) ainput, ninput, nword
          nerr =  1
        endif

 8705   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word 5 is not "random":  ',a)

        if ((aword(5) .ne. 'random') .and.
     &      (aword(5) .ne. '?'))     then
          write (   6, 8705) ainput, ninput, aword(5)
          write (iout, 8705) ainput, ninput, aword(5)
          nerr = 1
        endif

 8710   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named disk does not exist:  ',a)

        do 710 nd = 1, ndisks
          if (aword(4) .eq. adisk(nd)) then
            call dkout (nd, nd)
            go to 720
          endif
  710   continue
        write (   6, 8710) ainput, ninput, aword(4)
        write (iout, 8710) ainput, ninput, aword(4)
        nerr =  1

c....   Find the center point of the disk.

 8720   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named point does not exist:  ',a)

  720   do 730 np = 1, npoints
          if (apoint(np) .eq. acendk(nd)) then
            xtra = x(np)
            ytra = y(np)
            ztra = z(np)
            go to 740
          endif
  730   continue
        nerr = 1
        write (   6, 8720) ainput, ninput, acendk(nd)
        write (iout, 8720) ainput, ninput, acendk(nd)

c....   Find the point.

  740   call aptlocd (radkin(nd), radkout(nd), x(np), y(np), z(np),
     &                xnormdk(nd), ynormdk(nd), znormdk(nd), 1, tol,
     &                u, v, w, nerra)

 8740   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Bad input to aptlocd.')

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8740) ainput, ninput
          write (iout, 8740) ainput, ninput
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
          go to 999
        endif

        apoint(n)  = aword(2)
        lapoint(n) = lword(2)
        apoints    = aword(2)
        lapoints   = lword(2)

        x(n) = u
        y(n) = v
        z(n) = w

 8745   format ('  Selected radius ',1pe20.12)

        radran = sqrt ((u - x(np))**2 + (v - y(np))**2 + (w - z(np))**2)

        write (   6, 8745) radran
        write (iout, 8745) radran

c....   Print the coordinates of the point.

        amount = 'maximum'
        call pout (n, n, amount)

        go to 999

      endif

c===============================================================================

c.... Input line:  point PNAME tetrahedron TETNAME W1 W2 W3 W4
c.... Input line:  point PNAME tetrahedron TETNAME random

      if ((aword(3) .eq. 'tetrahedron') .or.
     &    (aword(3) .eq. 'tetra')        .or.
     &    (aword(3) .eq. 'tet'))         then

c....   See if tetrahedron exists.

 8405   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 5 or 8 words on input line:  ',i3)

        if ((nword .ne. 5)  .and.
     &      (nword .ne. 8)) then
          write (   6, 8405) ainput, ninput, nword
          write (iout, 8405) ainput, ninput, nword
          nerr =  1
        endif

 8410   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named tetrahedron does not exist:  ',a)

        do 410 nt = 1, ntets
          if (aword(4) .eq. atetra(nt)) then
            n1 = nt
            amount = 'maximum'
            call tetout (n1, n1, amount)
            go to 420
          endif
  410   continue
        write (   6, 8410) ainput, ninput, aword(4)
        write (iout, 8410) ainput, ninput, aword(4)
        nerr =  1

c....   Find the vertex points of the tetrahedron.

 8425 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named point does not exist:  ',a)

  420   do 425 np = 1, npoints
          if (apoint(np) .eq. atetpt(1,n1)) then
            xteta = x(np)
            yteta = y(np)
            zteta = z(np)
            go to 430
          endif
  425   continue
        nerr = 1
        write (   6, 8425) ainput, ninput, atetpt(1,n1)
        write (iout, 8425) ainput, ninput, atetpt(1,n1)

  430   do 435 np = 1, npoints
          if (apoint(np) .eq. atetpt(2,n1)) then
            xtetb = x(np)
            ytetb = y(np)
            ztetb = z(np)
            go to 440
          endif
  435   continue
        nerr = 1
        write (   6, 8425) ainput, ninput, atetpt(2,n1)
        write (iout, 8425) ainput, ninput, atetpt(2,n1)

  440   do 445 np = 1, npoints
          if (apoint(np) .eq. atetpt(3,n1)) then
            xtetc = x(np)
            ytetc = y(np)
            ztetc = z(np)
            go to 450
          endif
  445   continue
        nerr = 1
        write (   6, 8425) ainput, ninput, atetpt(3,n1)
        write (iout, 8425) ainput, ninput, atetpt(3,n1)

  450   do 455 np = 1, npoints
          if (apoint(np) .eq. atetpt(4,n1)) then
            xtetd = x(np)
            ytetd = y(np)
            ztetd = z(np)
            go to 460
          endif
  455   continue
        nerr = 1
        write (   6, 8425) ainput, ninput, atetpt(4,n1)
        write (iout, 8425) ainput, ninput, atetpt(4,n1)

c....   Find the point selection option.

  460   if (nword .eq. 8) then

c....   Find the weight factors.

 8460 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

        call varfind (5, 8)

        do 470 nw = 5, 8
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8460) ainput, ninput, aword(nw)
            write (iout, 8460) ainput, ninput, aword(nw)
            nerr = 1
          endif
  470   continue

        wa   = fword(5)
        wb   = fword(6)
        wc   = fword(7)
        wd   = fword(8)
        wsum = wa + wb + wc + wd
        werr = tol * (abs (wa) + abs (wb) + abs (wc) + abs (wd))

 8470 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Sum of weights is too small:  ',1pe20.12)

        if (wsum .le. werr) then
          write (   6, 8470) ainput, ninput, wsum
          write (iout, 8470) ainput, ninput, wsum
          nerr = 1
        endif

c....   Find the point.

        call aptetrw (wa, wb, wc, wd,
     &                xteta, yteta, zteta, xtetb, ytetb, ztetb,
     &                xtetc, ytetc, ztetc, xtetd, ytetd, ztetd,
     &                1, tol, u, v, w, nerra)

        go to 480

        elseif ((nword .eq. 5)            .and.
     &         ((aword(5) .eq. 'random')) .or.
     &          (aword(5) .eq. '?'))      then

c....   Find the point.

        call aptetrp (xteta, yteta, zteta, xtetb, ytetb, ztetb,
     &                xtetc, ytetc, ztetc, xtetd, ytetd, ztetd,
     &                1, 1, tol, u, v, w, wa, wb, wc, wd, nerra)

        go to 480

        else

 8475 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 5 is not "random":  ',a)

          write (   6, 8475) ainput, ninput, aword(5)
          write (iout, 8475) ainput, ninput, aword(5)
          nerr = 1

          go to 480

        endif

  480   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
          go to 999
        endif

        apoint(n)  = aword(2)
        lapoint(n) = lword(2)
        apoints    = aword(2)
        lapoints   = lword(2)

        x(n)     = u
        y(n)     = v
        z(n)     = w

 8480   format ('  Vertex wts w1,w2',1p2e20.12 /
     &          '             w3,w4',1p2e20.12 )

        write (   6, 8480) wa, wb, wc, wd
        write (iout, 8480) wa, wb, wc, wd

c....   Print the coordinates of the point.

        amount = 'maximum'
        call pout (n, n, amount)

        go to 999

      endif

c===============================================================================

c.... Input line:  point PNAME brick BRNAME FU FV FW
c.... Input line:  point PNAME brick BRNAME random

      if (((aword(3) .eq. 'brick') .or.
     &     (aword(3) .eq. 'br'))   .and.
     &    ((nword .eq. 5)          .or.
     &     (nword .eq. 7)))        then

 8505   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 5 words on input line:  ',i3)
 8506   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 7 words on input line:  ',i3)

        if (((aword(5) .eq. 'random') .or.
     &       (aword(5) .eq. '?'))     .and.
     &      (nword .ne. 5))           then
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
          nerr =  1
        endif

        if ((aword(5) .ne. 'random') .and.
     &      (aword(5) .ne. '?')      .and.
     &      (nword .ne. 7))          then
          write (   6, 8506) ainput, ninput, nword
          write (iout, 8506) ainput, ninput, nword
          nerr =  1
        endif

c....   See if brick exists.

 8510   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named brick does not exist:  ',a)

        do 510 nbr = 1, nbricks
          if (aword(4) .eq. abrick(nbr)) then
            nb = nbr
            amount = 'maximum'
            call brout (nb, nb, amount)
            go to 520
          endif
  510   continue
        write (   6, 8510) ainput, ninput, aword(4)
        write (iout, 8510) ainput, ninput, aword(4)
        nerr =  1

 8520   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Brick has unrecognized coordinate system:  ',a)

  520   if     (asysbr(nb) .eq. 'rect') then
          ksys = 0
        elseif (asysbr(nb) .eq. 'cyl')  then
          ksys = 1
        elseif (asysbr(nb) .eq. 'sph')  then
          ksys = 2
        else
          write (   6, 8520) ainput, ninput, asysbr(nb)
          write (iout, 8520) ainput, ninput, asysbr(nb)
          nerr =  1
        endif

 8530   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Brick has unrecognized angle units:  ',a)

        if (angbr(nb) .eq. 'radians') then
          iunit = 1
        elseif (angbr(nb) .eq. 'degrees') then
          iunit = 0
        else
          write (   6, 8530) ainput, ninput, angbr(nb)
          write (iout, 8530) ainput, ninput, angbr(nb)
          nerr =  1
          go to 535
        endif

c....   Find the point selection option.

        if (nword .eq. 7) then

c....     Find the point at specified fractional distances across the brick.

          call varfind (5, 7)

          do 524 nw = 5, 7
            if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
              write (   6, 8460) ainput, ninput, aword(nw)
              write (iout, 8460) ainput, ninput, aword(nw)
              nerr = 1
            endif
  524     continue

          fracu = fword(5)
          fracv = fword(6)
          fracw = fword(7)

          call aptbrkp (ksys, iunit,
     &                  ubr(nb), dubr(nb),
     &                  vbr(nb), dvbr(nb),
     &                  wbr(nb), dwbr(nb),
     &                  fracu, fracv, fracw, 1, tol,
     &                  u, v, w, nerra)

        elseif ((nword .eq. 5)            .and.
     &         ((aword(5) .eq. 'random')) .or.
     &          (aword(5) .eq. '?'))      then

c....     Find the point by randomly sampling.

          call aptbrks (ksys, iunit,
     &                  ubr(nb), dubr(nb),
     &                  vbr(nb), dvbr(nb),
     &                  wbr(nb), dwbr(nb), 1, tol,
     &                  u, v, w, fracu, fracv, fracw, nerra)

        else                          ! Word 5 not "random".

          write (   6, 8475) ainput, ninput, aword(5)
          write (iout, 8475) ainput, ninput, aword(5)
          nerr = 1
          go to 535

        endif                         ! Tested nword.

c....   Convert into rectangular coordinates, if necessary.

        call aptcsys (ksys, 0, iunit, u, v, w, 1, tol, nerra)

  535   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
          go to 999
        endif

 8535   format ('  Fractional wts  ',1p3e20.12 )

        write (   6, 8535) fracu, fracv, fracw
        write (iout, 8535) fracu, fracv, fracw

        apoint(n)  = aword(2)
        lapoint(n) = lword(2)
        apoints    = aword(2)
        lapoints   = lword(2)

        x(n)     = u
        y(n)     = v
        z(n)     = w

c....   Print the coordinates of the point.

        amount = 'maximum'
        call pout (n, n, amount)

        go to 999

      endif                           ! Word 5 was "brick".

c===============================================================================

c.... Input line:  point PNAME zone ZNAME random BRNAME

      if (((aword(3) .eq. 'zone')   .or.
     &     (aword(3) .eq. 'z'))     .and.
     &    ((aword(5) .eq. 'random') .or.
     &     (aword(5) .eq. '?')))   then

 8605   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 6 words on input line:  ',i3)

        if (nword .ne. 6) then
          write (   6, 8605) ainput, ninput, nword
          write (iout, 8605) ainput, ninput, nword
          ngerr =  1
        endif

c....   Find the zone.

 8610   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named zone does not exist:  ',a)

        do 610 nzn = 1, nzones
          if (aword(4) .eq. azone(nzn)) then
            nz = nzn
            call zout (nz, nz, 'min')
            go to 620
          endif
  610   continue
        write (   6, 8610) ainput, ninput, aword(4)
        write (iout, 8610) ainput, ninput, aword(4)
        nerr = 1

c....   Find the brick.

  620   do 630 nbr = 1, nbricks
          if (aword(6) .eq. abrick(nbr)) then
            nb = nbr
            call brout (nb, nb, 'min')
            go to 640
          endif
  630   continue
        write (   6, 8510) ainput, ninput, aword(6)
        write (iout, 8510) ainput, ninput, aword(6)
        nerr = 1

  640   if     (asysbr(nb) .eq. 'rect') then
          ksys = 0
        elseif (asysbr(nb) .eq. 'cyl')  then
          ksys = 1
        elseif (asysbr(nb) .eq. 'sph')  then
          ksys = 2
        else
          write (   6, 8520) ainput, ninput, asysbr(nb)
          write (iout, 8520) ainput, ninput, asysbr(nb)
          nerr =  1
        endif

c....   Find the angle units.

        if (angbr(nb) .eq. 'radians') then
          iunit = 1
        elseif (angbr(nb) .eq. 'degrees') then
          iunit = 0
        else
          write (   6, 8520) ainput, ninput, angbr(nb)
          write (iout, 8520) ainput, ninput, angbr(nb)
          nerr =  1
        endif

c....   Loop over trial points.

        do 680 ntest = 1, 10000

c....     Randomly sample a point in the brick.

          call aptbrks (ksys, iunit, ubr(nb), dubr(nb),
     &                  vbr(nb), dvbr(nb),
     &                  wbr(nb), dwbr(nb), 1, tol,
     &                  u, v, w, fracu, fracv, fracw, nerra)

          call aptcsys (ksys, 0, iunit, u, v, w, 1, tol, nerra)

          xp = u
          yp = v
          zp = w

c....     See if the point is in the zone.

          nside = 1                   ! Assume point is in zone.

c....     Test each boundary surface of this zone.

          do 670 nzq = 1, nzquads(nz) ! Loop over boundary surfaces.

 8650      format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &       '  Named zone boundary surface does not exist:  ',a)

            do 650 nqq = 1, nquads     ! Loop over all quadrics.
              if (aquad(nqq) .eq. azquad(nzq,nz)) then
                nq = nqq
                go to 660
              endif
  650       continue                  ! End of loop over all quadrics
            write (   6, 8650) ainput, ninput, azquad(nzq,nz)
            write (iout, 8650) ainput, ninput, azquad(nzq,nz)
            nerr = 1
            go to 690

c....       Find the side of the quadric surface the point is on.

  660       dside(nzq) = qc(nq) +
     &              qx(nq) * xp + qy(nq) * yp + qz(nq) * zp +
     &              qxy(nq) * xp * yp +
     &              qyz(nq) * yp * zp +
     &              qzx(nq) * zp * xp +
     &              qxx(nq) * xp**2 + qyy(nq) * yp**2 + qzz(nq) * zp**2

            derr  = tol * (abs (qc(nq)) +
     &              abs (qx(nq) * xp) +
     &              abs (qy(nq) * yp) +
     &              abs (qz(nq) * zp) +
     &              abs (qxy(nq) * xp * yp) +
     &              abs (qyz(nq) * yp * zp) +
     &              abs (qzx(nq) * zp * xp) +
     &              abs (qxx(nq)) * xp**2 +
     &              abs (qyy(nq)) * yp**2 +
     &              abs (qzz(nq)) * zp**2)

            if (abs (dside(nzq)) .lt. derr) then
              dside(nzq) = 0.0
            endif

c....       See if the point is inside (dside(nzq) has opposite sign of azsign)
c....         or outside the zone.

            if ((dside(nzq) .gt. 0.0)      .and.
     &          (azsign(nzq,nz) .eq. '+')) then
              nside = 0
            endif

            if ((dside(nzq) .lt. 0.0)      .and.
     &          (azsign(nzq,nz) .eq. '-')) then
              nside = 0
            endif

  670     continue                    ! End of loop over boundary surfaces.

          if (nside .eq. 1) then      ! Found a point in the zone.
            go to 690
          endif

  680   continue                      ! End of loop over test points.

 8680   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Could not find a point in the zone.')

        write (   6, 8680) ainput, ninput
        write (iout, 8680) ainput, ninput
        nerr = 1
        go to 690

c....   Test for errors.

  690   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
          go to 999
        endif

c....   Found a point in the zone.

        apoint(n)  = aword(2)
        lapoint(n) = lword(2)
        apoints    = aword(2)
        lapoints   = lword(2)

        x(n)     = xp
        y(n)     = yp
        z(n)     = zp

c....   Print the coordinates of the point.

        amount = 'maximum'
        call pout (n, n, amount)

        go to 999

      endif                           ! Randomly sampled a point in a zone.

c=======================================================================********

c.... Input line:  point PNAME01 polygon PGNAME

      if ((aword(3) .eq. 'polygon') .or.
     &    (aword(3) .eq. 'polyg')   .or.
     &    (aword(3) .eq. 'pg'))     then

c....   Test number of words in input line.

 8540 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 words on "point PNAME01 polygon PGNAME" line:  ',
     &  i3)

        if (nword .ne. 4) then
          write (   6, 8540) ainput, ninput, nword
          write (iout, 8540) ainput, ninput, nword
          nerr = 1
          go to 580
        endif

c....   Find the regular polygon, and its vertices.

 8550 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named regular polygon does not exist:  ',a)

        do 550 nn = 1, npolygs
          if (apolyg(nn) .eq. aword(4)) then
            npg = nn

            call pgout (npg, npg, 'min')

            go to 555
          endif
  550   continue
        write (   6, 8550) ainput, ninput, aword(4)
        write (iout, 8550) ainput, ninput, aword(4)
        nerr = 1
        go to 580

c....   See if the point names can be generated uniquely.

 8555 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' point names from:  ',a)

  555   anamex  = aword(2)
        lanamex = lword(2)
        nverx   = nverpg(npg) - 1

        call incrsubs (nverx, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8555) ainput, ninput, nver, aword(2)
          write (iout, 8555) ainput, ninput, nver, aword(2)
          go to 580
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
          go to 580
        endif

c....   Ignore the previous selection of the point array index.

        if (nrep .eq. 0) then
          if (n .eq. npoints) npoints = npoints - 1
        endif

c....   Store the vertex points.

        anamex  = aword(2)            ! Initial point name.
        lanamex = lword(2)            ! Initial point name length.

        do 560 nver = 1, nverpg(npg)

c....     Find the index in the point list for the new point.

          aword(2) = anamex
          lword(2) = lanamex

          call ptstore (nrep, n)

          if (n .eq. 0) go to 999

          apoint(n)  = anamex
          lapoint(n) = lanamex

          apoints    = anamex
          lapoints   = lanamex

          x(n)      = xver(nver)
          y(n)      = yver(nver)
          z(n)      = zver(nver)

          amount = 'maximum'
          call pout (n, n, amount)

c....     Find the next point name, by incrementing by 1.

          call incrsubs (1, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  560   continue

        go to 999

      endif                           ! Found vertex points of polygon.

c=======================================================================********

c.... Input line:  point PNAME01 polyhedron PHNAME

      if ((aword(3) .eq. 'polyhedron') .or.
     &    (aword(3) .eq. 'polyh')      .or.
     &    (aword(3) .eq. 'ph'))        then

c....   Test number of words in input line.

 8240 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 words on "point PNAME01 polyhedron PHNAME" line:  ',
     &  i3)

        if (nword .ne. 4) then
          write (   6, 8240) ainput, ninput, nword
          write (iout, 8240) ainput, ninput, nword
          nerr = 1
          go to 280
        endif

c....   Find the regular polyhedron, and its vertices.

 8250 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named regular polyhedron does not exist:  ',a)

        do 250 nn = 1, npolyhs
          if (apolyh(nn) .eq. aword(4)) then
            nph = nn

            call phout (nph, nph, 'min')

            go to 255
          endif
  250   continue
        write (   6, 8250) ainput, ninput, aword(4)
        write (iout, 8250) ainput, ninput, aword(4)
        nerr = 1
        go to 280

c....   See if the point names can be generated uniquely.

 8255 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate point names from:  ',a)

  255   anamex  = aword(2)
        lanamex = lword(2)
        nverx   = nverph(nph) - 1

        call incrsubs (nverx, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8255) ainput, ninput, aword(2)
          write (iout, 8255) ainput, ninput, aword(2)
          go to 280
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
          go to 280
        endif

c....   Ignore the previous selection of the point array index.

        if (nrep .eq. 0) then
          if (n .eq. npoints) npoints = npoints - 1
        endif

c....   Store the vertex points.

        anamex  = aword(2)            ! Initial point name.
        lanamex = lword(2)            ! Initial point name length.

        do 260 nver = 1, nverph(nph)  ! Loop over vertices of polyhedron.

c....     Find the index in the point list for the new point.

          aword(2) = anamex
          lword(2) = lanamex

          call ptstore (nrep, n)

          if (n .eq. 0) go to 999

          apoint(n)  = anamex
          lapoint(n) = lanamex

          apoints    = anamex
          lapoints   = lanamex

          x(n)      = xverh(nver)
          y(n)      = yverh(nver)
          z(n)      = zverh(nver)

          amount = 'maximum'
          call pout (n, n, amount)

c....     Find the next point name, by incrementing by 1.

          call incrsubs (1, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 280
          endif

  260   continue                      ! End of loop over vertices of polyhedron.

c....   Test for errors.

  280   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
        endif

        go to 999

      endif                           ! Found vertex points of polyhedron.

c=======================================================================********

c.... Input line:  point PNAME     X Y Z                       ! X    numerical.
c.... Input line:  point PNAME     RCYL THETA Z                ! RCYL numerical.
c.... Input line:  point PNAME     RSPH THETA PHI              ! RSPH numerical.
c.... Input line:  point PNAME [=] X Y Z
c.... Input line:  point PNAME [=] RCYL THETA Z
c.... Input line:  point PNAME [=] RSPH THETA PHI

      if ((nword .eq. 5)      .and.
     &   ((mtype(3) .eq. 2)   .or.
     &    (mtype(3) .eq. 3))) then
        nword    = 6
        aword(6) = aword(5)
        mtype(6) = mtype(5)
        iword(6) = iword(5)
        fword(6) = fword(5)
        lword(6) = lword(5)
        aword(5) = aword(4)
        mtype(5) = mtype(4)
        iword(5) = iword(4)
        fword(5) = fword(4)
        lword(5) = lword(4)
        aword(4) = aword(3)
        mtype(4) = mtype(3)
        iword(4) = iword(3)
        fword(4) = fword(3)
        lword(4) = lword(3)
        aword(3) = '='
        mtype(3) = 1
        iword(3) = 0
        fword(3) = 0.0
        lword(3) = 1
      endif

      if (aword(3) .eq. '=') then

c.... Test for wrong number of words on "point" line.

 8146 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 6 words on input line:  ', i3)

      if (nword .ne. 6) then
        write (   6, 8146) ainput, ninput, nword
        write (iout, 8146) ainput, ninput, nword
        nerr = 1
      endif

 8903 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      call varfind (4, nword)

      do 150 nw = 4, 6
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8903) ainput, ninput, aword(nw)
          write (iout, 8903) ainput, ninput, aword(nw)
          nerr = 1
        endif
  150 continue

 8170 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Coordinate system not specified.')

c.... Find the coordinates of the point.

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

      u = fword(4)
      v = fword(5)
      w = fword(6)

      fact = 1.0
      if (angunit .eq. 'degrees') fact = pi / 180.0

      if (asys .eq. 'cyl') then

 8173   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Angle theta is not from -180 to 360 degrees:  ',a)
 8193   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Angle theta is not from -pi to 2*pi radians:  ',a)

        if ((fact * v .lt. -pi) .or.
     &      (fact * v .gt. 2.0 * pi)) then
          if (angunit .eq. 'degees') then
            write (   6, 8173) ainput, ninput, aword(5)
            write (iout, 8173) ainput, ninput, aword(5)
          else
            write (   6, 8193) ainput, ninput, aword(5)
            write (iout, 8193) ainput, ninput, aword(5)
          endif
          nerr = 1
        endif

        call aptcsys (1, 0, iunit, u, v, w, 1, tol, nerra)

      elseif (asys .eq. 'sph') then

        if ((fact * v .lt. -pi) .or.
     &      (fact * v .gt. 2.0 * pi)) then
          if (angunit .eq. 'degees') then
            write (   6, 8173) ainput, ninput, aword(5)
            write (iout, 8173) ainput, ninput, aword(5)
          else
            write (   6, 8193) ainput, ninput, aword(5)
            write (iout, 8193) ainput, ninput, aword(5)
          endif
          nerr = 1
        endif

 8174   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Angle phi is not from 0 to 180 degrees:  ',a)
 8194   format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Angle phi is not from 0 to pi radians:  ',a)

        if ((fact * w .lt. 0.0) .or.
     &      (fact * w .gt. pi)) then
          if (angunit .eq. 'degrees') then
            write (   6, 8174) ainput, ninput, aword(6)
            write (iout, 8174) ainput, ninput, aword(6)
          else
            write (   6, 8194) ainput, ninput, aword(6)
            write (iout, 8194) ainput, ninput, aword(6)
          endif
          nerr = 1
        endif

        call aptcsys (2, 0, iunit, u, v, w, 1, tol, nerra)

      elseif (asys .ne. 'rect') then
        write (   6, 8170) ainput, ninput
        write (iout, 8170) ainput, ninput
        nerr = 1
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          if (n .eq. npoints) npoints = npoints - 1
        endif
        go to 999
      endif

      apoint(n)  = aword(2)
      lapoint(n) = lword(2)
      apoints    = aword(2)
      lapoints   = lword(2)

      x(n)     = u
      y(n)     = v
      z(n)     = w

c.... Print the coordinates of the point.

      amount = 'maximum'
      call pout (n, n, amount)

      go to 999

      endif                           ! Word 3 is "=".

c===============================================================================

 8285 format ('  point ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8285) ainput, ninput
      write (iout, 8285) ainput, ninput
      nerr = 1

c===============================================================================

c.... Test for errors.

  580   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. npoints) npoints = npoints - 1
          endif
        endif

  999 return

c.... End of subroutine point.        (+1 line)
      end
