      subroutine polyhed

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  polyhed  Updated     2005 April 5 17:45.
c               polyhed  Originated  1994 March 15 14:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create regular polyhedrons by specifying a
c               center, the first vertex point, and a point coplanar with
c               the center, first vertex and second vertex.
c
c     Logic:    Entry points:  polyhed.
c polyhed       Called by:  helpall, main.
c
c polyhed       Calls:  aptsubr, incrsubs, namechk, phout, ptfind, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8  amount             ! Amount of output to write in phout.

c.... Input line:  polyhedron [help]

 8110 format (
     &  '  Command "polyhedron":  acceptable input lines include:' /
     &  '  "polyhedron [help]"  Display this message.' /
     &  '  "polyhedron all"     Display all regular polyhedrons.' /
     &  '  "polyhedron list [PHNAME1 PHNAME2 PHNAME3 ...]"' /
     &  '     Display all listed regular polyhedrons (default is',
     &  ' all).' /
     &  '  "polyhedron PHNAME"  Display regular polyhedron PHNAME.' //
     &  '  "polyhedron PHNAME fit NVER PCEN PVER PCOP"' /
     &  '     Create regular polyhedron PHNAME with NVER',
     &  ' vertices,' /
     &  '     centered at point PCEN, with the first vertex at',
     &  ' point PVER,' /
     &  '     and with PCEN, PVER and the second vertex coplanar',
     &  ' with point PCOP.' /
     &  '     NVER must be 4 (tetrahedron), 6 (octahedron),',
     &  ' 8 (cube),' /
     &  '     12 (icosahedron) or 20 (dodecahedron).' )
 8112 format (/
     &  '  Note:  use +/- for PHNAME to incr/decr the last polyhedron',
     &  ' name:  ',a //
     &  '  Synonyms:  [help, h], [polyhedron, polyh, ph].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112) apolyhs
        write (iout, 8112) apolyhs
        go to 999
      endif

c===============================================================================

c.... Input line:  polyhedron all
c.... Input line:  polyhedron list

 8120 format (/ 'Number of polyhedrons = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        npolyhx = 0
        do 120 n = 1, npolyhs
          if (apolyh(n) .eq. ' ') go to 120
          npolyhx = npolyhx + 1
  120   continue
        if (npolyhx .eq. 1) then
          amount = 'maximum'
        else
          amount = 'minimum'
        endif
        call phout (1, npolyhs, amount)
        write (   6, 8120) npolyhx
        write (iout, 8120) npolyhx
        go to 999

      endif

c===============================================================================

c.... Input line:  polyhedron list PHNAME1 PHNAME2 PHNAME3 ...

 8130 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named polyhedron does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, npolyhs
            call aptsubr (apolyh(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. apolyh(n)) .or.
     &          (aword(nw) .eq. astem))    then
              nfind = nfind + 1
              call phout (n, n, 'min')
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c===============================================================================

c.... Input line:  polyhedron PHNAME

c.... See if user wants one polyhedron displayed.

      if (nword .eq. 2) then
        do 130 n = 1, npolyhs
          if (aword(2) .eq. apolyh(n)) then
            amount = 'maximum'
            call phout (n, n, amount)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test input for validity.

 8135 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "polyhedron" line not 7:  ',i3)
      if (nword .ne. 7) then
        write (   6, 8135) ainput, ninput, nword
        write (iout, 8135) ainput, ninput, nword
        go to 999
      endif

 8136 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of polyhedron not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

 8138 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of vertices not an integer or integer variable:  ',a)

      call varfind (4, 4)

      if (mtype(4) .ne. 2) then
        write (   6, 8138) ainput, ninput, aword(4)
        write (iout, 8138) ainput, ninput, aword(4)
        go to 999
      endif

      nverphx = iword(4)

 8139 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of vertices not 4, 6, 8, 12 or 20:  ',i5)

      if ((nverphx .ne.  4)  .and.
     &    (nverphx .ne.  6)  .and.
     &    (nverphx .ne.  8)  .and.
     &    (nverphx .ne. 12)  .and.
     &    (nverphx .ne. 20)) then
        write (   6, 8139) ainput, ninput, iword(4)
        write (iout, 8139) ainput, ninput, iword(4)
        go to 999
      endif

 8137 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      if (aword(3) .ne. 'fit') then
        write (   6, 8137) ainput, ninput
        write (iout, 8137) ainput, ninput
        go to 999
      endif

c===============================================================================

c.... See if last polyhedron name should be incremented and used.

 8128 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last polyhedron can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = apolyhs
        lword(2) = lapolyhs
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, apolyhs
          write (iout, 8128) ainput, ninput, apolyhs
          go to 999
        endif
      endif

 8129 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last polyhedron can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = apolyhs
        lword(2) = lapolyhs
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, apolyhs
          write (iout, 8129) ainput, ninput, apolyhs
          go to 999
        endif
      endif

 8905 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of polyhedron too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if polyhedron exists.

 8160 format ('  WARNING:  replacing existing polyhedron:  ',a)

      nrep = 0
      do 136 n = 1, npolyhs
        if (aword(2) .eq. apolyh(n)) then
          nrep = 1
          write (   6, 8160) apolyh(n)
          write (iout, 8160) apolyh(n)
          call phout (n, n, 'min')
          go to 138
        endif
  136 continue

      do 137 n = 1, npolyhs
        if (apolyh(n) .eq. ' ') go to 138
  137 continue

      npolyhs = npolyhs + 1

 8165 format ('  polyhed ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of polyhedrons exceeds limit ',i5)

      if (npolyhs .gt. npolyhm) then
        npolyhs = npolyhm
        write (   6, 8165) ainput, ninput, npolyhm
        write (iout, 8165) ainput, ninput, npolyhm
        go to 999
      endif

      n = npolyhs

  138 nerr = 0

c===============================================================================

c.... Input line:  polyhedron PHNAME fit NVER PCEN PVER PCOP

c.... Find the three points defining the polyhedron.

      call ptfind (5, ncen)

      if (ncen .eq. 0) then
        nerr = 1
        go to 150
      endif

  150 call ptfind (6, nver)

      if (nver .eq. 0) then
        nerr = 1
        go to 170
      endif

  170 call ptfind (7, ncop)

      if (ncop .eq. 0) then
        nerr = 1
        go to 190
      endif

c.... Find the vertex points and other polyhedron data.

  190 if     (nverphx .eq.  4) then   ! Tetrahedron.    4 faces.
        atypeh(n) = 'tetrahedron'
        nedgphx = 6
      elseif (nverphx .eq.  6) then   ! Octahedron      8 faces.
        atypeh(n) = 'octahedron'
        nedgphx = 12
      elseif (nverphx .eq.  8) then   ! Cube            6 faces.
        atypeh(n) = 'cube'
        nedgphx = 12
      elseif (nverphx .eq. 12) then   ! Icosahedron.   20 faces.
        atypeh(n) = 'icosahedron'
        nedgphx = 30
      elseif (nverphx .eq. 20) then   ! Dodecahedron.  12 faces.
        atypeh(n) = 'dodecahedron'
        nedgphx = 30
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          apolyh(n) = ' '
          if (n .eq. npolyhs) npolyhs = npolyhs - 1
        endif
        go to 999
      endif

c.... Store and display the polyhedron specifications.

      apolyh(n) = aword(2)
      apolyhs   = aword(2)
      lapolyhs  = lword(2)

      nverph(n) = nverphx
      nedgph(n) = nedgphx
      acenph(n) = apoint(ncen)
      averph(n) = apoint(nver)
      acopph(n) = apoint(ncop)

      amount = 'maximum'
      call phout (n, n, amount)

  999 return

c.... End of subroutine polyhed.      (+1 line)
      end
