      subroutine pout (n1, n2, amount)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  pout     Updated     2005 March 8 16:00.
c               pout     Originated  1993 November 3 13:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display the coordinates and mesh indices, if any,
c               of points n1 to n2.
c
c     Logic:    Entry points:  pout.
c pout          Called by:  area, clout, cloutcl, copy, delete, distance,
c               dkout, lout, matrix, mesh, meshblk, meshdel, meshexch,
c               meshins, meshinv, meshline, meshmove, meshout, meshrel,
c               meshrlxb, meshrlxm, meshtest, movex, pgout, phout, point,
c               project, ptfind, ptoper, ptstore, renamea, renamex, search,
c               side, tetout, tetstore, trout, twist, walk.
c
c pout          Calls:  plotpt, ptcoord.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8 amount

c.... Initialize.

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. npoints) nn2 = npoints

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

 8050 format ('p ',a,1p3e17.9)
 8100 format ('p ',a16,1p3e20.12)
 8200 format ('point  ',a8,' = ',1p3e20.12)
 8102 format ('  mesh point ',a,2x,3i6,7x,'! ASSIGNED')
 8104 format ('  mesh point ',a,2x,3i6,7x,'! VACUUM')
 8105 format ('    WARNING:  in cluster                 ',a)
 8110 format ('    WARNING:  defines line               ',a)
 8120 format ('    WARNING:  defines triangle           ',a)
 8125 format ('    WARNING:  defines regular polygon    ',a)
 8126 format ('    WARNING:  defines annular disk       ',a)
 8128 format ('    WARNING:  defines regular polyhedron ',a)
 8130 format ('    WARNING:  defines tetrahedron        ',a)

      do 150 np = nn1, nn2            ! Loop over points.
        if (apoint(np) .eq. ' ') go to 150

        call ptcoord (asys, angunit, x(np), y(np), z(np),
     &                u, v, w, vpos, nerra)

        if (lapoint(np) .gt. 16) then
          write (   6, 8050) apoint(np), u, v, w
          write (iout, 8050) apoint(np), u, v, w
        elseif (lapoint(np) .gt. 8) then
          write (   6, 8100) apoint(np), u, v, w
          write (iout, 8100) apoint(np), u, v, w
        else
          write (   6, 8200) apoint(np), u, v, w
          write (iout, 8200) apoint(np), u, v, w
        endif

        call plotpt (x(np), y(np), z(np), nerrg) ! Store if plot status is "on".

        if (vpos .gt. 0.0) then
          if (lapoint(np) .gt. 8) then
            write (   6, 8100) apoint(np), u, vpos, w
            write (iout, 8100) apoint(np), u, vpos, w
          else
            write (   6, 8200) apoint(np), u, vpos, w
            write (iout, 8200) apoint(np), u, vpos, w
          endif
        endif

        if ((kpoint(np) .ge. 1)  .and.
     &      (lpoint(np) .ge. 1)  .and.
     &      (mpoint(np) .ge. 1)) then

          klm = kpoint(np) + (lpoint(np) - 1) * kmax +
     &          (mpoint(np) - 1) * kmax * lmax

          if ((x(np) .le. -1.e99)  .or.
     &        (y(np) .le. -1.e99)  .or.
     &        (z(np) .le. -1.e99)) then

            write (   6, 8104) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np)
            write (iout, 8104) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np)

          else                        ! Not a void point.

            write (   6, 8102) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np)
            write (iout, 8102) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np)

          endif                       ! Tested for a void point.

        endif

        if ((amount .eq. 'max')      .or.
     &      (amount .eq. 'maximum')) then

          do 105 npr = 1, npairs
            if ((apairpt(npr) .eq. apoint(np)) .and.
     &          (apaircl(npr) .ne. ' '))      then
              write (   6, 8105) apaircl(npr)
              write (iout, 8105) apaircl(npr)
            endif
  105     continue

          do 110 nn = 1, nlines
            if (aline(nn) .eq. ' ') go to 110
            if ((alipt1(nn) .eq. apoint(np))  .or.
     &          (alipt2(nn) .eq. apoint(np))) then
              write (   6, 8110) aline(nn)
              write (iout, 8110) aline(nn)
            endif
  110     continue

          do 120 nn = 1, ntrigs
            if (atrig(nn) .eq. ' ') go to 120
            if ((atript1(nn) .eq. apoint(np))  .or.
     &          (atript2(nn) .eq. apoint(np))  .or.
     &          (atript3(nn) .eq. apoint(np))) then
              write (   6, 8120) atrig(nn)
              write (iout, 8120) atrig(nn)
            endif
  120     continue

          do 125 nn = 1, npolygs
            if (apolyg(nn) .eq. ' ') go to 125
            if ((acenpg(nn) .eq. apoint(np))  .or.
     &          (averpg(nn) .eq. apoint(np))  .or.
     &          (acoppg(nn) .eq. apoint(np))) then
              write (   6, 8125) apolyg(nn)
              write (iout, 8125) apolyg(nn)
            endif
  125     continue

          do 126 nn = 1, ndisks
            if (adisk(nn) .eq. ' ') go to 126
            if (acendk(nn) .eq. apoint(np)) then
              write (   6, 8126) adisk(nn)
              write (iout, 8126) adisk(nn)
            endif
  126     continue

          do 128 nn = 1, npolyhs
            if (apolyh(nn) .eq. ' ') go to 128
            if ((acenph(nn) .eq. apoint(np))  .or.
     &          (averph(nn) .eq. apoint(np))  .or.
     &          (acopph(nn) .eq. apoint(np))) then
              write (   6, 8128) apolyh(nn)
              write (iout, 8128) apolyh(nn)
            endif
  128     continue

          do 130 nn = 1, ntets
            if (atetra(nn) .eq. ' ') go to 130
            if ((atetpt(1,nn) .eq. apoint(np))  .or.
     &          (atetpt(2,nn) .eq. apoint(np))  .or.
     &          (atetpt(3,nn) .eq. apoint(np))  .or.
     &          (atetpt(4,nn) .eq. apoint(np))) then
              write (   6, 8130) atetra(nn)
              write (iout, 8130) atetra(nn)
            endif
  130     continue

        endif

  150 continue                        ! End of loop over points.

  999 return

c.... End of subroutine pout.         (+1 line)
      end
