      subroutine project

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  project  Updated     2004 November 23 13:30.
c               project  Originated  1994 February 18 14:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To project a point, a cluster, or all points onto a plane,
c               cylinder or sphere, with various options.
c
c     Logic:    Entry points:  project.
c project       Called by:  helpall, main.
c
c project       Calls:  aptpers, aptprcy, aptprop, aptprsp, aptradv, clout,
c               plout, pout, ptfind, varfind, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lproject/ amount        ! Amount of output data.
      character*8       amount        ! Amount of output data.

c.... Input line:  project [help]

 8110 format ('  Command "project":  acceptable input lines include:' /
     &  '  "project [help]"  Display this message.' //
     &  '   In the input lines below, "OBJTYPE OBJNAME" must be',
     &  ' "point PNAME",' /
     &  '     "cluster CLNAME" or "point all".' //
     &  '  "project OBJTYPE OBJNAME plane PLNAME"' /
     &  '     Project points normally onto plane PLNAME.' /
     &  '  "project OBJTYPE OBJNAME plane PCEN VNORM"' /
     &  '     Project points normally onto the plane through' /
     &  '     point PCEN with normal vector VNORM.' )
 8111 format (
     &  '  "project OBJTYPE OBJNAME perspective PVIEW PFOC',
     &  ' PTL PTR"' /
     &  '     Project points toward view point PVIEW, onto a plane' /
     &  '     through point PFOC perpendicular to the line from',
     &  ' PVIEW to PFOC,' /
     &  '     then rotate and translate onto plane z = 0, with PFOC',
     &  ' at the origin,' /
     &  '     and a horizon from point PTL on the left to point PTR',
     &  ' on the right.' /
     &  '  "project OBJTYPE OBJNAME viewfactor PVIEW VNAME"' /
     &  '     to make a geometric view factor projection of points' /
     &  '     based on the view from point PVIEW, in the direction of',
     &  ' the vector VNAME,' /
     &  '     and project the results onto plane z = 0,' )
 8112 format (
     &  '  "project OBJTYPE OBJNAME cylinder RCYL PCEN VAXIS',
     &  ' [cyl,sph,cos|cyl]"' /
     &  '     Project points onto the cylinder with radius RCYL,' /
     &  '     on an axis through point PCEN in the direction of the',
     &  ' vector VAXIS.' /
     &  '     Does not project point PCEN.' /
     &  '     Project [cyl] (default) toward the axis, or' /
     &  '     [sph] toward the central point PCEN, or' /
     &  '     [cos] to make the axial distance from point PCEN',
     &  ' equal to the cosine' /
     &  '     of the spherical polar angle PHI from the axis,',
     &  ' measured from point PCEN.' )
 8113 format (
     &  '  "project OBJTYPE OBJNAME sphere RSPH PCEN VAXIS',
     &  ' [sph,cyl,polar|sph]"' /
     &  '     Project points onto the sphere with radius RSPH,' /
     &  '     centered at point PCEN.  Does not project point PCEN.' /
     &  '     Project [sph] (default, and VAXIS is not needed)',
     &  ' toward the center PCEN, or' /
     &  '     [cyl] toward the polar axis through point PCEN in the',
     &  ' direction of the' /
     &  '     vector VAXIS, or' /
     &  '     [polar] in the direction of the polar axis.' )
 8114 format (/
     &  '  Synonyms:  [cosine, cos], [cylinder, cyl],',
     &  ' [cylindrical, cyl],' /
     &  '    [cluster, cl], [help, h], [perspective, pers],',
     &  ' [plane, pl],' /
     &  '    [point, pnt, pt, p], [project, proj], [sphere, sph],',
     &  ' [spherical, sph],' /
     &  '    [viewfactor, view]' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8111)
        write (iout, 8111)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8113)
        write (iout, 8113)
        write (   6, 8114)
        write (iout, 8114)
        go to 999
      endif                           ! Tested nword, aword(2).

c===============================================================================

c.... See if projecting one or all points (ncl = 0) or a cluster of points.

 8118   format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named cluster does not exist:  ',a)
 8120   format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Object type not recognized:  ',a)

      if     ((aword(2) .eq. 'point')   .or.
     &        (aword(2) .eq. 'pnt')     .or.
     &        (aword(2) .eq. 'pt')      .or.
     &        (aword(2) .eq. 'p'))      then
        ncl = 0
        go to 122
      elseif ((aword(2) .eq. 'cluster') .or.
     &        (aword(2) .eq. 'cl'))     then
        do 120 n = 1, nclusts
          if (aword(3) .eq. aclust(n)) then
            ncl = n
            call clout (ncl, ncl, 'min')
            if (aclust(ncl) .eq. ' ') go to 120
            go to 122
          endif
  120   continue
        write (   6, 8118) ainput, ninput, aword(3)
        write (iout, 8118) ainput, ninput, aword(3)
        go to 999
      else
        write (   6, 8120) ainput, ninput, aword(2)
        write (iout, 8120) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  project point   PNAME  plane PLNAME
c.... Input line:  project cluster CLNAME plane PLNAME
c.... Input line:  project point   all    plane PLNAME

  122 if (((aword(4) .eq. 'plane') .or.
     &     (aword(4) .eq. 'pl'))   .and.
     &     (nword .eq. 5))         then

c....   Find the plane.

 8130   format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named plane does not exist:  ',a)

        do 130 nq = 1, nquads
          if ((aword(5) .eq. aquad(nq))     .and.
     &        (aqtype(nq) .eq. aqtypes(0))) then
            call plout (nq, nq, 'min')
            go to 135
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(5)
        write (iout, 8130) ainput, ninput, aword(5)
        go to 999

c....   Find the point(s) to project.
  
  135   nproj = 0

        if (ncl .eq. 0) then          ! Project one or all points.

          do 140 np = 1, npoints      ! Loop over points. 

            if (apoint(np) .eq. ' ') go to 140

            if ((aword(3) .eq. apoint(np)) .or.
     &          (aword(3) .eq. 'all'))     then

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptprop (xcen(nq), ycen(nq), zcen(nq),
     &                      qx(nq),   qy(nq),   qz(nq),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    z(np), nerr)

              amount = 'maximum'
              call pout (np, np, amount)

            endif

  140     continue                    ! End of loop over points.

 8140     format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named point does not exist:  ',a)

          if (nproj .eq. 0) then
            write (   6, 8140) ainput, ninput, aword(3)
            write (iout, 8140) ainput, ninput, aword(3)
          endif

        else                          ! Project a cluster of points

          do 144 npr = 1, npairs      ! Loop over cluster pairs.
            if (apaircl(npr) .ne. aclust(ncl)) go to 144

            do 142 np = 1, npoints    ! Loop over points.
              if (apoint(np) .ne. apairpt(npr)) go to 142

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptprop (xcen(nq), ycen(nq), zcen(nq),
     &                      qx(nq),   qy(nq),   qz(nq),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    z(np), nerr)

              amount = 'maximum'
              call pout (np, np, amount)

              go to 144               ! Find next point in cluster.

  142       continue                  ! End of loop over points.

 8142       format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named cluster point does not exist:  ',a)

            write (   6, 8142) ainput, ninput, apairpt(npr)
            write (iout, 8142) ainput, ninput, apairpt(npr)
            go to 999

  144     continue                    ! End of loop over cluster pairs.

        endif                         ! Tested ncl.

        go to 999

      endif                           ! Option:  ... plane PLNAME.

c===============================================================================

c.... Input line:  project point   PNAME  plane PCEN VNORM
c.... Input line:  project cluster CLNAME plane PCEN VNORM
c.... Input line:  project point   all    plane PCEN VNORM

      if (((aword(4) .eq. 'plane') .or.
     &     (aword(4) .eq. 'pl'))   .and.
     &     (nword .eq. 6))         then

c....   Find the point in the plane.

        call ptfind (5, np)

        if (np .eq. 0) go to 999

        xcenp = x(np)
        ycenp = y(np)
        zcenp = z(np)

c....   Find the normal vector to the plane.

        call vectfind (6, nv)

        if (nv .eq. 0) go to 999

 8200 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named vector has zero length:  ',a)

        if (vlen(nv) .eq. 0.0) then
          write (   6, 8200) ainput, ninput, aword(6)
          write (iout, 8200) ainput, ninput, aword(6)
          go to 999
        endif

c....   Find the point(s) to project.
  
        nproj = 0

        if (ncl .eq. 0) then          ! Project one or all points.

          do 210 np = 1, npoints      ! Loop over points. 

            if (apoint(np) .eq. ' ') go to 210

            if ((aword(3) .eq. apoint(np)) .or.
     &          (aword(3) .eq. 'all'))     then

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptprop (xcenp,    ycenp,    zcenp,
     &                      xcom(nv), ycom(nv), zcom(nv),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    z(np), nerr)

              amount = 'maximum'
              call pout (np, np, amount)

            endif

  210     continue                    ! End of loop over points.

          if (nproj .eq. 0) then
            write (   6, 8140) ainput, ninput, aword(3)
            write (iout, 8140) ainput, ninput, aword(3)
          endif

        else                          ! Project a cluster of points

          do 214 npr = 1, npairs      ! Loop over cluster pairs.
            if (apaircl(npr) .ne. aclust(ncl)) go to 214

            do 212 np = 1, npoints    ! Loop over points.
              if (apoint(np) .ne. apairpt(npr)) go to 212

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptprop (xcenp,    ycenp,    zcenp,
     &                      xcom(nv), ycom(nv), zcom(nv),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    z(np), nerr)

              amount = 'maximum'
              call pout (np, np, amount)

              go to 214               ! Find next point in cluster.

  212       continue                  ! End of loop over points.

            write (   6, 8142) ainput, ninput, apairpt(npr)
            write (iout, 8142) ainput, ninput, apairpt(npr)
            go to 999

  214     continue                    ! End of loop over cluster pairs.

        endif                         ! Tested ncl.

        go to 999

      endif                           ! Option:  ... plane PCEN VNORM

c===============================================================================

c.... Input line:  project point   PNAME  perspective PVIEW PFOC PTL PTR
c.... Input line:  project cluster CLNAME perspective PVIEW PFOC PTL PTR
c.... Input line:  project point   all    perspective PVIEW PFOC PTL PTR

      if ((aword(4) .eq. 'perspective') .or.
     &    (aword(4) .eq. 'pers'))       then

 8220 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "project ... perspective" line not 8:  ',
     &  i3)

        if (nword .ne. 8) then
          write (   6, 8220) ainput, ninput, nword
          write (iout, 8220) ainput, ninput, nword
          go to 999
        endif

c....   Find the view point.

        call ptfind (5, np)

        if (np .eq. 0) go to 999

        xview = x(np)
        yview = y(np)
        zview = z(np)

c....   Find the focal point.

        call ptfind (6, np)

        if (np .eq. 0) go to 999

        xfoc = x(np)
        yfoc = y(np)
        zfoc = z(np)

c....   Find the left horizon point.

        call ptfind (7, np)

        if (np .eq. 0) go to 999

        xleft = x(np)
        yleft = y(np)
        zleft = z(np)

c....   Find the right horizon point.

        call ptfind (8, np)

        if (np .eq. 0) go to 999

        xright = x(np)
        yright = y(np)
        zright = z(np)

c....   Find the point(s) to project.
  
        nproj = 0

        if (ncl .eq. 0) then          ! Project one or all points.

          do 310 np = 1, npoints      ! Loop over points. 

            if (apoint(np) .eq. ' ') go to 310

            if ((aword(3) .eq. apoint(np)) .or.
     &          (aword(3) .eq. 'all'))     then

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptpers (xview,    yview,    zview,
     &                      xfoc,     yfoc,     zfoc,
     &                      xleft,    yleft,    zleft,
     &                      xright,   yright,   zright,
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    dap,   nerr)

              z(np) = 0.0

 8310         format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Point is behind view point.  Not projected:',a)

              if ((abs (x(np)) .ge. bignum)  .or.
     &            (abs (y(np)) .ge. bignum)) then
                write (   6, 8310) ainput, ninput, apoint(np)
                write (iout, 8310) ainput, ninput, apoint(np)
              endif

              amount = 'maximum'
              call pout (np, np, amount)

            endif

  310     continue                    ! End of loop over points.

          if (nproj .eq. 0) then
            write (   6, 8140) ainput, ninput, aword(3)
            write (iout, 8140) ainput, ninput, aword(3)
          endif

        else                          ! Project a cluster of points

          do 314 npr = 1, npairs      ! Loop over cluster pairs.
            if (apaircl(npr) .ne. aclust(ncl)) go to 314

            do 312 np = 1, npoints    ! Loop over points.
              if (apoint(np) .ne. apairpt(npr)) go to 312

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptpers (xview,    yview,    zview,
     &                      xfoc,     yfoc,     zfoc,
     &                      xleft,    yleft,    zleft,
     &                      xright,   yright,   zright,
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    dap,   nerr)

              z(np) = 0.0

              if ((abs (x(np)) .ge. bignum)  .or.
     &            (abs (y(np)) .ge. bignum)) then
                write (   6, 8310) ainput, ninput, apoint(np)
                write (iout, 8310) ainput, ninput, apoint(np)
              endif

              amount = 'maximum'
              call pout (np, np, amount)

              go to 314               ! Find next point in cluster.

  312       continue                  ! End of loop over points.

            write (   6, 8142) ainput, ninput, apairpt(npr)
            write (iout, 8142) ainput, ninput, apairpt(npr)
            go to 999

  314     continue                    ! End of loop over cluster pairs.

        endif                         ! Tested ncl.

        go to 999

      endif                           ! Option:  ... view PVIEW PFOC PTL PTR

c===============================================================================

c.... Input line:  project point   PNAME  viewfactor PVIEW VNORM
c.... Input line:  project cluster CLNAME viewfactor PVIEW VNORM
c.... Input line:  project point   all    viewfactor PVIEW VNORM

      if ((aword(4) .eq. 'viewfactor') .or.
     &    (aword(4) .eq. 'view'))      then

 8320 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "project ... viewfactor" line not 5:  ',
     &  i3)

        if (nword .ne. 6) then
          write (   6, 8320) ainput, ninput, nword
          write (iout, 8320) ainput, ninput, nword
          go to 999
        endif

c....   Find the point in the viewing plane.

        call ptfind (5, np)

        if (np .eq. 0) go to 999

        xview = x(np)
        yview = y(np)
        zview = z(np)

c....   Find the normal vector to the viewing plane.

        call vectfind (6, nv)

        if (nv .eq. 0) go to 999

        if (vlen(nv) .eq. 0.0) then
          write (   6, 8200) ainput, ninput, aword(6)
          write (iout, 8200) ainput, ninput, aword(6)
          go to 999
        endif

c....   Find the point(s) to project.
  
        nproj = 0

        if (ncl .eq. 0) then          ! Project one or all points.

          do 370 np = 1, npoints      ! Loop over points. 

            if (apoint(np) .eq. ' ') go to 370

            if ((aword(3) .eq. apoint(np)) .or.
     &          (aword(3) .eq. 'all'))     then

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptradv (xview,    yview,    zview,
     &                      xcom(nv), ycom(nv), zcom(nv),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),           nerr)

              z(np) = 0.0

              if ((abs (x(np)) .ge. bignum)  .or.
     &            (abs (y(np)) .ge. bignum)) then
                write (   6, 8310) ainput, ninput, apoint(np)
                write (iout, 8310) ainput, ninput, apoint(np)
              endif

              amount = 'maximum'
              call pout (np, np, amount)

            endif

  370     continue                    ! End of loop over points.

          if (nproj .eq. 0) then
            write (   6, 8140) ainput, ninput, aword(3)
            write (iout, 8140) ainput, ninput, aword(3)
          endif

        else                          ! Project a cluster of points.

          do 374 npr = 1, npairs      ! Loop over cluster pairs.
            if (apaircl(npr) .ne. aclust(ncl)) go to 374

            do 372 np = 1, npoints    ! Loop over points.
              if (apoint(np) .ne. apairpt(npr)) go to 372

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptradv (xview,    yview,    zview,
     &                      xcom(nv), ycom(nv), zcom(nv),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),           nerr)

              z(np) = 0.0

              if ((abs (x(np)) .ge. bignum)  .or.
     &            (abs (y(np)) .ge. bignum)) then
                write (   6, 8310) ainput, ninput, apoint(np)
                write (iout, 8310) ainput, ninput, apoint(np)
              endif

              amount = 'maximum'
              call pout (np, np, amount)

              go to 374               ! Find next point in cluster.

  372       continue                  ! End of loop over points.

            write (   6, 8142) ainput, ninput, apairpt(npr)
            write (iout, 8142) ainput, ninput, apairpt(npr)
            go to 999

  374     continue                    ! End of loop over cluster pairs.

        endif                         ! Tested ncl.

        go to 999

      endif                           ! Option:  ... viewfactor PVIEW VNORM

c===============================================================================

c.... Input line:  project point   PNAME  cylinder RCYL PCEN VAXIS
c.... Input line:  project cluster CLNAME cylinder RCYL PCEN VAXIS
c.... Input line:  project point   all    cylinder RCYL PCEN VAXIS

c.... Input line:  project point   PNAME  cylinder RCYL PCEN VAXIS cylindrical
c.... Input line:  project cluster CLNAME cylinder RCYL PCEN VAXIS cylindrical
c.... Input line:  project point   all    cylinder RCYL PCEN VAXIS cylindrical

c.... Input line:  project point   PNAME  cylinder RCYL PCEN VAXIS spherical
c.... Input line:  project cluster CLNAME cylinder RCYL PCEN VAXIS spherical
c.... Input line:  project point   all    cylinder RCYL PCEN VAXIS spherical

c.... Input line:  project point   PNAME  cylinder RCYL PCEN VAXIS cosine
c.... Input line:  project cluster CLNAME cylinder RCYL PCEN VAXIS cosine
c.... Input line:  project point   all    cylinder RCYL PCEN VAXIS cosine

      if ((aword(4) .eq. 'cylinder') .or.
     &    (aword(4) .eq. 'cyl'))     then

c....   Find the radius of the cylinder.

 8380 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

        call varfind (5, 5)

        if ((mtype(5) .ne. 2) .and. (mtype(5) .ne. 3)) then
          write (   6, 8380) ainput, ninput, aword(5)
          write (iout, 8380) ainput, ninput, aword(5)
          go to 999
        endif

c....   Find the point on the axis of the cylinder.

        call ptfind (6, np)

        if (np .eq. 0) go to 999

        xcyl = x(np)
        ycyl = y(np)
        zcyl = z(np)

 8410 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not project point on axis of cylinder:  ',a)

        if (aword(3) .eq. aword(6)) then
          write (   6, 8410) ainput, ninput, aword(3)
          write (iout, 8410) ainput, ninput, aword(3)
          go to 999
        endif

c....   Find the axis of the cylinder.

        call vectfind (7, nv)

        if (nv .eq. 0) go to 999

        if (vlen(nv) .eq. 0.0) then
          write (   6, 8200) ainput, ninput, aword(7)
          write (iout, 8200) ainput, ninput, aword(7)
          go to 999
        endif

c....   Find the projection option.

 8430 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Option on "project ... cylinder ..." line is not',
     &  ' recognized:  ',a)
 8435 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "project ... cylinder ..."',
     &  ' line not 7 or 8:  ',i3)

        if (nword .eq. 7) then
          nopt = 0
        elseif (nword .eq. 8) then
          if     ((aword(8) .eq. 'cylindrical') .or.
     &            (aword(8) .eq. 'cyl'))        then
            nopt = 0
          elseif ((aword(8) .eq. 'spherical')   .or.
     &            (aword(8) .eq. 'sph'))        then
            nopt = 1
          elseif ((aword(8) .eq. 'cosine')      .or.
     &            (aword(8) .eq. 'cos'))        then
            nopt = 2
          else
            write (   6, 8430) ainput, ninput, aword(8)
            write (iout, 8430) ainput, ninput, aword(8)
            go to 999
          endif
        else
          write (   6, 8435) ainput, ninput, nword
          write (iout, 8435) ainput, ninput, nword
          go to 999
        endif

c....   Find the point(s) to project.
  
        nproj = 0

        if (ncl .eq. 0) then          ! Project one or all points.

          do 440 np = 1, npoints      ! Loop over points. 

            if (apoint(np) .eq. ' ') go to 440

            if (apoint(np) .eq. aword(6)) go to 440  ! Do not project PCEN.

            if ((aword(3) .eq. apoint(np)) .or.
     &          (aword(3) .eq. 'all'))     then

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptprcy (nopt,
     &                      xcyl,     ycyl,     zcyl,
     &                      xcom(nv), ycom(nv), zcom(nv), fword(5),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    z(np), nerr)

 8440         format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Point is on axis.  Not projected:  ',a)

              if ((abs (x(np)) .ge. bignum)  .or.
     &            (abs (y(np)) .ge. bignum)  .or.
     &            (abs (z(np)) .ge. bignum)) then
                write (   6, 8440) ainput, ninput, apoint(np)
                write (iout, 8440) ainput, ninput, apoint(np)
              endif

              amount = 'maximum'
              call pout (np, np, amount)

            endif

  440     continue                    ! End of loop over points.

          if (nproj .eq. 0) then
            write (   6, 8140) ainput, ninput, aword(3)
            write (iout, 8140) ainput, ninput, aword(3)
          endif

        else                          ! Project a cluster of points.

          do 444 npr = 1, npairs      ! Loop over cluster pairs.
            if (apaircl(npr) .ne. aclust(ncl)) go to 444

            do 442 np = 1, npoints    ! Loop over points.
              if (apoint(np) .ne. apairpt(npr)) go to 442

              if (apoint(np) .eq. aword(6)) go to 442  ! Do not project PCEN.

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptprcy (nopt,
     &                      xcyl,     ycyl,     zcyl,
     &                      xcom(nv), ycom(nv), zcom(nv), fword(5),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    z(np), nerr)

              if ((abs (x(np)) .ge. bignum)  .or.
     &            (abs (y(np)) .ge. bignum)  .or.
     &            (abs (z(np)) .ge. bignum)) then
                write (   6, 8440) ainput, ninput, apoint(np)
                write (iout, 8440) ainput, ninput, apoint(np)
              endif

              amount = 'maximum'
              call pout (np, np, amount)

              go to 444               ! Find next point in cluster.

  442       continue                  ! End of loop over points.

            write (   6, 8142) ainput, ninput, apairpt(npr)
            write (iout, 8142) ainput, ninput, apairpt(npr)
            go to 999

  444     continue                    ! End of loop over cluster pairs.

        endif                         ! Tested ncl.

        go to 999

      endif                           ! Option:  ... cyl RCYL PCEN VAXIS ...

c===============================================================================

c.... Input line:  project point   PNAME  sphere RSPH PCEN
c.... Input line:  project cluster CLNAME sphere RSPH PCEN
c.... Input line:  project point   all    sphere RSPH PCEN

c.... Input line:  project point   PNAME  sphere RSPH PCEN VAXIS cylindrical
c.... Input line:  project cluster CLNAME sphere RSPH PCEN VAXIS cylindrical
c.... Input line:  project point   all    sphere RSPH PCEN VAXIS cylindrical

c.... Input line:  project point   PNAME  sphere RSPH PCEN VAXIS polar
c.... Input line:  project cluster CLNAME sphere RSPH PCEN VAXIS polar
c.... Input line:  project point   all    sphere RSPH PCEN VAXIS polar

      if ((aword(4) .eq. 'sphere') .or.
     &    (aword(4) .eq. 'sph'))   then

c....   Find the radius of the sphere.

        call varfind (5, 5)

        if ((mtype(5) .ne. 2) .and. (mtype(5) .ne. 3)) then
          write (   6, 8380) ainput, ninput, aword(5)
          write (iout, 8380) ainput, ninput, aword(5)
          go to 999
        endif

c....   Find the center of the sphere.

        call ptfind (6, np)

        if (np .eq. 0) go to 999

        xsph = x(np)
        ysph = y(np)
        zsph = z(np)

 8510 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not project point at center of sphere:  ',a)

        if (aword(3) .eq. aword(6)) then
          write (   6, 8510) ainput, ninput, aword(3)
          write (iout, 8510) ainput, ninput, aword(3)
          go to 999
        endif

c....   Find the polar axis of the sphere.

        if (nword .gt. 6) then

          call vectfind (7, nv)

          if (nv .eq. 0) go to 999

          if (vlen(nv) .eq. 0.0) then
            write (   6, 8200) ainput, ninput, aword(7)
            write (iout, 8200) ainput, ninput, aword(7)
            go to 999
          endif

        endif                         ! Found the polar axis of the sphere.

c....   Find the projection option.

 8530 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Unrecognized option on "project ... sphere ..."',
     &  ' line:  ',a)
 8535 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "project ... sphere ..."',
     &  ' line not 6 to 8:  ',i3)

        if ((nword .eq. 6)  .or.
     &      (nword .eq. 7)) then
          nopt = 0
        elseif (nword .eq. 8) then
          if     ((aword(8) .eq. 'spherical')    .or.
     &            (aword(8) .eq. 'sph'))         then
            nopt = 0
          elseif ((aword(8) .eq. 'cylindrical')  .or.
     &            (aword(8) .eq. 'cyl'))         then
            nopt = 1
          elseif  (aword(8) .eq. 'polar')        then
            nopt = 2
          else
            write (   6, 8530) ainput, ninput, aword(8)
            write (iout, 8530) ainput, ninput, aword(8)
            go to 999
          endif
        else
          write (   6, 8535) ainput, ninput, nword
          write (iout, 8535) ainput, ninput, nword
          go to 999
        endif

c....   Find the point(s) to project.
  
        nproj = 0

        if (ncl .eq. 0) then          ! Project one or all points.

          do 540 np = 1, npoints      ! Loop over points. 

            if (apoint(np) .eq. ' ') go to 540

            if (apoint(np) .eq. aword(6)) go to 540  ! Do not project PCEN.

            if ((aword(3) .eq. apoint(np)) .or.
     &          (aword(3) .eq. 'all'))     then

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptprsp (nopt,
     &                      xsph,     ysph,     zsph,
     &                      xcom(nv), ycom(nv), zcom(nv), fword(5),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    z(np), nerr)

 8540         format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Projection of point is indeterminate:  ',a)

              if ((abs (x(np)) .ge. bignum)  .or.
     &            (abs (y(np)) .ge. bignum)  .or.
     &            (abs (z(np)) .ge. bignum)) then
                write (   6, 8540) ainput, ninput, apoint(np)
                write (iout, 8540) ainput, ninput, apoint(np)
              endif

              amount = 'maximum'
              call pout (np, np, amount)

            endif

  540     continue                    ! End of loop over points.

          if (nproj .eq. 0) then
            write (   6, 8140) ainput, ninput, aword(3)
            write (iout, 8140) ainput, ninput, aword(3)
          endif

        else                          ! Project a cluster of points.

          do 544 npr = 1, npairs      ! Loop over cluster pairs.
            if (apaircl(npr) .ne. aclust(ncl)) go to 544

            do 542 np = 1, npoints    ! Loop over points.
              if (apoint(np) .ne. apairpt(npr)) go to 542

              if (apoint(np) .eq. aword(6)) go to 542  ! Do not project PCEN.

              nproj = nproj + 1

              call pout (np, np, 'min')

              xpt = x(np)
              ypt = y(np)
              zpt = z(np)

              call aptprsp (nopt,
     &                      xsph,     ysph,     zsph,
     &                      xcom(nv), ycom(nv), zcom(nv), fword(5),
     &                      xpt,      ypt,      zpt,   1, tol,
     &                      x(np),    y(np),    z(np), nerr)

              if ((abs (x(np)) .ge. bignum)  .or.
     &            (abs (y(np)) .ge. bignum)  .or.
     &            (abs (z(np)) .ge. bignum)) then
                write (   6, 8540) ainput, ninput, apoint(np)
                write (iout, 8540) ainput, ninput, apoint(np)
              endif

              amount = 'maximum'
              call pout (np, np, amount)

              go to 544               ! Find next point in cluster.

  542       continue                  ! End of loop over points.

            write (   6, 8142) ainput, ninput, apairpt(npr)
            write (iout, 8142) ainput, ninput, apairpt(npr)
            go to 999

  544     continue                    ! End of loop over cluster pairs.

        endif                         ! Tested ncl.

        go to 999

      endif                           ! Option:  ... sphere RSPH PCEN VAXIS ...

c===============================================================================

c.... Unrecognized input line.

 8999 format ('  project ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8999) ainput, ninput
      write (iout, 8999) ainput, ninput

  999 return

c.... End of subroutine project.      (+1 line)
      end
