      subroutine prox

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  prox     Updated     1999 March 29 13:30.
c               prox     Originated  1994 October 11 11:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the proximal point on a quadric surface, relative to a
c               specified point.
c               Subroutine aptwhis is called first.  If it fails, which should
c               not happen unless the quadric surface equation has very strange
c               coefficients, then random directions from an isotropic
c               distribution, then from a cosine-power distribution of
c               increasing power are tried, then an iterative method using radii
c               of curvature and tangent planes is tried.
c
c     Logic:    Entry points:  prox.
c prox          Called by:  helpall, main.
c
c prox          Calls:  aptqnor, aptqprr, aptqprt, aptvdis, aptvusz, aptwhis,
c               plotpt, ptfind, quout, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*16 aside
      common /lprox/ aqp(20)          ! Type of quadric surface point.
      common /lprox/ qpx(20)          ! Coordinate x of quadric surface point.
      common /lprox/ qpy(20)          ! Coordinate y of quadric surface point.
      common /lprox/ qpz(20)          ! Coordinate z of quadric surface point.
      common /lprox/ ccx(4)           ! Coordinate x of proximal point.
      common /lprox/ ccy(4)           ! Coordinate y of proximal point.
      common /lprox/ ccz(4)           ! Coordinate z of proximal point.
      common /lprox/ ddist(4)         ! Distance to proximal point.

c.... Input line:  proximal [help]

 8110 format ('  Command "proximal":  acceptable input lines',
     &  ' include:' /
     &  '  "proximal [help]"       Display this message.' //
     &  '  "proximal PNAME QNAME [NUMDIR|100]"' /
     &  '   Find the proximal point on quadric surface QNAME,',
     &  ' for point PNAME,' /
     &  '   If the standard method fails (rarely),' /
     &  '   try NUMDIR isotropic random directions from',
     &  ' point PNAME,' /
     &  '   then 10 * NUMDIR cosine-power random directions from',
     &  ' point PNAME,' /
     &  '   directed in the last best direction, with powers',
     &  ' geometrically increasing' /
     &  '   from 1 to 1.e10.  Convergence is determined by tol.' //
     &  '  Synonyms:  [help, h], [proximal, prox].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Initialize.

      pdist   = -1.e99

      proxptx = -1.e99
      proxpty = -1.e99
      proxptz = -1.e99

      proxvx  = -1.e99
      proxvy  = -1.e99
      proxvz  = -1.e99

c=======================================================================********

c.... Test input for validity.

      nerr = 0

      if (nword .eq. 3) then
        nword = 4
        mtype(4) = 2
        iword(4) = 100
      endif

 8120 format ('  prox ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Too many words on input line:  ',i3)
      if (nword .gt. 4) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c=======================================================================********

c.... Input line:  proximal PNAME QNAME [NUMDIR|100]

c.... Find the point.

      call ptfind (2, np)

      if (np .eq. 0) nerr = 1

c=======================================================================********

c.... Find the quadric surface.

      do 130 nq = 1, nquads
        if (aword(3) .eq. aquad(nq)) then

          call quout (nq, nq, 'min', nerrq)

          go to 140

        endif                         ! Finding side of a quadric surface.
  130 continue                        ! End of loop over quadric surfaces.

 8130 format ('  prox ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      nerr = 1
      write (   6, 8130) ainput, ninput, aword(3)
      write (iout, 8130) ainput, ninput, aword(3)

c=======================================================================********

c.... Find the number of random directions to try.

  140 call varfind (4, 4)

 8140   format ('  prox ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMDIR is not a positive integer:  ',a)
 8150   format ('  prox ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMDIR is not an integer or integer variable:  ',a)

      if (mtype(4) .eq. 2) then
        numdir = iword(4)
        if (numdir .le. 0) then
          nerr = 1
          write (   6, 8140) ainput, ninput, aword(4)
          write (iout, 8140) ainput, ninput, aword(4)
        endif
      else
        nerr = 1
        write (   6, 8150) ainput, ninput, aword(4)
        write (iout, 8150) ainput, ninput, aword(4)
      endif

      if (nerr .ne. 0) go to 999

c=======================================================================********

c.... See if the point is on the quadric surface, or an exact proximal point
c....   can be found.

      call aptwhis (x(np), y(np), z(np), qc(nq),
     &              qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), tol,
     &              dside, pnx, pny, pnz, nprox, ccx, ccy, ccz,
     &              ddist, nerra)

c.... Note: (pnx, pny, pnz) is the normal vector at the external point P.

 8170 format (/ 'Side measure      ',1pe20.12,2x,a /
     &          'Normal pnx,pny,pnz',1p3e20.12 )
      aside = ' '
      if (dside .eq. 0.0) then        ! On surface.

        aside = '***ON SURFACE***'

        proxptx = x(np)
        proxpty = y(np)
        proxptz = z(np)
  
        proxvx  = 0.0
        proxvy  = 0.0
        proxvz  = 0.0

      endif                           ! On surface.

      write (   6, 8170) dside, aside, pnx, pny, pnz
      write (iout, 8170) dside, aside, pnx, pny, pnz

      if (dside .eq. 0.0) go to 999

c.... See if exact proximal point was found.

      if (nprox .ge. 1) then          ! Found exact proximal point(s).

        pdist = ddist(1)

 8194   format ('Three points on a proximal circle:')
 8196   format ('Four points on a proximal sphere:')
 8190   format ('Proximal x,y,z    ',1p3e20.12)
 8201   format ('Proximal dx,dy,dz ',1p3e20.12)
 8192   format ('  Exact distance  ',1pe20.12)
 8193   format ('Normal qnx,qny,qnz',1p3e20.12)

        if (nprox .eq. 3) then
          write (   6, 8194)
          write (iout, 8194)
        elseif (nprox .eq. 4) then
          write (   6, 8196)
          write (iout, 8196)
        endif

        do 160 n = 1, nprox           ! Loop over proximal points

c....     Find normal vector on surface, at proximal point.

          call aptqnor (ccx(n), ccy(n), ccz(n),
     &                  qc(nq), qx(nq), qy(nq), qz(nq),
     &                  qxy(nq), qyz(nq), qzx(nq),
     &                  qxx(nq), qyy(nq), qzz(nq), tol,
     &                  sc, sx, sy, sz, slen, nerr)

          write (   6, 8193) sx, sy, sz
          write (iout, 8193) sx, sy, sz

          call aptvdis (x(np), y(np), z(np), ccx(n), ccy(n), ccz(n),
     &                  1, tol, vgoodx, vgoody, vgoodz, distx, nerra)

          write (   6, 8190) ccx(n), ccy(n), ccz(n)
          write (iout, 8190) ccx(n), ccy(n), ccz(n)
          call plotpt (ccx(n), ccy(n), ccz(n), nerrg)

          write (   6, 8201) vgoodx, vgoody, vgoodz
          write (iout, 8201) vgoodx, vgoody, vgoodz

          if (n .eq. 1) then
            proxptx = ccx(n)
            proxpty = ccy(n)
            proxptz = ccz(n)
  
            proxvx  = vgoodx
            proxvy  = vgoody
            proxvz  = vgoodz
          endif

          write (   6, 8192) ddist(n)
          write (iout, 8192) ddist(n)

  160   continue                      ! End of loop over proximal points.

        go to 999

      endif                           ! Found exact proximal point.

c=======================================================================********

 8180 format (/ 'Iterate with random direction method.')

      write (   6, 8180)
      write (iout, 8180)

c.... Find the vector from point PNAME to the approximate proximal point.

      vx = ccx(1) - x(np)
      vy = ccy(1) - y(np)
      vz = ccz(1) - z(np)

      call aptvusz (vx, vy, vz, tol, ux, uy, uz, vlentest)

      if (vlentest .le. 0.0) then     ! PNAME is at the center point.
        ux = 1.0
        uy = 0.0
        uz = 0.0
      endif

c.... Try random directions to find the proximal point.

      call aptqprr (x(np), y(np), z(np), ux, uy, uz,
     &              qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), numdir, tol,
     &              nprox, sgoodx, sgoody, sgoodz, distgood,
     &              vgoodx, vgoody, vgoodz, snx, sny, snz,
     &              vcosmax, nerr)

c.... Find the angle between the best trial vector and normal at intersection.

      vang = acos (vcosmax)
      if (abs (vang) .le. tol) vang = 0.0
      if (abs (vang - pi) .le. tol) vang = pi

      if (angunit .eq. 'degrees') then
        vang = vang * 180.0 / pi
      endif

c.... See if a proximal point was found.

      if (nprox .eq. -1) then         ! No intersection.

 8230   format ('  No intersections found.')

        write (   6, 8230)
        write (iout, 8230)

        go to 999

      elseif (nprox .eq. 0) then      ! No convergence.

 8270   format (/ '  Distance and angle did not converge.')
 8208   format (/ 'Proximal x,y,z   ?',1p3e20.12 /
     &            'Proximal dx,dy,dz?',1p3e20.12 /
     &            '  Approx distance?',1pe20.12 )
 8215   format ('  Cosine of angle ',1pe20.12,'  (from normal)' )
 8220   format ('  Angle (',a,')',1pe20.12,'  (from normal)' )

        write (   6, 8270)
        write (iout, 8270)
        write (   6, 8208) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood
        write (iout, 8208) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood

        write (   6, 8215) vcosmax
        write (iout, 8215) vcosmax
        write (   6, 8220) angunit, vang
        write (iout, 8220) angunit, vang

      else                            ! Converged.

 8240   format (/ '  Distance and angle converged.')
 8210   format (/ 'Proximal x,y,z    ',1p3e20.12 /
     &            'Proximal dx,dy,dz ',1p3e20.12 /
     &            '  Exact distance  ',1pe20.12 )

        write (   6, 8240)
        write (iout, 8240)
        write (   6, 8210) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood
        write (iout, 8210) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood
        call plotpt (sgoodx, sgoody, sgoodz, nerrg)

        proxptx = sgoodx
        proxpty = sgoody
        proxptz = sgoodz

        proxvx  = vgoodx
        proxvy  = vgoody
        proxvz  = vgoodz

        write (   6, 8215) vcosmax
        write (iout, 8215) vcosmax
        write (   6, 8220) angunit, vang
        write (iout, 8220) angunit, vang

        pdist = distgood

        go to 999

      endif                           ! Tested nprox.

c=======================================================================********

 8310 format (/ 'Iterate with center of curvature and tangent plane',
     &  ' methods.')

      write (   6, 8310)
      write (iout, 8310)

      vtestx = vgoodx
      vtesty = vgoody
      vtestz = vgoodz

      call aptqprt (x(np), y(np), z(np), vtestx, vtesty, vtestz,
     &              qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), tol,
     &              niter, nprox, sgoodx, sgoody, sgoodz, distgood,
     &              vgoodx, vgoody, vgoodz, snx, sny, snz,
     &              vcosmax, nerr)

      if     (nerr  .ne.  0) then

 8200   format ('  prox ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Error in subroutine aptqprt.  nerr = ',i2)

        write (   6, 8200) ainput, ninput, nerr
        write (iout, 8200) ainput, ninput, nerr

      elseif (nprox .eq.  1) then

 8245   format (/ '  Distance and angle converged in',i6,
     &    ' iterations.')
 8212   format (/ 'Proximal x,y,z    ',1p3e20.12 /
     &            'Proximal dx,dy,dz ',1p3e20.12 /
     &            '  Exact distance  ',1pe20.12 )

        write (   6, 8245) niter
        write (iout, 8245) niter
        write (   6, 8212) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood
        write (iout, 8212) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood
        call plotpt (sgoodx, sgoody, sgoodz, nerrg)

        proxptx = sgoodx
        proxpty = sgoody
        proxptz = sgoodz

        proxvx  = vgoodx
        proxvy  = vgoody
        proxvz  = vgoodz

        write (   6, 8215) vcosmax
        write (iout, 8215) vcosmax

c....   Find the angle between "d" and "sn".

        vang = acos (vcosmax)
        if (abs (vang) .le. tol) vang = 0.0
        if (abs (vang - pi) .le. tol) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

        write (   6, 8220) angunit, vang
        write (iout, 8220) angunit, vang

        pdist = distgood

      elseif (nprox .eq.  0) then

 8275   format (/ '  Distance and angle did not converge in',
     &    i6,' iterations.')

        write (   6, 8275) niter
        write (iout, 8275) niter

 8400   format (/ 'Proximal x,y,z   ?',1p3e20.12 /
     &            'Proximal dx,dy,dz?',1p3e20.12 /
     &            '  Approx distance?',1pe20.12 )

        write (   6, 8400) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood
        write (iout, 8400) sgoodx, sgoody, sgoodz,
     &    vgoodx, vgoody, vgoodz, distgood

        proxptx = sgoodx
        proxpty = sgoody
        proxptz = sgoodz

        proxvx  = vgoodx
        proxvy  = vgoody
        proxvz  = vgoodz

        write (   6, 8215) vcosmax
        write (iout, 8215) vcosmax

c....   Find the angle between "d" and "sn".

        vang = acos (vcosmax)
        if (abs (vang) .le. tol) vang = 0.0
        if (abs (vang - pi) .le. tol) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

        write (   6, 8220) angunit, vang
        write (iout, 8220) angunit, vang

      elseif (nprox .eq. -1) then

 8450   format (/ 'First trial direction missed quadric surface.' /
     &            '  vector     (xyz)',1p3e20.12)

        write (   6, 8450) vtestx, vtesty, vtestz
        write (iout, 8450) vtestx, vtesty, vtestz

      elseif (nprox .eq. -2) then

 8340   format (/ '  Normal vector has zero length.  Cannot proceed.')

        write (   6, 8340)
        write (iout, 8340)

      elseif (nprox .eq. -3) then

 8350   format (/ '  Did not converge.  Correction too small',
     &    ' to continue.')

        write (   6, 8350)
        write (iout, 8350)

      elseif (nprox .eq. -4) then

 8380   format (/  '  Missed quadric entirely 100 times.  Quit.')

        write (   6, 8380)
        write (iout, 8380)

      elseif (nprox .eq. -5) then

 8430   format (/  '  Failed to decrease distance 100',
     &    ' times.  Quit.')

        write (   6, 8430)
        write (iout, 8430)

      elseif (nprox .eq. -6) then

 8360   format (/ '  Test vector has zero length.  Cannot proceed.')

        write (   6, 8360)
        write (iout, 8360)

      endif                           ! Tested nerr, nprox.

c=======================================================================********

  999 return

c.... End of subroutine prox.         (+1 line)
      end
