      subroutine ptbrick (kopt, np, nb, nside, fracu, fracv, fracw)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  ptbrick  Updated     1995 December 9 15:00.
c               ptbrick  Originated  1994 March 22 11:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find whether or not a point is in a brick, and the distances
c               from the point to each of the six bounding surfaces.
c
c     Logic:    Entry points:  ptbrick.
c ptbrick       Called by:  cluster, distance, where.
c
c ptbrick       Calls:  aptbrkn, aptcsys.
c
c     Biblio:   ?
c
c     Glossary:
c
c     kopt      Input    Surface distance display option, 0 for none, 1 for all.
c
c     np        Input    Index of point.
c
c     nside     Output   1 if point is in brick.
c
c     nb        Input    Index of brick.
c
c     fracu     Output   Fractional distance of point from ubr to ubr + dubr.
c
c     fracv     Output   Fractional distance of point from vbr to vbr + dvbr.
c
c     fracw     Output   Fractional distance of point from wbr to wbr + dwbr.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lptbr/ dsurf(6)         ! Distances from point to brick faces.

c.... Find the coordinates of the point in the coordinate system of the
c....   brick.

      u = x(np)
      v = y(np)
      w = z(np)

      if     (asysbr(nb) .eq. 'rect') then
        ksys = 0
      elseif (asysbr(nb) .eq. 'cyl') then
        ksys = 1
      elseif (asysbr(nb) .eq. 'sph') then
        ksys = 2
      endif

      if (angbr(nb) .eq. 'degrees') then
        iunit  = 0
      else
        iunit  = 1
      endif

      call aptcsys (0, ksys, iunit, u, v, w, 1, tol, nerra)

c.... See if the point is in this brick.

      call aptbrkn (ksys, iunit,
     &              ubr(nb), dubr(nb), 
     &              vbr(nb), dvbr(nb),
     &              wbr(nb), dwbr(nb),
     &              u, v, w, 1, tol,
     &              nside, fracu, fracv, fracw, dsurf, nerra)

c.... See if distances from the point to the six surfaces are to be displayed.

 8110 format ('  Dist to u,v,wmin',1p3e20.12 /
     &        '  Dist to u,v,wmax',1p3e20.12 )

      if (kopt .eq. 1) then           ! Display distances to surfaces.

        write (   6, 8110) dsurf(1), dsurf(3), dsurf(5),
     &    dsurf(2), dsurf(4), dsurf(6)
        write (iout, 8110) dsurf(1), dsurf(3), dsurf(5),
     &    dsurf(2), dsurf(4), dsurf(6)

      endif

      pdist   = -1.e99

      proxptx = -1.e99
      proxpty = -1.e99
      proxptz = -1.e99

      proxvx  = -1.e99
      proxvy  = -1.e99
      proxvz  = -1.e99

      return

c.... End of subroutine ptbrick.      (+1 line)
      end
