      subroutine quadric

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  quadric  Updated     2005 April 5 17:45.
c               quadric  Originated  1993 December  1 14:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create quadric surfaces, which may be planes,
c                 spheres, ellipsoids, cylinders, cones, axisymmetric quadrics,
c                 general quadrics, slices parallel to a major plane through a
c                 quadric surface, or cylindrical quadrics through the
c                 intersection of a plane and another quadric, and perpendicular
c                 to the plane, or cylindrical quadrics having the outline of
c                 another quadric projected in a specified direction.
c
c     Logic:    Entry points:  quadric.
c quadric       Called by:  helpall, main.
c
c quadric       Calls:  aptplis, aptplqu, aptqexc, aptqfit, aptqper, aptqupr,
c               aptsubr, incrsubs, namechk, plout, ptfind, quadfind, qumove,
c               quoper, quout, qustore, varfind, vout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lquadx/ ex(4)           ! Coordinate x of an extreme point.
      common /lquadx/ ey(4)           ! Coordinate y of an extreme point.
      common /lquadx/ ez(4)           ! Coordinate z of an extreme point.
      common /lquadx/ adir(4)         ! Direction of normal of extreme point.
      character*8     adir            ! Direction of normal of extreme point.
      character*8     amount          ! Amount of output data.

      common /lquadx/ pex(2)          ! Coordinate x of planar pt prox to "e".
      common /lquadx/ pey(2)          ! Coordinate y of planar pt prox to "e".
      common /lquadx/ pez(2)          ! Coordinate z of planar pt prox to "e".

      common /lquadx/ dpex(2)         ! Component x of distance "pe" to "e".
      common /lquadx/ dpey(2)         ! Component y of distance "pe" to "e".
      common /lquadx/ dpez(2)         ! Component z of distance "pe" to "e".

      common /lquadx/ dplex(2)        ! Distance from point "pe" to point "e".

      common /lquadx/ actype          ! Type of intersection curve.
      character*24    actype          ! Type of intersection curve.

      common /lquadx/ nlen            ! Number of int radii, lengths, distances.
      common /lquadx/ alen(6)         ! Type of int radius, length or distance.
      character*16    alen            ! Type of int radius, length or distance.
      common /lquadx/ dlen(6)         ! Radius, length or distance.

      common /lquadx/ npin            ! Number of points of intersection.
      common /lquadx/ apin(11)        ! Type of intersection point.
      character*16    apin            ! Type of intersection point.
      common /lquadx/ pinx(11)        ! Coordinate x of an intersection point.
      common /lquadx/ piny(11)        ! Coordinate y of an intersection point.
      common /lquadx/ pinz(11)        ! Coordinate z of an intersection point.

      common /lquadx/ nvin            ! Number of vectors of intersection.
      common /lquadx/ avin(2)         ! Type of intersection vector.
      character*16    avin            ! Type of intersection vector.
      common /lquadx/ vinx(2)         ! Component x of an intersection vector.
      common /lquadx/ viny(2)         ! Component y of an intersection vector.
      common /lquadx/ vinz(2)         ! Component z of an intersection vector.

c=======================================================================********

c.... Initialize.

c.... Data statements for types of quadrics (qxx is always positive).

      if (nword .eq. 0) then

        aqtypes( 0) =
     &   'Plane, simple (type 0):                                 x = 0'
        aqtypes( 1) =
     &   'Planes, coincident:                                  x**2 = 0'
        aqtypes( 2) =
     &   'Planes, parallel:                           -1 + qxx*x**2 = 0'
        aqtypes( 3) =
     &   'Planes, intersecting:                   x**2 - |qyy|*y**2 = 0'
        aqtypes( 4) =
     &   'Cylinder, parabolic:                       -|qy|*y + x**2 = 0'
        aqtypes( 5) =
     &   'Cylinder, hyperbolic:           1 + qxx*x**2 - |qyy|*y**2 = 0'
        aqtypes( 6) =
     &   'Cylinder, elliptic:            -1 + qxx*x**2 +  qyy *y**2 = 0'
        aqtypes( 7) =
     &   'Cylinder, circular (type 7):     -1 + qxx * (x**2 + y**2) = 0'
        aqtypes( 8) =
     &   'Paraboloid, hyperbolic:       -|qz|*z + x**2 - |qyy|*y**2 = 0'
        aqtypes( 9) =
     &   'Paraboloid, elliptic:         -|qz|*z + x**2 +  qyy *y**2 = 0'
        aqtypes(10) =
     &   'Paraboloid, circular:         -|qz|*z + x**2 +       y**2 = 0'
        aqtypes(11) =
     &   'Cone, elliptic:               x**2 + qyy*y**2 -|qzz|*z**2 = 0'
        aqtypes(12) =
     &   'Cone, circular (type 12):         x**2 + y**2 -|qzz|*z**2 = 0'
        aqtypes(13) =
     &   'Hyperboloid of 1 sheet:  -1+qxx*x**2+qyy*y**2 -|qzz|*z**2 = 0'
        aqtypes(14) =
     &   'Hyperboloid of 2 sheets:  1+qxx*x**2+qyy*y**2 -|qzz|*z**2 = 0'
        aqtypes(15) =
     &   'Ellipsoid:            -1 + qxx*x**2 + qyy*y**2 + qzz*z**2 = 0'
        aqtypes(16) =
     &   'Sphere (type 16):         -1 + qxx * (x**2 + y**2 + z**2) = 0'
        aqtypes(17) =
     &   'Imaginary planes, parallel:                  1 + qxx*x**2 = 0'
        aqtypes(18) =
     &   'Imaginary planes, intersecting (line):    x**2 + qyy*y**2 = 0'
        aqtypes(19) =
     &   'Imaginary cylinder, elliptic:     1 + qxx*x**2 + qyy*y**2 = 0'
        aqtypes(20) =
     &   'Imaginary cylinder, circular:     1 + qxx * (x**2 + y**2) = 0'
        aqtypes(21) =
     &   'Imaginary cone, elliptic (pt): x**2 + qyy*y**2 + qzz*z**2 = 0'
        aqtypes(22) =
     &   'Imaginary cone, circular (pt):     x**2 + y**2 + qzz*z**2 = 0'
        aqtypes(23) =
     &   'Imaginary ellipsoid:   1 + qxx*x**2 + qyy*y**2 + qzz*z**2 = 0'
        aqtypes(24) =
     &   'Imaginary sphere:          1 + qxx * (x**2 + y**2 + z**2) = 0'

        go to 999

      endif

c=======================================================================********

c.... Input line:  quadric [help]

 8110 format (
     &  '  Command "quadric":  acceptable input lines include:' /
     &  '  "quadric [help]"  Display this message.' /
     &  '  "quadric types"   Display all quadric types.' /
     &  '  "quadric all"     Display all quadric surfaces.' /
     &  '  "quadric list [QNAME1 QNAME2 QNAME3 ...]"' /
     &  '     Display all listed quadrics (default is all',
     &  ' quadrics).' /
     &  '  "quadric QNAME"   Display quadric surface QNAME.' //
     &  '  "quadric QNAME = QC QX QY QZ [QXY QYZ QZX QXX',
     &  ' QYY QZZ]"' /
     &  '     Create quadric QNAME, with implicit equation',
     &  ' coefficients' /
     &  '     QC, QX, QY, QZ [, QXY, QYZ, QZX, QXX, QYY and QZZ].' //
     &  '  "quadric QNAME & CFNAME1 CFVAL1 [CFNAME2 CFVAL2 ...]"' /
     &  '     For existing quadric QNAME, replace coefficient CFNAME1' /
     &  '     with new value CFVAL1, where CFNAME1 may be any of the' /
     &  '     coefficient names QC, QX, QY, QZ, QXY, QYZ, QZX, QXX,',
     &  ' QYY, QZZ.' /
     &  '     Likewise for CFNAME2, CFVAL2, ...' //
     &  '  "quadric QNAME fit PNAME1 PNAME2 ... PNAME9"' /
     &  '     Create quadric QNAME, through the nine points PNAME1,',
     &  ' PNAME2, ..., PNAME9,' /
     &  '     if a unique solution exists.' //
     &  '  "quadric QNAME [* /] FACTOR"' /
     &  '     [Multiply, Divide] the implicit equation coefficients',
     &  ' of quadric QNAME' /
     &  '     by FACTOR.' //
     &  '  "quadric QNAME sum QNAME1 QNAME2 [QMULT1|1] [QMULT2|1]"' /
     &  '     Create quadric QNAME, the weighted sum QMULT1 * QNAME1',
     &  ' + QMULT2 * QNAME2,' /
     &  '     where QMULT1 and QMULT2 default to 1.' //
     &  '  "quadric QNAME plane PLNAME1 PLNAME2"' /
     &  '     Create quadric QNAME, representing the two coincident,' /
     &  '     parallel or intersecting planes PLNAME1 and PLNAME2.' )

 8111 format (/
     &  '  "quadric QNAME project VNAME QNAME2"' /
     &  '     Create quadric QNAME, a cylindrical quadric',
     &  ' parallel to' /
     &  '     vector VNAME, and tangent to quadric surface QNAME2.' //
     &  '  "quadric QNAME intersect PLNAME QNAME2"' /
     &  '     Create quadric QNAME, a cylindrical quadric',
     &  ' perpendicular to' /
     &  '     plane PLNAME, and through the intersection of plane',
     &  ' PLNAME' /
     &  '     and quadric surface QNAME2.' //
     &  '  Type <return> for more help.' )

 8112 format (a)

 8113 format (/
     &  '  "quadric QNAME slice QNAME2 [x,y,z|x] [VAL|0.0]"' /
     &  '     Create quadric QNAME, a slice through quadric QNAME2' /
     &  '     at [x, y, z] = VAL (defaults are x and 0.0).' //
     &  '  "quadric QNAME ortho QNAME1 QNAME2"' /
     &  '     Create quadric QNAME, the locus of points where the' /
     &  '     family of quadric surfaces QNAME1 is orthogonal to the' /
     &  '     family of quadric surfaces QNAME2.' //
     &  '  "quadric QNAME02 NUMQ INC move QNAME01 VMOVE"' /
     &  '     Create a family of NUMQ quadric surfaces',
     &  ' QNAME02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     spaced at intervals of vector VMOVE, starting from',
     &  ' quadric surface QNAME01.' //
     &  '  "quadric QNAME02 rotate NUMQ INC QNAME01 OPNAME PINV"' /
     &  '     Create a family of NUMQ quadric surfaces QNAME02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by rotating the preceding quadric surface with operator',
     &  ' OPNAME' /
     &  '     and invariant point PINV, starting from quadric',
     &  ' surface QNAME01.' )

 8114 format (/
     &  '  "quadric QNAME02 scale NUMQ INC QNAME01 OPNAME PINV"' /
     &  '     Create a family of NUMQ quadric surfaces QNAME02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by scaling the preceding quadric surface with operator',
     &  ' OPNAME' /
     &  '     and invariant point PINV, starting from quadric',
     &  ' surface QNAME01.' //
     &  '  Note:  use +/- for QNAME or QNAME02 to incr/decr the last',
     &  ' quadric name:' / 2x,a //
     &  '  Synonyms:  [help, h], [intersect, inter, int],' /
     &  '    [move, mv, translate, trans], [plane, pl],',
     &  ' [quadric, quad, q],' /
     &  '    [project, proj], [rotate, rot], [scale, sc],',
     &  ' [slice, cut], [sum, add].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8111)
        write (iout, 8111)
        if (ainput .eq. 'tty') then
          read  (   5, 8112) ans
        endif
        write (   6, 8113)
        write (iout, 8113)
        write (   6, 8114) aquads
        write (iout, 8114) aquads
        go to 999
      endif

c=======================================================================********

c.... Input line:  quadric types

 8117 format ('  Quadric types, in standard form' /
     &        '  (all coefficients must have the preceding sign):')
 8119 format (2x,i2,2x,a)

      if (aword(2) .eq. 'types') then
        write (   6, 8117)
        write (iout, 8117)
        write (   6, 8119) (n, aqtypes(n), n = 0, 24)
        write (iout, 8119) (n, aqtypes(n), n = 0, 24)
        go to 999
      endif

c=======================================================================********

c.... Input line:  quadric all
c.... Input line:  quadric list

 8120 format (/ 'Number of quadric surfaces = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nquadx = 0
        call quout (1, nquads, 'min', nerrq)
        do 120 n = 1, nquads
          if (aquad(n) .eq. ' ') go to 120
          nquadx = nquadx + 1
  120   continue
        write (   6, 8120) nquadx
        write (iout, 8120) nquadx
        go to 999
      endif

c=======================================================================********

c.... Input line:  quadric list QNAME1 QNAME2 QNAME3 ...

 8130 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, nquads
            call aptsubr (aquad(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aquad(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call quout (n, n, 'min', nerrq)
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c=======================================================================********

c.... Input line:  quadric QNAME

      if (nword .eq. 2) then
        do 130 n = 1, nquads
          if (aword(2) .eq. aquad(n)) then
            amount = 'maximum'
            call quout (n, n, amount, nerrq)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

 8136 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of quadric not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... See if last quadric name should be incremented and used.

 8128 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last quadric can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = aquads
        lword(2) = laquads
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, aquads
          write (iout, 8128) ainput, ninput, aquads
          go to 999
        endif
      endif

 8129 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last quadric can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = aquads
        lword(2) = laquads
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, aquads
          write (iout, 8129) ainput, ninput, aquads
          go to 999
        endif
      endif

 8905 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of quadric too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  quadric QNAME02 move NUMQ INC QNAME01 VMOVE

      if ((aword(3) .eq. 'move')      .or.
     &    (aword(3) .eq. 'mv')        .or.
     &    (aword(3) .eq. 'translate') .or.
     &    (aword(3) .eq. 'trans'))    then

        call qumove

        go to 999

      endif                           ! Created translated quadric(s).

c===============================================================================

c.... Input line:  quadric QNAME02 rotate NUMQ INC QNAME01 OPNAME PINV
c.... Input line:  quadric QNAME02 scale  NUMQ INC QNAME01 OPNAME PINV

      if ((aword(3) .eq. 'scale')   .or.
     &    (aword(3) .eq. 'sc')      .or.
     &    (aword(3) .eq. 'rotate')  .or.
     &    (aword(3) .eq. 'rot'))    then

        call quoper

        go to 999

      endif                           ! Created rotated or scaled quadric(s).

c===============================================================================

c.... Find the index in the quadric surface array at which to store this
c....   quadric surface.
c....   Set nrep = 1 if replacing an existing quadric surface.

      call qustore (-1, nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c=======================================================================********

c.... Input line:  quadric QNAME = QC QX QY QZ QXY QYZ QZX QXX QYY QZZ

      if (aword(3) .eq. '=') then

 8310 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 or 13 words on input line:  ',i4)

      if ((nword .ne. 7) .and. (nword .ne. 13)) then
        write (   6, 8310) ainput, ninput, nword
        write (iout, 8310) ainput, ninput, nword
        nerr = 1
      endif

c.... Change missing words to default values.

      do 310 nw = 8, 13
        if (nw .gt. nword) then
          mtype(nw) = 3
          fword(nw) = 0.0
        endif
  310 continue

      nword = 13

c.... Find the values of words 4-13

 8320 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      call varfind (4, 13)

      do 330 nw = 4, 13
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8320) ainput, ninput, aword(nw)
          write (iout, 8320) ainput, ninput, aword(nw)
          nerr = 1
        endif
  330 continue

c.... See if any coefficients except qc are not zero.

 8340 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  All coefficients after QC are zero.')

      do 340 nw = 5,13
        if (fword(nw) .ne. 0.0) go to 350
  340 continue
      write (   6, 8340) ainput, ninput, aword(nw)
      write (iout, 8340) ainput, ninput, aword(nw)
      nerr = 1

  350 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Print the quadric specifications.

      aquads   = aword(2)
      laquads  = lword(2)

      aquad(n) = aword(2)
      laquad(n)= lword(2)

      qc(n)    = fword(4)
      qx(n)    = fword(5)
      qy(n)    = fword(6)
      qz(n)    = fword(7)

      qxy(n)   = fword(8)
      qyz(n)   = fword(9)
      qzx(n)   = fword(10)

      qxx(n)   = fword(11)
      qyy(n)   = fword(12)
      qzz(n)   = fword(13)

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

      endif                           ! Specify quadric coefficients.

c=======================================================================********

c.... Input line:  quadric QNAME & CFNAME1 CFVALUE1 CFNAME2 CFVALUE2 ...

      if (aword(3) .eq. '&') then

      nerr = 0

 8315 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on input line not odd:  ',i4)

      if (mod (nword, 2) .ne. 1) then
        nerr = 1
        write (   6, 8315) ainput, ninput, nword
        write (iout, 8315) ainput, ninput, nword
      endif

c.... Make sure quadric surface exists.

      if (nrep .ne. 1) then
        nerr = 1
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
      endif

c.... See if any coefficient names duplicated.

c*************************************************************

c.... Find the values of words 5, 7, ...

      call varfind (5, nword)

      do 320 nw = 5, nword, 2
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          nerr = 1
          write (   6, 8320) ainput, ninput, aword(nw)
          write (iout, 8320) ainput, ninput, aword(nw)
        endif
  320 continue

c.... Find the new coefficient values.

 8325 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Coefficient name not qc, qx, qy, qz, qxy, qyz, qxz,'
     &  ' qxx, qyy or qzz:  ' / 2x,a)

      do 325 nw = 4, nword, 2
        if     (aword(nw) .eq. 'qc' ) then
          qc(n)  = fword(nw+1)
        elseif (aword(nw) .eq. 'qx' ) then
          qx(n)  = fword(nw+1)
        elseif (aword(nw) .eq. 'qy' ) then
          qy(n)  = fword(nw+1)
        elseif (aword(nw) .eq. 'qz' ) then
          qz(n)  = fword(nw+1)
        elseif (aword(nw) .eq. 'qxy') then
          qxy(n) = fword(nw+1)
        elseif (aword(nw) .eq. 'qyz') then
          qyz(n) = fword(nw+1)
        elseif (aword(nw) .eq. 'qzx') then
          qzx(n) = fword(nw+1)
        elseif (aword(nw) .eq. 'qxx') then
          qxx(n) = fword(nw+1)
        elseif (aword(nw) .eq. 'qyy') then
          qyy(n) = fword(nw+1)
        elseif (aword(nw) .eq. 'qzz') then
          qzz(n) = fword(nw+1)
        else
          nerr = 1
          write (   6, 8320) ainput, ninput, aword(nw)
          write (iout, 8320) ainput, ninput, aword(nw)
        endif
  325 continue

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Print the quadric specifications.

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

      endif                           ! Specify quadric coefficients.

c=======================================================================********


c.... Input line:  quadric QNAME * FACTOR
c.... Input line:  quadric QNAME / FACTOR

      if ((aword(3) .eq. '*')  .or.
     &    (aword(3) .eq. '/')) then

 8610 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 words on input line:  ',i4)

      if (nword .ne. 4) then
        write (   6, 8610) ainput, ninput, nword
        write (iout, 8610) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the value of FACTOR.

 8620 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      call varfind (4, 4)

      if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
        write (   6, 8620) ainput, ninput, aword(4)
        write (iout, 8620) ainput, ninput, aword(4)
        nerr = 1
      endif

c.... Make sure quadric surface exists.

      if (nrep .ne. 1) then
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        nerr = 1
      endif

c.... See if FACTOR is zero.

 8640 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  ' FACTOR is zero.')

      if (fword(4) .eq. 0.0) then
        write (   6, 8640) ainput, ninput
        write (iout, 8640) ainput, ninput
        nerr = 1
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

      if (aword(3) .eq. '/') then
        factor = 1.0 / fword(4)
      else
        factor = fword(4)
      endif

c.... Print the quadric specifications.

      aquads    = aword(2)
      laquads   = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      qc(n)     = qc(n) * factor

      qx(n)     = qx(n) * factor
      qy(n)     = qy(n) * factor
      qz(n)     = qz(n) * factor

      qxy(n)    = qxy(n) * factor
      qyz(n)    = qyz(n) * factor
      qzx(n)    = qzx(n) * factor

      qxx(n)    = qxx(n) * factor
      qyy(n)    = qyy(n) * factor
      qzz(n)   = qzz(n) * factor

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

      endif                           ! Specify quadric coefficients.

c=======================================================================********

c.... Input line:  quadric QNAME sum QNAME1 QNAME2 [QMULT1|1 [QMULT2|1]]

 8650 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are not 5-7 words on "sum" command line:  ',i1)

      if ((aword(3) .eq. 'add')  .or.
     &    (aword(3) .eq. 'sum')) then

        if ((nword .lt. 5) .or. (nword .gt. 7)) then
          nerr = 1
          write (   6, 8650) ainput, ninput, nword
          write (iout, 8650) ainput, ninput, nword
          go to 690
        endif

c....   Find the first quadric.

        call quadfind (4, nq1)

        if (nq1 .eq. 0) then
          nerr = 1
        endif

c....   Find the second quadric.

        call quadfind (5, nq2)

        if (nq2 .eq. 0) then
          nerr = 1
        endif

c....   Find the multipliers.

        qmult1 = 1.0
        qmult2 = 1.0

 8685 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

        if (nword .gt. 5) then

          call varfind (6, 6)

          if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
            write (   6, 8685) ainput, ninput, aword(6)
            write (iout, 8685) ainput, ninput, aword(6)
            nerr = 1
            go to 690
          endif

          qmult1 = fword(6)

        endif

        if (nword .eq. 7) then

          call varfind (7, 7)

          if ((mtype(7) .ne. 2) .and. (mtype(7) .ne. 3)) then
            write (   6, 8685) ainput, ninput, aword(7)
            write (iout, 8685) ainput, ninput, aword(7)
            nerr = 1
            go to 690
          endif

          qmult2 = fword(7)

        endif

  690   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nquads) nquads = nquads - 1
          endif
          go to 999
        endif

c....   Find the weighted sum of the two quadrics.

        aquads    = aword(2)
        laquads   = lword(2)
  
        aquad(n)  = aword(2)
        laquad(n) = lword(2)
        qc (n)    = qmult1 * qc (nq1) + qmult2 * qc (nq2)
        qx (n)    = qmult1 * qx (nq1) + qmult2 * qx (nq2)
        qy (n)    = qmult1 * qy (nq1) + qmult2 * qy (nq2)
        qz (n)    = qmult1 * qz (nq1) + qmult2 * qz (nq2)
        qxy(n)    = qmult1 * qxy(nq1) + qmult2 * qxy(nq2)
        qyz(n)    = qmult1 * qyz(nq1) + qmult2 * qyz(nq2)
        qzx(n)    = qmult1 * qzx(nq1) + qmult2 * qzx(nq2)
        qxx(n)    = qmult1 * qxx(nq1) + qmult2 * qxx(nq2)
        qyy(n)    = qmult1 * qyy(nq1) + qmult2 * qyy(nq2)
        qzz(n)    = qmult1 * qzz(nq1) + qmult2 * qzz(nq2)

c....   Print the quadric specifications.

        amount = 'maximum'
        call quout (n, n, amount, nerrq)

        go to 999

      endif                           ! Weighted sum of two quadrics.

c=======================================================================********

c.... Input line:  quadric QNAME ortho QNAME1 QNAME2

 8655 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are not 5 words on "ortho" command line:  ',i1)

      if (aword(3) .eq. 'ortho') then

        if (nword .ne. 5) then
          nerr = 1
          write (   6, 8655) ainput, ninput, nword
          write (iout, 8655) ainput, ninput, nword
          go to 695
        endif

c....   Find the first quadric.

        call quadfind (4, nq1)

        if (nq1 .eq. 0) then
          nerr = 1
        endif

c....   Find the second quadric.

        call quadfind (5, nq2)

        if (nq2 .eq. 0) then
          nerr = 1
        endif

  695   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nquads) nquads = nquads - 1
          endif
          go to 999
        endif

c....   Find the quadric surface where the two quadric families are orthogonal.

        call aptqper (qc(nq1), qx(nq1), qy(nq1), qz(nq1),
     &                qxy(nq1), qyz(nq1), qzx(nq1),
     &                qxx(nq1), qyy(nq1), qzz(nq1),
     &                qc(nq2), qx(nq2), qy(nq2), qz(nq2),
     &                qxy(nq2), qyz(nq2), qzx(nq2),
     &                qxx(nq2), qyy(nq2), qzz(nq2), tol,
     &                qc(n), qx(n), qy(n), qz(n),
     &                qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &                nerra)

        aquads    = aword(2)
        laquads   = lword(2)
  
        aquad(n)  = aword(2)
        laquad(n) = lword(2)

c....   Print the quadric specifications.

        amount = 'maximum'
        call quout (n, n, amount, nerrq)

        go to 999

      endif                           ! Quadric where two quadrics orthogonal.

c=======================================================================********

c.... Input line:  quadric QNAME intersect PLNAME QNAME2

      if ((aword(3) .eq. 'intersect') .or.
     &    (aword(3) .eq. 'inter')     .or.
     &    (aword(3) .eq. 'int'))      then

 8450 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words not 5 on input line:  ',i4)

      if (nword .ne. 5) then
        write (   6, 8450) ainput, ninput, nword
        write (iout, 8450) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the intersection plane.

 8460 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named plane does not exist:  ',a)

      do 460 nq = 1, nquads
        if (aword(4) .eq. aquad(nq)) then
          if (aqtype(nq) .ne. aqtypes(0)) go to 460
          npl = nq
          call plout (npl, npl)
          go to 465
        endif
  460 continue
      write (   6, 8460) ainput, ninput, aword(4)
      write (iout, 8460) ainput, ninput, aword(4)
      nerr = 1

c.... Find the second quadric surface.

 8470 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Second quadric surface does not exist:  ',a)

  465 do 470 nq = 1, nquads
        if (aword(5) .eq. aquad(nq)) then
          call quout (nq, nq, 'min', nerrq)
          go to 475
        endif
  470 continue
      write (   6, 8470) ainput, ninput, aword(5)
      write (iout, 8470) ainput, ninput, aword(5)
      nerr = 1

c.... See if input data is good.

  475 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the coefficients of the cylinderical quadric surface QNAME,
c....   perpendicular to the plane PLNAME, and through the intersection
c....   of the plane with the second quadric surface QNAME2.

      call aptplqu (xcen(npl), ycen(npl), zcen(npl),
     &              qx(npl), qy(npl), qz(npl),
     &              qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), tol,
     &              nqtype, px, py, pz, qcenx, qceny, qcenz,
     &              dplqu, dpx, dpy, dpz, nextr, ex, ey, ez,
     &              dplex, pex, pey, pez, dpex, dpey, dpez,
     &              actype, nlen, alen, dlen,
     &              npin, apin, pinx, piny, pinz,
     &              nvin, avin, vinx, viny, vinz,
     &              qc(n), qx(n), qy(n), qz(n),
     &              qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &              nerr)

      aquads    = aword(2)
      laquads   = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

      endif

c=======================================================================********

c.... Input line:  quadric QNAME project VNAME QNAME2

      if ((aword(3) .eq. 'project') .or.
     &    (aword(3) .eq. 'proj'))   then

 8550 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words not 5 on input line:  ',i4)

      if (nword .ne. 5) then
        write (   6, 8550) ainput, ninput, nword
        write (iout, 8550) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the projection vector.

 8560 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named vector does not exist:  ',a)

      do 560 nv = 1, nvectors
        if (aword(4) .eq. avector(nv)) then
          ax = xcom(nv)
          ay = ycom(nv)
          az = zcom(nv)
          call vout (nv, nv)
          go to 565
        endif
  560 continue
      write (   6, 8560) ainput, ninput, aword(4)
      write (iout, 8560) ainput, ninput, aword(4)
      nerr = 1

 8565 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Direction vector has zero length:  ',a)

  565 if (vlen(nv) .eq. 0.0) then
        nerr = 1
        write (   6, 8565) ainput, ninput, aword(4)
        write (iout, 8565) ainput, ninput, aword(4)
      endif

c.... Find the second quadric surface.

 8570 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Second quadric surface does not exist:  ',a)

      do 570 nq = 1, nquads
        if (aword(5) .eq. aquad(nq)) then
          call quout (nq, nq, 'min', nerrq)
          go to 575
        endif
  570 continue
      write (   6, 8570) ainput, ninput, aword(5)
      write (iout, 8570) ainput, ninput, aword(5)
      nerr = 1

c.... See if input data is good.

  575 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find any plane passing through the curve on quadric surface QNAME
c....   where the normal vector is perpendicular to vector "a".

      sc =       qx (nq) * ax +       qy (nq) * ay +       qz (nq) * az
      sx = 2.0 * qxx(nq) * ax +       qxy(nq) * ay +       qzx(nq) * az
      sy =       qxy(nq) * ax + 2.0 * qyy(nq) * ay +       qyz(nq) * az
      sz =       qzx(nq) * ax +       qyz(nq) * ay + 2.0 * qzz(nq) * az

      slen  = sqrt (sx**2 + sy**2 + sz**2)

 8572 format ('  There is no unique plane of intersection.')
 8575 format (/ 'Plane of intersection:' /
     &          '  Center     (xyz)',1p3e20.12 /
     &          '  qc = N dot P    ',1pe20.12 /
     &          '  Norm vect  (xyz)',1p3e20.12 )

      if (slen .ne. 0.0) then
        sc = sc / slen
        sx = sx / slen
        sy = sy / slen
        sz = sz / slen
        px = -sc * sx
        py = -sc * sy
        pz = -sc * sz
        write (   6, 8575) px, py, pz, sc, sx, sy, sz
        write (iout, 8575) px, py, pz, sc, sx, sy, sz
      else
        write (   6, 8572)
        write (iout, 8572)
      endif

c.... Find the coefficients of the cylinderical quadric surface QNAME,
c....   with cylindrical axis parallel to vector "a", and tangent to the
c....   second quadric surface QNAME2.

      call aptqupr (ax, ay, az, qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), tol,
     &              qc(n), qx(n), qy(n), qz(n), qxy(n), qyz(n), qzx(n),
     &              qxx(n), qyy(n), qzz(n), ntypeq, nerra)

 8580 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There is no projected outline.')

      if (nerra .ne. 0) then
        write (   6, 8580) ainput, ninput
        write (iout, 8580) ainput, ninput
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

      aquads    = aword(2)
      laquads   = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

      endif

c=======================================================================********

c.... Input line:  quadric QNAME slice QNAME2 [x,y,z|x] [VAL|0]

      if ((aword(3) .eq. 'slice') .or.
     &    (aword(3) .eq. 'cut')) then

 8360 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words not 4-6 on input line:  ',i4)

      if ((nword .lt. 4) .or. (nword .gt. 6)) then
        write (   6, 8360) ainput, ninput, nword
        write (iout, 8360) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the second quadric surface.

 8370 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Second quadric surface does not exist:  ',a)

      do 370 nq = 1, nquads
        if (aword(4) .eq. aquad(nq)) then
          n1 = nq
          call quout (n1, n1, 'min', nerrq)
          go to 375
        endif
  370 continue
      write (   6, 8370) ainput, ninput, aword(4)
      write (iout, 8370) ainput, ninput, aword(4)
      nerr = 1

c.... Change missing words to default values.

  375 if (nword .lt. 5) then
        mtype(5) = 1
        aword(5) = 'x'
      endif

      if (nword .lt. 6) then
        mtype(6) = 3
        fword(6) = 0.0
      endif

      nword = 6

c.... Find the value of word 6.

 8380 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 6 not integer or floating point value or variable:  ',a)

      call varfind (6, 6)

      if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
        write (   6, 8380) ainput, ninput, aword(6)
        write (iout, 8380) ainput, ninput, aword(6)
        nerr = 1
      endif

c.... Find the major plane parallel to the slice.

 8390 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 5 is not x, y or z')

  390 if ((aword(5) .eq. 'x')  .or.
     &    (aword(5) .eq. 'X')) then
        kpl = 1
      elseif ((aword(5) .eq. 'y')  .or.
     &        (aword(5) .eq. 'Y')) then
        kpl = 2
      elseif ((aword(5) .eq. 'z')  .or.
     &        (aword(5) .eq. 'Z')) then
        kpl = 3
      else
        write (   6, 8390) ainput, ninput, aword(5)
        write (iout, 8390) ainput, ninput, aword(5)
        nerr = 1
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the slice through the quadric.

      call aptplis (kpl, fword(6),
     &              qc(n1), qx(n1), qy(n1), qz(n1),
     &              qxy(n1), qyz(n1), qzx(n1),
     &              qxx(n1), qyy(n1), qzz(n1), 1, tol,
     &              qc(n), qx(n), qy(n), qz(n), qxy(n), qyz(n), qzx(n),
     &              qxx(n), qyy(n), qzz(n), nerra)

      aquads    = aword(2)
      laquads   = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)

 8395 format (/ '  Slice through quadric ',a,' at ',a1,' = ',1pe20.12 /
     &          '  (ignore extension in ',a1,' direction)')

      write (   6, 8395) aquad(n1), aword(5), fword(6), aword(5)
      write (iout, 8395) aquad(n1), aword(5), fword(6), aword(5)

      call quout (n, n, 'min', nerrq)

c....   Find any extremities of the curve.

        if (kpl .eq. 1) then          ! At constant x.

          call aptqexc (qc(n), qy(n), qz(n), qyz(n), qyy(n), qzz(n),
     &                  tol, np, ey, ez, adir, nerra)

          do 410 n = 1, np
            ex(n) = fword(6)
            if (adir(n) .eq. 'ucon') adir(n) = 'Const y'
            if (adir(n) .eq. 'uint') adir(n) = 'Inter y'
            if (adir(n) .eq. 'umax') adir(n) = 'Max y'
            if (adir(n) .eq. 'umin') adir(n) = 'Min y'
            if (adir(n) .eq. 'vcon') adir(n) = 'Const z'
            if (adir(n) .eq. 'vint') adir(n) = 'Inter z'
            if (adir(n) .eq. 'vmax') adir(n) = 'Max z'
            if (adir(n) .eq. 'vmin') adir(n) = 'Min z'
  410     continue

        elseif (kpl .eq. 2) then      ! At constant y.

          call aptqexc (qc(n), qz(n), qx(n), qzx(n), qzz(n), qxx(n),
     &                  tol, np, ez, ex, adir, nerra)

          do 420 n = 1, np
            ey(n) = fword(6)
            if (adir(n) .eq. 'ucon') adir(n) = 'Const z'
            if (adir(n) .eq. 'uint') adir(n) = 'Inter z'
            if (adir(n) .eq. 'umax') adir(n) = 'Max z'
            if (adir(n) .eq. 'umin') adir(n) = 'Min z'
            if (adir(n) .eq. 'vcon') adir(n) = 'Const x'
            if (adir(n) .eq. 'vint') adir(n) = 'Inter x'
            if (adir(n) .eq. 'vmax') adir(n) = 'Max x'
            if (adir(n) .eq. 'vmin') adir(n) = 'Min x'
  420     continue

        else                          ! At constant z.

          call aptqexc (qc(n), qx(n), qy(n), qxy(n), qxx(n), qyy(n),
     &                  tol, np, ex, ey, adir, nerra)

          do 430 n = 1, np
            ez(n) = fword(6)
            if (adir(n) .eq. 'ucon') adir(n) = 'Const x'
            if (adir(n) .eq. 'uint') adir(n) = 'Inter x'
            if (adir(n) .eq. 'umax') adir(n) = 'Max x'
            if (adir(n) .eq. 'umin') adir(n) = 'Min x'
            if (adir(n) .eq. 'vcon') adir(n) = 'Const y'
            if (adir(n) .eq. 'vint') adir(n) = 'Inter y'
            if (adir(n) .eq. 'vmax') adir(n) = 'Max y'
            if (adir(n) .eq. 'vmin') adir(n) = 'Min y'
  430     continue

        endif                         ! Tested kpl.

 8440   format ('  ',a,'   (xyz)',1p3e20.12 )

        do 440 n = 1, np
          write (   6, 8440) adir(n), ex(n), ey(n), ez(n)
          write (iout, 8440) adir(n), ex(n), ey(n), ez(n)
  440   continue

      go to 999

      endif                           ! Find slice through quadric.

c=======================================================================********

c.... Input line:  quadric QNAME plane PLNAME1 PLNAME2

      if ((aword(3) .eq. 'plane') .or.
     &    (aword(3) .eq. 'pl'))   then

 8200 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words not 5 on input line:  ',i4)

      if (nword .ne. 5) then
        write (   6, 8200) ainput, ninput, nword
        write (iout, 8200) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the first plane.

 8210 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  First plane does not exist:  ',a)

      do 210 nq = 1, nquads
        if ((aword(4) .eq. aquad(nq))   .and.
     &      (aqtype(nq) .eq. aqtypes(0))) then
          n1 = nq
          call quout (n1, n1, 'min', nerrq)
          go to 220
        endif
  210 continue
      write (   6, 8210) ainput, ninput, aword(4)
      write (iout, 8210) ainput, ninput, aword(4)
      nerr = 1

c.... Find the second plane.

 8230 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Second plane does not exist:  ',a)

  220 do 230 nq = 1, nquads
        if ((aword(5) .eq. aquad(nq))   .and.
     &      (aqtype(nq) .eq. aqtypes(0))) then
          n2 = nq
          call quout (n2, n2, 'min', nerrq)
          go to 240
        endif
  230 continue
      write (   6, 8230) ainput, ninput, aword(4)
      write (iout, 8230) ainput, ninput, aword(4)
      nerr = 1

  240 if (nerr .eq. 1) go to 999

      aquads    = aword(2)
      laquads   = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      qc(n)     = qc(n1) * qc(n2)
      qx(n)     = qc(n1) * qx(n2) + qx(n1) * qc(n2)
      qerr      = tol * (abs (qc(n1) * qx(n2)) + abs (qx(n1) * qc(n2)))
      if (abs (qx(n)) .le. qerr) qx(n) = 0.0
      qy(n)     = qc(n1) * qy(n2) + qy(n1) * qc(n2)
      qerr      = tol * (abs (qc(n1) * qy(n2)) + abs (qy(n1) * qc(n2)))
      if (abs (qy(n)) .le. qerr) qy(n) = 0.0
      qz(n)     = qc(n1) * qz(n2) + qz(n1) * qc(n2)
      qerr      = tol * (abs (qc(n1) * qz(n2)) + abs (qz(n1) * qc(n2)))
      if (abs (qz(n)) .le. qerr) qz(n) = 0.0
      qxy(n)    = qx(n1) * qy(n2) + qy(n1) * qx(n2)
      qerr      = tol * (abs (qx(n1) * qy(n2)) + abs (qy(n1) * qx(n2)))
      if (abs (qxy(n)) .le. qerr) qxy(n) = 0.0
      qyz(n)    = qy(n1) * qz(n2) + qz(n1) * qy(n2)
      qerr      = tol * (abs (qy(n1) * qz(n2)) + abs (qz(n1) * qy(n2)))
      if (abs (qyz(n)) .le. qerr) qyz(n) = 0.0
      qzx(n)    = qz(n1) * qx(n2) + qx(n1) * qz(n2)
      qerr      = tol * (abs (qz(n1) * qx(n2)) + abs (qx(n1) * qz(n2)))
      if (abs (qzx(n)) .le. qerr) qzx(n) = 0.0
      qzx(n)    = qz(n1) * qx(n2) + qx(n1) * qz(n2)
      qxx(n)    = qx(n1) * qx(n2)
      qyy(n)    = qy(n1) * qy(n2)
      qzz(n)    = qz(n1) * qz(n2)

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

      endif                           ! Create quadric from two planes.

c=======================================================================********

c.... Input line:  quadric QNAME fit PNAME1 PNAME2 ... PNAME9

      if (aword(3) .eq. 'fit') then

      nerr = 0

 8220 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words not 12 on input line:  ',i4)

      if (nword .ne. 12) then
        write (   6, 8220) ainput, ninput, nword
        write (iout, 8220) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the nine points defining the quadric surface.

      call ptfind (4, n1)

      if (n1 .eq. 0) nerr = 1

      call ptfind (5, n2)

      if (n2 .eq. 0) nerr = 1

      call ptfind (6, n3)

      if (n3 .eq. 0) nerr = 1

      call ptfind (7, n4)

      if (n4 .eq. 0) nerr = 1

      call ptfind (8, n5)

      if (n5 .eq. 0) nerr = 1

      call ptfind (9, n6)

      if (n6 .eq. 0) nerr = 1

      call ptfind (10, n7)

      if (n7 .eq. 0) nerr = 1

      call ptfind (11, n8)

      if (n8 .eq. 0) nerr = 1

      call ptfind (12, n9)

      if (n9 .eq. 0) nerr = 1

      if (nerr .ne. 0) go to 999

c.... Find any unique quadric suface that fits the nine points.

      call aptqfit (x(n1), y(n1), z(n1), x(n2), y(n2), z(n2),
     &              x(n3), y(n3), z(n3), x(n4), y(n4), z(n4),
     &              x(n5), y(n5), z(n5), x(n6), y(n6), z(n6),
     &              x(n7), y(n7), z(n7), x(n8), y(n8), z(n8),
     &              x(n9), y(n9), z(n9), tol,
     &              ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz, nerra)

 8140 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Singular matrix.  No solution in 10 tries.' /
     &  '  Points are coincident or no unique solution exists.' )

      if (nerra .eq. 1) then
        write (   6, 8140) ainput, ninput
        write (iout, 8140) ainput, ninput
        go to 999
      endif

 8150 format ('  quadric WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Inaccurate solution, but may be usable.')

      if (nerra .gt. 1) then
        write (   6, 8150) ainput, ninput
        write (iout, 8150) ainput, ninput
      endif

c.... Store data for the new quadric surface.

      aquads    = aword(2)
      laquads   = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      qc(n)     = ac
      qx(n)     = ax
      qy(n)     = ay
      qz(n)     = az
      qxy(n)    = axy
      qyz(n)    = ayz
      qzx(n)    = azx
      qxx(n)    = axx
      qyy(n)    = ayy
      qzz(n)    = azz

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

      endif

c=======================================================================********

c.... Input line not recognized.

 8810 format ('  quadric ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

      if (nrep .eq. 0) then
        aquad(n) = ' '
        if (n .eq. nquads) nquads = nquads - 1
      endif

c=======================================================================********

  999 return

c.... End of subroutine quadric.      (+1 line)
      end
