      subroutine qutype (awordx, nq, nopt, nerrq)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  qutype   Updated     1996 December 20 17:20.
c               qutype   Originated  1994 January 12 10:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To check the quadric type specification awordx for consistency
c               with the actual quadric type aqtype(nq) or asym(nq):
c
c               awordx                Requirement
c               ------------------    ------------------------
c               plane, pl             aqtype(nq) = aqtypes(0)    (simple)
c               sphere, sph           aqtype(nq) = aqtypes(16)
c               cylinder, cyl         aqtype(nq) = aqtypes(7)    (circular)
c               cone                  aqtype(nq) = aqtypes(12)   (circular)
c               ellipsoid, ellipse,
c                 ell, el             aqtype(nq) = aqtypes(15)
c               axisym                asym(nq)   = 'axisymmetric'
c               quadric, quad, q      (all types match)
c
c     Logic:    Entry points:  qutype.
c qutype        Called by:  copy, delete, matrix, movex, operator, point,
c               qumove, quoper, renamea, renamex, track, vector.
c
c     Biblio:   ?
c
c     Glossary:
c
c     awordx    Input    Quadric type specification:  plane, sphere, cylinder,
c                          cone, ellipsoid, axisym or quadric (or a synonym).
c
c     nerrq     Output   The result indicator, 0 for no error, 1 for an error.
c
c     nopt      Input    Option to display error message, if 1.
c
c     nq        Input    The index of the quadric surface to test.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*80 awordx

 8105 format ('  qutype ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric index is not in table:  ',i3)
 8110 format ('  qutype ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric is not a plane:  ',a)
 8120 format ('  qutype ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric is not a sphere:  ',a)
 8130 format ('  qutype ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric is not a circular cylinder:  ',a)
 8140 format ('  qutype ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric is not a circular cone:  ',a)
 8150 format ('  qutype ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric is not an ellipsoid:  ',a)
 8160 format ('  qutype ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric is not axisymmetric:  ',a)

      nerrq = 0

      if ((nq .lt. 1) .or. (nq .gt. nquads)) then
        nqerr = 1
        if (nopt .eq. 1) then
          write (   6, 8105) ainput, ninput, nq
          write (iout, 8105) ainput, ninput, nq
        endif
        go to 999
      endif

      if (((awordx .eq. 'plane') .or.
     &     (awordx .eq. 'pl'))   .and.
     &     (aqtype(nq) .ne. aqtypes(0))) then
        nerrq = 1
        if (nopt .eq. 1) then
          write (   6, 8110) ainput, ninput, aquad(nq)
          write (iout, 8110) ainput, ninput, aquad(nq)
        endif
        go to 999
      endif

      if (((awordx .eq. 'sphere') .or.
     &     (awordx .eq. 'sph'))   .and.
     &     (aqtype(nq) .ne. aqtypes(16))) then
        nerrq = 1
        if (nopt .eq. 1) then
          write (   6, 8120) ainput, ninput, aquad(nq)
          write (iout, 8120) ainput, ninput, aquad(nq)
        endif
        go to 999
      endif

      if (((awordx .eq. 'cylinder') .or.
     &     (awordx .eq. 'cyl'))     .and.
     &     (aqtype(nq) .ne. aqtypes(7))) then
        nerrq = 1
        if (nopt .eq. 1) then
          write (   6, 8130) ainput, ninput, aquad(nq)
          write (iout, 8130) ainput, ninput, aquad(nq)
        endif
        go to 999
      endif

      if ((awordx .eq. 'cone')       .and.
     &    (aqtype(nq) .ne. aqtypes(12))) then
        nerrq = 1
        if (nopt .eq. 1) then
          write (   6, 8140) ainput, ninput, aquad(nq)
          write (iout, 8140) ainput, ninput, aquad(nq)
        endif
        go to 999
      endif

      if (((awordx .eq. 'ellipsoid')  .or.
     &     (awordx .eq. 'ellipse')    .or.
     &     (awordx .eq. 'ell')        .or.
     &     (awordx .eq. 'el'))        .and.
     &     (aqtype(nq) .ne. aqtypes(15))) then
        nerrq = 1
        if (nopt .eq. 1) then
          write (   6, 8150) ainput, ninput, aquad(nq)
          write (iout, 8150) ainput, ninput, aquad(nq)
        endif
        go to 999
      endif

      if ((awordx .eq. 'axisym')       .and.
     &    (asym(nq) .ne. 'axisymmetric')) then
        nerrq = 1
        if (nopt .eq. 1) then
          write (   6, 8160) ainput, ninput, aquad(nq)
          write (iout, 8160) ainput, ninput, aquad(nq)
        endif
        go to 999
      endif

  999 return

c.... End of subroutine qutype.       (+1 line)
      end
