      subroutine side

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  side     Updated     1999 March 23 15:30.
c               side     Originated  1993 December 7 16:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the side of a plane, sphere, circular cylinder, circular
c               cone, ellipsoid, axisymmetric quadric surface or general quadric
c               surface that a point is on, the exact or estimated minimum
c               distance from the point to the surface, and the coordinates of,
c               and direction to, the proximal point.
c
c     Logic:    Entry points:  side.
c side          Called by:  helpall, main.
c
c side          Calls:  pout, ptdist, quout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input line:  side [help]

 8110 format ('  Command "side":  acceptable input lines include:' /
     &  '  "side [help]"        Display this message.' //
     &  '  "side PNAME QNAME"   Find the distance from point PNAME' /
     &  '     to quadric surface QNAME, and which side of surface'
     &  ' QNAME' /
     &  '     point PNAME is on.' /
     &  '  "side PNAME [all]"   Do the same for point PNAME and',
     &  ' all quadric surfaces.' //
     &  '  "side all QNAME"     Do the same for all points and',
     &  ' quadric surface QNAME.' //
     &  '  "side all [all]"     Do the same for all points and',
     &  ' all quadric surfaces.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Test input for validity.

      if (nword .eq. 2) then
        nword = 3
        aword(3) = 'all'
      endif

 8180 format ('  side ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Too many words on input line:  ',i3)
      if (nword .gt. 3) then
        write (   6, 8180) ainput, ninput, nword
        write (iout, 8180) ainput, ninput, nword
        go to 999
      endif

c=======================================================================********

c.... Input line:  side PNAME [QNAME,all]
c.... Input line:  side all [QNAME,all]

      np = 0

      do 180 n = 1, npoints           ! Loop over points.
        if (apoint(n) .eq. ' ') go to 180
        if ((aword(2) .eq. apoint(n)) .or.
     &      (aword(2) .eq. 'all'))    then

          np = n

c....     Find the quadric surface.

          nq = 0

          do 160 nn = 1, nquads       ! Loop over quadric surfaces.
            if ((aword(3) .eq. aquad(nn)) .or.
     &          (aword(3) .eq. 'all'))   then

              nq = nn

              call pout (np, np, 'min')
              call quout (nq, nq, 'min', nerrq)

              call ptdist (np, nq)

              write (   6, "(/)")     ! Blank line.
              write (iout, "(/)")     ! Blank line.

            endif                     ! Finding side of a quadric surface.
  160     continue                    ! End of loop over quadric surfaces.

 8132     format ('  side ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named quadric surface does not exist:  ',a)

          if (nq .eq. 0) then
            write (   6, 8132) ainput, ninput, aword(3)
            write (iout, 8132) ainput, ninput, aword(3)
            go to 999
          endif

        endif
  180 continue                        ! End of loop over points.

 8190 format ('  side ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named point does not exist:  ',a)

      if (np .eq. 0) then
        write (   6, 8190) ainput, ninput, aword(2)
        write (iout, 8190) ainput, ninput, aword(2)
      endif

c=======================================================================********

  999 return

c.... End of subroutine side.         (+1 line)
      end
