      subroutine sort

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  sort     Updated     2006 January 6 15:45.
c               sort     Originated  1994 March 10 15:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c
c     Purpose:  To sort aliases, markers, variables, vectors, operators, points,
c               (by name or mesh index), clusters, lines, triangles,
c               annular disks, regular polygons, planes, spheres,
c               circular cylinders, circular cones, hyperbolic paraboloids,
c               ellipsoids, axisymmetric quadric surfaces, general quadric
c               surfaces, regular polyhedrons, tetrahedrons, bricks, zones,
c               bins, and probability distribution functions (pdfs),
c               into increasing ASCII order of their names, and list them.
c
c     Logic:    Entry points:  sort.
c sort          Called by:  helpall, main.
c
c sort          Calls:  compsubs, list, repack.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*80 anamex

      common /asort/ mlkcl(npairm)

c.... Input line:  sort [help]

 8110 format (
     &  '  Command "sort":  acceptable input lines include:' //
     &  '  "sort [help]"       Display this message.' //
     &  '  "sort alias       [+,-,?|+]"   Sort aliases.' /
     &  '  "sort all         [+,-,?|+]"   Sort all objects.' /
     &  '  "sort axisym      [+,-,?|+]"   Sort all quadric surfaces.' /
     &  '  "sort bin         [+,-,?|+]"   Sort probability bins.' /
     &  '  "sort brick       [+,-,?|+]"   Sort bricks.' /
     &  '  "sort cluster     [+,-,?|+]"   Sort clusters.' /
     &  '  "sort cone        [+,-,?|+]"   Sort all quadric surfaces.' /
     &  '  "sort cylinder    [+,-,?|+]"   Sort all quadric surfaces.' /
     &  '  "sort disk        [+,-,?|+]"   Sort annular disks.' /
     &  '  "sort ellipsoid   [+,-,?|+]"   Sort all quadric surfaces.' /
     &  '  "sort hyperb      [+,-,?|+]"   Sort all quadric surfaces.' /
     &  '  "sort line        [+,-,?|+]"   Sort lines.' /
     &  '  "sort marker      [+,-,?|+]"   Sort markers.' /
     &  '  "sort mesh        [+,-,?|+]"   Sort mesh points by mesh',
     &  ' indices.' )

 8112 format (
     &  '  "sort operator    [+,-,?|+]"   Sort operators.' /
     &  '  "sort pdf         [+,-,?|+]"   Sort pdfs.' /
     &  '  "sort plane       [+,-,?|+]"   Sort all quadric surfaces.' /
     &  '  "sort point       [+,-,?|+]"   Sort points.' /
     &  '  "sort polygon     [+,-,?|+]"   Sort regular polygons.' /
     &  '  "sort polyhedron  [+,-,?|+]"   Sort regular polyhedrons.' /
     &  '  "sort quadric     [+,-,?|+]"   Sort all quadric surfaces.' /
     &  '  "sort sphere      [+,-,?|+]"   Sort all quadric surfaces.' /
     &  '  "sort symbol      [+,-,?|+]"   Sort symbols.' /
     &  '  "sort tetrahedron [+,-,?|+]"   Sort tetrahedrons.' /
     &  '  "sort triangle    [+,-,?|+]"   Sort triangles.' /
     &  '  "sort variable    [+,-,?|+]"   Sort variables.' /
     &  '  "sort vector      [+,-,?|+]"   Sort vectors.' /
     &  '  "sort zone        [+,-,?|+]"   Sort zones.' //
     &  '  where [+,-,?|+] means to sort in',
     &  ' [increasing, decreasing, random] order,' /
     &  '  with increasing order the default option.' )

 8114 format (/
     &  '  Synonyms:  [?, random], [alias, al, a], [brick, br],',
     &  ' [cluster, cl],' /
     &  '    [cylinder, cyl], [disk, disc, dk],',
     &  ' [ellipsoid, ellipse, ell, el], [help, h],' /
     &  '    [hyperb, saddle], [line, ln, l], [marker, mark, m],' /
     &  '    [operator, oper, op, o],' /
     &  '    [point, pnt, pt, p], [polygon, polyg, pg],',
     &  ' [polyhedron, polyh, ph],' /
     &  '    [quadric, quad, q], [sphere, sph], [symbol, symb, sym],' /
     &  '    [tetrahedron, tetra, tet],  [triangle, tri],' /
     &  '    [variable, var, set, s], [vector, vect, v],',
     &  ' [zone, zn, z].' )

      if (nword .eq. 0) go to 999

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        go to 999
      endif

      nsort = 0

c=======================================================================********

c.... Test for errors.

 9980 format ('  sort ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  More than 3 words in sort command:  ', i3,'.' )
 9982 format ('  sort ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 3 is not "+", "-", "random" or "?":  ',a)

      if (nword .gt. 3) then
        write (   6, 9980) ainput, ninput, nword
        write (iout, 9980) ainput, ninput, nword
        go to 999
      endif

      if (nword .eq. 3) then
        if ((aword(3) .ne. '+')       .and.
     &      (aword(3) .ne. '-')       .and.
     &      (aword(3) .ne. 'random')  .and.
     &      (aword(3) .ne. '?'))      then
          write (   6, 9982) ainput, ninput, aword(3)
          write (iout, 9982) ainput, ninput, aword(3)
          go to 999
        endif
      endif

c=======================================================================********

c.... Remove blanked objects.

      nword = 2

      call repack

c=======================================================================********

c.... Input line:  sort alias [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'alias') .or.
     &    (aword(2) .eq. 'all')   .or.
     &    (aword(2) .eq. 'al')    .or.
     &    (aword(2) .eq. 'a'))    then

        nsort = 1

c....   Sort aliases.

        do 520 ns = 1, nalias
          do 510 nn = ns + 1, nalias
cold            if (alias1(nn) .lt. alias1(ns)) then
            call compsubs (alias1(nn), alias1(ns), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex      = alias1(ns)
              alias1(ns)  = alias1(nn)
              alias1(nn)  = anamex
              anamex      = alias2(ns)
              alias2(ns)  = alias2(nn)
              alias2(nn)  = anamex
              lanamex     = lalias2(ns)
              lalias2(ns) = lalias2(nn)
              lalias2(nn) = lanamex
            endif
  510     continue
  520   continue

      endif                           ! Sorted aliases.

c=======================================================================********

c.... Input line:  sort big [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'big')  .or.
     &    (aword(2) .eq. 'all')) then

        nsort = 1

c....   Do not need to sort big integers.

      endif                           ! Ignored big integers.

c=======================================================================********

c.... Input line:  sort bin [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'bin')  .or.
     &    (aword(2) .eq. 'all')) then

        nsort = 1

c....   Sort probability bins.

        do 440 nb = 1, nbins
          do 430 nn = nb + 1, nbins
cold            if (abin(nn) .lt. abin(nb)) then
            call compsubs (abin(nn), abin(nb), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex     = abin(nb)
              abin(nb)   = abin(nn)
              abin(nn)   = anamex
              anamex     = atypeb(nb)
              atypeb(nb) = atypeb(nn)
              atypeb(nn) = anamex
              problx     = probl(nb)
              probl(nb)  = probl(nn)
              probl(nn)  = problx
              probrx     = probr(nb)
              probr(nb)  = probr(nn)
              probr(nn)  = probrx
              vranlx     = vranl(nb)
              vranl(nb)  = vranl(nn)
              vranl(nn)  = vranlx
              vranrx     = vranr(nb)
              vranr(nb)  = vranr(nn)
              vranr(nn)  = vranrx
              prbinx     = prbin(nb)
              prbin(nb)  = prbin(nn)
              prbin(nn)  = prbinx
              prbintx    = prbint(nb)
              prbint(nb) = prbint(nn)
              prbint(nn) = prbintx
            endif
  430     continue
  440   continue
cbugc___DEBUG begins.
cbug 9912 format ('sort DEBUG X2.  PDF/bin list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9912) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbug      write (iout, 9912) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbugc___DEBUG ends.

        do 460 nb = 1, npdfxs
          do 450 nn = nb + 1, npdfxs
cold            if (abinx(nn) .lt. abinx(nb)) then
            call compsubs (abinx(nn), abinx(nb), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex    = abinx(nb)
              abinx(nb) = abinx(nn)
              abinx(nn) = anamex
              anamex    = apdfx(nb)
              apdfx(nb) = apdfx(nn)
              apdfx(nn) = anamex
            endif
  450     continue
  460   continue
cbugc___DEBUG begins.
cbug 9913 format ('sort DEBUG Y2.  PDF/bin list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9913) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbug      write (iout, 9913) (nn, apdfx(nn), abinx(nn), nn = 1, npdfxs)
cbugc___DEBUG ends.

      endif                           ! Sorted bins.

c=======================================================================********

c.... Input line:  sort brick [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'brick') .or.
     &    (aword(2) .eq. 'all')   .or.
     &    (aword(2) .eq. 'br'))   then

        nsort = 1

c....   Sort bricks.

        do 280 nb = 1, nbricks
          do 270 nn = nb + 1, nbricks
cold            if (abrick(nn) .lt. abrick(nb)) then
            call compsubs (abrick(nn), abrick(nb), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex     = abrick(nb)
              abrick(nb) = abrick(nn)
              abrick(nn) = anamex
              anamex     = asysbr(nb)
              asysbr(nb) = asysbr(nn)
              asysbr(nn) = anamex
              anamex     = angbr(nb)
              angbr(nb)  = angbr(nn)
              angbr(nn)  = anamex
              ubrx       = ubr(nb)
              ubr(nb)    = ubr(nn)
              ubr(nn)    = ubrx
              dubrx      = dubr(nb)
              dubr(nb)   = dubr(nn)
              dubr(nn)   = dubrx
              vbrx       = vbr(nb)
              vbr(nb)    = vbr(nn)
              vbr(nn)    = vbrx
              dvbrx      = dvbr(nb)
              dvbr(nb)   = dvbr(nn)
              dvbr(nn)   = dvbrx
              wbrx       = wbr(nb)
              wbr(nb)    = wbr(nn)
              wbr(nn)    = wbrx
              dwbrx      = dwbr(nb)
              dwbr(nb)   = dwbr(nn)
              dwbr(nn)   = dwbrx
              volbrx     = volbr(nb)
              volbr(nb)  = volbr(nn)
              volbr(nn)  = volbrx
            endif
  270     continue
  280   continue

      endif                           ! Sorted regular bricks.

c=======================================================================********

c.... Input line:  sort cluster [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'cluster') .or.
     &    (aword(2) .eq. 'all')     .or.
     &    (aword(2) .eq. 'cl'))     then

        nsort = 1

c....   Sort clusters.

        do 170 ncl = 1, nclusts
          do 160 nn = ncl + 1, nclusts
cold            if (aclust(nn) .lt. aclust(ncl)) then
            call compsubs (aclust(nn), aclust(ncl), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex       = aclust(ncl)
              aclust(ncl)  = aclust(nn)
              aclust(nn)   = anamex
              lanamex      = laclust(ncl)
              laclust(ncl) = laclust(nn)
              laclust(nn)  = lanamex
            endif
  160     continue
  170   continue

      endif                           ! Sorted clusters.

c=======================================================================********

c.... Input line:  sort disk [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'disk') .or.
     &    (aword(2) .eq. 'all')  .or.
     &    (aword(2) .eq. 'disc') .or.
     &    (aword(2) .eq. 'dk'))  then

        nsort = 1

c....   Sort disks.

        do 420 nd = 1, ndisks
          do 410 nn = nd + 1, ndisks
cold            if (adisk(nn) .lt. adisk(nd)) then
            call compsubs (adisk(nn), adisk(nd), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex      = adisk(nd)
              adisk(nd)   = adisk(nn)
              adisk(nn)   = anamex
              radkinx     = radkin(nd)
              radkin(nd)  = radkin(nn)
              radkin(nn)  = radkinx
              radkoutx    = radkout(nd)
              radkout(nd) = radkout(nn)
              radkout(nn) = radkoutx

              anamex      = acendk(nd)
              acendk(nd)  = acendk(nn)
              acendk(nn)  = anamex
              xnormdkx    = xnormdk(nd)
              xnormdk(nd) = xnormdk(nn)
              xnormdk(nn) = xnormdkx
              ynormdkx    = ynormdk(nd)
              ynormdk(nd) = ynormdk(nn)
              ynormdk(nn) = ynormdkx
              znormdkx    = znormdk(nd)
              znormdk(nd) = znormdk(nn)
              znormdk(nn) = znormdkx
            endif
  410     continue
  420   continue

      endif                           ! Sorted annular disks.

c=======================================================================********

c.... Input line:  sort line [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'line') .or.
     &    (aword(2) .eq. 'all')  .or.
     &    (aword(2) .eq. 'ln')   .or.
     &    (aword(2) .eq. 'l'))   then

        nsort = 1

c....   Sort lines.

        do 190 nl = 1, nlines
          do 180 nn = nl + 1, nlines
cold            if (aline(nn) .lt. aline(nl)) then
            call compsubs (aline(nn), aline(nl), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex      = aline(nl)
              aline(nl)   = aline(nn)
              aline(nn)   = anamex
              anamex      = alipt1(nl)
              alipt1(nl)  = alipt1(nn)
              alipt1(nn)  = anamex
              lanamex     = lalipt1(nl)
              lalipt1(nl) = lalipt1(nn)
              lalipt1(nn) = lanamex
              anamex      = alipt2(nl)
              alipt2(nl)  = alipt2(nn)
              alipt2(nn)  = anamex
              lanamex     = lalipt2(nl)
              lalipt2(nl) = lalipt2(nn)
              lalipt2(nn) = lanamex
            endif
  180     continue
  190   continue

      endif                           ! Sorted lines.

c=======================================================================********

c.... Input line:  sort marker [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'm'))     then

        nsort = 1

c....   Sort markers.

        do 540 nm = 1, nmarks
          do 530 nn = nm + 1, nmarks
            if (amark(nn) .lt. amark(nm)) then
              nsmall = 1
            elseif (amark(nn) .gt. amark(nm)) then
              nsmall = 2
            else
              nsmall = 0
            endif
cold            call compsubs (amark(nn), amark(nm), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex     = amark(nm)
              amark(nm)  = amark(nn)
              amark(nn)  = anamex
              anamex     = amarkt(nm)
              amarkt(nm) = amarkt(nn)
              amarkt(nn) = anamex
            endif
  530     continue
  540   continue

      endif                           ! Sorted markers.

c=======================================================================********

c.... Input line:  sort mesh [+,-,?|+]

      if ((aword(2) .eq. 'mesh') .and.
     &    (kmax * lmax * mmax .gt. 0)) then

        nsort = 1

c....   Sort mesh points.

        do 174 np = 1, npoints
          do 172 nn = np + 1, npoints

            mlknn = mpoint(nn) + (lpoint(nn) - 1) * mmax +
     &              (kpoint(nn) - 1) * mmax * lmax
            mlknp = mpoint(np) + (lpoint(np) - 1) * mmax +
     &              (kpoint(np) - 1) * mmax * lmax
            if (mlknn .lt. mlknp) then
              anamex      = apoint(np)
              apoint(np)  = apoint(nn)
              apoint(nn)  = anamex
              lanamex     = lapoint(np)
              lapoint(np) = lapoint(nn)
              lapoint(nn) = lanamex
              xx          = x(np)
              x(np)       = x(nn)
              x(nn)       = xx
              yy          = y(np)
              y(np)       = y(nn)
              y(nn)       = yy
              zz          = z(np)
              z(np)       = z(nn)
              z(nn)       = zz
              kk          = kpoint(np)
              kpoint(np)  = kpoint(nn)
              kpoint(nn)  = kk
              ll          = lpoint(np)
              lpoint(np)  = lpoint(nn)
              lpoint(nn)  = ll
              mm          = mpoint(np)
              mpoint(np)  = mpoint(nn)
              mpoint(nn)  = mm
            endif
  172     continue
  174   continue

c....   Sort cluster list by mesh indices.

c....   First find the mlk index for each cluster pair.

        do 620 npr = 1, npairs
          mlkcl(npr) = 0
          do 610 np = 1, npoints
            if (apoint(np) .eq. apairpt(npr)) then
              mlkcl(npr) = mpoint(np) + (lpoint(np) - 1) * mmax +
     &                     (kpoint(np) - 1) * mmax * lmax
            endif
  610     continue
  620   continue
cbugc***DEBUG begins.
cbug 9991 format ('DEBUG sort',i3,'  Cluster pairs.')
cbug 9992 format (i5,2x,a,2x,a,2x,i5)
cbug      nbug = 1
cbug      write (iout, 9991) nbug
cbug      write (iout, 9992) (n, apairpt(n), apaircl(n),
cbug     &  mlkcl(n), n = 1, npairs)
cbugc***DEBUG ends.

c....   Then sort the cluster pairs by their mlk indices.

        do 640 np = 1, npairs
          do 630 nn = np + 1, npairs
            if (mlkcl(nn) .lt. mlkcl(np)) then
              anamex      = apairpt(np)
              apairpt(np) = apairpt(nn)
              apairpt(nn) = anamex
              anamex      = apaircl(np)
              apaircl(np) = apaircl(nn)
              apaircl(nn) = anamex
              mlkclx      = mlkcl(np)
              mlkcl(np)   = mlkcl(nn)
              mlkcl(nn)   = mlkclx
            endif
  630     continue
  640   continue
cbugc***DEBUG begins.
cbug      nbug = 2
cbug      write (iout, 9991) nbug
cbug      write (iout, 9992) (n, apairpt(n), apaircl(n),
cbug     &  mlkcl(n), n = 1, npairs)
cbugc***DEBUG ends.

      endif                           ! Sorted points by mesh index.

c=======================================================================********

c.... Input line:  sort operator [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'operator') .or.
     &    (aword(2) .eq. 'all')      .or.
     &    (aword(2) .eq. 'oper')     .or.
     &    (aword(2) .eq. 'op'))      then

        nsort = 1

c....   Sort operators.

        do 390 nop = 1, nopers
          do 380 nn = nop + 1, nopers
cold            if (aoper(nn) .lt. aoper(nop)) then
            call compsubs (aoper(nn), aoper(nop), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex       = aoper(nop)
              aoper(nop)   = aoper(nn)
              aoper(nn)    = anamex
              anamex       = aoptype(nop)
              aoptype(nop) = aoptype(nn)
              aoptype(nn)  = anamex
              anamex       = aoptyps(nop)
              aoptyps(nop) = aoptyps(nn)
              aoptyps(nn)  = anamex

              do 370 i = 1, 3
                do 365 j = 1, 3
                  toperx         = toper(i,j,nop)
                  toper(i,j,nop) = toper(i,j,nn)
                  toper(i,j,nn)  = toperx
                  topinx         = topin(i,j,nop)
                  topin(i,j,nop) = topin(i,j,nn)
                  topin(i,j,nn)  = topinx
  365           continue
  370         continue

            endif
  380     continue
  390   continue

      endif                           ! Sorted operators.

c=======================================================================********

c.... Input line:  sort pdf [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'pdf')     .or.
     &    (aword(2) .eq. 'all'))    then

        nsort = 1

c....   Sort probability distribution functions.

        do 480 npx = 1, npdfs
          do 470 nn = npx + 1, npdfs
cold            if (apdf(nn) .lt. apdf(npx)) then
            call compsubs (apdf(nn), apdf(npx), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex      = apdf(npx)
              apdf(npx)   = apdf(nn)
              apdf(nn)    = anamex
              prpdfx      = prpdf(npx)
              prpdf(npx)  = prpdf(nn)
              prpdf(nn)   = prpdfx
              prpdftx     = prpdft(npx)
              prpdft(npx) = prpdft(nn)
              prpdft(nn)  = prpdftx
            endif
  470     continue
  480   continue

      endif                           ! Sorted pdfs.

c=======================================================================********

c.... Input line:  sort point [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'all')   .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))    then

        nsort = 1

c....   Sort points.

        do 150 np = 1, npoints
          do 140 nn = np + 1, npoints
cold            if (apoint(nn) .lt. apoint(np)) then
            call compsubs (apoint(nn), apoint(np), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex      = apoint(np)
              apoint(np)  = apoint(nn)
              apoint(nn)  = anamex
              lanamex     = lapoint(np)
              lapoint(np) = lapoint(nn)
              lapoint(nn) = lanamex
              xx          = x(np)
              x(np)       = x(nn)
              x(nn)       = xx
              yy          = y(np)
              y(np)       = y(nn)
              y(nn)       = yy
              zz          = z(np)
              z(np)       = z(nn)
              z(nn)       = zz
              kk          = kpoint(np)
              kpoint(np)  = kpoint(nn)
              kpoint(nn)  = kk
              ll          = lpoint(np)
              lpoint(np)  = lpoint(nn)
              lpoint(nn)  = ll
              mm          = mpoint(np)
              mpoint(np)  = mpoint(nn)
              mpoint(nn)  = mm
            endif
  140     continue
  150   continue
cbugc___DEBUG begins.
cbug 9912 format ('sort DEBUG X2.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9912) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9912) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.

c....   Sort point/cluster pairs.

        do 158 np = 1, npairs
          do 156 nn = np + 1, npairs
cold            if (apairpt(nn) .lt. apairpt(np)) then
            call compsubs (apairpt(nn), apairpt(np), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex      = apairpt(np)
              apairpt(np) = apairpt(nn)
              apairpt(nn) = anamex
              anamex      = apaircl(np)
              apaircl(np) = apaircl(nn)
              apaircl(nn) = anamex
            endif
  156     continue
  158   continue
cbugc___DEBUG begins.
cbug 9913 format ('sort DEBUG Y2.  Pair list is' /
cbug     &  (i3,2x,a,2x,a))
cbug      write (   6, 9913) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbug      write (iout, 9913) (nn, apaircl(nn), apairpt(nn), nn = 1, npairs)
cbugc___DEBUG ends.

      endif                           ! Sorted points.

c=======================================================================********

c.... Input line:  sort polygon [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'polygon') .or.
     &    (aword(2) .eq. 'all')     .or.
     &    (aword(2) .eq. 'polyg')   .or.
     &    (aword(2) .eq. 'pg'))     then

        nsort = 1

c....   Sort regular polygons.

        do 260 ng = 1, npolygs
          do 250 nn = ng + 1, npolygs
cold            if (apolyg(nn) .lt. apolyg(ng)) then
            call compsubs (apolyg(nn), apolyg(ng), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex     = apolyg(ng)
              apolyg(ng) = apolyg(nn)
              apolyg(nn) = anamex
              nverpgx    = nverpg(ng)
              nverpg(ng) = nverpg(nn)
              nverpg(nn) = nverpgx
              anamex     = acenpg(ng)
              acenpg(ng) = acenpg(nn)
              acenpg(nn) = anamex
              anamex     = averpg(ng)
              averpg(ng) = averpg(nn)
              averpg(nn) = anamex
              anamex     = acoppg(ng)
              acoppg(ng) = acoppg(nn)
              acoppg(nn) = anamex
            endif
  250     continue
  260   continue

      endif                           ! Sorted regular polygons.

c=======================================================================********

c.... Input line:  sort polyhedron [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'polyhedron') .or.
     &    (aword(2) .eq. 'all')     .or.
     &    (aword(2) .eq. 'polyh')   .or.
     &    (aword(2) .eq. 'ph'))     then

        nsort = 1

c....   Sort regular polyhedrons.

        do 300 nh = 1, npolyhs
          do 290 nn = nh + 1, npolyhs
cold            if (apolyh(nn) .lt. apolyh(nh)) then
            call compsubs (apolyh(nn), apolyh(nh), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex     = apolyh(nh)
              apolyh(nh) = apolyh(nn)
              apolyh(nn) = anamex
              anamex     = atypeh(nh)
              atypeh(nh) = atypeh(nn)
              atypeh(nn) = anamex
              nverphx    = nverph(nh)
              nverph(nh) = nverph(nn)
              nverph(nn) = nverphx
              nedgphx    = nedgph(nh)
              nedgph(nh) = nedgph(nn)
              nedgph(nn) = nedgphx
              anamex     = acenph(nh)
              acenph(nh) = acenph(nn)
              acenph(nn) = anamex
              anamex     = averph(nh)
              averph(nh) = averph(nn)
              averph(nn) = anamex
              anamex     = acopph(nh)
              acopph(nh) = acopph(nn)
              acopph(nn) = anamex
            endif
  290     continue
  300   continue

      endif                           ! Sorted regular polyhedrons.

c=======================================================================********

c.... Input line:  sort quadric [+,-,?|+]
c.... Input line:  sort   axisym [+,-,?|+]
c.... Input line:  sort   cone [+,-,?|+]
c.... Input line:  sort   cylinder [+,-,?|+]
c.... Input line:  sort   ellipsoid [+,-,?|+]
c.... Input line:  sort   hyperb [+,-,?|+]
c.... Input line:  sort   plane [+,-,?|+]
c.... Input line:  sort   sphere [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'quadric')   .or.
     &    (aword(2) .eq. 'quad')      .or.
     &    (aword(2) .eq. 'q')         .or.
     &    (aword(2) .eq. 'plane')     .or.
     &    (aword(2) .eq. 'pl')        .or.
     &    (aword(2) .eq. 'sphere')    .or.
     &    (aword(2) .eq. 'sph')       .or.
     &    (aword(2) .eq. 'cylinder')  .or.
     &    (aword(2) .eq. 'cyl')       .or.
     &    (aword(2) .eq. 'cone')      .or.
     &    (aword(2) .eq. 'hyperb')    .or.
     &    (aword(2) .eq. 'saddle')    .or.
     &    (aword(2) .eq. 'ellipsoid') .or.
     &    (aword(2) .eq. 'ellipse')   .or.
     &    (aword(2) .eq. 'ell')       .or.
     &    (aword(2) .eq. 'el')        .or.
     &    (aword(2) .eq. 'axisym')    .or.
     &    (aword(2) .eq. 'all'))      then

        nsort = 1

c....   Sort quadric surfaces.

        do 320 nq = 1, nquads
          do 310 nn = nq + 1, nquads
            call compsubs (aquad(nn), aquad(nq), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then

              anamex     = aquad(nq)
              aquad(nq)  = aquad(nn)
              aquad(nn)  = anamex

              lanamex    = laquad(nq)
              laquad(nq) = laquad(nn)
              laquad(nn) = lanamex

              anamex     = aqtype(nq)
              aqtype(nq) = aqtype(nn)
              aqtype(nn) = anamex

              anamex     = aqform(nq)
              aqform(nq) = aqform(nn)
              aqform(nn) = anamex

              anamex     = asym(nq)
              asym(nq)   = asym(nn)
              asym(nn)   = anamex

              xcent      = xcen(nq)
              xcen(nq)   = xcen(nn)
              xcen(nn)   = xcent
              ycent      = ycen(nq)
              ycen(nq)   = ycen(nn)
              ycen(nn)   = ycent
              zcent      = zcen(nq)
              zcen(nq)   = zcen(nn)
              zcen(nn)   = zcent

              qct        = qc(nq)
              qc(nq)     = qc(nn)
              qc(nn)     = qct
              qxt        = qx(nq)
              qx(nq)     = qx(nn)
              qx(nn)     = qxt
              qyt        = qy(nq)
              qy(nq)     = qy(nn)
              qy(nn)     = qyt
              qzt        = qz(nq)
              qz(nq)     = qz(nn)
              qz(nn)     = qzt

              qxyt       = qxy(nq)
              qxy(nq)    = qxy(nn)
              qxy(nn)    = qxyt
              qyzt       = qyz(nq)
              qyz(nq)    = qyz(nn)
              qyz(nn)    = qyzt
              qzxt       = qzx(nq)
              qzx(nq)    = qzx(nn)
              qzx(nn)    = qzxt

              qxxt       = qxx(nq)
              qxx(nq)    = qxx(nn)
              qxx(nn)    = qxxt
              qyyt       = qyy(nq)
              qyy(nq)    = qyy(nn)
              qyy(nn)    = qyyt
              qzzt       = qzz(nq)
              qzz(nq)    = qzz(nn)
              qzz(nn)    = qzzt
            endif
  310     continue
  320   continue

      endif                           ! Sorted quadric surfaces.

c=======================================================================********

c.... Input line:  sort symbol [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'symbol') .or.
     &    (aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'symb')   .or.
     &    (aword(2) .eq. 'sym'))   then

        nsort = 1

c....   Sort symbols.

        do 560 nsymb = 1, nsymbs
          do 550 nn = nsymb + 1, nsymbs
cold            if (asymbl(nn) .lt. asymbl(nsymb)) then
            call compsubs (asymbl(nn), asymbl(nsymb), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex        = asymbl(nsymb)
              asymbl(nsymb) = asymbl(nn)
              asymbl(nn)    = anamex
              anamex        = asymbr(nsymb)
              asymbr(nsymb) = asymbr(nn)
              asymbr(nn)    = anamex
              fsymbx        = fsymb(nsymb)
              fsymb(nsymb)  = fsymb(nn)
              fsymb(nn)     = fsymbx
              isymbx        = isymb(nsymb)
              isymb(nsymb)  = isymb(nn)
              isymb(nn)     = isymbx
              lsymbx        = lsymb(nsymb)
              lsymb(nsymb)  = lsymb(nn)
              lsymb(nn)     = lsymbx
              msymbx        = msymb(nsymb)
              msymb(nsymb)  = msymb(nn)
              msymb(nn)     = msymbx
            endif
  550     continue
  560   continue

      endif                           ! Sorted symbols.

c=======================================================================********

c.... Input line:  sort tetrahedron [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'tetrahedron') .or.
     &    (aword(2) .eq. 'all')         .or.
     &    (aword(2) .eq. 'tetra')       .or.
     &    (aword(2) .eq. 'tet'))        then

        nsort = 1

c....   Sort tetrahedrons.

        do 240 nt = 1, ntets
          do 230 nn = nt + 1, ntets
cold            if (atetra(nn) .lt. atetra(nt)) then
            call compsubs (atetra(nn), atetra(nt), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex     = atetra(nt)
              atetra(nt) = atetra(nn)
              atetra(nn) = anamex
              anamex     = acenpg(nt)
              acenpg(nt) = acenpg(nn)
              acenpg(nn) = anamex

              do 225 nv = 1, 4
                anamex         = atetpt(nv,nt)
                atetpt(nv,nt)  = atetpt(nv,nn)
                atetpt(nv,nn)  = anamex
                lanamex        = latetpt(nv,nt)
                latetpt(nv,nt) = latetpt(nv,nn)
                latetpt(nv,nn) = lanamex
  225         continue

              volx       = voltet(nt)
              voltet(nt) = voltet(nn)
              voltet(nn) = volx
            endif
  230     continue
  240   continue

      endif                           ! Sorted tetrahedrons.

c=======================================================================********

c.... Input line:  sort triangle [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'triangle') .or.
     &    (aword(2) .eq. 'all')      .or.
     &    (aword(2) .eq. 'tri'))     then

        nsort = 1

c....   Sort triangles.

        do 220 nt = 1, ntrigs
          do 210 nn = nt + 1, ntrigs
cold            if (atrig(nn) .lt. atrig(nt)) then
            call compsubs (atrig(nn), atrig(nt), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex       = atrig(nt)
              atrig(nt)    = atrig(nn)
              atrig(nn)    = anamex
              anamex       = atript1(nt)
              atript1(nt)  = atript1(nn)
              atript1(nn)  = anamex
              lanamex      = latript1(nt)
              latript1(nt) = latript1(nn)
              latript1(nn) = lanamex
              anamex       = atript2(nt)
              atript2(nt)  = atript2(nn)
              atript2(nn)  = anamex
              lanamex      = latript2(nt)
              latript2(nt) = latript2(nn)
              latript2(nn) = lanamex
              anamex       = atript3(nt)
              atript3(nt)  = atript3(nn)
              atript3(nn)  = anamex
              lanamex      = latript3(nt)
              latript3(nt) = latript3(nn)
              latript3(nn) = lanamex
            endif
  210     continue
  220   continue

      endif                           ! Sorted triangles.

c=======================================================================********

c.... Input line:  sort variable [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'variable') .or.
     &    (aword(2) .eq. 'all')      .or.
     &    (aword(2) .eq. 'var')      .or.
     &    (aword(2) .eq. 'set')      .or.
     &    (aword(2) .eq. 's'))       then

        nsort = 1

c....   Sort variables.

        do 130 nvar = 1, nvars
          do 120 nn = nvar + 1, nvars
cold            if (avarbl(nn) .lt. avarbl(nvar)) then
            call compsubs (avarbl(nn), avarbl(nvar), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex       = avarbl(nvar)
              avarbl(nvar) = avarbl(nn)
              avarbl(nn)   = anamex
              anamex       = avar(nvar)
              avar(nvar)   = avar(nn)
              avar(nn)     = anamex
              fvarx        = fvar(nvar)
              fvar(nvar)   = fvar(nn)
              fvar(nn)     = fvarx
              ivarx        = ivar(nvar)
              ivar(nvar)   = ivar(nn)
              ivar(nn)     = ivarx
              lvarx        = lvar(nvar)
              lvar(nvar)   = lvar(nn)
              lvar(nn)     = lvarx
              mvarx        = mvar(nvar)
              mvar(nvar)   = mvar(nn)
              mvar(nn)     = mvarx
              anamex        = avartag(nvar)
              avartag(nvar) = avartag(nn)
              avartag(nn)   = anamex
            endif
  120     continue
  130   continue

      endif                           ! Sorted variables.

c=======================================================================********

c.... Input line:  sort vector [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'vector') .or.
     &    (aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'vect')   .or.
     &    (aword(2) .eq. 'v'))     then

        nsort = 1

c....   Sort vectors.

        do 360 nv = 1, nvectors
          do 350 nn = nv + 1, nvectors
cold            if (avector(nn) .lt. avector(nv)) then
            call compsubs (avector(nn), avector(nv), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex       = avector(nv)
              avector(nv)  = avector(nn)
              avector(nn)  = anamex
              lanamex      = lavector(nv)
              lavector(nv) = lavector(nn)
              lavector(nn) = lanamex

              xcomx        = xcom(nv)
              xcom(nv)     = xcom(nn)
              xcom(nn)     = xcomx
              ycomx        = ycom(nv)
              ycom(nv)     = ycom(nn)
              ycom(nn)     = ycomx
              zcomx        = zcom(nv)
              zcom(nv)     = zcom(nn)
              zcom(nn)     = zcomx

              xbndx        = xbnd(nv)
              xbnd(nv)     = xbnd(nn)
              xbnd(nn)     = xbndx
              ybndx        = ybnd(nv)
              ybnd(nv)     = ybnd(nn)
              ybnd(nn)     = ybndx
              zbndx        = zbnd(nv)
              zbnd(nv)     = zbnd(nn)
              zbnd(nn)     = zbndx

              vlenx        = vlen(nv)
              vlen(nv)     = vlen(nn)
              vlen(nn)     = vlenx
            endif
  350     continue
  360   continue

      endif                           ! Sorted vectors.

c=======================================================================********

c.... Input line:  sort zone [+,-,?|+]
c.... Input line:  sort all [+,-,?|+]

      if ((aword(2) .eq. 'zone') .or.
     &    (aword(2) .eq. 'all')     .or.
     &    (aword(2) .eq. 'zn')    .or.
     &    (aword(2) .eq. 'z'))       then

        nsort = 1

c....   Sort zones.

        do 340 nz = 1, nzones
          do 330 nn = nz + 1, nzones
cold            if (azone(nn) .lt. azone(nz)) then
            call compsubs (azone(nn), azone(nz), nsmall, nerrc)
            if (aword(3) .eq. '-') nsmall = 3 - nsmall
            if (aword(3) .eq. '?') nsmall = 1.5 + ranf ()
            if (nsmall .eq. 1) then
              anamex      = azone(nz)
              azone(nz)   = azone(nn)
              azone(nn)   = anamex
              nzquadx     = nzquads(nz)
              nzquads(nz) = nzquads(nn)
              nzquads(nn) = nzquadx

              do 325 nzq = 1, nzquadm
                 anamex          = azquad(nzq,nz)
                 azquad(nzq,nz)  = azquad(nzq,nn)
                 azquad(nzq,nn)  = anamex
                 lanamex         = lazquad(nzq,nz)
                 lazquad(nzq,nz) = lazquad(nzq,nn)
                 lazquad(nzq,nn) = lanamex
                 anamex          = azsign(nzq,nz)
                 azsign(nzq,nz)  = azsign(nzq,nn)
                 azsign(nzq,nn)  = anamex
  325         continue

              volzx       = volz(nz)
              volz(nz)    = volz(nn)
              volz(nn)    = volzx
              volzerrx    = volzerr(nz)
              volzerr(nz) = volzerr(nn)
              volzerr(nn) = volzerrx
            endif
  330     continue
  340   continue

      endif                           ! Sorted zones.

c=======================================================================********

 9990 format ('  sort ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Object type not recognized:  ',a)

      if (nsort .eq. 0) then
        write (   6, 9990) ainput, ninput, aword(2)
        write (iout, 9990) ainput, ninput, aword(2)
        go to 999
      endif

      nword = 2

      call list

c=======================================================================********

  999 return

c.... End of subroutine sort.         (+1 line)
      end
