      subroutine sphere

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  sphere   Updated     2005 April 5 17:45.
c               sphere   Originated  1993 December 16 10:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create spheres.
c
c     Logic:    Entry points:  sphere.
c sphere        Called by:  helpall, main.
c
c sphere        Calls:  aptspha, aptsphp, aptsubr, aptvdis, incrsubs, namechk,
c               ptfind, qumove, quoper, quout, qustore, varfind, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lsph/ rotm(3,3)         ! Principle axis rotation tensor.
      common /lsph/ nfit(4)           ! Indices of points to fit sphere to.

      character*8  amount             ! Amount of output data.
      character*24 anamex             ! Temporary name of a sphere.

c.... Input line:  sphere [help]

 8110 format (
     &  '  Command "sphere":  acceptable input lines include:' //
     &  '  "sphere [help]"   Display this message.' //
     &  '  "sphere all"      Display all spheres.' //
     &  '  "sphere list [SPHNAME1 SPHNAME2 SPHNAME3 ...]"' /
     &  '     Display all listed spheres (default is all spheres).' //
     &  '  "sphere SPHNAME"  Display sphere SPHNAME.' //
     &  '  "sphere SPHNAME = PCEN RADIUS"' /
     &  '     Create sphere SPHNAME, centered at point PCEN,' /
     &  '     with radius RADIUS.' //
     &  '  "sphere SPHNAME fit PAXIS VAXIS PNAME1 PNAME2"' /
     &  '     Create sphere SPHNAME, centered on the axis through',
     &  ' point PAXIS,' /
     &  '     in the direction of vector VAXIS, and with its surface' /
     &  '     through the two points PNAME1 and PNAME2.' //
     &  '  "sphere SPHNAME point PNAME1 PNAME2 PNAME3 PNAME4"' /
     &  '     Create sphere SPHNAME, with its surface through' /
     &  '     the four points PNAME1, PNAME2, PNAME3 and PNAME4.' )

 8112 format (/
     &  '  "sphere SPHNAME concentric NUMSPH INC PCEN RAD1 RAD2"' /
     &  '     Create a family of NUMSPH concentric spheres SPHNAME,'
     &  ' ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     centered at point PCEN, with radii equally spaced from' /
     &  '     radius RAD1 to radius RAD2.' //
     &  '  "sphere SPH02 move NUMSPH INC SPH01 VMOVE"' /
     &  '     Create a family of NUMSPH spheres SPH02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     and spaced at intervals of vector VMOVE,' /
     &  '     starting from sphere SPH01.' //
     &  '  Type <return> for more help.' )

 8113 format (a)

 8114 format (/
     &  '  "sphere SPH02 rotate NUMSPH INC SPH01 OPNAME PINV"' /
     &  '     Create a family of NUMSPH spheres SPH02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by rotating the preceding sphere with operator OPNAME' /
     &  '     and invariant point PINV, starting from sphere SPH01.' //
     &  '  "sphere SPH02 scale NUMSPH INC SPH01 OPNAME PINV"' /
     &  '     Create a family of NUMSPH quadric surfaces SPH02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by scaling the preceding quadric surfaces with operator',
     &  ' OPNAME' /
     &  '     and invariant point PINV, starting from sphere SPH01.' /
     &  '     WARNING:  a linear or radial scaling operator will',
     &  ' produce ellipsoids.' //
     &  '  Note:  use +/- for SPHNAME or SPH02 to incr/decr the last',
     &  ' sphere name:  ',a //
     &  '  Synonyms:  [help, h], [concentric, conc]',
     &  ' [move, mv, translate, trans],' /
     &  '    [point, pnt, pt, p], [rotate, rot], [scale, sc],',
     &  ' [sphere, sph].' )	

      if (nword .eq. 0) go to 999

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8113) ans
        endif
        write (   6, 8114) aspheres
        write (iout, 8114) aspheres
        go to 999
      endif

c=======================================================================********

c.... Input line:  sphere all
c.... Input line:  sphere list

 8120 format (/ 'Number of spheres = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nspherex = 0
        do 120 n = 1, nquads
          if (aquad(n) .eq. ' ') go to 120
          if (aqtype(n) .ne. aqtypes(16)) go to 120
          call quout (n, n, 'min', nerrq)
          nspherex = nspherex + 1
  120   continue
        write (   6, 8120) nspherex
        write (iout, 8120) nspherex
        go to 999
      endif

c=======================================================================********

c.... Input line:  sphere list SPHNAME1 SPHNAME2 SPHNAME3 ...

 8130 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named sphere does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, nquads
            if (aqtype(n) .ne. aqtypes(16)) go to 125
            call aptsubr (aquad(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aquad(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call quout (n, n, 'min', nerrq)
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c=======================================================================********

c.... Input line:  sphere SPHNAME

      if (nword .eq. 2) then
        do 130 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(16)) go to 130
          if (aword(2) .eq. aquad(n)) then
            amount = 'maximum'
            call quout (n, n, amount, nerrq)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

 8136 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of sphere not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... See if last sphere name should be incremented and used.

 8128 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last sphere can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = aspheres
        lword(2) = laspheres
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, aspheres
          write (iout, 8128) ainput, ninput, aspheres
          go to 999
        endif
      endif

 8129 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last sphere can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = aspheres
        lword(2) = laspheres
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, aspheres
          write (iout, 8129) ainput, ninput, aspheres
          go to 999
        endif
      endif

 8905 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of sphere too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Input line:  sphere SPHNAME concentric NUMSPH INC PCEN RAD1 RAD2

      if ((aword(3) .eq. 'concentric') .or.
     &    (aword(3) .eq. 'conc'))      then

        nerr = 0

 8505   format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 8 words in input line:  ',i3)

        if (nword .ne. 8) then
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
          nerr = 1
        endif

 8520   format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMSPH is less than 2:  ',a)
 8525   format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMSPH is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numsph = iword(4)
          if (numsph .lt. 2) then
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
            nerr = 1
          endif
        else
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
          nerr = 1
        endif

 8510   format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8515   format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8510) ainput, ninput
            write (iout, 8510) ainput, ninput
            nerr = 1
          endif
        else
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
          nerr = 1
        endif

c....   Find the center of the sphere.

        call ptfind (6, np)

        if (np .eq. 0) then
          nerr = 1
        endif

c....   Find the inner and outer radii.

 8540   format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word not integer or floating point value or variable:  ',a)

        call varfind (7, 8)

        if ((mtype(7) .ne. 2) .and. (mtype(7) .ne. 3)) then
          write (   6, 8540) ainput, ninput, aword(7)
          write (iout, 8540) ainput, ninput, aword(7)
          nerr = 1
        endif

        rad1 = fword(7)

        if ((mtype(8) .ne. 2) .and. (mtype(8) .ne. 3)) then
          write (   6, 8540) ainput, ninput, aword(8)
          write (iout, 8540) ainput, ninput, aword(8)
          nerr = 1
        endif

        rad2 = fword(8)

c....   Find the radial increment between the spheres.
c....     Use fuzz factor 1.e-99 in case numsph = 1 (an error).

        drad = (rad2 - rad1) / (numsph - 1 + 1.e-99)

c....   See if the sphere names can be generated uniquely.

 8570 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' sphere names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numsph - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8570) ainput, ninput, numsph, aword(2)
          write (iout, 8570) ainput, ninput, numsph, aword(2)
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

c....   See if all radii are positive.

 8575 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not make all radii sufficiently positive.')

        if (np .ne. 0) then
          errsq = tol * (x(np)**2 + y(np)**2 + z(np)**2)
        else                          ! Center point does not exist.
          errsq = 0.0
        endif

        if ((rad1**2 .le. errsq)  .or.
     &      (rad2**2 .le. errsq)  .or.
     &      (rad1    .le. 0.0)    .or.
     &      (rad2    .le. 0.0))   then
          write (   6, 8575) ainput, ninput
          write (iout, 8575) ainput, ninput
          nerr = 1
        endif

        if (nerr .ne. 0) go to 999

c....   Create the concentric spheres.

        anamex  = aword(2)            ! Initial sphere name.
        lanamex = lword(2)            ! Initial sphere name length.

        do 580 nsph = 1, numsph        ! Loop over the new spheres.

c....     Find the index in the quadric surface list for the new sphere.

          aword(2) = anamex
          lword(2) = lanamex

          call qustore (16, nrep, n)

          if (n .eq. 0) go to 999

          aspheres  = anamex
          laspheres = lanamex
          aquads    = anamex
          laquads   = lanamex
          axisyms   = anamex
          laxisyms  = lanamex

          aquad(n)  = anamex
          laquad(n) = lanamex
          aqtype(n) = aqtypes(16)
          asym(n)   = 'axisymmetric'

          radius    = rad1 + (nsph - 1) * drad
    
          qc(n)     = x(np)**2 +
     &                y(np)**2 +
     &                z(np)**2 - radius**2
          qx(n)     = -2.0 * x(np)
          qy(n)     = -2.0 * y(np)
          qz(n)     = -2.0 * z(np)
          qxy(n)    = 0.0
          qyz(n)    = 0.0
          qzx(n)    = 0.0
          qxx(n)    = 1.0
          qyy(n)    = 1.0
          qzz(n)    = 1.0
    
          xcen(n) = x(np)
          ycen(n) = y(np)
          zcen(n) = z(np)

          call quout (n, n, 'min', nerrq)

c....     Find the next sphere name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  580   continue                      ! End of loop over the new spheres.

c....   Test for errors.

  590   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nquads) nquads = nquads - 1
          endif
        endif

        go to 999

      endif                           ! Created concentric sphere(s).

c===============================================================================

c.... Input line:  sphere SPH02 move NUMSPH INC SPH01 VMOVE

      if ((aword(3) .eq. 'move')      .or.
     &    (aword(3) .eq. 'mv')        .or.
     &    (aword(3) .eq. 'translate') .or.
     &    (aword(3) .eq. 'trans'))    then

        call qumove

        go to 999

      endif                           ! Created translated sphere(s).

c===============================================================================

c.... Input line:  sphere SPH02 rotate NUMSPH INC SPH01 OPNAME PINV
c.... Input line:  sphere SPH02 scale  NUMSPH INC SPH01 OPNAME PINV

      if ((aword(3) .eq. 'scale')   .or.
     &    (aword(3) .eq. 'sc')      .or.
     &    (aword(3) .eq. 'rotate')  .or.
     &    (aword(3) .eq. 'rot'))    then

        call quoper

        go to 999

      endif                           ! Created rotated or scaled sphere(s).

c===============================================================================

c.... Find the index in the quadric surface array at which to store this sphere.
c....   Set nrep = 1 if replacing an existing quadric surface.

      call qustore (16, nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c=======================================================================********

c.... Input line:  sphere SPHNAME fit PAXIS VAXIS PNAME1 PNAME2

      if ((nword .eq. 7)        .and.
     &    (aword(3) .eq. 'fit')) then

c....   Find the axis point.  The sphere center is on the axis.

        call ptfind (4, npaxis)

        if (npaxis .eq. 0) then
          nerr = 1
        endif

c....   Find the axis vector.

        call vectfind (5, nv)

        if (nv .eq. 0) then
          nerr = 1
        endif

c....   See if the axis vector has nonzero length.

 8560 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Axis vector has zero length:  ',a)

        if (vlen(nv) .eq. 0.0) then
          write (   6, 8560) ainput, ninput, aword(5)
          write (iout, 8560) ainput, ninput, aword(5)
          nerr = 1
        endif

c....   Find the two points on the surface of the sphere.

        call ptfind (6, np1)

        if (np1 .eq. 0) then
          nerr = 1
        endif

        call ptfind (7, np2)

        if (np2 .eq. 0) then
          nerr = 1
        endif

c....   See if the points are coincident.

        call aptvdis (x(np1), y(np1), z(np1),
     &                x(np2), y(np2), z(np2), 1, tol, 
     &                dx,     dy,     dz,     d12, nerra)

 8400 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are coincident.  The center of the sphere may be' /
     &  '  anywhere on the axis.')

        if (d12 .eq. 0.0) then
          write (   6, 8400) ainput, ninput
          write (iout, 8400) ainput, ninput
          nerr = 1
        endif

        if (nerr .ne. 0) go to 470

        call aptspha (x(npaxis), y(npaxis), z(npaxis),
     &                xcom(nv), ycom(nv), zcom(nv),
     &                x(np1),   y(np1),   z(np1),
     &                x(np2),   y(np2),   z(np2), 1, tol,
     &                xsph,     ysph,     zsph, radius, nerra)

 8410 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are on a line perpendicular to the axis.' /
     &  '  If they are equidistant from the axis, the center of' /
     &  '  the sphere may be anywhere on the axis.' /
     &  '  Otherwise, there is no solution.')

        if (radius .ge. 1.e99) then
          write (   6, 8410) ainput, ninput
          write (iout, 8410) ainput, ninput
          nerr = 1
        endif

        go to 470

      endif                           ! Fit sphere to axis and two points.

c=======================================================================********

c.... Input line:  sphere SPHNAME point PNAME1 PNAME2 PNAME3 PNAME4

      if ((nword .eq. 7)          .and.
     &   ((aword(3) .eq. 'point') .or.
     &    (aword(3) .eq. 'pt')    .or.
     &    (aword(3) .eq. 'p')))   then

c....   Find four points to fit sphere to.

        do 420 nw = 4, 7

          call ptfind (nw, np)

          if (np .eq. 0) then
            nerr = 1
            go to 420
          endif

          nfit(nw-3) = np

  420   continue

        if (nerr .ne. 0) go to 470

        n1 = nfit(1)
        n2 = nfit(2)
        n3 = nfit(3)
        n4 = nfit(4)

        call aptsphp (x(n1), y(n1), z(n1), x(n2), y(n2), z(n2),
     &                x(n3), y(n3), z(n3), x(n4), y(n4), z(n4), 1, tol,
     &                xsph, ysph, zsph, radius, nerra)

 8420 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are coincident, colinear, or coplanar.')

        if (radius .ge. 1.e99) then
          write (   6, 8420) ainput, ninput
          write (iout, 8420) ainput, ninput
          nerr = 1
        endif

        go to 470

      endif                           ! Fit sphere to 4 points.

c=======================================================================********

c.... Input line:  sphere SPHNAME = PCEN RADIUS

      if ((nword .eq. 5)       .and.
     &    (aword(3) .eq. '=')) then
cbugc***DEBUG begins.
cbug 9901 format ('sphere DEBUG.  nbug, n, aword, lword, aquad, laquad=' /
cbug     &  2i3,2x,a24,2x,i3,2x,a24,2x,i3)
cbug      nbug = 1
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.

c....   Find the radius of the sphere.

 8440   format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word not integer or floating point value or variable:  ',a)

        call varfind (5, 5)

        if ((mtype(5) .ne. 2) .and. (mtype(5) .ne. 3)) then
          write (   6, 8440) ainput, ninput, aword(5)
          write (iout, 8440) ainput, ninput, aword(5)
          nerr = 1
        endif

        radius = fword(5)
cbugc***DEBUG begins.
cbug      nbug = 2
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.

 8450   format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Radius of sphere is not sufficiently positive:  ',1pe20.12)

        if (radius .le. 0.0) then
          write (   6, 8450) ainput, ninput, radius
          write (iout, 8450) ainput, ninput, radius
          nerr = 1
        endif

c....   Find the center of the sphere.

        call ptfind (4, np)

        if (np .eq. 0) then
          nerr = 1
          go to 470
        endif
cbugc***DEBUG begins.
cbug      nbug = 3
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.

        xsph = x(np)
        ysph = y(np)
        zsph = z(np)

        errsq = tol * (xsph**2 + ysph**2 + zsph**2)

        if ((radius**2 .le. errsq) .or.
     &      (radius    .le. 0.0))  then
          write (   6, 8450) ainput, ninput, radius
          write (iout, 8450) ainput, ninput, radius
          nerr = 1
        endif
cbugc***DEBUG begins.
cbug      nbug = 4
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.

        go to 470

      endif                           ! Create sphere at point with radius.

c-----------------------------------------------------------------------********

 8465 format ('  sphere ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized')

      nerr = 1
      write (   6, 8465) ainput, ninput
      write (iout, 8465) ainput, ninput

c-----------------------------------------------------------------------********

  470 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the coefficients of the quadric equation for the sphere.
cbugc***DEBUG begins.
cbug      nbug = 5
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.

      aspheres  = aword(2)
      laspheres = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
cbugc***DEBUG begins.
cbug      nbug = 6
cbug      write (   6, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbug      write (iout, 9901) nbug, n, aword(2), lword(2),
cbug     &                            aquad(n), laquad(n)
cbugc***DEBUG ends.

      qc(n)  = xsph**2 + ysph**2 + zsph**2 - radius**2
      qx(n)  = -2.0 * xsph
      qy(n)  = -2.0 * ysph
      qz(n)  = -2.0 * zsph
      qxy(n) = 0.0
      qyz(n) = 0.0
      qzx(n) = 0.0
      qxx(n) = 1.0
      qyy(n) = 1.0
      qzz(n) = 1.0

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

c=======================================================================********

  999 return

c.... End of subroutine sphere.       (+1 line)
      end
