      subroutine triangle

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  triangle Updated     2007 January 16 17:40.
c               triangle Originated  1993 November 29 10:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create triangles by specifying three points.
c
c     Logic:    Entry points:  triangle.
c triangle      Called by:  helpall, main, triangle* .
c
c triangle      Calls:  aptsubr, aptvpln, incrsubs, namechk, ptfind, tristore,
c               trout, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*8  amount             ! Amount of output to write in trout.
      character*24 anamex             ! Temporary name of a triangle.
      character*24 apointx1           ! Temporary name of a vertex point.
      character*24 apointx2           ! Temporary name of a vertex point.
      character*24 apointx3           ! Temporary name of a vertex point.

c.... Input line:  triangle [help]
c.... Input line:  steiner [help]

 8110 format (
     &  '  Command "triangle":  acceptable input lines include:' /
     &  '  "triangle [help]"  Display this message.' /
     &  '  "triangle all"     Display all triangles.' /
     &  '  "triangle list [TRNAME1 TRNAME2 TRNAME3 ...]"' /
     &  '     Display all listed triangles (default is all',
     &  ' triangles).' /
     &  '  "triangle TRNAME"  Display triangle TRNAME.' //
     &  '  "triangle TRNAME = PNAME1 PNAME2 PNAME3"' /
     &  '     Create triangle TRNAME through the 3 points',
     &  ' PNAME1, PNAME2 and' /
     &  '     PNAME3.' )

 8112 format (/
     &  '  "triangle TR02 increment NUMTR INC TR01 IP1 [IP2 [IP3]]"' /
     &  '     Create a family of NUMTET triangles TR02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by incrementing the names of the three vertex',
     &  ' points of' /
     &  '     the preceding triangle by the number of characters',
     &  ' indicated' /
     &  '     by the integers IP1, IP2, IP3, starting from',
     &  ' triangle TR01.' //
     &  '  "steiner [help]"  Display this message.' /
     &  '  "steiner TRNAME"  Display triangle TRNAME',
     &  ' (Steiner vertex).' //
     &  '  Note:  use +/- for TRNAME to incr/decr the last triangle',
     &  ' name:  ',a //
     &  '  Synonyms:  [help, h], [increment, incr],',
     &  ' [steiner, stein], [triangle, tri].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112) atrigs
        write (iout, 8112) atrigs
        go to 999
      endif

c===============================================================================

c.... Input line:  triangle all
c.... Input line:  triangle list

 8120 format (/ 'Number of triangles = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        ntrigx = 0
        do 120 n = 1, ntrigs
          if (atrig(n) .eq. ' ') go to 120
          ntrigx = ntrigx + 1
  120   continue
        if (ntrigx .eq. 1) then
          amount = 'maximum'
        else
          amount = 'minimum'
        endif
        call trout (1, ntrigs, amount)
        write (   6, 8120) ntrigx
        write (iout, 8120) ntrigx
        go to 999
      endif

c===============================================================================

c.... Input line:  triangle list TRNAME1 TRNAME2 TRNAME3 ...

 8130 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named triangle does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, ntrigs
            call aptsubr (atrig(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. atrig(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call trout (n, n, 'min')
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c===============================================================================

c.... Input line:  triangle TRNAME
c.... Input line:  steiner TRNAME

c.... See if user wants one triangle displayed.

      if (nword .eq. 2) then
        do 130 n = 1, ntrigs
          if (aword(2) .eq. atrig(n)) then
            amount = 'maximum'
            call trout (n, n, amount)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test input for validity.
cbug
cbug 8135 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
cbug     &  '  Number of words on "triangle" line not 6:  ',i3)
cbug      if (nword .ne. 6) then
cbug        write (   6, 8135) ainput, ninput, nword
cbug        write (iout, 8135) ainput, ninput, nword
cbug        go to 999
cbug      endif

 8136 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of triangle not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

cbug 8137 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
cbug     &  '  Input line is not recognized.')
cbug
cbug      if (aword(3) .ne. '=') then
cbug        write (   6, 8137) ainput, ninput
cbug        write (iout, 8137) ainput, ninput
cbug        go to 999
cbug      endif

c===============================================================================

c.... See if last triangle name should be incremented and used.

 8128 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last triangle can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = atrigs
        lword(2) = latrigs
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, atrigs
          write (iout, 8128) ainput, ninput, atrigs
          go to 999
        endif
      endif

 8129 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last triangle can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = atrigs
        lword(2) = latrigs
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, atrigs
          write (iout, 8129) ainput, ninput, atrigs
          go to 999
        endif
      endif

 8905 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of triangle too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  triangle TR02 increment NUMTR INC TR01 [IP1 [IP2 [IP3]]]

      if ((aword(3) .eq. 'increment') .or.
     &    (aword(3) .eq. 'incr'))     then

        nerr   = 0

 8505   format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Wrong number of words in input line:  ',i3)

        if ((nword .lt. 6) .or. (nword .gt. 9)) then
          nerr = 1
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
        endif

 8520   format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMTR is not a positive integer:  ',a)
 8525   format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMTR is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numtr = iword(4)
          if (numtr .lt. 1) then
            nerr = 1
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
          endif
        else
          nerr = 1
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
        endif

 8510   format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8515   format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            nerr = 1
            write (   6, 8510) ainput, ninput
            write (iout, 8510) ainput, ninput
          endif
        else
          nerr = 1
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
        endif

c....   Find the base triangle.

        do 530 ntr = 1, ntrigs
          if (aword(6) .eq. atrig(ntr)) then
            nbase = ntr
            call trout (nbase, nbase, 'min')
            go to 540
          endif
  530   continue
        nerr = 1
        write (   6, 8130) ainput, ninput, aword(6)
        write (iout, 8130) ainput, ninput, aword(6)

c....   Find the increments to the vertex point names.

  540   call varfind (7, nword)

 8560   format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word is not an integer or integer variable:  ',a)

        inctr1 = 0

        if (mtype(7) .eq. 2) then
          inctr1 = iword(7)
        else
          nerr = 1
          write (   6, 8560) ainput, ninput, aword(7)
          write (iout, 8560) ainput, ninput, aword(7)
        endif

        inctr2 = 0

        if (nword .ge. 8) then
          if (mtype(8) .eq. 2) then
            inctr2 = iword(8)
          else
            nerr = 1
            write (   6, 8560) ainput, ninput, aword(nw)
            write (iout, 8560) ainput, ninput, aword(nw)
          endif
        endif

        inctr3 = 0

        if (nword .eq. 9) then
          if (mtype(9) .eq. 2) then
            inctr3 = iword(9)
          else
            nerr = 1
            write (   6, 8560) ainput, ninput, aword(nw)
            write (iout, 8560) ainput, ninput, aword(nw)
          endif
        endif

c....   See if the triangle names can be generated uniquely.

 8570   format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not generate',i5,' triangle names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numtr - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8570) ainput, ninput, numtr, aword(2)
          write (iout, 8570) ainput, ninput, numtr, aword(2)
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

        if (nerr .ne. 0) go to 999

c....   Create the new triangle.

        anamex  = aword(2)            ! Initial triangle name.
        lanamex = lword(2)            ! Initial triangle name.

        do 590 ntr = 1, numtr         ! Loop over the new triangles.

c....     Find the index in the triangle list for the new triangle.

          aword(2) = anamex
          lword(2) = lanamex

          call tristore (nrep, n)

          if (n .eq. 0) go to 999

c....     Find the new names of the vertex points of the triangle.

          apointx1  = atript1(nbase)
          lapointx1 = latript1(nbase)

          if (inctr1 .eq. 0) go to 580

          call incrsubs (inctr1, apointx1, 1, lapointx1, nerra)

 8580     format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Can not increment point name:  ',a)

          if (nerra .ne. 0) then
            write (   6, 8580) ainput, ninput, atript1(nbase)
            write (iout, 8580) ainput, ninput, atript1(nbase)
            go to 999
          endif

          apointx2  = atript2(nbase)
          lapointx2 = latript2(nbase)

          if (inctr2 .eq. 0) go to 580

          call incrsubs (inctr2, apointx2, 1, lapointx2, nerra)

          if (nerra .ne. 0) then
            write (   6, 8580) ainput, ninput, atript2(nbase)
            write (iout, 8580) ainput, ninput, atript2(nbase)
            go to 999
          endif

          apointx3  = atript3(nbase)
          lapointx3 = latript3(nbase)

          if (inctr3 .eq. 0) go to 580

          call incrsubs (inctr3, apointx3, 1, lapointx3, nerra)

          if (nerra .ne. 0) then
            write (   6, 8580) ainput, ninput, atript3(nbase)
            write (iout, 8580) ainput, ninput, atript3(nbase)
            go to 999
          endif

  580     continue

c....     See if the new vertex points exist.

          aword(7) = apointx1
          lword(7) = lapointx1

          call ptfind (7, np)

          if (np .eq. 0) go to 999

          aword(8)  = apointx2
          lword(8) = lapointx2

          call ptfind (8, np)

          if (np .eq. 0) go to 999

          aword(9)  = apointx3
          lword(9) = lapointx3

          call ptfind (9, np)

          if (np .eq. 0) go to 999

c....     Store and display the new triangle data.
    
          atrigs   = anamex
          latrigs  = lanamex

          atrig(n)    = anamex
          atript1(n)  = apointx1
          atript2(n)  = apointx2
          atript3(n)  = apointx3
          latript1(n) = lapointx1
          latript2(n) = lapointx2
          latript3(n) = lapointx3

          call trout (n, n, 'min')

          nbase     = n

c....     Find the next triangle name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  590   continue                      ! End of loop over the new triangles.

        go to 999

      endif

c===============================================================================

c.... Find the index in the triangle array at which to store this triangle.
c....   Set nrep = 1 if replacing an existing triangle.

      call tristore (nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c===============================================================================

c.... Input line:  triangle TRNAME = PNAME1 PNAME2 PNAME3

      if (aword(3) .eq. '=') then

      nerr   = 0

 8185 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '   Not 6 words in input line:  ',i3)

      if (nword .ne. 6) then
        write (   6, 8185) ainput, ninput, nword
        write (iout, 8185) ainput, ninput, nword
        if (nrep .eq. 0) then
          atrig(n) = ' '
          if (n .eq. ntrigs) ntrigs = ntrigs - 1
        endif
        go to 999
      endif

c.... Find the three points defining the triangle.

      call ptfind (4, n1)
      call ptfind (5, n2)
      call ptfind (6, n3)

      if ((n1 .eq. 0)  .or.
     &    (n2 .eq. 0)  .or.
     &    (n3 .eq. 0)) then
        if (nrep .eq. 0) then
          atrig(n) = ' '
          if (n .eq. ntrigs) ntrigs = ntrigs - 1
        endif
        go to 999
      endif

c.... See if the points are unique.

 8190 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are not unique:  ' / a / a / a)

      if ((n1 .eq. n2)  .or.
     &    (n2 .eq. n3)  .or.
     &    (n3 .eq. n1)) then
        write (   6, 8190) ainput, ninput, aword(4), aword(5), aword(6)
        write (iout, 8190) ainput, ninput, aword(4), aword(5), aword(6)
        if (nrep .eq. 0) then
          atrig(n) = ' '
          if (n .eq. ntrigs) ntrigs = ntrigs - 1
        endif
        go to 999
      endif

c.... See if the points are coincident or colinear.

c....   Find the normal vector.

        call aptvpln (x(n1), y(n1), z(n1), x(n2), y(n2), z(n2),
     &                x(n3), y(n3), z(n3), 1, tol,
     &                du, dv, dw, vlength, nerra)

 8200 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are coincident or colinear:  ' / a / a / a )

      if (vlength .eq. 0.0) then
        write (   6, 8200) ainput, ninput, aword(4), aword(5), aword(6)
        write (iout, 8200) ainput, ninput, aword(4), aword(5), aword(6)
        if (nrep .eq. 0) then
          atrig(n) = ' '
          if (n .eq. ntrigs) ntrigs = ntrigs - 1
        endif
        go to 999
      endif

c.... Store and display the triangle specifications.

      atrigs   = aword(2)
      latrigs  = lword(2)

      atrig(n)    = aword(2)
      atript1(n)  = apoint(n1)
      atript2(n)  = apoint(n2)
      atript3(n)  = apoint(n3)
      latript1(n) = lapoint(n1)
      latript2(n) = lapoint(n2)
      latript3(n) = lapoint(n3)

      amount = 'maximum'
      call trout (n, n, amount)

      go to 999

      endif                           ! Input line "triangle ... = ...".

c===============================================================================


 8285 format ('  triangle ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8285) ainput, ninput
      write (iout, 8285) ainput, ninput

c===============================================================================

  999 return

c.... End of subroutine triangle.     (+1 line)
      end
