      subroutine trig

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  trig     Updated     2006 April 10 18:30.
c               trig     Originated  1995 May 31 11:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To create and display temporary triangles by specifying three
c               edges and/or angles.
c
c     Logic:    Entry points:  trig.
c trig          Called by:  helpall, main.
c
c trig          Calls:  trout, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*8 amount              ! Amount of output to write in trout.

c.... Input line:  triangle [help]

 8110 format (
     &  '  Command "trig":  acceptable input lines include:' /
     &  '  "trig [help]"  Display this message.' /
     &  '  "trig [angle,edge] VA [angle,edge] VB [angle,edge] VC"' /
     &  '     Find the missing parts of a triangle with given parts' /
     &  '     [angle,edge] VA, [angle,edge] VB, [angle,edge] VC,' /
     &  '     in counterclockwise order.' /
     &  '     The last two arguments default to "edge 1".' //
     &  '  Synonyms:  [angle, ang], [edge, side], [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... Replace any missing arguments 6 and 7.

      if (nword .eq. 5) then
        nword = 7
        mtype(6) = 1
        aword(6) = 'edge'
        mtype(7) = 2
        iword(7) = 1
        fword(7) = 1.0
      endif

c===============================================================================

c.... Test input for validity.

      nerr = 0

 8120 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "trig" line not 7:  ',i3)

      if (nword .ne. 7) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        nerr = 1
      endif

 8130 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.  Bad argument:  ',a)

      do 130 nw = 2, 6, 2             ! Loop over part labels.
        if (aword(nw) .eq. 'ang') aword(nw) = 'angle'
        if ((aword(nw) .ne. 'angle')  .and.
     &      (aword(nw) .ne. 'edge' )  .and.
     &      (aword(nw) .ne. 'side' )) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          nerr = 1
        endif
  130 continue                        ! End of loop over part labels.

 8140 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)
 8150 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Angle or edge length is not positive:  ',1pe22.14)
 8160 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Angle is 180 degrees (pi radians) or more:  ',1pe22.14)

      angsum = fword(3) + fword(5) + fword(7)

      do 140 nw = 3, 7, 2             ! Loop over numerical values.

        call varfind (nw, nw)
  
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8140) ainput, ninput, aword(nw)
          write (iout, 8140) ainput, ninput, aword(nw)
          nerr = 1
        endif

        if (fword(nw) .le. 0.0) then
          write (   6, 8150) ainput, ninput, fword(nw)
          write (iout, 8150) ainput, ninput, fword(nw)
          nerr = 1
        endif

        if (aword(nw-1) .eq. 'angle') then  ! Test for angle range.

          ang = fword(nw)

          if (angunit .eq. 'degrees') then  ! Convert to radians.
            fword(nw) = (fword(nw) / 180.0) * pi
          endif

          if (fword(nw) .ge. pi) then
            write (   6, 8160) ainput, ninput, ang
            write (iout, 8160) ainput, ninput, ang
            nerr = 1
          endif

        endif                         ! End of angle range test.

  140 continue                        ! End of loop over numerical values.

 8170 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No room to store temporary vertex points.')

      if ((npoints + 3) .gt. npointm) then
        write (   6, 8170) ainput, ninput
        write (iout, 8170) ainput, ninput
        nerr = 1
      endif

 8180 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No room to store temporary triangle.')

      if ((ntrigs + 1) .gt. ntrigm) then
        write (   6, 8180) ainput, ninput
        write (iout, 8180) ainput, ninput
        nerr = 1
      endif

      if (nerr .ne. 0) go to 999

c===============================================================================

c.... Input line:  trig edge VA edge VB edge VC

      if (((aword(2) .eq. 'edge' ) .or. (aword(2) .eq. 'side'))  .and.
     &    ((aword(4) .eq. 'edge' ) .or. (aword(4) .eq. 'side'))  .and.
     &    ((aword(6) .eq. 'edge' ) .or. (aword(6) .eq. 'side'))) then

        costh  = 0.5 *       (fword(7)**2 + fword(3)**2 - fword(5)**2) /
     &                       (fword(7) * fword(3))
        coserr = 2.0 * tol * (fword(7)**2 + fword(3)**2 + fword(5)**2) /
     &                       (fword(7) * fword(3))

        if (abs (costh) .le. coserr) costh = 0.0

        if (abs (abs (costh) - 1.0) .le. coserr) then
          costh = sign (1.0, costh)
        endif

 8210 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  One edge length not less than sum of other two.')

        if (abs (costh) .ge. 1.0) then
          write (   6, 8210) ainput, ninput
          write (iout, 8210) ainput, ninput
          go to 999
        endif

        sinth = sqrt (1.0 - costh**2)
        if (abs (sinth) .le. tol) sinth = 0.0

        np1          = npoints + 1
        apoint(np1)  = 'vertex_1'
        lapoint(np1) = 8
        x(np1)       = 0.0
        y(np1)       = 0.0
        z(np1)       = 0.0

        np2          = npoints + 2
        apoint(np2)  = 'vertex_2'
        lapoint(np2) = 8
        x(np2)       = fword(3)
        y(np2)       = 0.0
        z(np2)       = 0.0

        np3          = npoints + 3
        apoint(np3)  = 'vertex_3'
        lapoint(np3) = 8
        x(np3)       = fword(7) * costh
        y(np3)       = fword(7) * sinth
        z(np3)       = 0.0

        nt           = ntrigs + 1
        atrig(nt)    = 'type_SSS'

        go to 500

      endif                           ! Type SSS.

c===============================================================================

c.... Input line:  trig angle VA edge VB angle VC

      if ((aword(2) .eq. 'angle')  .and.
     &   ((aword(4) .eq. 'edge' ) .or. (aword(4) .eq. 'side'))  .and.
     &    (aword(6) .eq. 'angle')) then

        costh1  = cos (fword(3))
        if (abs (costh1) .le. tol) costh1 = 0.0
        sinth1  = sqrt (1.0 - costh1**2)

        sinth2  = sin (fword(7))
        sinth12 = sin (fword(3) + fword(7))
        if (abs (sinth12) .le. tol) sinth12 = 0.0

 8220 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Sum of two angles is 180 degrees (pi radians) or more.')

        if (sinth12 .le. 0.0) then
          write (   6, 8220) ainput, ninput
          write (iout, 8220) ainput, ninput
          go to 999
        endif

        np1          = npoints + 1
        apoint(np1)  = 'vertex_1'
        lapoint(np1) = 8
        x(np1)       = 0.0
        y(np1)       = 0.0
        z(np1)       = 0.0

        np2          = npoints + 2
        apoint(np2)  = 'vertex_2'
        lapoint(np2) = 8
        x(np2)       = fword(5)
        y(np2)       = 0.0
        z(np2)       = 0.0

        np3          = npoints + 3
        apoint(np3)  = 'vertex_3'
        lapoint(np3) = 8
        x(np3)       = fword(5) * costh1 * sinth2 / sinth12
        y(np3)       = fword(5) * sinth1 * sinth2 / sinth12
        z(np3)       = 0.0

        nt           = ntrigs + 1
        atrig(nt)    = 'type_ASA'

        go to 500

      endif                           ! Type ASA.

c===============================================================================

c.... Input line:  trig edge VA angle VB angle VC

      if (((aword(2) .eq. 'edge' ) .or. (aword(2) .eq. 'side'))  .and.
     &     (aword(4) .eq. 'angle')  .and.
     &     (aword(6) .eq. 'angle')) then

        costh2  = cos (fword(5))
        if (abs (costh2) .le. tol) costh2 = 0.0
        sinth2  = sqrt (1.0 - costh2**2)

        sinth3  = sin (fword(7))

        costh23 = cos (fword(5) + fword(7))
        if (abs (costh23) .le. tol) costh23 = 0.0
        sinth23 = sin (fword(5) + fword(7))
        if (abs (sinth23) .le. tol) sinth23 = 0.0

        if (sinth23 .le. 0.0) then
          write (   6, 8220) ainput, ninput
          write (iout, 8220) ainput, ninput
          go to 999
        endif

        sinth23 = sqrt (1.0 - costh23**2)

        np1          = npoints + 1
        apoint(np1)  = 'vertex_1'
        lapoint(np1) = 8
        x(np1)       = 0.0
        y(np1)       = 0.0
        z(np1)       = 0.0

        np2          = npoints + 2
        apoint(np2)  = 'vertex_2'
        lapoint(np2) = 8
        x(np2)       = fword(3)
        y(np2)       = 0.0
        z(np2)       = 0.0

        np3          = npoints + 3
        apoint(np3)  = 'vertex_3'
        lapoint(np3) = 8
        x(np3)       = -fword(3) * sinth2 * costh23 / sinth3
        y(np3)       =  fword(3) * sinth2 * sinth23 / sinth3
        z(np3)       = 0.0

        nt           = ntrigs + 1
        atrig(nt)    = 'type_SAA'

        go to 500

      endif                           ! Type SAA.

c===============================================================================

c.... Input line:  trig angle VA angle VB edge VC

      if ((aword(2) .eq. 'angle')  .and.
     &    (aword(4) .eq. 'angle')  .and.
     &   ((aword(6) .eq. 'edge' ) .or. (aword(6) .eq. 'side'))) then

        costh1  = cos (fword(5))
        if (abs (costh1) .le. tol) costh1 = 0.0
        sinth1  = sqrt (1.0 - costh1**2)

        sinth3  = sin (fword(3))

        sinth31 = sin (fword(5) + fword(3))
        if (abs (sinth31) .le. tol) sinth31 = 0.0

        if (sinth31 .le. 0.0) then
          write (   6, 8220) ainput, ninput
          write (iout, 8220) ainput, ninput
          go to 999
        endif

        np1          = npoints + 1
        apoint(np1)  = 'vertex_1'
        lapoint(np1) = 8
        x(np1)       = 0.0
        y(np1)       = 0.0
        z(np1)       = 0.0

        np2          = npoints + 2
        apoint(np2)  = 'vertex_2'
        lapoint(np2) = 8
        x(np2)       = fword(7)
        y(np2)       = 0.0
        z(np2)       = 0.0

        np3          = npoints + 3
        apoint(np3)  = 'vertex_3'
        lapoint(np3) = 8
        x(np3)       = fword(7) * costh1 * sinth31 / sinth3
        y(np3)       = fword(7) * sinth1 * sinth31 / sinth3
        z(np3)       = 0.0

        nt          = ntrigs + 1
        atrig(nt)   = 'type_AAS'

        go to 500

      endif                           ! Type AAS.

c===============================================================================

c.... Input line:  trig edge VA angle VB edge VC

      if (((aword(2) .eq. 'edge' ) .or. (aword(2) .eq. 'side'))  .and.
     &     (aword(4) .eq. 'angle')  .and.
     &    ((aword(6) .eq. 'edge' ) .or. (aword(6) .eq. 'side'))) then

        costh1  = cos (fword(5))
        if (abs (costh1) .le. tol) costh1 = 0.0
        sinth1  = sqrt (1.0 - costh1**2)

        np1          = npoints + 1
        apoint(np1)  = 'vertex_1'
        lapoint(np1) = 8
        x(np1)       = 0.0
        y(np1)       = 0.0
        z(np1)       = 0.0

        np2          = npoints + 2
        apoint(np2)  = 'vertex_2'
        lapoint(np2) = 8
        x(np2)       = fword(7)
        y(np2)       = 0.0
        z(np2)       = 0.0

        np3          = npoints + 3
        apoint(np3)  = 'vertex_3'
        lapoint(np3) = 8
        x(np3)       = fword(3) * costh1
        y(np3)       = fword(3) * sinth1
        z(np3)       = 0.0

        nt           = ntrigs + 1
        atrig(nt)    = 'type_SAS'

        go to 500

      endif                           ! Type SAS.

c===============================================================================

c.... Input line:  trig edge VA edge VB angle VC

      if (((aword(2) .eq. 'edge' ) .or. (aword(2) .eq. 'side'))  .and.
     &    ((aword(4) .eq. 'edge' ) .or. (aword(4) .eq. 'side'))  .and.
     &     (aword(6) .eq. 'angle')) then

        costh2  = cos (fword(7))
        if (abs (costh2) .le. tol) costh2 = 0.0
        sinth2  = sqrt (1.0 - costh2**2)

 8230 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No triangle is possible.')

        arg    = fword(3)**2 - fword(5)**2 * sinth2**2
        argerr = 2.0 * tol * (fword(3)**2 + fword(5)**2 * sinth2**2)
        if (abs (arg) .le. argerr) arg = 0.0

        if (arg .lt. 0.0) then
          write (   6, 8230) ainput, ninput
          write (iout, 8230) ainput, ninput
          go to 999
        endif

        sqarg = sqrt (arg)

        np1          = npoints + 1
        apoint(np1)  = 'vertex_1'
        lapoint(np1) = 8
        x(np1)       = 0.0
        y(np1)       = 0.0
        z(np1)       = 0.0

        np2          = npoints + 2
        apoint(np2)  = 'vertex_2'
        lapoint(np2) = 8
        x(np2)       = fword(5)
        y(np2)       = 0.0
        z(np2)       = 0.0

        np3          = npoints + 3
        apoint(np3)  = 'vertex_3'
        lapoint(np3) = 8
        x(np3)       = fword(5) * sinth2 * sinth2 - sqarg * costh2
        y(np3)       = fword(5) * costh2 * sinth2 + sqarg * sinth2
        z(np3)       = 0.0

        nt           = ntrigs + 1
        atrig(nt)    = 'type_SSA'

        atript1(nt)  = apoint(np1)
        atript2(nt)  = apoint(np2)
        atript3(nt)  = apoint(np3)

        latript1(nt) = lapoint(np1)
        latript2(nt) = lapoint(np2)
        latript3(nt) = lapoint(np3)
  
        ntrigs       = ntrigs + 1
        npoints      = npoints + 3

 8240   format (/ 'First  of two triangles:' /)

        if (arg .ne. 0.0) then        ! Two triangles exist.
          write (   6, 8240)
          write (iout, 8240)
        endif
  
        amount = 'maximum'
        call trout (nt, nt, amount)

 8250   format (/ 'Second of two triangles:' /)

        if (arg .ne. 0.0) then        ! Two triangles exist.

          write (   6, 8250)
          write (iout, 8250)

          x(np3)      = fword(5) * sinth2 * sinth2 + sqarg * costh2
          y(np3)      = fword(5) * sinth2 * costh2 - sqarg * sinth2

          call trout (nt, nt, amount)

        endif
  
        ntrigs      = ntrigs - 1
        npoints     = npoints - 3
  
        go to 999

      endif                           ! Type SSA.

c===============================================================================

c.... Input line:  trig angle VA edge VB edge VC

      if ((aword(2) .eq. 'angle')  .and.
     &   ((aword(4) .eq. 'edge' ) .or. (aword(4) .eq. 'side'))  .and.
     &   ((aword(6) .eq. 'edge' ) .or. (aword(6) .eq. 'side'))) then

        costh1  = cos (fword(3))
        if (abs (costh1) .le. tol) costh1 = 0.0
        sinth1  = sqrt (1.0 - costh1**2)

        arg    = fword(7)**2 - fword(5)**2 * sinth1**2
        argerr = 2.0 * tol * (fword(7)**2 + fword(5)**2 * sinth1**2)
        if (abs (arg) .le. argerr) arg = 0.0

        if (arg .lt. 0.0) then
          write (   6, 8230) ainput, ninput
          write (iout, 8230) ainput, ninput
          go to 999
        endif

        sqarg = sqrt (arg)

        np1          = npoints + 1
        apoint(np1)  = 'vertex_1'
        lapoint(np1) = 8
        x(np1)       = 0.0
        y(np1)       = 0.0
        z(np1)       = 0.0

        np2          = npoints + 2
        apoint(np2)  = 'vertex_2'
        lapoint(np2) = 8
        x(np2)       = fword(5)
        y(np2)       = 0.0
        z(np2)       = 0.0

        np3          = npoints + 3
        apoint(np3)  = 'vertex_3'
        lapoint(np3) = 8
        x(np3)       = fword(5) * costh1 * costh1 + sqarg * costh1
        y(np3)       = fword(5) * costh1 * sinth1 + sqarg * sinth1
        z(np3)       = 0.0

        nt           = ntrigs + 1
        atrig(nt)    = 'type_ASS'

        atript1(nt)  = apoint(np1)
        atript2(nt)  = apoint(np2)
        atript3(nt)  = apoint(np3)

        latript1(nt) = lapoint(np1)
        latript2(nt) = lapoint(np2)
        latript3(nt) = lapoint(np3)
  
        ntrigs       = ntrigs + 1
        npoints      = npoints + 3

        if (arg .ne. 0.0) then        ! Two triangles exist.
          write (   6, 8240)
          write (iout, 8240)
        endif
  
        amount = 'maximum'
        call trout (nt, nt, amount)

        if (arg .ne. 0.0) then        ! Two triangles exist.

          write (   6, 8250)
          write (iout, 8250)

          x(np3)      = fword(5) * costh1 * costh1 - sqarg * costh1
          y(np3)      = fword(5) * costh1 * sinth1 - sqarg * sinth1

          call trout (nt, nt, amount)

        endif
  
        ntrigs      = ntrigs - 1
        npoints     = npoints - 3
  
        go to 999

      endif                           ! Type ASS.

c===============================================================================

c.... Input line:  trig angle VA angle VB angle VC


      if ((aword(2) .eq. 'angle')  .and.
     &    (aword(4) .eq. 'angle')  .and.
     &    (aword(6) .eq. 'angle')) then

 8260   format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Sum of three angles is not 180 degrees (pi radians):  ',
     &    1pe22.14)

        angtot = fword(3) + fword(5) + fword(7)

        if (abs (angtot - pi) .gt. tol) then

          write (   6, 8260) ainput, ninput, angsum
          write (iout, 8260) ainput, ninput, angsum

          go to 999

        endif

        costh1  = cos (fword(5))
        if (abs (costh1) .le. tol) costh1 = 0.0
        sinth1  = sqrt (1.0 - costh1**2)

        sinth3  = sin (fword(3))

        sinth31 = sin (fword(5) + fword(3))

        if (sinth31 .le. 0.0) then
          write (   6, 8220) ainput, ninput
          write (iout, 8220) ainput, ninput
          go to 999
        endif

        np1          = npoints + 1
        apoint(np1)  = 'vertex_1'
        lapoint(np1) = 8
        x(np1)       = 0.0
        y(np1)       = 0.0
        z(np1)       = 0.0

        np2          = npoints + 2
        apoint(np2)  = 'vertex_2'
        lapoint(np2) = 8
        x(np2)       = 1.0
        y(np2)       = 0.0
        z(np2)       = 0.0

        np3          = npoints + 3
        apoint(np3)  = 'vertex_3'
        lapoint(np3) = 8
        x(np3)       = costh1 * sinth31 / sinth3
        y(np3)       = sinth1 * sinth31 / sinth3
        z(np3)       = 0.0

        nt           = ntrigs + 1
        atrig(nt)    = 'type_AAA'

        go to 500

      endif                         ! Type AAA.
c===============================================================================

 8500 format ('  trig ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8500) ainput, ninput
      write (iout, 8500) ainput, ninput

      go to 999

c===============================================================================

c.... Store and display the triangle specifications.

  500 atript1(nt)  = apoint(np1)
      atript2(nt)  = apoint(np2)
      atript3(nt)  = apoint(np3)

      latript1(nt) = lapoint(np1)
      latript2(nt) = lapoint(np2)
      latript3(nt) = lapoint(np3)

      ntrigs       = ntrigs + 1
      npoints      = npoints + 3

      amount = 'maximum'
      call trout (nt, nt, amount)

      ntrigs      = ntrigs - 1
      npoints     = npoints - 3

      go to 999

c===============================================================================

  999 return

c.... End of subroutine trig.         (+1 line)
      end
