      subroutine voper

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  voper    Updated     2005 April 5 17:45.
c               voper    Originated  1994 July 26 14:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To create a family of new vectors by repeatedly
c               rotating or scaling a base vector.
c
c     Logic:    Entry points:  voper.
c voper         Called by:  vector.
c
c voper         Calls:  aptmopv, incrsubs, namechk, opout, ptfind, varfind,
c               vectfind, vout, vstore.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*24 anamex             ! Word VNAME02.
      character*16 aopx               ! Word "rotate" or "scale".

c=======================================================================********

c.... Input line:  vector VNAME02 rotate NUMV INC VNAME01 OPNAME PINV
c.... Input line:  vector VNAME02 scale  NUMV INC VNAME01 OPNAME PINV

      nerr = 0

 8605 format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 8 words in input line:  ',i3)

      if (nword .ne. 8) then
        write (   6, 8605) ainput, ninput, nword
        write (iout, 8605) ainput, ninput, nword
        nerr = 1
      endif

 8620 format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word NUMV is not a positive integer:  ',a)
 8625 format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word NUMV is not an integer or integer variable:  ',a)

      call varfind (4, 4)

      if (mtype(4) .eq. 2) then
        numv = iword(4)
        if (numv .lt. 1) then
          write (   6, 8620) ainput, ninput, aword(4)
          write (iout, 8620) ainput, ninput, aword(4)
          nerr = 1
        endif
      else
        write (   6, 8625) ainput, ninput, aword(4)
        write (iout, 8625) ainput, ninput, aword(4)
        nerr = 1
      endif

 8610 format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word INC is zero:  ',a)
 8615 format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word INC is not an integer or integer variable:  ',a)

      call varfind (5, 5)

      if (mtype(5) .eq. 2) then
        inc = iword(5)
        if (inc .eq. 0) then
          write (   6, 8610) ainput, ninput, aword(5)
          write (iout, 8610) ainput, ninput, aword(5)
          nerr = 1
        endif
      else
        write (   6, 8615) ainput, ninput, aword(5)
        write (iout, 8615) ainput, ninput, aword(5)
        nerr = 1
      endif

c.... Find the base vector.

      call vectfind (6, nbase)

      if (nbase .eq. 0) then
        nerr = 1
      endif

c.... Find the operator.

 8650 format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named operator does not exist:  ',a)

      do 650 n = 1, nopers
        if (aword(7) .eq. aoper(n)) then
          nop = n
          call opout (nop, nop, 'min')
          go to 660
        endif
  650 continue
      write (   6, 8650) ainput, ninput, aword(7)
      write (iout, 8650) ainput, ninput, aword(7)
      nerr = 1

c.... See if the operator type agrees with scale or rotate.

 8660 format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Operator type is not ',a,':  ',a)

  660 if ((aword(3) .eq. 'rotate') .or.
     &    (aword(3) .eq. 'rot'))   then

        aopx = 'rotate'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8660) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8660) ainput, ninput, aopx, aoptype(nop)
          nerr = 1
        endif

      elseif ((aword(3) .eq. 'scale') .or.
     &        (aword(3) .eq. 'sc'))   then

        aopx = 'scale'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8660) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8660) ainput, ninput, aopx, aoptype(nop)
          nerr = 1
        endif

      endif

c.... Find the invariant point.

      call ptfind (8, ninv)

      if (ninv .eq. 0) then
        nerr = 1
      endif

c.... See if the vector names can be generated uniquely.

 8680 format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' vector names from:  ',a)
 8905 format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of vector too dangerous to use:  ',a)

  680 namex   = inc * (numv - 1)
      anamex  = aword(2)
      lanamex = lword(2)

      call incrsubs (namex, anamex, 1, lanamex, nerra)

      if (nerra .ne. 0) then
        write (   6, 8680) ainput, ninput, numv, aword(2)
        write (iout, 8680) ainput, ninput, numv, aword(2)
        nerr = 1
      endif

      call namechk (anamex, lanamex, nerrchk)

      if (nerrchk .ne. 0) then
        nerr = 1
        write (   6, 8905) ainput, ninput, anamex
        write (iout, 8905) ainput, ninput, anamex
      endif

      if (nerr .ne. 0) go to 999

c.... Save the base vector data.

      xcoma = xcom(nbase)
      ycoma = ycom(nbase)
      zcoma = zcom(nbase)
      xbnda = xbnd(nbase)
      ybnda = ybnd(nbase)
      zbnda = zbnd(nbase)

c.... Create the rotated or scaled vectors.

      nerr    = 0
      anamex  = aword(2)              ! Initial vector name.
      lanamex = lword(2)              ! Initial vector name.

      do 690 nvc = 1, numv            ! Loop over the new vectors.

c....   Find the index in the vector list for the new vector.

        aword(2) = anamex
        lword(2) = lanamex

        call vstore (nrep, n)

        if (n .eq. 0) go to 999

c....   Rotate or scale the base vector.

        call aptmopv (toper(1,1,nop), 0, 0., 0., 0.,
     &                xcoma, ycoma, zcoma, 1, tol, nerra)

        call aptmopv (toper(1,1,nop), 0, x(ninv), y(ninv), z(ninv),
     &                xbnda, ybnda, zbnda, 1, tol, nerra)

 8690   format ('  voper ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Unacceptable name.  Too dangerous:  ',a)

        if ((anamex .eq. 'random') .or.
     &      (anamex .eq. '?'))     then
          write (   6, 8690) ainput, ninput, anamex
          write (iout, 8690) ainput, ninput, anamex
          go to 999
        endif

        avector(n)  = anamex
        lavector(n) = lanamex
        xcom(n)     = xcoma
        ycom(n)     = ycoma
        zcom(n)     = zcoma
        xbnd(n)     = xbnda
        ybnd(n)     = ybnda
        zbnd(n)     = zbnda

        avectors    = anamex
        lavectors   = lanamex

        call vout (n, n)

c....   Find the next vector name, by incrementing by inc.

        call incrsubs (inc, anamex, 1, lanamex, nerra)

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
          go to 999
        endif

  690 continue                        ! End of loop over the new vectors.

  999 return

c.... End of subroutine voper.        (+1 line)
      end
