      subroutine where

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  where    Updated     2007 January 24 16:40.
c               where    Originated  1994 March 22 11:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find whether or not one point, a cluster of points, or
c               all points are in one or all bricks, tetrahedrons, or zones.
c
c     Logic:    Entry points:  where.
c where         Called by:  helpall, main.
c
c where         Calls:  brout, clout, ptbrick, ptfind, pttet, ptzone, tetout,
c               zout.
c               Called by:  helpall, main.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      dimension dside(nzquadm)        ! Value of F(x,y,z) at point.

      character*8 amount              ! Amount of output data.

c.... Input line:  where [help]

 8110 format ('  Command "where":  acceptable input lines include:' /
     &  '  "where [help]"  Display this message.' //
     &  '  "where point PNAME OBJTYPE OBJNAME"' /
     &  '     See if point PNAME is in object type OBJTYPE' /
     &  '     (brick, tetrahedron or zone) with name OBJNAME.' /
     &  '  "where point PNAME OBJTYPE all"' /
     &  '     See if point PNAME is in any object type OBJTYPE' /
     &  '     (brick, tetrahedron or zone).' /
     &  '  "where point PNAME"' /
     &  '     See if point PNAME is in any',
     &  ' brick, tetrahedron or zone.' /
     &  '  "where cluster CLNAME OBJTYPE OBJNAME"' /
     &  '     See if any points in cluster CLNAME are in object',
     &  ' type OBJTYPE' /
     &  '     (brick, tetrahedron or zone) with name OBJNAME.' /
     &  '  "where cluster CLNAME OBJTYPE all"' /
     &  '     See if any points in cluster CLNAME are in any',
     &  ' object type OBJTYPE' /
     &  '     (brick, tetrahedron or zone).' )
 8112 format (
     &  '  "where cluster CLNAME"' /
     &  '     See if any points in cluster CLNAME are in any' /
     &  '     brick, tetrahedron or zone.' /
     &  '  "where point all OBJTYPE OBJNAME"' /
     &  '     See if any points are in object type OBJTYPE' /
     &  '     (brick, tetrahedron or zone) with name OBJNAME.' /
     &  '  "where point all OBJTYPE all"' /
     &  '     See if any points are in any object type OBJTYPE' /
     &  '     (brick, tetrahedron or zone).' /
     &  '  "where point all"' /
     &  '     See if any points are in any',
     &  ' brick, tetrahedron or zone.' //
     &  '  Synonyms:  [brick, br], [cluster, cl], [help, h],' /
     &  '    [point, pnt, pt, p], [tetrahedron, tetra, tet],',
     &  ' [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        go to 999
      endif

c=======================================================================********

c.... See if any points, bricks, tetrahedrons, or zones exist.

 8120   format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  No points exist.')

      if (npoints .eq. 0) then
        write (   6, 8120) ainput, ninput
        write (iout, 8120) ainput, ninput
        go to 999
      endif

 8130   format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  No bricks exist.')

      if (nbricks .eq. 0) then
        if ((aword(4) .eq. 'brick') .or.
     &      (aword(4) .eq. 'br'))   then
          write (   6, 8130) ainput, ninput
          write (iout, 8130) ainput, ninput
          go to 999
        endif
      endif

 8140   format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  No tetrahedrons exist.')

      if (ntets .eq. 0) then
        if ((aword(4) .eq. 'tetrahedron') .or.
     &      (aword(4) .eq. 'tetra')   .or.
     &      (aword(4) .eq. 'tet'))   then
          write (   6, 8140) ainput, ninput
          write (iout, 8140) ainput, ninput
          go to 999
        endif
      endif

 8150   format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  No zones exist.')

      if (nzones .eq. 0) then
        if ((aword(4) .eq. 'zone') .or.
     &      (aword(4) .eq. 'zn')   .or.
     &      (aword(4) .eq. 'z'))   then
          write (   6, 8150) ainput, ninput
          write (iout, 8150) ainput, ninput
          go to 999
        endif
      endif

 8160   format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  No bricks, tetrahedrons, or zones exist.')

      if ((nbricks .eq. 0)  .and.
     &    (ntets   .eq. 0)  .and.
     &    (nzones  .eq. 0)) then
        if (nword .eq. 3) then
          write (   6, 8160) ainput, ninput
          write (iout, 8160) ainput, ninput
          go to 999
        endif
      endif

c=======================================================================********

c.... Initialize.

      nerr     = 0                    ! Error flag.
      ndone    = 0                    ! Objects tested.
      ndonep   = 0                    ! Points tested.
      ndonebr  = 0                    ! Bricks tested.
      ndonetet = 0                    ! Tetrahedrons tested.
      ndonez   = 0                    ! Zones tested.

c=======================================================================********

c.... See if a single point is to be tested.

      if (((aword(2) .eq. 'point') .or.
     &     (aword(2) .eq. 'pnt')   .or.
     &     (aword(2) .eq. 'pt')    .or.
     &     (aword(2) .eq. 'p'))    .and.
     &     (aword(3) .ne. 'all'))  then

        call ptfind (3, np)

        if (np .eq. 0) go to 999

        ndonep = ndonep + 1

c=======================================================================********

c.... Input line:  where point   PNAME  brick       all
c.... Input line:  where point   PNAME  brick       BRNAME

c.... Find object type.

        if ((aword(4) .eq. 'brick') .or.
     &      (aword(4) .eq. 'br')    .or.
     &      (nword .eq. 3))         then

c=======================================================================********

c.... Input line:  where point   PNAME  brick       BRNAME

c....     Find object option.

          if ((nword .eq. 5)         .and.
     &        (aword(5) .ne. 'all')) then

 8210       format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named brick does not exist:  ',a)

            do 210 nb = 1, nbricks
              if (aword(5) .eq. abrick(nb)) then
                call brout (nb, nb, 'min')
                go to 220
              endif
  210       continue
            write (   6, 8210) ainput, ninput, aword(5)
            write (iout, 8210) ainput, ninput, aword(5)
            go to 999

  220       nsidebr = 0
            ndone   = ndone    + 1
            ndonebr = ndonebr  + 1

            call ptbrick (0, np, nb, nside, fracu, fracv, fracw)

 8226       format ('point ',a,' is in      brick       ',a)
 8227       format ('  Fractional wts  ',1p3e20.12 )
 8228       format ('point ',a,' is not in  brick       ',a)

            if (nside .eq. 1) then
              nsidebr = nsidebr + 1
              write (   6, 8226) apoint(np), abrick(nb)
              write (iout, 8226) apoint(np), abrick(nb)
              write (   6, 8227) fracu, fracv, fracw
              write (iout, 8227) fracu, fracv, fracw
            else
              write (   6, 8228) apoint(np), abrick(nb)
              write (iout, 8228) apoint(np), abrick(nb)
            endif

c=======================================================================********

c.... Input line:  where point   PNAME  brick       all
c.... Input line:  where point   PNAME

          elseif ((nword .eq. 3)         .or.
     &            (aword(5) .eq. 'all')) then

            nsidebr = 0

            do 230 nb = 1, nbricks
              if (abrick(nb) .eq. ' ') go to 230

              ndone   = ndone    + 1
              ndonebr = ndonebr  + 1

              call ptbrick (0, np, nb, nside, fracu, fracv, fracw)

              if (nside .eq. 1) then
                nsidebr = nsidebr + 1
                write (   6, 8226) apoint(np), abrick(nb)
                write (iout, 8226) apoint(np), abrick(nb)
                write (   6, 8227) fracu, fracv, fracw
                write (iout, 8227) fracu, fracv, fracw
              endif

  230       continue

 8292       format ('point ',a,' is in ',i4,' bricks.')

            write (   6, 8292) apoint(np), nsidebr
            write (iout, 8292) apoint(np), nsidebr

          else                        ! Option not recognized.

            nerr = 1
            go to 710

          endif                       ! Tested object option.

        endif                         ! Did one or all bricks.

c=======================================================================********

c.... Input line:  where point   PNAME  tetrahedron all
c.... Input line:  where point   PNAME  tetrahedron TETNAME

        if ((aword(4) .eq. 'tetrahedron') .or.
     &      (aword(4) .eq. 'tetra')       .or.
     &      (aword(4) .eq. 'tet')         .or.
     &      (nword .eq. 3))               then

c....     Find object option.

c=======================================================================********

c.... Input line:  where point   PNAME  tetrahedron TETNAME

          if ((nword .eq. 5)         .and.
     &        (aword(5) .ne. 'all')) then

 8240       format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named tetrahedron does not exist:  ',a)

            do 240 nt = 1, ntets
              if (aword(5) .eq. atetra(nt)) then
                call tetout (nt, nt, 'min')
                go to 250
              endif
  240       continue
            write (   6, 8240) ainput, ninput, aword(5)
            write (iout, 8240) ainput, ninput, aword(5)
            go to 999

  250       nsidetet = 0
            ndone    = 1
            ndonetet = ndonetet + 1

            call pttet (0, np, nt, nside, wa, wb, wc, wd, nerra)

            if (nerra .ne. 0) go to 999

 8256       format ('point ',a,' is in      tetrahedron ',a)
 8257       format ('Vertex wts w1, w2 ',1p2e20.12 /
     &              'Vertex wts w3, w4 ',1p2e20.12 )
 8258       format ('point ',a,' is not in  tetrahedron ',a)

            if (nside .eq. 1) then
              nsidetet = nsidetet + 1
              write (   6, 8256) apoint(np), atetra(nt)
              write (iout, 8256) apoint(np), atetra(nt)
              write (   6, 8257) wa, wb, wc, wd
              write (iout, 8257) wa, wb, wc, wd
            else
              write (   6, 8258) apoint(np), atetra(nt)
              write (iout, 8258) apoint(np), atetra(nt)
            endif

c=======================================================================********

c.... Input line:  where point   PNAME  tetrahedron all
c.... Input line:  where point   PNAME

          elseif ((nword .eq. 3)         .or.
     &            (aword(5) .eq. 'all')) then

            nsidetet = 0

            do 260 nt = 1, ntets
              if (atetra(nt) .eq. ' ') go to 260

              ndone    = ndone    + 1
              ndonetet = ndonetet + 1

              call pttet (0, np, nt, nside, wa, wb, wc, wd, nerra)

              if (nerra .ne. 0) go to 999

              if (nside .eq. 1) then
                nsidetet = nsidetet + 1
                write (   6, 8256) apoint(np), atetra(nt)
                write (iout, 8256) apoint(np), atetra(nt)
                write (   6, 8257) wa, wb, wc, wd
                write (iout, 8257) wa, wb, wc, wd
              endif

  260       continue

 8294       format ('point ',a,' is in ',i4,' tetrahedrons.')

            write (   6, 8294) apoint(np), nsidetet
            write (iout, 8294) apoint(np), nsidetet

c=======================================================================********

          else                        ! Option not recognized.

            nerr = 1
            go to 710

          endif                       ! Tested object option.

        endif                         ! Did one or all tetrahedrons.

c=======================================================================********

c.... Input line:  where point   PNAME  zone        all
c.... Input line:  where point   PNAME  zone        ZNAME

        if ((aword(4) .eq. 'zone') .or.
     &      (aword(4) .eq. 'zn')   .or.
     &      (aword(4) .eq. 'z')    .or.
     &      (nword .eq. 3))        then

c....     Find object option.

          if ((nword .eq. 5)         .and.
     &        (aword(5) .ne. 'all')) then

c=======================================================================********

c.... Input line:  where point   PNAME  zone        ZNAME

 8270       format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named zone does not exist:  ',a)

            do 270 nz = 1, nzones
              if (aword(5) .eq. azone(nz)) then
                call zout (nz, nz, 'min')
                go to 280
              endif
  270       continue
            write (   6, 8270) ainput, ninput, aword(5)
            write (iout, 8270) ainput, ninput, aword(5)
            go to 999

  280       nsidez   = 0
            ndone    = ndone    + 1
            ndonez   = ndonez   + 1

            amount = 'maximum'

            call ptzone (0, np, nz, amount, dside, nside, nerra)

            if (nerra .ne. 0) go to 999

 8286       format ('point ',a,' is in      zone        ',a)
 8287       format ('  dside ',a,2x,1pe20.12)
 8288       format ('point ',a,' is not in  zone        ',a)

            if (nside .eq. 1) then
              nsidez = nsidez + 1
              write (   6, 8286) apoint(np), azone(nz)
              write (iout, 8286) apoint(np), azone(nz)
              write (   6, 8287) (azquad(nzq,nz), dside(nzq),
     &          nzq = 1, nzquads(nz))
              write (iout, 8287) (azquad(nzq,nz), dside(nzq),
     &          nzq = 1, nzquads(nz))
            else
              write (   6, 8288) apoint(np), azone(nz)
              write (iout, 8288) apoint(np), azone(nz)
            endif

c=======================================================================********

c.... Input line:  where point   PNAME  zone        all
c.... Input line:  where point   PNAME

          elseif ((nword .eq. 3)         .or.
     &            (aword(5) .eq. 'all')) then

            nsidez = 0

            do 290 nz = 1, nzones
              if (azone(nz) .eq. ' ') go to 290

              ndone    = ndone    + 1
              ndonez   = ndonez   + 1

              amount = 'minimum'

              call ptzone (0, np, nz, amount, dside, nside, nerra)

              if (nerra .ne. 0) go to 999

              if (nside .eq. 1) then
                nsidez = nsidez + 1
                write (   6, 8286) apoint(np), azone(nz)
                write (iout, 8286) apoint(np), azone(nz)
                write (   6, 8287) (azquad(nzq,nz), dside(nzq),
     &            nzq = 1, nzquads(nz))
                write (iout, 8287) (azquad(nzq,nz), dside(nzq),
     &            nzq = 1, nzquads(nz))
              endif

  290       continue

 8296       format ('point ',a,' is in ',i4,' zones.')

            write (   6, 8296) apoint(np), nsidez
            write (iout, 8296) apoint(np), nsidez

c=======================================================================********

          else                        ! Option not recognized.

            nerr = 1
            go to 710

          endif                       ! Tested object option.

        endif                         ! Did one or all zones.

c=======================================================================********

c....   See if any objects were tested.

        if (ndone .eq. 0) then
          nerr = 1
        endif

        go to 710

      endif                           ! Did a single point.

c=======================================================================********

c.... See if a cluster of points is to be tested.

      if ((aword(2) .eq. 'cluster') .or.
     &    (aword(2) .eq. 'cl'))     then

 8310   format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named cluster does not exist:  ',a)

        do 310 nc = 1, nclusts
          if (aword(3) .eq. aclust(nc)) then
            call clout (nc, nc, 'min')
            go to 315
          endif
  310   continue
        write (   6, 8310) ainput, ninput, aword(3)
        write (iout, 8310) ainput, ninput, aword(3)
        go to 999

  315   continue

c....   Find object type.

c=======================================================================********

c.... Input line:  where cluster CLNAME brick       all
c.... Input line:  where cluster CLNAME brick       BRNAME

        if ((aword(4) .eq. 'brick') .or.
     &      (aword(4) .eq. 'br')    .or.
     &      (nword .eq. 3))         then

c....     Find object option.

c=======================================================================********

c.... Input line:  where cluster CLNAME brick       BRNAME

          if ((nword .eq. 5)         .and.
     &        (aword(5) .ne. 'all')) then

 8320       format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named brick does not exist:  ',a)

            do 320 nb = 1, nbricks
              if (aword(5) .eq. abrick(nb)) then
                call brout (nb, nb, 'min')
                go to 325
              endif
  320       continue
            write (   6, 8320) ainput, ninput, aword(5)
            write (iout, 8320) ainput, ninput, aword(5)
            go to 999

  325       continue

            do 330 npr = 1, npairs
              if (apaircl(npr) .ne. aword(3)) go to 330
              do 328 np = 1, npoints
                if (apoint(np) .ne. apairpt(npr)) go to 328

                nsidebr  = 0
                ndone    = ndone    + 1
                ndonep   = ndonep   + 1
                ndonebr  = ndonebr  + 1

                call ptbrick (0, np, nb, nside, fracu, fracv, fracw)

                if (nside .eq. 1) then
                  nsidebr = nsidebr + 1
                  write (   6, 8226) apoint(np), abrick(nb)
                  write (iout, 8226) apoint(np), abrick(nb)
                  write (   6, 8227) fracu, fracv, fracw
                  write (iout, 8227) fracu, fracv, fracw
                endif

  328         continue
  330       continue

c=======================================================================********

c.... Input line:  where cluster CLNAME brick       all
c.... Input line:  where cluster CLNAME

          elseif ((nword .eq. 3)         .or.
     &            (aword(5) .eq. 'all')) then

            ndonep = 0

            do 335 npr = 1, npairs
              if (apaircl(npr) .ne. aword(3)) go to 335

              do 332 np = 1, npoints
                if (apoint(np) .ne. apairpt(npr)) go to 332

                nsidebr = 0
                ndonep  = ndonep + 1

                do 331 nb = 1, nbricks
                  if (abrick(nb) .eq. ' ') go to 331

                  ndone    = ndone    + 1
                  ndonebr  = ndonebr  + 1

                  call ptbrick (0, np, nb, nside, fracu, fracv, fracw)

                  if (nside .eq. 1) then
                    nsidebr = nsidebr + 1
                    write (   6, 8226) apoint(np), abrick(nb)
                    write (iout, 8226) apoint(np), abrick(nb)
                    write (   6, 8227) fracu, fracv, fracw
                    write (iout, 8227) fracu, fracv, fracw
                  endif

  331           continue

                if (nsidebr .ne. 0) then
                  write (   6, 8292) apoint(np), nsidebr
                  write (iout, 8292) apoint(np), nsidebr
                endif
  332         continue
  335       continue

          else                        ! Option not recognized.

            nerr = 1
            go to 710

          endif                       ! Tested object option.

        endif                         ! Did one or all bricks.

c=======================================================================********

c.... Input line:  where cluster CLNAME tetrahedron all
c.... Input line:  where cluster CLNAME tetrahedron TETNAME

        if ((aword(4) .eq. 'tetrahedron') .or.
     &      (aword(4) .eq. 'tetra')       .or.
     &      (aword(4) .eq. 'tet')         .or.
     &      (nword .eq. 3))               then

c....     Find object option.

c=======================================================================********

c.... Input line:  where cluster CLNAME tetrahedron TETNAME

          if ((nword .eq. 5)         .and.
     &        (aword(5) .ne. 'all')) then

 8345       format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named tetrahedron does not exist:  ',a)

            do 345 nt = 1, ntets
              if (aword(5) .eq. atetra(nt)) then
                call tetout (nt, nt, 'min')
                go to 350
              endif
  345       continue
            write (   6, 8345) ainput, ninput, aword(5)
            write (iout, 8345) ainput, ninput, aword(5)
            go to 999

  350       continue

            do 355 npr = 1, npairs
              if (apaircl(npr) .ne. aword(3)) go to 355
              do 352 np = 1, npoints
                if (apoint(np) .ne. apairpt(npr)) go to 352

                nsidetet = 0
                ndone    = ndone    + 1
                ndonep   = ndonep   + 1
                ndonetet = ndonetet + 1

                call pttet (0, np, nt, nside, wa, wb, wc, wd, nerra)

                if (nerra .ne. 0) go to 999

                if (nside .eq. 1) then
                  nsidetet = nsidetet + 1
                  write (   6, 8256) apoint(np), atetra(nt)
                  write (iout, 8256) apoint(np), atetra(nt)
                  write (   6, 8257) wa, wb, wc, wd
                  write (iout, 8257) wa, wb, wc, wd
                endif

  352         continue
  355       continue

c=======================================================================********

c.... Input line:  where cluster CLNAME tetrahedron all
c.... Input line:  where cluster CLNAME

          elseif ((nword .eq. 3)         .or.
     &            (aword(5) .eq. 'all')) then

            ndonep = 0

            do 360 npr = 1, npairs
              if (apaircl(npr) .ne. aword(3)) go to 360
              do 358 np = 1, npoints
                if (apoint(np) .ne. apairpt(npr)) go to 358

                nsidetet = 0
                ndonep   = ndonep + 1

                do 357 nt = 1, ntets
                  if (atetra(nt) .eq. ' ') go to 357

                  ndone    = ndone    + 1
                  ndonetet = ndonetet + 1

                  call pttet (0, np, nt, nside, wa, wb, wc, wd, nerra)

                  if (nerra .ne. 0) go to 999

                  if (nside .eq. 1) then
                    nsidetet = nsidetet + 1
                    write (   6, 8256) apoint(np), atetra(nt)
                    write (iout, 8256) apoint(np), atetra(nt)
                    write (   6, 8257) wa, wb, wc, wd
                    write (iout, 8257) wa, wb, wc, wd
                  endif

  357           continue

                if (nsidetet .ne. 0) then
                  write (   6, 8294) apoint(np), nsidetet
                  write (iout, 8294) apoint(np), nsidetet
                endif
  358         continue
  360       continue


c=======================================================================********

          else                        ! Option not recognized.

            nerr = 1
            go to 710

          endif                       ! Tested object option.

        endif                         ! Did one or all tetrahedrons.

c=======================================================================********

c.... Input line:  where cluster CLNAME zone        all
c.... Input line:  where cluster CLNAME zone        ZNAME

        if ((aword(4) .eq. 'zone') .or.
     &      (aword(4) .eq. 'zn')   .or.
     &      (aword(4) .eq. 'z')    .or.
     &      (nword .eq. 3))        then

c....     Find object option.

          if ((nword .eq. 5)         .and.
     &        (aword(5) .ne. 'all')) then

c=======================================================================********

c.... Input line:  where cluster CLNAME zone        ZNAME

 8370       format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named zone does not exist:  ',a)

            do 370 nz = 1, nzones
              if (aword(5) .eq. azone(nz)) then
                call zout (nz, nz, 'min')
                go to 375
              endif
  370       continue
            write (   6, 8370) ainput, ninput, aword(5)
            write (iout, 8370) ainput, ninput, aword(5)
            go to 999

  375       continue

            do 380 npr = 1, npairs
              if (apaircl(npr) .ne. aword(3)) go to 380
              do 378 np = 1, npoints
                if (apoint(np) .ne. apairpt(npr)) go to 378

                nsidez   = 0
                ndone    = ndone    + 1
                ndonep   = ndonep   + 1
                ndonez   = ndonez   + 1

                amount = 'minimum'

                call ptzone (0, np, nz, amount, dside, nside, nerra)

                if (nerra .ne. 0) go to 999

                if (nside .eq. 1) then
                  nsidez = nsidez + 1
                  write (   6, 8286) apoint(np), azone(nz)
                  write (iout, 8286) apoint(np), azone(nz)
                  write (   6, 8287) (azquad(nzq,nz), dside(nzq),
     &              nzq = 1, nzquads(nz))
                  write (iout, 8287) (azquad(nzq,nz), dside(nzq),
     &              nzq = 1, nzquads(nz))
                endif

  378         continue
  380       continue

c=======================================================================********

c.... Input line:  where cluster CLNAME zone        all
c.... Input line:  where cluster CLNAME

          elseif ((nword .eq. 3)         .or.
     &            (aword(5) .eq. 'all')) then

            ndonep = 0

            do 385 npr = 1, npairs
              if (apaircl(npr) .ne. aword(3)) go to 385
              do 382 np = 1, npoints
                if (apoint(np) .ne. apairpt(npr)) go to 382

                nsidez = 0
                ndonep = ndonep + 1

                do 381 nz = 1, nzones
                  if (azone(nz) .eq. ' ') go to 381

                  ndone    = ndone    + 1
                  ndonez   = ndonez   + 1

                  amount = 'minimum'

                  call ptzone (0, np, nz, amount, dside, nside, nerra)

                  if (nerra .ne. 0) go to 999

                  if (nside .eq. 1) then
                    nsidez = nsidez + 1
                    write (   6, 8286) apoint(np), azone(nz)
                    write (iout, 8286) apoint(np), azone(nz)
                    write (   6, 8287) (azquad(nzq,nz), dside(nzq),
     &                nzq = 1, nzquads(nz))
                    write (iout, 8287) (azquad(nzq,nz), dside(nzq),
     &                nzq = 1, nzquads(nz))
                  endif

  381           continue

              if (nsidez .ne. 0) then
                write (   6, 8296) apoint(np), nsidez
                write (iout, 8296) apoint(np), nsidez
              endif
  382         continue
  385       continue

c=======================================================================********

          else                        ! Option not recognized.

            nerr = 1
            go to 710

          endif                       ! Tested object option.

        endif                         ! Did one or all zones.

c=======================================================================********

c....   See if any objects were tested.

        if (ndone .eq. 0) then
          nerr = 1
        endif

        go to 710

      endif                           ! Did a cluster of points.

c=======================================================================********

c.... See if all points are to be tested.

      if (((aword(2) .eq. 'point') .or.
     &     (aword(2) .eq. 'pnt')   .or.
     &     (aword(2) .eq. 'pt')    .or.
     &     (aword(2) .eq. 'p'))    .and.
     &     (aword(3) .eq. 'all'))  then

c....   Find object type.

c=======================================================================********

c.... Input line:  where point   all    brick       all
c.... Input line:  where point   all    brick       BRNAME

        if ((aword(4) .eq. 'brick') .or.
     &      (aword(4) .eq. 'br')    .or.
     &      (nword .eq. 3))         then

c....     Find object option.

c=======================================================================********

c.... Input line:  where point   all    brick       BRNAME

          if ((nword .eq. 5)         .and.
     &        (aword(5) .ne. 'all')) then

 8420       format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named brick does not exist:  ',a)

            do 420 nb = 1, nbricks
              if (aword(5) .eq. abrick(nb)) then
                call brout (nb, nb, 'min')
                go to 425
              endif
  420       continue
            write (   6, 8420) ainput, ninput, aword(5)
            write (iout, 8420) ainput, ninput, aword(5)
            go to 999

  425       continue

            do 430 np = 1, npoints
              if (apoint(np) .eq. ' ') go to 430

              nsidebr  = 0
              ndone    = ndone    + 1
              ndonep   = ndonep   + 1
              ndonebr  = ndonebr  + 1

              call ptbrick (0, np, nb, nside, fracu, fracv, fracw)

              if (nside .eq. 1) then
                nsidebr = nsidebr + 1
                write (   6, 8226) apoint(np), abrick(nb)
                write (iout, 8226) apoint(np), abrick(nb)
                write (   6, 8227) fracu, fracv, fracw
                write (iout, 8227) fracu, fracv, fracw
              endif

  430       continue

c=======================================================================********

c.... Input line:  where point   all    brick       all
c.... Input line:  where point   all

          elseif ((nword .eq. 3)          .or.
     &            (aword(5) .eq. 'all'))  then

            ndonep = 0

            do 435 np = 1, npoints
              if (apoint(np) .eq. ' ') go to 435

              nsidebr = 0
              ndonep  = ndonep + 1

              do 434 nb = 1, nbricks
                if (abrick(nb) .eq. ' ') go to 434

                ndone    = ndone    + 1
                ndonebr  = ndonebr  + 1

                call ptbrick (0, np, nb, nside, fracu, fracv, fracw)

                if (nside .eq. 1) then
                  nsidebr = nsidebr + 1
                  write (   6, 8226) apoint(np), abrick(nb)
                  write (iout, 8226) apoint(np), abrick(nb)
                  write (   6, 8227) fracu, fracv, fracw
                  write (iout, 8227) fracu, fracv, fracw
                endif

  434         continue

              if (nsidebr .ne. 0) then
                write (   6, 8292) apoint(np), nsidebr
                write (iout, 8292) apoint(np), nsidebr
              endif
  435       continue

          else                        ! Option not recognized.

            nerr = 1
            go to 710

          endif                       ! Tested object option.

        endif                         ! Did one or all bricks.

c=======================================================================********

c.... Input line:  where point   all    tetrahedron all
c.... Input line:  where point   all    tetrahedron TETNAME

        if ((aword(4) .eq. 'tetrahedron') .or.
     &      (aword(4) .eq. 'tetra')       .or.
     &      (aword(4) .eq. 'tet')         .or.
     &      (nword .eq. 3))               then

c....     Find object option.

c=======================================================================********

c.... Input line:  where point   all    tetrahedron TETNAME

          if ((nword .eq. 5)          .and.
     &        (aword(5) .ne. 'all'))  then

 8445       format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named tetrahedron does not exist:  ',a)

            do 445 nt = 1, ntets
              if (aword(5) .eq. atetra(nt)) then
                call tetout (nt, nt, 'min')
                go to 450
              endif
  445       continue
            write (   6, 8445) ainput, ninput, aword(5)
            write (iout, 8445) ainput, ninput, aword(5)
            go to 999

  450       continue

            do 455 np = 1, npoints
              if (apoint(np) .eq. ' ') go to 455

              nsidetet = 0
              ndone    = ndone    + 1
              ndonep   = ndonep   + 1
              ndonetet = ndonetet + 1

              call pttet (0, np, nt, nside, wa, wb, wc, wd, nerra)

              if (nerra .ne. 0) go to 999

              if (nside .eq. 1) then
                nsidetet = nsidetet + 1
                write (   6, 8256) apoint(np), atetra(nt)
                write (iout, 8256) apoint(np), atetra(nt)
                write (   6, 8257) wa, wb, wc, wd
                write (iout, 8257) wa, wb, wc, wd
              endif

  455       continue


c=======================================================================********

c.... Input line:  where point   all    tetrahedron all
c.... Input line:  where point   all

          elseif ((nword .eq. 3)          .or.
     &            (aword(5) .eq. 'all'))  then

            ndonep = 0

            do 460 np = 1, npoints
              if (apoint(np) .eq. ' ') go to 460

              nsidetet = 0
              ndonep   = ndonep + 1

              do 459 nt = 1, ntets
                if (atetra(nt) .eq. ' ') go to 459

                ndone    = ndone    + 1
                ndonetet = ndonetet + 1

                call pttet (0, np, nt, nside, wa, wb, wc, wd, nerra)

                if (nerra .ne. 0) go to 999

                if (nside .eq. 1) then
                  nsidetet = nsidetet + 1
                  write (   6, 8256) apoint(np), atetra(nt)
                  write (iout, 8256) apoint(np), atetra(nt)
                  write (   6, 8257) wa, wb, wc, wd
                  write (iout, 8257) wa, wb, wc, wd
                endif

  459         continue

              if (nsidetet .ne. 0) then
                write (   6, 8294) apoint(np), nsidetet
                write (iout, 8294) apoint(np), nsidetet
              endif
  460       continue

c=======================================================================********

          else                        ! Option not recognized.

            nerr = 1
            go to 710

          endif                       ! Tested object option.

        endif                         ! Did one or all tetrahedrons.

c=======================================================================********

c.... Input line:  where point   all    zone        all
c.... Input line:  where point   all    zone        ZNAME

        if ((aword(4) .eq. 'zone') .or.
     &      (aword(4) .eq. 'zn')   .or.
     &      (aword(4) .eq. 'z')    .or.
     &      (nword .eq. 3))        then

c....     Find object option.

          if ((nword .eq. 5)          .and.
     &        (aword(5) .ne. 'all'))  then

c=======================================================================********

c.... Input line:  where point   all    zone        ZNAME

 8470       format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Named zone does not exist:  ',a)

            do 470 nz = 1, nzones
              if (aword(5) .eq. azone(nz)) then
                call zout (nz, nz, 'min')
                go to 475
              endif
  470       continue
            write (   6, 8470) ainput, ninput, aword(5)
            write (iout, 8470) ainput, ninput, aword(5)
            go to 999

  475       continue

            do 480 np = 1, npoints

              if (apoint(np) .eq. ' ') go to 480

              nsidez   = 0
              ndone    = ndone    + 1
              ndonep   = ndonep   + 1
              ndonez   = ndonez   + 1

              amount = 'minimum'

              call ptzone (0, np, nz, amount, dside, nside, nerra)

              if (nerra .ne. 0) go to 999

              if (nside .eq. 1) then
                nsidez = nsidez + 1
                write (   6, 8286) apoint(np), azone(nz)
                write (iout, 8286) apoint(np), azone(nz)
                write (   6, 8287) (azquad(nzq,nz), dside(nzq),
     &            nzq = 1, nzquads(nz))
                write (iout, 8287) (azquad(nzq,nz), dside(nzq),
     &            nzq = 1, nzquads(nz))
              endif

  480       continue


c=======================================================================********

c.... Input line:  where point   all    zone        all
c.... Input line:  where point   all

          elseif ((nword .eq. 3)         .or.
     &            (aword(5) .eq. 'all')) then

            ndonep = 0

            do 485 np = 1, npoints
              if (apoint(np) .eq. ' ') go to 485

              nsidez = 0
              ndonep = ndonep + 1

              do 484 nz = 1, nzones
                if (azone(nz) .eq. ' ') go to 484

                ndone    = ndone    + 1
                ndonez   = ndonez   + 1

                amount = 'minimum'

                call ptzone (0, np, nz, amount, dside, nside, nerra)

                if (nerra .ne. 0) go to 999

                if (nside .eq. 1) then
                  nsidez = nsidez + 1
                  write (   6, 8286) apoint(np), azone(nz)
                  write (iout, 8286) apoint(np), azone(nz)
                  write (   6, 8287) (azquad(nzq,nz), dside(nzq),
     &              nzq = 1, nzquads(nz))
                  write (iout, 8287) (azquad(nzq,nz), dside(nzq),
     &              nzq = 1, nzquads(nz))
                endif

  484         continue

              if (nsidez .ne. 0) then
                write (   6, 8296) apoint(np), nsidez
                write (iout, 8296) apoint(np), nsidez
              endif
  485       continue

c=======================================================================********

          else                        ! Option not recognized.

            nerr = 1
            go to 710

          endif                       ! Tested object option.

        endif                         ! Did one or all zones.

c=======================================================================********

c....   See if any objects were tested.

        if (ndone .eq. 0) then
          nerr = 1
        endif

        go to 710

      endif                           ! Did all points.

c=======================================================================********

 8710 format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized, or no objects tested.')
 8720 format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No points were tested.')
 8730 format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No bricks, tetrahedrons, or zones were tested.')
 8740 format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No bricks were tested.')
 8750 format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No tetrahedrons were tested.')
 8760 format ('  where ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No zones were tested.')

  710 if (nerr .ne. 0) then
        write (   6, 8710) ainput, ninput
        write (iout, 8710) ainput, ninput
      endif

      if (ndonep .eq. 0) then
        write (   6, 8720) ainput, ninput
        write (iout, 8720) ainput, ninput
      endif

      if ((nword .eq. 3) .and. (ndone .eq. 0)) then
        write (   6, 8730) ainput, ninput
        write (iout, 8730) ainput, ninput
      endif

      if (((aword(4) .eq. 'brick') .or.
     &     (aword(4) .eq. 'br'))   .and.
     &     (ndonebr .eq. 0))       then
        write (   6, 8740) ainput, ninput
        write (iout, 8740) ainput, ninput
      endif

      if (((aword(4) .eq. 'tetrahedron') .or.
     &     (aword(4) .eq. 'tetra')       .or.
     &     (aword(4) .eq. 'tet'))       .and.
     &     (ndonetet .eq. 0))            then
        write (   6, 8750) ainput, ninput
        write (iout, 8750) ainput, ninput
      endif

      if (((aword(4) .eq. 'zone') .or.
     &     (aword(4) .eq. 'zn')   .or.
     &     (aword(4) .eq. 'z'))   .and.
     &     (ndonez .eq. 0))       then
        write (   6, 8760) ainput, ninput
        write (iout, 8760) ainput, ninput
      endif

  999 return

c.... End of subroutine where.        (+1 line)
      end
